@@ -1619,6 +1619,7 @@ serial_port_dialog()
 
 #endif /* MAC */
 
-unit_advance_dialog() {}
-global_advance_dialog() {}
-unit_build_dialog(Unit *unit, Task *task) { return 0; }
+unit_advance_dialog(Unit *unit) { auto_pick_new_research(unit); }
+global_advance_dialog(Side *side, Unit *unit, int s) { auto_pick_new_research(unit); }
+unit_build_dialog(Unit *unit) { return auto_pick_new_build_task(unit); }
+unit_plan_dialog(Unit *unit) { auto_pick_new_plan(unit); }
