@@ -600,7 +600,7 @@ Unit *unit;
 	plan_desc(spbuf, unit);
 	draw_text(closeupwin, 0, 4, spbuf);
 	for (task = unit->plan->tasks; task != NULL; task = task->next) {
-	    task_desc(spbuf, unit->side, task);
+	    task_desc(spbuf, unit->side, unit, task);
 	    draw_text(closeupwin, 0, row++, spbuf);
 	}
     }
