@@ -16,26 +16,25 @@ any later version.  See the file COPYING.  */
    but should never call do_foo_action directly. */
 
 #include "conq.h"
-extern int lookup_action_type PARAMS ((char *name));
-extern int type_can_sit_on_conn PARAMS ((int u, int x, int y));
+extern int lookup_action_type(char *name);
+extern int type_can_sit_on_conn(int u, int x, int y);
 #include "kernel.h"
 #include "kpublic.h"
 
-static int check_create_common PARAMS ((Unit *unit, Unit *unit2, int u3,
-					int x, int y));
-static void try_detonate_on_approach PARAMS ((int x, int y));
-static void set_created_unit_props PARAMS ((Unit *unit, int u2, Side *side,
-					    int u2mor));
-static int give_away PARAMS ((Unit *unit, int m, int amt));
-static void notify_action PARAMS ((Unit *unit, Action *action));
-static void play_action_movies PARAMS ((Unit *unit, Action *action));
+static int check_create_common(Unit *unit, Unit *unit2, int u3,
+					int x, int y);
+static void set_created_unit_props(Unit *unit, int u2, Side *side,
+					    int u2mor);
+static int give_away(Unit *unit, int m, int amt);
+static void notify_action(Unit *unit, Action *action);
+static void play_action_movies(Unit *unit, Action *action);
 
 /* We can't declare all the action functions as static because some of them
    are in other files, but don't let them be visible to all files. */
 
 #undef  DEF_ACTION
 #define DEF_ACTION(name,code,args,prepfn,netprepfn,DOFN,checkfn,ARGDECL,doc)  \
-  extern int DOFN PARAMS (ARGDECL);
+  extern int DOFN ARGDECL;
 
 #include "action.def"
 
@@ -44,12 +43,12 @@ static void play_action_movies PARAMS ((Unit *unit, Action *action));
 ActionDefn actiondefns[] = {
 
 #undef  DEF_ACTION
-#define DEF_ACTION(NAME,CODE,ARGS,prepfn,netprepfn,DOFN,CHECKFN,argdecl,doc)  \
-    { CODE, NAME, ARGS, DOFN, CHECKFN },
+#define DEF_ACTION(NAME,CODE,ARGS,prepfn,netprepfn,dofn,checkfn,argdecl,doc)  \
+    { CODE, NAME, ARGS },
 
 #include "action.def"
 
-    { -1, NULL, NULL, NULL }
+    { -1, NULL, NULL }
 };
 
 /* This is used to indicate that a move is a retreat; for normal movement it will
@@ -66,15 +65,14 @@ char *actiondesigbuf = NULL;
 /* Do any action-related initialization. */
 
 void
-init_actions()
+init_actions(void)
 {
 }
 
 /* Just a placeholder action, so not much to do here. */
 
 int
-prep_none_action(unit, unit2)
-Unit *unit, *unit2;
+prep_none_action(Unit *unit, Unit *unit2)
 {
     if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
@@ -84,8 +82,7 @@ Unit *unit, *unit2;
 }
 
 int
-net_prep_none_action(unit, unit2)
-Unit *unit, *unit2;
+net_prep_none_action(Unit *unit, Unit *unit2)
 {
     int rslt;
 
@@ -95,15 +92,13 @@ Unit *unit, *unit2;
 }
 
 int
-do_none_action(unit, unit2)
-Unit *unit, *unit2;
+do_none_action(Unit *unit, Unit *unit2)
 {
     return A_ANY_DONE;
 }
 
 int
-check_none_action(unit, unit2)
-Unit *unit, *unit2;
+check_none_action(Unit *unit, Unit *unit2)
 {
     return A_ANY_DONE;
 }
@@ -420,9 +415,15 @@ int m, amount;
 	amount = min(amount, um_load_max(to->type, m));
     }
     amt2 = min(amount, um_storage_x(to->type, m) - origto);
-    if (um_gives_to_treasury(to->type, m)) {
-	side = (to->side ? to->side : indepside);
-	side->treasury[m] += (amount - amt2);
+
+    /* Only do this if the unit has a real side. Allowing indepside to
+       use a treasury does prevent crashes, but defeats the purpose of
+       having independent cities that should not cooperate with each
+       other. */
+    /* (should rethink this though -sts) */
+    
+    if (to->side != NULL && um_gives_to_treasury(to->type, m)) {
+	to->side->treasury[m] += (amount - amt2);
     } else {
 	amount = amt2;
     }
@@ -589,8 +590,8 @@ int u, oldtech, newtech;
 
 /* Toolup action. */
 
-/* Before a unit can build another, it may need to take some time to prepare by
-   "tooling up". */
+/* Before a unit can build another, it may need to take some time to
+   prepare by "tooling up". */
 
 int
 prep_toolup_action(unit, unit2, u3)
@@ -677,7 +678,7 @@ adjust_tooling_crossover(unit, u2)
 Unit *unit;
 int u2;
 {
-    int u3, uucross, cross, tp2, oldtp3;
+    int u3, uucross, cross, oldtp3;
 
     /* Perhaps nothing to cross over with. */
     if (unit->tooling == NULL)
@@ -735,6 +736,7 @@ int u3;
     return TRUE;
 }
 
+int
 net_prep_create_in_action(unit, unit2, u3, dest)
 Unit *unit, *unit2, *dest;
 int u3;
@@ -755,13 +757,21 @@ int u3;
     Unit *newunit;
 
     /* Doublecheck that the new unit is still allowed. */
-    if (!new_unit_allowed_on_side(u3, unit->side))
+
+    /* has_advance_to_build no longer included by type_allowed_on_side. */
+    if (!new_unit_allowed_on_side(u3, unit->side)
+	|| !has_advance_to_build(unit->side, u3))
       return A_ANY_ERROR;
     /* Make the new unit. */
     newunit = create_unit(u3, FALSE);
     if (newunit != NULL) {
 	/* Fill in various properties. */
 	set_created_unit_props(newunit, u2, unit->side, unit2->morale);
+	/* Transfer the builders stash of cps to newunit if permitted. */
+    	if (uu_builder_can_reuse_cp(u2, u3)) {
+	    newunit->cp += min(unit2->cp_stash, u_cp(u3) - newunit->cp);
+	    unit2->cp_stash -= min(unit2->cp_stash, u_cp(u3) - newunit->cp);
+	}
 	/* Put the new unit inside the designated transport. */
 	enter_transport(newunit, dest);
 	/* Unit might have started out complete. */
@@ -783,7 +793,9 @@ int u3;
 		    consum = unit2->supply[m];
 		}
 		unit2->supply[m] -= consum;
-		unit2->side->treasury[m] -= consum2;
+		/* Null side crash fix. */
+		if (unit2->side)
+		  unit2->side->treasury[m] -= consum2;
 	    }
 	}
 	use_up_acp(unit, uu_acp_to_create(u2, u3));
@@ -831,6 +843,9 @@ int u3;
       return A_ANY_CANNOT_DO;
     if (!can_have_enough_acp(unit, uu_acp_to_create(u2, u3)))
       return A_ANY_CANNOT_DO;
+   /* No longer included in type_allowed_on_side. */
+   if (!has_advance_to_build(unit->side, u3))
+      return A_ANY_CANNOT_DO;
     /* Don't permit the creation of units that are not allowed on this
        side. */
     if (!type_allowed_on_side(u3, unit->side))
@@ -854,7 +869,8 @@ int u3;
         && !uu_occ_can_build(unit2->transport->type, u2))
       return A_ANY_ERROR;
     for_all_material_types(m) {
-	totsup = unit2->supply[m] + unit2->side->treasury[m];
+ 	/* Null side crash fix. */
+	totsup = unit2->supply[m] + (unit2->side != NULL ? unit2->side->treasury[m] : 0);
     	if (totsup < um_to_create(u3, m))
     	  return A_ANY_NO_MATERIAL;
     	if (totsup < um_consumption_on_creation(u3, m))
@@ -878,6 +894,14 @@ Side *side;
     newunit->cp = uu_creation_cp(u2, u3);
     set_unit_side(newunit, side);
     set_unit_origside(newunit, side);
+
+    /* Turn on automation for indeps and AI controlled sides. */      
+    if (indep(newunit) || !side_has_display(side) || side_has_ai(side)) {
+	newunit->autoplan = TRUE;
+     	newunit->autobuild = TRUE;
+     	newunit->autoresearch = TRUE;
+    }
+
     /* Always number the unit when first created. */
     assign_unit_number(newunit);
     /* Set all supplies to their just-created levels. */
@@ -943,18 +967,26 @@ int u3, x, y, z;
     Unit *newunit;
 
     /* Doublecheck that the new unit is still allowed. */
-    if (!new_unit_allowed_on_side(u3, unit->side))
+
+    /* has_advance_to_build no longer included by type_allowed_on_side. */
+    if (!new_unit_allowed_on_side(u3, unit->side)
+	|| !has_advance_to_build(unit->side, u3))
       return A_ANY_ERROR;
     /* Make the new unit. */
     newunit = create_unit(u3, FALSE);
     if (newunit != NULL) {
 	/* Fill in various properties. */
 	set_created_unit_props(newunit, u2, unit->side, unit2->morale);
+	/* Transfer the builders stash of cps to newunit if permitted. */
+    	if (uu_builder_can_reuse_cp(u2, u3)) {
+	    newunit->cp += min(unit2->cp_stash, u_cp(u3) - newunit->cp);
+	    unit2->cp_stash -= min(unit2->cp_stash, u_cp(u3) - newunit->cp);
+	}
 	/* Put it at a correct location. */
 	if (can_occupy_cell(newunit, x, y)) {
 	    enter_cell(newunit, x, y);
 	} else if (can_occupy_cell_without(newunit, x, y, unit2)
-		   && can_occupy(unit2, newunit)) {
+		&& can_occupy(unit2, newunit)) {
 	    /* Let the builder occupy its incomplete work. */
 	    leave_cell(unit2);
 	    enter_cell(newunit, x, y);
@@ -984,7 +1016,9 @@ int u3, x, y, z;
 		    consum = unit2->supply[m];
 		}
 		unit2->supply[m] -= consum;
-		unit2->side->treasury[m] -= consum2;
+		/* Null side crash fix. */
+		if (unit2->side)
+		  unit2->side->treasury[m] -= consum2;
 	    }
 	}
 	use_up_acp(unit, uu_acp_to_create(u2, u3));
@@ -1010,7 +1044,7 @@ int u3, x, y, z;
     /* Check that the desired cell has room for the unit, either by
        stacking, or by having the builder occupy its construction work. */
     if (!(type_can_occupy_cell(u3, x, y)
-          || (can_occupy_type(unit2, u3)
+  	 || (can_occupy_type(unit2, u3)
               && type_can_occupy_cell_without(u3, x, y, unit2))))
       return A_ANY_ERROR;
     t = terrain_at(x, y);
@@ -1027,7 +1061,7 @@ int u3, x, y, z;
 /* This action makes progress on a construction effort, possibly completing
    the new unit and making it available. */
 
-int build_step_consumption PARAMS ((int u2, int u3, int m, int cp));
+int build_step_consumption(int u2, int u3, int m, int cp);
 
 int
 prep_build_action(unit, unit2, newunit)
@@ -1242,7 +1276,15 @@ Unit *unit;
 	compute_acp(unit);
 	if (unit->act->initacp > 0) {
 	    side = (unit->side ? unit->side : indepside);
-	    side->actionvector = add_unit_to_vector(side->actionvector, unit, 0);
+
+	   /* Make_unit_complete may be called by advanced units at
+	      turn 0 before compose_actionvectors has been called! */
+	   if (side->actionvector == NULL) {
+		side->actionvector = make_unit_vector(max(numunits, 100));
+		clear_unit_vector(side->actionvector);
+	   }
+	   side->actionvector =
+	     add_unit_to_vector(side->actionvector, unit, 0);
 	}
     }
     /* Inform all sides that should know about the completion. */
@@ -1255,7 +1297,8 @@ Unit *unit;
 	    observers = add_side_to_set(unit->side, observers);
 	}
     }
-    record_event(H_UNIT_COMPLETED, observers, side_number(unit->side), unit->id);
+    record_event(H_UNIT_COMPLETED, observers, side_number(unit->side),
+		 unit->id);
     /* (should add to any per-side tallies) */
     Dprintf("%s is completed\n", unit_desig(unit));
 }
@@ -1418,7 +1461,7 @@ int m, amt;
     /* Then to any unit in the cell. */
     if (amt > 0) {
 	for_all_stack(unit->x, unit->y, unit2) {
-	    if (unit2 != unit && 1 /* friendly */) {
+	    if (unit2 != unit && unit_trusts_unit(unit, unit2)) {
 		amt = give_away(unit2, m, amt);
 	    }
 	}
@@ -1428,7 +1471,7 @@ int m, amt;
 	for_all_directions(dir) {
 	    if (interior_point_in_dir(unit->x, unit->y, dir, &x1, &y1)) {
 		for_all_stack(x1, y1, unit2) {
-		    if (1 /* friendly */) {
+		    if (unit_trusts_unit(unit, unit2)) {
 			amt = give_away(unit2, m, amt);
 		    }
 		}
@@ -1523,7 +1566,10 @@ int parts;
     part_hp = u_hp(u2) / u_parts(u2);
     if (unit3 == NULL) {
 	/* Doublecheck that the new unit is still allowed. */
-	if (!new_unit_allowed_on_side(u2, unit->side))
+
+    /* has_advance_to_build no longer included by type_allowed_on_side. */
+    if (!new_unit_allowed_on_side(u2, unit->side)
+	|| !has_advance_to_build(unit->side, u2))
 	  return A_ANY_ERROR;
 	/* Create a new unit with parts from unit2. */
 	unit3 = create_unit(u2, TRUE);
@@ -1658,6 +1704,9 @@ int u3;
       return A_ANY_CANNOT_DO;
     if (!can_have_enough_acp(unit, acp))
       return A_ANY_CANNOT_DO;
+    /* No longer included in type_allowed_on_side. */
+    if (!has_advance_to_build(unit2->side, u3))
+      return A_ANY_CANNOT_DO;
     if (!type_allowed_on_side(u3, unit2->side))
       return A_ANY_CANNOT_DO;
     if (!has_enough_acp(unit, acp))
@@ -1792,7 +1841,6 @@ Unit *unit, *unit2;
 int x, y, t;
 {
     int u, u2, oldt, acpr, acpa, m, amt, excess;
-    Side *side;
 
     u = unit->type;  u2 = unit2->type;
     oldt = terrain_at(x, y);
@@ -1894,7 +1942,7 @@ do_add_terrain_action(unit, unit2, x, y, dir, t)
 Unit *unit, *unit2;
 int x, y, dir, t;
 {
-    int u = unit->type, oldval, newval, x1, y1, m, amt;
+    int u = unit->type, oldval, newval, m, amt;
     Side *side;
 
     switch (t_subtype(t)) {
@@ -2010,7 +2058,7 @@ do_remove_terrain_action(unit, unit2, x, y, dir, t)
 Unit *unit, *unit2;
 int x, y, dir, t;
 {
-    int u = unit->type, oldval, newval, x1, y1, m, amt, excess;
+    int u = unit->type, oldval, newval, m, amt, excess;
     Side *side;
 
     switch (t_subtype(t)) {
@@ -2100,23 +2148,12 @@ int x, y, dir, t;
 /* (assumes unit can act in the first place - valid?) */
 
 int
-execute_action(unit, action)
-Unit *unit;
-Action *action;
+execute_action(Unit *unit, Action *action)
 {
     char *argtypestr;
-    int u = unit->type, rslt = A_ANY_ERROR, n, i;
-    long args[4];
-    Unit *unit2, *argunit;
-    Side *argside, *side2;
-#ifdef THINK_C
-    /* Think C can be excessively picky sometimes. */
-    int (*checkfn) PARAMS ((Unit *unit, Unit *unit2, ...));
-    int (*dofn) PARAMS ((Unit *unit, Unit *unit2, ...));
-#else
-    int (*checkfn) ();
-    int (*dofn) ();
-#endif
+    int u = unit->type, rslt = A_ANY_ERROR, n, i, a[4];
+    Unit *unit2, *aunits[4];
+    Side *asides[4];
     extern int numsoundplays;
 
     Dprintf("%s doing %s with %d acp left\n",
@@ -2137,26 +2174,22 @@ Action *action;
 	  case 'y':
 	  case 'z':
 	  case 'd':
-	    args[i] = action->args[i];
+	    a[i] = action->args[i];
 	    break;
 	  case 'U':
-	    argunit = find_unit(action->args[i]);
+	    aunits[i] = find_unit(action->args[i]);
 	    /* It may be that the argunit cannot be found - perhaps the
 	       unit died before the action could be applied to it.  In any
 	       case, let the per-action code handle the case. */
-	    args[i] = (long) argunit;
 	    break;
 	  case 'S':
-	    argside = side_n(action->args[i]);
-	    args[i] = (long) argside;
+	    asides[i] = side_n(action->args[i]);
 	    break;
 	  default:
 	    case_panic("action argument type", argtypestr[i]);
 	    break;
 	}
     }
-    checkfn = actiondefns[(int) action->type].checkfn;
-    dofn = actiondefns[(int) action->type].dofn;
     if (action->actee == 0) {
 	unit2 = unit;
     } else {
@@ -2165,95 +2198,221 @@ Action *action;
     if (unit2 == NULL) {
 	return A_ANY_ERROR;
     }
-    switch (n) {
-      case 0:
-	rslt = (*checkfn)(unit, unit2);
+    switch (actiondefns[(int) action->type].typecode) {
+      case ACTION_NONE:
+	rslt = check_none_action(unit, unit2);
 	break;
-      case 1:
-	rslt = (*checkfn)(unit, unit2, args[0]);
+      case ACTION_MOVE:
+	rslt = check_move_action(unit, unit2, a[0], a[1], a[2]);
 	break;
-      case 2:
-	rslt = (*checkfn)(unit, unit2, args[0], args[1]);
+      case ACTION_ENTER:
+	rslt = check_enter_action(unit, unit2, aunits[0]);
 	break;
-      case 3:
-	rslt = (*checkfn)(unit, unit2, args[0], args[1], args[2]);
+      case ACTION_ATTACK:
+	rslt = check_attack_action(unit, unit2, aunits[0], a[1]);
 	break;
-      case 4:
-	rslt = (*checkfn)(unit, unit2, args[0], args[1], args[2], args[3]);
+      case ACTION_OVERRUN:
+	rslt = check_overrun_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_FIRE_AT:
+	rslt = check_fire_at_action(unit, unit2, aunits[0], a[1]);
+	break;
+      case ACTION_FIRE_INTO:
+	rslt = check_fire_into_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_CAPTURE:
+	rslt = check_capture_action(unit, unit2, aunits[0]);
+	break;
+      case ACTION_DETONATE:
+	rslt = check_detonate_action(unit, unit2, a[0], a[1], a[2]);
+	break;
+      case ACTION_PRODUCE:
+	rslt = check_produce_action(unit, unit2, a[0], a[1]);
+	break;
+      case ACTION_EXTRACT:
+	rslt = check_extract_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_TRANSFER:
+	rslt = check_transfer_action(unit, unit2, a[0], a[1], aunits[2]);
+	break;
+      case ACTION_DEVELOP:
+	rslt = check_develop_action(unit, unit2, a[0]);
+	break;
+      case ACTION_TOOL_UP:
+	rslt = check_toolup_action(unit, unit2, a[0]);
+	break;
+      case ACTION_CREATE_IN:
+	rslt = check_create_in_action(unit, unit2, a[0], aunits[1]);
+	break;
+      case ACTION_CREATE_AT:
+	rslt = check_create_at_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_BUILD:
+	rslt = check_build_action(unit, unit2, aunits[0]);
+	break;
+      case ACTION_REPAIR:
+	rslt = check_repair_action(unit, unit2, aunits[0]);
+	break;
+      case ACTION_DISBAND:
+	rslt = check_disband_action(unit, unit2);
+	break;
+      case ACTION_TRANSFER_PART:
+	rslt = check_transfer_part_action(unit, unit2, a[0], aunits[1]);
+	break;
+      case ACTION_CHANGE_TYPE:
+	rslt = check_change_type_action(unit, unit2, a[0]);
+	break;
+      case ACTION_CHANGE_SIDE:
+	rslt = check_change_side_action(unit, unit2, asides[0]);
+	break;
+      case ACTION_ALTER_TERRAIN:
+	rslt = check_alter_cell_action(unit, unit2, a[0], a[1], a[2]);
+	break;
+      case ACTION_ADD_TERRAIN:
+	rslt = check_add_terrain_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_REMOVE_TERRAIN:
+	rslt = check_remove_terrain_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case NUMACTIONTYPES:
+	/* should be error */
 	break;
-      default:
-	case_panic("action arg count", n);
     }
     numsoundplays = 0; /* kind of a hack */
-    if (valid(rslt)) {
-	if (g_action_notices() != lispnil)
-	  notify_action(unit, action);
-	if (g_action_movies() != lispnil)
-	  play_action_movies(unit, action);
-	switch (n) {
-	  case 0:
-	    rslt = (*dofn)(unit, unit2);
-	    break;
-	  case 1:
-	    rslt = (*dofn)(unit, unit2, args[0]);
-	    break;
-	  case 2:
-	    rslt = (*dofn)(unit, unit2, args[0], args[1]);
-	    break;
-	  case 3:
-	    rslt = (*dofn)(unit, unit2, args[0], args[1], args[2]);
-	    break;
-	  case 4:
-	    rslt = (*dofn)(unit, unit2, args[0], args[1], args[2], args[3]);
-	    break;
-	  default:
-	    case_panic("action arg count", n);
-	}
-	Dprintf("%s action %s result is %s, %d acp left\n",
-		unit_desig(unit), action_desig(action), hevtdefns[rslt].name,
-		(unit->act ? unit->act->acp : -9999));
-	if (unit->plan) {
-	    unit->plan->lastaction = *action;
-	    unit->plan->lastresult = rslt;
-	}
-	if (unit->side && side_has_display(unit->side)) {
-	    update_action_result_display(unit->side, unit, rslt, TRUE);
-	}
-	/* Show other sides that some action has occurred. */
-	update_side_display_all_sides(unit->side, TRUE);
-	/* If of a type that might be spotted if it does anything,
-	   check each side to see if they notice.  Note that the type
-	   is from *before* the action, not after (some actions may
-	   cause type changes). */
-	if (u_spot_action(u)
-	    && !all_see_all
-	    && !u_see_always(u)
-	    && in_area(unit->x, unit->y)) {
-	    for_all_sides(side2) {
-		if (cover(side2, unit->x, unit->y) > 0) {
-		    /* (should call some sort of "glimpsing" routine) */
-		}
-	    }
-	}
-	/* Check on any scorekeepers that run after each action.  Note
-	   that one or more sides may be out of the game after this
-	   returns. */
-	if (any_post_action_scores)
-	  check_post_action_scores();
-    } else {
+    if (!valid(rslt)) {
 	if (unit->plan) {
 	    unit->plan->lastaction = *action;
 	    unit->plan->lastresult = rslt;
 	}
 	Dprintf("%s action %s can't be done, result is %s\n",
 		unit_desig(unit), action_desig(action), hevtdefns[rslt].name);
+	return rslt;
     }
+    if (g_action_notices() != lispnil)
+      notify_action(unit, action);
+    if (g_action_movies() != lispnil)
+      play_action_movies(unit, action);
+    switch (actiondefns[(int) action->type].typecode) {
+      case ACTION_NONE:
+	rslt = do_none_action(unit, unit2);
+	break;
+      case ACTION_MOVE:
+	rslt = do_move_action(unit, unit2, a[0], a[1], a[2]);
+	break;
+      case ACTION_ENTER:
+	rslt = do_enter_action(unit, unit2, aunits[0]);
+	break;
+      case ACTION_ATTACK:
+	rslt = do_attack_action(unit, unit2, aunits[0], a[1]);
+	break;
+      case ACTION_OVERRUN:
+	rslt = do_overrun_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_FIRE_AT:
+	rslt = do_fire_at_action(unit, unit2, aunits[0], a[1]);
+	break;
+      case ACTION_FIRE_INTO:
+	rslt = do_fire_into_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_CAPTURE:
+	rslt = do_capture_action(unit, unit2, aunits[0]);
+	break;
+      case ACTION_DETONATE:
+	rslt = do_detonate_action(unit, unit2, a[0], a[1], a[2]);
+	break;
+      case ACTION_PRODUCE:
+	rslt = do_produce_action(unit, unit2, a[0], a[1]);
+	break;
+      case ACTION_EXTRACT:
+	rslt = do_extract_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_TRANSFER:
+	rslt = do_transfer_action(unit, unit2, a[0], a[1], aunits[2]);
+	break;
+      case ACTION_DEVELOP:
+	rslt = do_develop_action(unit, unit2, a[0]);
+	break;
+      case ACTION_TOOL_UP:
+	rslt = do_toolup_action(unit, unit2, a[0]);
+	break;
+      case ACTION_CREATE_IN:
+	rslt = do_create_in_action(unit, unit2, a[0], aunits[1]);
+	break;
+      case ACTION_CREATE_AT:
+	rslt = do_create_at_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_BUILD:
+	rslt = do_build_action(unit, unit2, aunits[0]);
+	break;
+      case ACTION_REPAIR:
+	rslt = do_repair_action(unit, unit2, aunits[0]);
+	break;
+      case ACTION_DISBAND:
+	rslt = do_disband_action(unit, unit2);
+	break;
+      case ACTION_TRANSFER_PART:
+	rslt = do_transfer_part_action(unit, unit2, a[0], aunits[1]);
+	break;
+      case ACTION_CHANGE_TYPE:
+	rslt = do_change_type_action(unit, unit2, a[0]);
+	break;
+      case ACTION_CHANGE_SIDE:
+	rslt = do_change_side_action(unit, unit2, asides[0]);
+	break;
+      case ACTION_ALTER_TERRAIN:
+	rslt = do_alter_cell_action(unit, unit2, a[0], a[1], a[2]);
+	break;
+      case ACTION_ADD_TERRAIN:
+	rslt = do_add_terrain_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case ACTION_REMOVE_TERRAIN:
+	rslt = do_remove_terrain_action(unit, unit2, a[0], a[1], a[2], a[3]);
+	break;
+      case NUMACTIONTYPES:
+	/* should be error */
+	break;
+    }
+    Dprintf("%s action %s result is %s, %d acp left\n",
+	    unit_desig(unit), action_desig(action), hevtdefns[rslt].name,
+	    (unit->act ? unit->act->acp : -9999));
+    if (unit->plan) {
+	unit->plan->lastaction = *action;
+	unit->plan->lastresult = rslt;
+    }
+    /* Report the result to the unit's owner. */
+    if (unit->side && side_has_display(unit->side)) {
+	update_action_result_display(unit->side, unit, rslt, TRUE);
+    }
+    /* Show other sides that some action has occurred. */
+    update_side_display_all_sides(unit->side, TRUE);
+#if 0 /* doesn't do anything yet */
+    /* If of a type that might be spotted if it does anything, check
+       each side to see if they notice.  Note that the type is from
+       *before* the action, not after (some actions may cause type
+       changes). */
+    if (u_spot_action(u)
+	&& !all_see_all
+	&& !u_see_always(u)
+	&& in_area(unit->x, unit->y)) {
+	Side *side2;
+	for_all_sides(side2) {
+	    if (cover(side2, unit->x, unit->y) > 0) {
+		/* (should call some sort of "glimpsing" routine) */
+	    }
+	}
+    }
+#endif
+    /* Check on any scorekeepers that run after each action.  Note
+       that one or more sides may be out of the game after this
+       returns. */
+    if (any_post_action_scores)
+      check_post_action_scores();
     /* Return success/failure so caller can use. */
     return rslt;
 }
 
-static int pattern_matches_action PARAMS ((Obj *pattern, Unit *unit, Action *action));
-static void action_desc_from_list PARAMS ((Side *side, Obj *lis, Unit *unit, Action *action, char *buf));
+static int pattern_matches_action(Obj *pattern, Unit *unit, Action *action);
+static void action_desc_from_list(Side *side, Obj *lis, Unit *unit, Action *action, char *buf);
 
 static void
 notify_action(unit, action)
@@ -2261,8 +2420,8 @@ Unit *unit;
 Action *action;
 {
     int found = FALSE;
-    char *atypename, *soundname, abuf[BUFSIZE];
-    Obj *rest, *head, *pat, *parms, *msgdesc;
+    char *atypename, abuf[BUFSIZE];
+    Obj *rest, *head, *pat, *msgdesc;
 
     atypename = actiondefns[(int) action->type].name;
     for_all_list(g_action_notices(), rest) {
@@ -2297,12 +2456,9 @@ Action *action;
 }
 
 static int
-pattern_matches_action(parms, unit, action)
-Obj *parms;
-Unit *unit;
-Action *action;
+pattern_matches_action(Obj *parms, Unit *unit, Action *action)
 {
-    Obj *rest, *head;
+    Obj *head;
     Unit *actee;
 
     head = car(parms);
