@@ -10,13 +10,9 @@ any later version.  See the file COPYING.  */
 #include "kernel.h"
 /* (should do a better separation of kernel in set_side_ai) */
 #include "kpublic.h"
-extern void try_to_draw PARAMS ((Side *side, int flag, char *ainame));
-extern void give_up PARAMS ((Side *side, char *ainame));
-
-/* The number of AI types defined.  Increment for each new AI type
-   that is compiled in. */
-
-#define numaitypes 2
+extern void try_to_draw(Side *side, int flag, char *ainame);
+extern void give_up(Side *side, char *ainame);
+#include "ai.h"
 
 /* Declarations of all AI types. */
 
@@ -39,8 +35,8 @@ Side *side;
 {
     int i;
     char *typename;
-    int (*test) PARAMS ((void));
-    void (*fn) PARAMS ((Side *side));
+    int (*test)(void);
+    void (*fn)(Side *side);
 
     if (side_wants_ai(side)) {
 	/* (should merge with same bit in set_side_ai) */
@@ -108,14 +104,25 @@ char *typename;
 	side->ai = NULL;
     } else {
 	init_ai(side);
+	/* Turn on automation for all units when switching on the
+	   side's AI. Note that the reverse is NOT true. Switching
+	   from AI to human control will therefore leave all units on
+	   automatic. */
+	for_all_side_units(side, unit) {
+	    unit->autoplan = TRUE;
+	    unit->autoresearch = TRUE;
+	    unit->autobuild = TRUE;
+	}
     }
 }
 
+/* The following functions dispatch on AI type. */
+
 void
 ai_init_turn(side)
 Side *side;
 {
-    void (*fn) PARAMS ((Side *side));
+    void (*fn)(Side *side);
 
     fn = (all_ai_ops[side->aitype])->to_init_turn;
     if (fn)
@@ -127,7 +134,7 @@ ai_decide_plan(side, unit)
 Side *side;
 Unit *unit;
 {
-    void (*fn) PARAMS ((Side *side, Unit *unit));
+    void (*fn)(Side *side, Unit *unit);
 
     fn = ((all_ai_ops[side->aitype])->to_decide_plan);
     if (fn)
@@ -142,7 +149,7 @@ Side *side;
 Unit *unit;
 int rslt;
 {
-    void (*fn) PARAMS ((Side *side, Unit *unit, int rslt));
+    void (*fn)(Side *side, Unit *unit, int rslt);
 
     fn = ((all_ai_ops[side->aitype])->to_react_to_action_result);
     if (fn)
@@ -158,7 +165,7 @@ Unit *unit;
 Task *task;
 TaskOutcome rslt;
 {
-    void (*fn) PARAMS ((Side *side, Unit *unit, Task *task, TaskOutcome rslt));
+    void (*fn)(Side *side, Unit *unit, Task *task, TaskOutcome rslt);
 
     fn = ((all_ai_ops[side->aitype])->to_react_to_task_result);
     if (fn)
@@ -169,7 +176,7 @@ void
 ai_react_to_new_side(side, side2)
 Side *side, *side2;
 {
-    void (*fn) PARAMS ((Side *side, Side *side2));
+    void (*fn)(Side *side, Side *side2);
 
     fn = ((all_ai_ops[side->aitype])->to_react_to_new_side);
     if (fn)
@@ -181,7 +188,7 @@ ai_adjust_plan(side, unit)
 Side *side;
 Unit *unit;
 {
-    int (*fn) PARAMS ((Side *side, Unit *unit));
+    int (*fn)(Side *side, Unit *unit);
 
     fn = ((all_ai_ops[side->aitype])->to_adjust_plan);
     if (fn)
@@ -190,11 +197,35 @@ Unit *unit;
       return FALSE;
 }
 
+int
+ai_update_plan(Side *side, Unit *unit)
+{
+    int (*fn)(Unit *unit);
+
+    fn = ((all_ai_ops[side->aitype])->to_update_plan);
+    if (fn)
+      return (*fn)(unit);
+    else
+      return FALSE;
+}
+
+int
+ai_handle_emergency(Side *side, Unit *unit)
+{
+    int (*fn)(Unit *unit);
+
+    fn = ((all_ai_ops[side->aitype])->to_handle_emergency);
+    if (fn)
+      return (*fn)(unit);
+    else
+      return FALSE;
+}
+
 void
 ai_finish_movement(side)
 Side *side;
 {
-    void (*fn) PARAMS ((Side *side));
+    void (*fn)(Side *side);
 
     fn = ((all_ai_ops[side->aitype])->to_finish_movement);
     if (fn)
@@ -208,7 +239,7 @@ ai_receive_message(side, sender, str)
 Side *side, *sender;
 char *str;
 {
-    void (*fn) PARAMS ((Side *side, Side *sender, char *str));
+    void (*fn)(Side *side, Side *sender, char *str);
 
     fn = ((all_ai_ops[side->aitype])->to_receive_message);
     if (fn)
@@ -219,7 +250,7 @@ void
 ai_save_state(side)
 Side *side;
 {
-    Obj *(*fn) PARAMS ((Side *side)), *state = lispnil;
+    Obj *(*fn)(Side *side), *state = lispnil;
 
     fn = ((all_ai_ops[side->aitype])->to_save_state);
     if (fn)
@@ -227,15 +258,20 @@ Side *side;
     /* Don't bother if there's no AI state to mess with. */
     if (side->aidata == lispnil && state == lispnil)
       return;
-    side->aidata = replace_at_key(side->aidata, side->player->aitypename, state);
+    side->aidata = replace_at_key(side->aidata, side->player->aitypename,
+				  state);
 }
 
+/* Return a numerical value indicate the AI's use of the given location.
+   For instance, each number could represent a region or theater of
+   operations. */
+
 int
 ai_region_at(side, x, y)
 Side *side;
 int x, y;
 {
-    int (*fn) PARAMS ((Side *side, int x, int y));
+    int (*fn)(Side *side, int x, int y);
 
     fn = ((all_ai_ops[side->aitype])->region_at);
     if (fn)
@@ -244,12 +280,15 @@ int x, y;
       return 0;
 }
 
+/* Provide textual information about the AI's usage or opinion of the
+   given location. */
+
 char *
 ai_at_desig(side, x, y)
 Side *side;
 int x, y;
 {
-    char *(*fn) PARAMS ((Side *side, int x, int y));
+    char *(*fn)(Side *side, int x, int y);
 
     fn = ((all_ai_ops[side->aitype])->at_desig);
     if (fn)
@@ -258,11 +297,10 @@ int x, y;
       return NULL;
 }
 
+/* Set willingness to declare a draw. */
+
 void
-try_to_draw(side, flag, ainame)
-Side *side;
-int flag;
-char *ainame;
+try_to_draw(Side *side, int flag, char *ainame)
 {
     /* If no change, nothing to say or do. */
     if (flag == side->willingtodraw)
@@ -284,24 +322,21 @@ char *ainame;
 /* If an AI resigns, it tries to help its friends. */
 
 void
-give_up(side, ainame)
-Side *side;
-char *ainame;
+give_up(Side *side, char *ainame)
 {
     Side *side1;
 
     /* Can't resign if we've been forbidden to do so. */
-    if (!side->ai_may_resign) {
-	/* If there is a human player, speak up. */
-	if (side_has_display(side))
-	  notify(side, "Your AI %s recommends resignation", ainame);
+    if (side_has_display(side) && !side->ai_may_resign) {
+	/* But speak up about it. */
+	notify(side, "Your AI %s recommends resignation", ainame);
 	return;
     }
     /* Try to give away all of our units to an ally. */
     for_all_sides(side1) {
 	if (side != side1 && allied_side(side, side1) && side1->ingame) {
 	    if (side_has_display(side))
-	      notify(side, "Your AI resigns, giving everthing to %s!",
+	      notify(side, "Your AI resigns, giving its assets to %s!",
 		     short_side_title(side1));
 	    net_resign_game(side, side1);
 	    return;
@@ -422,6 +457,10 @@ Goal *goal;
 	}
       case GOAL_CELL_OCCUPIED:
 	return 0;
+      case GOAL_UNIT_OCCUPIED:
+	return 0;
+      case GOAL_COLONIZE:
+	return 0;
       case GOAL_HAS_UNIT_TYPE:
 	return 0;
       case GOAL_HAS_UNIT_TYPE_NEAR:
@@ -542,6 +581,15 @@ ai_init_shared()
 	set_u_is_base_builder(u1, FALSE);
 	set_u_can_make(u1, FALSE);
 	set_u_can_capture(u1, FALSE);
+	set_u_is_ground_mobile(u1, FALSE);
+	set_u_is_naval_mobile(u1, FALSE);
+	set_u_is_air_mobile(u1, FALSE);
+	set_u_is_colonizer(u1, FALSE);
+	set_u_is_facility(u1, FALSE);
+
+	/* DON'T set u_advanced to FALSE. It is already set by the
+           game file. */
+
 	numbuilders = 0;
 /*	ave_build_time[u1] = 0;  */
 	for_all_unit_types(u2) {
@@ -557,6 +605,9 @@ ai_init_shared()
 		numbuilders++;
 /*		ave_build_time[u1] += uu_make(u2,u1);  */
 		set_u_can_make(u2, TRUE);
+		/* A colonizer is a mobile builder of advanced units. */
+		if (u_advanced(u2) && mobile(u1))
+		  set_u_is_colonizer(u1, TRUE);
 	    }
 	    if (uu_capture(u1, u2) > 0 || uu_indep_capture(u1, u2) > 0) {
 		set_u_can_capture(u1, TRUE);
@@ -582,6 +633,49 @@ ai_init_shared()
     }
     for_all_unit_types(u) {
 	set_u_bw(u, basic_worth(u));
+	set_u_offensive_worth(u, offensive_worth(u));
+	set_u_defensive_worth(u, defensive_worth(u));
+	set_u_explorer_worth(u, exploring_worth(u));
+	set_u_colonizer_worth(u, colonizing_worth(u));
+	set_u_facility_worth(u, facility_worth(u));
+	set_u_random_worth(u, random_worth(u));
+	set_u_siege_worth(u, siege_worth(u));
+
+	if (mobile(u)) {
+	    int maxacp = u_acp(u);
+		
+	    if (u_acp_turn_max(u) >= 0)
+	      maxacp = min(maxacp, u_acp_turn_max(u));
+	    maxacp = (u_acp_max(u) < 0 ? maxacp : u_acp_max(u));
+	    /* Assume this is an air unit by default. */
+	    set_u_is_air_mobile(u, TRUE);
+	    for_all_terrain_types(t) {
+	        /* Check that our unit can have enough mps to enter the
+		   terrain. */
+	        if (ut_mp_to_enter(u, t) * u_acp_to_move(u) <= maxacp	
+		    /* And that there is room for our unit in the terrain. */
+		    && ut_size(u, t) <= t_capacity(t)
+		    /* And that it does not vanish in the terrain. */	
+		    && ut_vanishes_on(u, t) != TRUE) {
+		    /* It is naval mobile if it can enter at least one
+		       liquid terrain. */
+		    if (t_liquid(t))
+		      set_u_is_naval_mobile(u, TRUE);
+		    /* It is ground mobile if it can enter at least one
+		       non-liquid terrain. */
+		    if (!t_liquid(t))
+		      set_u_is_ground_mobile(u, TRUE);
+		    /* Air units should have zero size in all terrains. */
+		    if (ut_size(u, t) > 0)
+		      set_u_is_air_mobile(u, FALSE);
+		} else {
+		    /* Air units should be able to enter all terrains. */
+		    set_u_is_air_mobile(u, FALSE);
+	        }
+	    }
+	/* Kind of crude, but OK for now. */
+	} else
+	  set_u_is_facility(u, TRUE);
     }
     for_all_unit_types(u) {
 	for_all_unit_types(u2) {
@@ -598,8 +692,8 @@ ai_init_shared()
       display_assessment();
 }
 
-int basic_transport_worth PARAMS ((int u1, int u2));
-void set_uu_btw PARAMS ((int u1, int u2, int v));
+int basic_transport_worth(int u1, int u2);
+void set_uu_btw(int u1, int u2, int v);
 
 #define DICE(N,NUMDICE,SPOTS,OFFSET)  \
   (((N) >> 14 == 0 || (N) >> 14 == 3) ?  \
@@ -643,8 +737,131 @@ int u;
     return worth;
 }
 
-/* A basic estimate of the payoff of one unit type attacking another type
-   directly.  This is "context-free", does not account for overall goals etc. */
+int
+offensive_worth(int u)
+{
+    int offvalue = 0;
+    int defenders = 0;
+    int worth = 0;
+    int u2;
+
+    for_all_unit_types(u2) {
+	if (uu_acp_to_attack(u, u2) > 0) {
+	    ++defenders;
+	    offvalue += uu_hit(u, u2) * uu_damage(u, u2);
+	}
+	if (u_acp_to_fire(u) > 0) {
+	    ++defenders;
+	    offvalue += (uu_fire_hit(u, u2) != -1 ? uu_fire_hit(u, u2) : uu_hit(u, u2))
+	      * (uu_fire_damage(u, u2) != -1 ? uu_fire_damage(u, u2) : uu_damage(u, u2));
+	}
+    }
+    if (defenders > 0)
+      offvalue /= defenders;
+    worth =  u_acp(u) * offvalue * u_range(u) * u_speed(u) / 100;
+    DMprintf("unit type %s offensive worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+defensive_worth(int u)
+{
+    int defvalue = 0;
+    int attackers = 0;
+    int worth = 0;
+    int u2;
+
+    for_all_unit_types(u2) {
+	if (uu_acp_to_attack(u2, u) > 0) {
+	    ++attackers;
+	    defvalue += 100 * u_hp_max(u) - uu_hit(u2, u) * uu_damage(u2, u);
+	}
+	if (u_acp_to_fire(u2) > 0) {
+	    ++attackers;   
+	    defvalue += 100 * u_hp_max(u) - (uu_fire_hit(u2, u) != -1 ? uu_fire_hit(u2, u) : uu_hit(u2, u)) 
+	      * (uu_fire_damage(u2, u) != -1 ? uu_fire_damage(u2, u) : uu_damage(u2, u));
+	}
+    }
+    if (attackers > 0)
+      defvalue /= attackers;
+    worth = u_acp(u) * defvalue / 10;
+    DMprintf("unit type %s defensive worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+exploring_worth(int u)
+{
+    int worth = u_acp(u) * u_speed(u) / 100;
+
+    DMprintf("unit type %s exploring worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+colonizing_worth(int u)
+{
+    int colvalue = 0;
+    int worth = 0;
+    int u2;
+ 
+    /* A colonizer is a mobile builder of advanced units. */
+    if (!mobile(u))
+      return 0;
+    for_all_unit_types(u2) {
+	if (could_create(u, u2)
+	    && u_advanced(u2)) {
+	    /* It is more valuable if it can build bigger cities. */
+	    colvalue += u_reach(u2) + 1;
+	}
+    }    	
+    worth = u_acp(u) * u_speed(u) * colvalue / 10;
+    DMprintf("unit type %s colonizing worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+facility_worth(int u)
+{
+    int worth = 0;
+
+    /* Kind of crude, but OK for now. */
+    if (!mobile(u))
+      worth = 1;
+
+    DMprintf("unit type %s facility worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+int
+siege_worth(int u)
+{
+    int siegevalue = 0;
+    int worth = 0;
+    int u2;
+ 
+    for_all_unit_types(u2) {
+	siegevalue += uu_capture(u, u2);
+    }           
+    siegevalue = siegevalue / numutypes;
+ 
+    worth = u_acp(u) * u_range(u) * siegevalue; 
+
+    DMprintf("unit type %s siege worth %d \n ", u_type_name(u), worth);
+    return worth;
+}
+
+/* Assign the same worth to all unit types. */
+
+int
+random_worth(int u)
+{
+    return 1;
+}
+
+/* A basic estimate of the payoff of one unit type attacking another
+   type directly.  This is "context-free", does not account for
+   overall goals etc. */
 
 /* (should account for number of attacks possible in one turn) */
 
@@ -814,6 +1031,21 @@ void set_u_can_make(u, n) int u, n; {  utypes[u].can_make = n;  }
 void set_u_can_capture(u, n) int u, n; {  utypes[u].can_capture = n;  }
 void set_u_bw(u, n) int u, n; {  utypes[u].bw = n;  }
 
+void set_u_offensive_worth(u, n) int u, n; {  utypes[u].ow = n;  }
+void set_u_defensive_worth(u, n) int u, n; {  utypes[u].dw = n;  }
+void set_u_explorer_worth(u, n) int u, n; {  utypes[u].ew = n;  }
+void set_u_colonizer_worth(u, n) int u, n; {  utypes[u].cw = n;  }
+void set_u_facility_worth(u, n) int u, n; {  utypes[u].fw = n;  }
+void set_u_random_worth(u, n) int u, n; {  utypes[u].rw = n;  }
+void set_u_siege_worth(u, n) int u, n; {  utypes[u].sw = n;  }
+
+void set_u_is_ground_mobile(u, n) int u, n; {  utypes[u].is_ground_mobile = n;  }
+void set_u_is_naval_mobile(u, n) int u, n; {  utypes[u].is_naval_mobile = n;  }
+void set_u_is_air_mobile(u, n) int u, n; {  utypes[u].is_air_mobile = n;  }
+void set_u_is_advanced(u, n) int u, n; {  utypes[u].is_advanced = n;  }
+void set_u_is_colonizer(u, n) int u, n; {  utypes[u].is_colonizer = n;  }
+void set_u_is_facility(u, n) int u, n; {  utypes[u].is_facility = n;  }
+	
 int bhwtab = -1;
 int bfwtab = -1;
 int bcwtab = -1;
