@@ -1,5 +1,5 @@
 /* Command line parsing for Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,15 +16,15 @@ any later version.  See the file COPYING.  */
 extern short initially_no_ai;
 #include "cmdline.h"
 
-static void version_info PARAMS ((void));
-static void general_usage_info PARAMS ((void));
-static void game_usage_info PARAMS ((void));
-static void unixify_variant_name PARAMS ((char *buf, char *varname));
-static void player_usage_info PARAMS ((void));
-static void parse_world_option PARAMS ((char *str));
-static void parse_realtime_option PARAMS ((char *subopt, char *arg));
-static void parse_variant PARAMS ((char *str));
-static void add_a_raw_player_spec PARAMS ((char *specstr));
+static void version_info(void);
+static void general_usage_info(void);
+static void game_usage_info(void);
+static void unixify_variant_name(char *buf, char *varname);
+static void player_usage_info(void);
+static void parse_world_option(char *str);
+static void parse_realtime_option(char *subopt, char *arg);
+static void parse_variant(char *str);
+static void add_a_raw_player_spec(char *specstr);
 
 /* The startup-settable options. */
 
@@ -73,8 +73,8 @@ struct gdl_spec {
   struct gdl_spec *next;
 } *pre_gdl_specs, *last_pre_gdl_spec, *post_gdl_specs, *last_post_gdl_spec;
 
-static void add_a_gdl_spec PARAMS ((char *specstr, struct gdl_spec **first,
-				    struct gdl_spec **last));
+static void add_a_gdl_spec(char *specstr, struct gdl_spec **first,
+				    struct gdl_spec **last);
 
 /* The list of accumulated variant choices. */
 
@@ -207,7 +207,9 @@ char *argv[];
 	    } else if (strcmp(arg, "-h") == 0) {
 		REQUIRE_ONE_ARG;
 		PLAYER_OPTION;
-		option_num_to_wait_for = atoi(argv[i+1]);
+		n = atoi(argv[i+1]);
+		while (n-- > 0)
+		  add_a_raw_player_spec("x@");
 	    } else if (strcmp(arg, "-help") == 0) {
 		GENERAL_OPTION;
 		help_wanted = TRUE;
@@ -798,10 +800,11 @@ set_players_from_options()
 void
 print_instructions()
 {
-    Obj *instructions = mainmodule->instructions, *rest;
+    Obj *instructions, *rest;
 
     printf("\n");
-    if (instructions != lispnil) {
+    if (mainmodule != NULL
+	&& (instructions = mainmodule->instructions) != lispnil) {
 	if (stringp(instructions)) {
 	    printf("%s\n", c_string(instructions));
 	} else if (consp(instructions)) {
