@@ -10,35 +10,31 @@ any later version.  See the file COPYING.  */
 #include "kernel.h"
 #include "kpublic.h"
 
-static int one_attack PARAMS ((Unit *atker, Unit *defender));
-static void fire_on_unit PARAMS ((Unit *atker, Unit *other));
-static void attack_unit PARAMS ((Unit *atker, Unit *other));
-static void maybe_hit_unit PARAMS ((Unit *atker, Unit *other, int fire,
-				    int fallsoff));
-static void hit_unit PARAMS ((Unit *unit, int hit, Unit *atker));
+static int one_attack(Unit *atker, Unit *defender);
+static void fire_on_unit(Unit *atker, Unit *other);
+static void attack_unit(Unit *atker, Unit *other);
+static void maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff);
+static void hit_unit(Unit *unit, int hit, Unit *atker);
 
-static void reckon_damage PARAMS ((int fire));
-static void reckon_damage_here PARAMS ((int x, int y));
-static void report_damage PARAMS ((Unit *unit, Unit *atker, Unit *mainunit));
-static int will_report_damage PARAMS ((Unit *unit));
-static void rescue_one_occupant PARAMS ((Unit *occ));
-static int retreat_unit PARAMS ((Unit *unit, Unit *atker));
-static int retreat_in_dir PARAMS ((Unit *unit, int dir));
-static void attempt_to_capture_unit PARAMS ((Unit *atker, Unit *other));
-static void capture_unit_2 PARAMS ((Unit *unit, Unit *pris,
-				    Side *prevside));
-static void capture_occupant PARAMS ((Unit *unit, Unit *pris, Unit *occ,
-				      Side *newside));
-static void detonate_on_cell PARAMS ((int x, int y));
-static void hit_unit_with_detonation PARAMS ((Unit *unit, int hit,
-					      Unit *atker));
+static void reckon_damage(int fire);
+static void reckon_damage_here(int x, int y);
+static void report_damage(Unit *unit, Unit *atker, Unit *mainunit);
+static int will_report_damage(Unit *unit);
+static void rescue_one_occupant(Unit *occ);
+static int retreat_unit(Unit *unit, Unit *atker);
+static int retreat_in_dir(Unit *unit, int dir);
+static void attempt_to_capture_unit(Unit *atker, Unit *other);
+static void capture_unit_2(Unit *unit, Unit *pris, Side *prevside);
+static void capture_occupant(Unit *unit, Unit *pris, Unit *occ, Side *newside);
+static void detonate_on_cell(int x, int y);
+static void hit_unit_with_detonation(Unit *unit, int hit, Unit *atker);
 
 /* We can't declare all the action functions as static because some of them
    are in other files, but don't let them be visible to all files. */
 
 #undef  DEF_ACTION
 #define DEF_ACTION(name,code,args,prepfn,netprepfn,DOFN,checkfn,ARGDECL,doc)  \
-  extern int DOFN PARAMS (ARGDECL);
+  extern int DOFN ARGDECL;
 
 #include "action.def"
 
@@ -251,14 +247,23 @@ int x, y, z, n;
 		gotonext = TRUE;
 	    }
 	}
-	if (!gotonext)
-	  one_attack(unit2, defender);
-	if (alive(unit))
-	  use_up_acp(unit, uu_acp_to_attack(u2, u3));
-	acpused += uu_acp_to_attack(u2, u3);
-	/* The target of an attack has to take time to defend itself. */
-	if (!gotonext && alive(defender)) {
-	    use_up_acp(defender, uu_acp_to_defend(u, u3));
+	if (can_fire(unit)) {
+	    do_fire_at_action(unit, unit, unit2, -1); 
+	    acpused += u_acp_to_fire(u2);
+	    /* Unlike the case below, target acp consumption has
+	       already been dealt with in fire_on_unit, which is
+	       called by do_fire_at_action. */
+	} else if (can_attack(unit)) {
+	    if (!gotonext)
+	      one_attack(unit2, defender);
+	    if (alive(unit))
+	      use_up_acp(unit, uu_acp_to_attack(u2, u3));
+	    acpused += uu_acp_to_attack(u2, u3);
+	    /* The target of an attack has to take time to defend
+	       itself. */
+	    if (!gotonext && alive(defender)) {
+		use_up_acp(defender, uu_acp_to_defend(u, u3));
+	    }
 	}
     }
     if (!alive(unit2))
@@ -308,10 +313,8 @@ int x, y, z, n;
       return A_ANY_ERROR;
     u = unit->type;
     u2 = unit2->type;
-    /* (should think about this some more - overrunning into a cell with no
-        resistance should work perhaps, even for noncombat units) */
-    if (!type_can_attack(u))
-      return A_ANY_CANNOT_DO;
+    /* if (!type_can_attack(u)) return A_ANY_CANNOT_DO check deleted to enable 
+    overruning after killing  targets by fire, or into empty cells by noncombat units. */
     if (!has_enough_acp(unit, 1))
       return A_ANY_NO_ACP;
     /* Check whether we can attack from inside a transport. */
@@ -440,11 +443,26 @@ Unit *unit, *unit2, *defender;
 int m;
 {
     int ux = unit->x, uy = unit->y, ox, oy, oz;
+    SideMask sidemask;
+    Side *side;
 
     action_point(unit2->side, defender->x, defender->y);
     action_point(defender->side, defender->x, defender->y);
-    update_fire_at_display(unit2->side, unit, defender, m, TRUE);
-    update_fire_at_display(defender->side, unit, defender, m, TRUE);
+    /* Make up the list of sides that will see the fire. */
+    sidemask = NOSIDES;
+    for_all_sides(side) {
+    /* Let all sides that can see the attacker's cell see it. */
+	if (units_visible(side, unit2->x, unit2->y))
+		sidemask = add_side_to_set(side, sidemask);
+    /* Let all sides that can see the defender's cell see it. */
+	if (units_visible(side, defender->x, defender->y))
+		sidemask = add_side_to_set(side, sidemask);
+    }
+    /* Show the fire. */
+    for_all_sides(side) {
+	if (side_in_set(side, sidemask))
+		update_fire_at_display(side, unit, defender, m, TRUE);
+    }
     ox = defender->x;  oy = defender->y;  oz = defender->z;
     amain = unit;  omain = defender;
     fire_on_unit(unit, defender);
@@ -465,6 +483,23 @@ int m;
     /* Actually, everybody might be seeing the combat. */
     all_see_cell(ux, uy);
     all_see_cell(ox, oy);
+    /* Permit attempts to capture after fire, but only from adjacent
+       or same cell. */
+    if (distance(unit2->x, unit2->y, defender->x, defender->y) < 2) {
+	Side *os = defender->side;
+
+	attempt_to_capture_unit(unit2, defender);
+	/* If the defender was not captured, it might turn the tables! */
+	/* (Note that we cannot cache the attacker's and defender's
+	   types, because the type might have changed due to damage,
+	   and we want to know if the new type might countercapture.)  */
+	if (alive(defender)
+	    && alive(unit2)
+	    && defender->side == os
+	    && uu_countercapture(unit2->type, defender->type) > 0) {
+	    attempt_to_capture_unit(defender, unit2);
+	}
+    }
     /* Always expend the ammo (but only if m is a valid mtype). */
     return A_ANY_DONE;
 }
@@ -572,18 +607,21 @@ int x, y, z, m;
     SideMask sidemask;
     Unit *other;
     Side *side;
-
-    /* Show the firing unit doing its attack. */
-    update_fire_into_display(unit->side, unit2, x, y, z, m, TRUE);
-    /* Make up the list of sides that will see the incoming fire. */
+    
+    /* Make up the list of sides that will see the fire. */
     sidemask = NOSIDES;
-    for_all_stack(x, y, other) {
-	if (other->side)
-	  sidemask = add_side_to_set(other->side, sidemask);
+    for_all_sides(side) {
+    /* Let all sides that can see the attacker's cell see it. */
+	if (units_visible(side, unit2->x, unit2->y))
+		sidemask = add_side_to_set(side, sidemask);
+    /* Let all sides that can see the fired-into cell see it. */
+	if (units_visible(side, x, y))
+		sidemask = add_side_to_set(side, sidemask);
     }
+    /* Show the fire. */
     for_all_sides(side) {
 	if (side_in_set(side, sidemask))
-    	  update_fire_into_display(side, unit2, x, y, z, m, TRUE);
+		update_fire_into_display(side, unit2, x, y, z, m, TRUE);
     }
     /* If any units at target, hit them. */
     for_all_stack(x, y, other) {
@@ -668,9 +706,10 @@ int x, y, z, m;
 
 static int tmpx0, tmpy0, tmpe0, tmpe1, tmpdist01, cellwid, fangle;
 
+static int elevation_blocks(int x, int y);
+
 static int
-elevation_blocks(x, y)
-int x, y;
+elevation_blocks(int x, int y)
 {
     int interpol, celldist0, dist0, el;
 
@@ -799,18 +838,17 @@ Unit *atker, *other;
    protection that the transport provides. */
 
 static void
-maybe_hit_unit(atker, other, fire, fallsoff)
-Unit *atker, *other;
-int fire, fallsoff;
+maybe_hit_unit(Unit *atker, Unit *other, int fire, int fallsoff)
 {
     int chance, t, hit = 0, a = atker->type, o = other->type;
-    int cxpeffect, cxpmax, effect, prot, morchange;
+    int cxpeffect, cxpmax, effect, prot;
     int dist, disthit, rangedelta, hitdelta, rangeamt;
     int dmgspec;
     char *hitmovietype;
     Unit *occ, *unit2;
     Side *side3;
-    
+    SideMask sidemask;    
+
     Dprintf("%s tries to hit %s", unit_desig(atker), unit_desig(other));
     if (fire && uu_fire_hit(a, o) != -1)
       chance = uu_fire_hit(a, o);
@@ -839,13 +877,6 @@ int fire, fallsoff;
 	      chance = (chance * prot) / 100;
 	}
     }
-    if (other->transport
-	&& in_play(other->transport)
-	&& completed(other->transport)) {
-	prot = uu_protection(other->transport->type, o);
-	if (prot != 100)
-	  chance = (chance * prot) / 100;
-    }
     for_all_stack(other->x, other->y, unit2) {
 	if (unit2 != other
 	    && in_play(unit2)
@@ -856,6 +887,37 @@ int fire, fallsoff;
 	      chance = (chance * prot) / 100;
 	}
     }
+    /* Note that this code differs from that above in that it treats
+       all units in the same cell equally, whether occs, suboccs or
+       cellmates. This also means that one occ can protect another occ
+       in the same transport. Moreover, the protective unit also
+       protects itself. These rules simulate real situations such as
+       when triple-A protects all nearby units (including itself)
+       against bombers, or when a city wall protects all other
+       occupants against ground attack. */
+
+    for_all_stack_with_occs(other->x, other->y, unit2) {
+	if (is_active(unit2)
+	    /* We also extend protection to our buddies! */
+	    && trusted_side(unit2->side, other->side)) {
+
+		/* This is when a unit, such as triple-A, extends
+		   unique protection against a specific attacker, such
+		   as bombers, to all other units in the cell. */
+		prot = uu_cellwide_protection_against(unit2->type, a);
+		if (prot != 100)
+		  chance = (chance * prot) / 100;
+
+		/* This is when a unit (such as a garrison)
+		   specifically protects a second unit, such as a fort
+		   (but not other nearby units), against all forms of
+		   attack. It thus works the same way as uu_protection
+		   and uu_stack_protection. */
+		prot = uu_cellwide_protection_for(unit2->type, o);	    
+		if (prot != 100)
+		  chance = (chance * prot) / 100;
+	}
+    }
     if (fallsoff) {
 	dist = distance(atker->x, atker->y, other->x, other->y);
     	disthit = uu_hit_max_range_effect(a, o);
@@ -911,21 +973,31 @@ int fire, fallsoff;
 	}
     }
     hit_unit(other, hit, atker);
+    /* Pick the hit movie to show. */
+    hitmovietype = ((hit >= other->hp) ? "death" : ((hit > 0) ? "hit" : "miss"));
+    /* Make up the list of sides that will see it. */
+    sidemask = NOSIDES;
     for_all_sides(side3) {
-    	if (active_display(side3)
-	    && (all_see_all
-    	    	|| (side3 == atker->side || side3 == other->side))) {
-	    hitmovietype = ((hit >= other->hp) ? "death" :
-			    ((hit > 0) ? "hit" : "miss"));
-	    schedule_movie(side3, hitmovietype, other->id);
-    	}
+    /* Let all sides that can see the attacker's cell see it. */
+	if (units_visible(side3, atker->x, atker->y))
+		sidemask = add_side_to_set(side3, sidemask);
+    /* Let all sides that can see the defender's cell see it. */
+	if (units_visible(side3, other->x, other->y))
+		sidemask = add_side_to_set(side3, sidemask);
+    }
+    /* Show the movie. */
+    for_all_sides(side3) {
+	if (side_in_set(side3, sidemask))
+		schedule_movie(side3, hitmovietype, other->id);
     }
     Dprintf("\n");
     /* Recurse into occupants, maybe hit them too.  */
     for_all_occupants(other, occ) {
-	if (probability(100 - uu_protection(o, occ->type))) {
-	    maybe_hit_unit(atker, occ, fire, fallsoff);
-	}
+	if (is_active(other)) {
+		if (probability(uu_protection(o, occ->type)))
+		 maybe_hit_unit(atker, occ, fire, fallsoff);
+     	/* No protection by incomplete transport. */
+    	} else maybe_hit_unit(atker, occ, fire, fallsoff);
     }
     /* We get combat experience only if there could have been some damage. */
     if (chance > 0) {
@@ -1164,10 +1236,9 @@ enum damage_reasons dmgreason;
 }
 
 void
-rescue_occupants(unit)
-Unit *unit;
+rescue_occupants(Unit *unit)
 {
-    Unit *occ, *other;
+    Unit *occ;
 
     for_all_occupants(unit, occ) {
 	if (probability(uu_occ_escape(unit->type, occ->type))) {
@@ -1177,8 +1248,7 @@ Unit *unit;
 }
 
 static void
-rescue_one_occupant(occ)
-Unit *occ;
+rescue_one_occupant(Unit *occ)
 {
     int rslt, dir, tmp, nx, ny;
     Unit *other;
@@ -1416,6 +1486,37 @@ Unit *atker, *other;
 		  chance = (chance * prot) / 100;
 	    }
 	}
+    /* Note that this code differs from that above in that it treats
+       all units in the same cell equally, whether occs, suboccs or
+       part of the stack. This also means that one occ can protect
+       another occ in the same transport. Moreover, the protective
+       unit also protects itself. These rules simulate real situations
+       such as when triple-A protects all nearby units (including
+       itself) against bombers, or when a city wall protects all other
+       occupants against ground attack. */
+
+    for_all_stack_with_occs(other->x, other->y, unit2) {
+	if (is_active(unit2)
+	    /* We also extend protection to our buddies! */
+	    && trusted_side(unit2->side, other->side)) {
+
+		/* This is when a unit, such as triple-A, provides
+		   unique protection against a specific attacker, such
+		   as bombers, to all other units in the cell. */
+		prot = uu_cellwide_protection_against(unit2->type, a);
+		if (prot != 100)
+			chance = (chance * prot) / 100;
+
+		/* This is when a unit (such as a garrison)
+		   specifically protects a second unit, such as a fort
+		   (but not other nearby units), against all forms of
+		   attack. It thus works the same way as uu_protection
+		   and uu_stack_protection. */
+		prot = uu_cellwide_protection_for(unit2->type, o);	    
+		if (prot != 100)
+			chance = (chance * prot) / 100;
+	}
+    }
 	/* Test whether the capture actually happens. */
 	if (probability(chance)) {
 	    capture_unit(atker, other, H_UNIT_CAPTURED);
@@ -1469,7 +1570,7 @@ int captype;
     /* Return a unit to its original side if we are buds with that side. */
     if (pris->origside != newside && trusted_side(us, pris->origside))
       newside = pris->origside;
-    if (probability(uu_scuttle(pris->type, unit->type))) {
+    if (probability(uu_scuttle(pris->type, u))) {
 	/* (should add terrain effect on success too) */
 	/* (should characterize as a scuttle) */
 	kill_unit(pris, H_UNIT_DISBANDED);
@@ -1504,20 +1605,7 @@ int captype;
 		change_control_side_around(px, py, pris->type, unit->side);
 	    }
 	}
-    
-// --- USER LAYER FIX -----------------------------------------
-
-    kick_out_enemy_users(unit->side, px, py);
-
-//    	/* Kick out enemy users if we control this cell */
-//   	 if (user_at(px, py) != NULL 
-//    		&& (!user_at(px, py)->side || !trusted_side(user_at(px, py)->side, unit->side))) {
-//			user_at(px, py)->usedcells -=1;
-//    			set_user_at(px, py, NULL);
-//   	 }
-
-// ---------------------------------------------------------
-
+         kick_out_enemy_users(unit->side, px, py);
     }
 
     /* Update everybody's view of the situation. */
@@ -1632,9 +1720,7 @@ Side *prevside;
 /* A detonate action just blasts the vicinity indiscriminately. */
 
 int
-prep_detonate_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+prep_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
@@ -1647,9 +1733,7 @@ int x, y, z;
 }
 
 int
-net_prep_detonate_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+net_prep_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int rslt;
 
@@ -1659,9 +1743,7 @@ int x, y, z;
 }
 
 int
-do_detonate_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+do_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int u2 = unit2->type;
 
@@ -1678,9 +1760,7 @@ int x, y, z;
 }
 
 int
-check_detonate_action(unit, unit2, x, y, z)
-Unit *unit, *unit2;
-int x, y, z;
+check_detonate_action(Unit *unit, Unit *unit2, int x, int y, int z)
 {
     int u, u2, acp;
 
@@ -1712,12 +1792,9 @@ static int tmpdetx, tmpdety;
    routine makes the detonation effects happen, pyrotechnics and all. */
 
 int
-detonate_unit(unit, x, y, z)
-Unit *unit;
-int x, y, z;
+detonate_unit(Unit *unit, int x, int y, int z)
 {
-    int u = unit->type, dir, x1, y1, dmg, t, maxrange;
-    char *hitmovietype;
+    int u = unit->type, dir, x1, y1, dmg, maxrange;
     Unit *unit2;
     Side *side;
 
@@ -1794,15 +1871,17 @@ int x, y, z;
 }
 
 static void
-detonate_on_cell(x, y)
-int x, y;
+detonate_on_cell(int x, int y)
 {
-    int dist, dmg, sdmg, t;
+    int dist, dmg, sdmg;
     Unit *unit2;
 
     dist = distance(tmpdetx, tmpdety, x, y);
     if (dist > 1 && dist <= max_u_detonate_effect_range) {
-	for_all_stack(x, y, unit2) {
+	/* Since this code bypasses the occ recursion in
+	   maybe_hit_unit, we should also hit all occupants directly
+	   with the blast. */
+	for_all_stack_with_occs(x, y, unit2) {
 	    if (dist <= uu_detonation_range(tmpunit->type, unit2->type)) {
 		dmg = uu_detonation_damage_adj(tmpunit->type, unit2->type);
 		/* Reduce by inverse square of the distance. */
@@ -1818,9 +1897,7 @@ int x, y;
 }
 
 static void
-hit_unit_with_detonation(unit, hit, atker)
-Unit *unit, *atker;
-int hit;
+hit_unit_with_detonation(Unit *unit, int hit, Unit *atker)
 {
     char *hitmovietype;
     Side *side;
@@ -1839,10 +1916,9 @@ int hit;
 }
 
 void
-damage_terrain(u, x, y)
-int u, x, y;
+damage_terrain(int u, int x, int y)
 {
-    int t, t2, dir, x1, y1;
+    int t, t2, dir;
 
     /* Damage the cell's terrain. */
     t = terrain_at(x, y);
@@ -2018,3 +2094,58 @@ Unit *unit;
 {
     return (u_acp_to_detonate(unit->type) > 0);
 }
+
+int
+can_capture_directly(Unit *unit)
+{
+    return type_can_capture_directly(unit->type);
+}
+
+/* Tests if type can capture without first attacking. */
+
+int
+type_can_capture_directly(int u)
+{
+    int u2;
+
+    for_all_unit_types(u2) {
+	if (uu_acp_to_capture(u, u2) > 0
+	    && (uu_capture(u, u2) > 0 
+		|| uu_indep_capture(u, u2) > 0)) {
+	    return TRUE;
+	}
+    }
+    return FALSE;
+}
+
+/* Tests if type can have any occupants at all. */
+
+int
+type_can_carry(int u)
+{
+    int u2;
+
+    for_all_unit_types(u2) {
+	if (could_carry(u, u2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+/* True if first type can both occupy and protect second type. */
+
+int
+occ_can_defend_transport(int o, int t)
+{
+    /* Never true if transport cannot carry occupant. */
+    if (!could_carry(t, o))
+      return FALSE;
+    /* Specific protection of transport by its occupant. */
+    /* Note: zero protection is 100, full protection is 0! */
+    if (uu_protection(o, t) < 100)
+      return TRUE;
+    /* Cellwide protection of all units by occupant. */
+    if (uu_cellwide_protection_for(o, t) < 100)
+      return TRUE;
+    return FALSE;	
+}
