@@ -53,7 +53,7 @@ enum terrain_subtype {
 #define VARLO -2000000000
 #define VARHI 2000000000
 
-// --- USED IN ADVANCES CODE ----------------------------------
+/* Used in advances code. */
 
 #define DONE (-1)
 
@@ -63,8 +63,6 @@ enum terrain_subtype {
 
 #define NOUNIT (-1)
 
-// ------------------------------------------------------
-
 #include "lisp.h"
 
 /* This is the structure representing info about a property
@@ -72,9 +70,9 @@ enum terrain_subtype {
 
 typedef struct propertydefn {
     char *name;
-    int (*intgetter) PARAMS ((int));
-    char *(*strgetter) PARAMS ((int));
-    Obj *(*objgetter) PARAMS ((int));
+    int (*intgetter)(int);
+    char *(*strgetter)(int);
+    Obj *(*objgetter)(int);
     short offset;
     char *doc;
     short dflt;
@@ -86,7 +84,7 @@ typedef struct propertydefn {
 
 typedef struct tabledefn {
     char *name;			/* name of the table */
-    int (*getter) PARAMS ((int, int));  /* accessor function */
+    int (*getter)(int, int);	/* accessor function */
     char *doc;			/* documentation string */
     short **table;		/* pointer to table itself */
     short dflt;			/* default value of entries */
@@ -99,16 +97,16 @@ typedef struct tabledefn {
 
 typedef struct vardefn {
     char *name;			/* name of the global */
-    int   (*intgetter) PARAMS ((void));  /* accessor if integer type */
-    char *(*strgetter) PARAMS ((void));  /* accessor if string type */
-    Obj  *(*objgetter) PARAMS ((void));  /* accessor if object type */
-    void (*intsetter) PARAMS ((int));  /* setter if integer type */
-    void (*strsetter) PARAMS ((char *));  /* setter if string type */
-    void (*objsetter) PARAMS ((Obj *));  /* setter if object type */
+    int   (*intgetter)(void);	/* accessor if integer type */
+    char *(*strgetter)(void);	/* accessor if string type */
+    Obj  *(*objgetter)(void);	/* accessor if object type */
+    void (*intsetter)(int);	/* setter if integer type */
+    void (*strsetter)(char *);  /* setter if string type */
+    void (*objsetter)(Obj *);	/* setter if object type */
     char *doc;			/* documentation string */
     int dflt;			/* default value if integer type */
     char *dfltstr;		/* default value if string type */
-    void (*dfltfn) PARAMS ((void));  /* default value if object type */
+    void (*dfltfn)(void);	/* default value if object type */
     int lo, hi;			/* bounds of integer value */
 } VarDefn;
 
@@ -208,53 +206,53 @@ typedef struct a_globals {
 /* Declarations of the functions accessing and setting type properties. */
 
 #undef  DEF_UPROP_I
-#define DEF_UPROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME PARAMS ((int u));
+#define DEF_UPROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME(int u);
 #undef  DEF_UPROP_S
-#define DEF_UPROP_S(name,FNAME,doc,slot,dflt)  char *FNAME PARAMS ((int u));
+#define DEF_UPROP_S(name,FNAME,doc,slot,dflt)  char *FNAME(int u);
 #undef  DEF_UPROP_L
-#define DEF_UPROP_L(name,FNAME,doc,slot)  Obj *FNAME PARAMS ((int u));
+#define DEF_UPROP_L(name,FNAME,doc,slot)  Obj *FNAME(int u);
 
 #include "utype.def"
 
 #undef  DEF_MPROP_I
-#define DEF_MPROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME PARAMS ((int m));
+#define DEF_MPROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME(int m);
 #undef  DEF_MPROP_S
-#define DEF_MPROP_S(name,FNAME,doc,slot,dflt)  char *FNAME PARAMS ((int m));
+#define DEF_MPROP_S(name,FNAME,doc,slot,dflt)  char *FNAME(int m);
 #undef  DEF_MPROP_L
-#define DEF_MPROP_L(name,FNAME,doc,slot)  Obj *FNAME PARAMS ((int m));
+#define DEF_MPROP_L(name,FNAME,doc,slot)  Obj *FNAME(int m);
 
 #include "mtype.def"
 
 #undef  DEF_TPROP_I
-#define DEF_TPROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME PARAMS ((int t));
+#define DEF_TPROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME(int t);
 #undef  DEF_TPROP_S
-#define DEF_TPROP_S(name,FNAME,doc,slot,dflt)  char *FNAME PARAMS ((int t));
+#define DEF_TPROP_S(name,FNAME,doc,slot,dflt)  char *FNAME(int t);
 #undef  DEF_TPROP_L
-#define DEF_TPROP_L(name,FNAME,doc,slot)  Obj *FNAME PARAMS ((int t));
+#define DEF_TPROP_L(name,FNAME,doc,slot)  Obj *FNAME(int t);
 
 #include "ttype.def"
 
 #undef  DEF_APROP_I
-#define DEF_APROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME PARAMS ((int t));
+#define DEF_APROP_I(name,FNAME,doc,slot,lo,dflt,hi)  int FNAME(int a);
 #undef  DEF_APROP_S
-#define DEF_APROP_S(name,FNAME,doc,slot,dflt)  char *FNAME PARAMS ((int t));
+#define DEF_APROP_S(name,FNAME,doc,slot,dflt)  char *FNAME(int a);
 #undef  DEF_APROP_L
-#define DEF_APROP_L(name,FNAME,doc,slot)  Obj *FNAME PARAMS ((int t));
+#define DEF_APROP_L(name,FNAME,doc,slot)  Obj *FNAME(int a);
 
 #include "atype.def"
 
 #undef  DEF_VAR_I
 #define DEF_VAR_I(str,FNAME,SETFNAME,doc,var,lo,dflt,hi)  \
-  int FNAME PARAMS ((void));  \
-  void SETFNAME PARAMS ((int val));
+  int FNAME(void);  \
+  void SETFNAME(int val);
 #undef  DEF_VAR_S
 #define DEF_VAR_S(str,FNAME,SETFNAME,doc,var,dflt)  \
-  char *FNAME PARAMS ((void));  \
-  void SETFNAME PARAMS ((char *val));
+  char *FNAME(void);  \
+  void SETFNAME(char *val);
 #undef  DEF_VAR_L
 #define DEF_VAR_L(str,FNAME,SETFNAME,doc,var,dflt)  \
-  Obj *FNAME PARAMS ((void));  \
-  void SETFNAME PARAMS ((Obj *val));
+  Obj *FNAME(void);  \
+  void SETFNAME(Obj *val);
 
 #include "gvar.def"
 
@@ -263,47 +261,47 @@ typedef struct a_globals {
 
 #undef  DEF_UU_TABLE
 #define DEF_UU_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int u1, int u2));  \
+  int FNAME(int u1, int u2);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_UM_TABLE
 #define DEF_UM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int u, int m));  \
+  int FNAME(int u, int m);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_UT_TABLE
 #define DEF_UT_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int u, int t));  \
+  int FNAME(int u, int t);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_TM_TABLE
 #define DEF_TM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int t, int m));  \
+  int FNAME(int t, int m);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_TT_TABLE
 #define DEF_TT_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int t1, int t2));  \
+  int FNAME(int t1, int t2);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_MM_TABLE
 #define DEF_MM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int m1, int m2));  \
+  int FNAME(int m1, int m2);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_UA_TABLE
 #define DEF_UA_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int u, int a));  \
+  int FNAME(int u, int a);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_AM_TABLE
 #define DEF_AM_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int a, int m));  \
+  int FNAME(int a, int m);  \
   extern short *TABLE, CNST;
 
 #undef  DEF_AA_TABLE
 #define DEF_AA_TABLE(name,FNAME,doc,TABLE,CNST,lo,dflt,hi,valtype)  \
-  int FNAME PARAMS ((int a1, int a2));  \
+  int FNAME(int a1, int a2);  \
   extern short *TABLE, CNST;
 
 #include "table.def"
@@ -332,19 +330,6 @@ extern TableDefn tabledefns[];
 
 extern VarDefn vardefns[];
 
-//--- ADVANCED UNIT AI SUPPORT -----------------------------------------
-
-#define for_all_advanced_utypes(i)	for (i = 0; i < numadv; i++)
-#define for_all_colonizer_utypes(i)	for (i = 0; i < numcol; i++)
-#define for_all_offensive_utypes(i)	for (i = 0; i < numoff; i++)
-#define for_all_defensive_utypes(i)	for (i = 0; i < numdef; i++)
-#define for_all_improver_utypes(i)	for (i = 0; i < numimp; i++)
-#define for_all_facility_utypes(i)	for (i = 0; i < numfac; i++)
-#define for_all_explorer_utypes(i)	for (i = 0; i < numexp; i++)
-#define for_all_mobile_utypes(i)	for (i = 0; i < nummob; i++)
-
-// ---------------------------------------------------------------
-
 /* Macros for iterating over types. */
 
 #define for_all_unit_types(v)      for (v = 0; v < numutypes; ++v)
@@ -434,21 +419,20 @@ extern short tmpmtype;
 extern short tmpttype;
 extern short tmpatype;
 
-extern void utype_error  PARAMS ((int u));
-extern void mtype_error  PARAMS ((int m));
-extern void ttype_error  PARAMS ((int t));
-extern void atype_error  PARAMS ((int s));
+extern void utype_error(int u);
+extern void mtype_error(int m);
+extern void ttype_error(int t);
+extern void atype_error(int s);
 
-extern void init_types PARAMS ((void));
-extern void init_globals PARAMS ((void));
-extern void default_unit_type PARAMS ((int x));
-extern void default_material_type PARAMS ((int x));
-extern void default_terrain_type PARAMS ((int x));
-extern void default_advance_type PARAMS ((int x));
+extern void init_types(void);
+extern void init_globals(void);
+extern void default_unit_type(int x);
+extern void default_material_type(int x);
+extern void default_terrain_type(int x);
+extern void default_advance_type(int x);
 
-extern char *index_type_name();
-extern void allocate_table PARAMS ((int tbl, int reset));
-extern int numtypes_from_index_type PARAMS ((int x));
-extern char *index_type_name PARAMS ((int x));
+extern void allocate_table(int tbl, int reset);
+extern int numtypes_from_index_type(int x);
+extern char *index_type_name(int x);
 
-extern void set_g_synth_methods_default PARAMS ((void));
+extern void set_g_synth_methods_default(void);
