@@ -1,5 +1,5 @@
 /* Low-level support for variables and properties in Xconq GDL.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1998 Stanley T. Shebs.
+   Copyright (C) 1991-1996, 1998-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -141,7 +141,7 @@ init_globals()
 #undef  DEF_VAR_L
 #define DEF_VAR_L(str,fname,SETFNAME,doc,var,DFLT)  \
     if ((DFLT) != NULL) {  \
-	void (*fn) PARAMS ((void)) = (DFLT);  \
+	void (*fn)(void) = (DFLT);  \
 	if (fn != NULL) (*fn)();  \
     } else { \
 	SETFNAME(lispnil);  \
