@@ -13,10 +13,11 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 
-int any_ut_capacity_x PARAMS ((int u));
-int any_mp_to_enter_unit PARAMS ((int u));
-int any_mp_to_leave_unit PARAMS ((int u));
-int any_enter_indep PARAMS ((int u));
+int may_detonate(int u);
+int any_ut_capacity_x(int u);
+int any_mp_to_enter_unit(int u);
+int any_mp_to_leave_unit(int u);
+int any_enter_indep(int u);
 
 /* Obstack allocation and deallocation routines.  */
 
@@ -29,59 +30,83 @@ extern int error_is_bug;
 extern int free ();
 #endif
 
-static void describe_help_system PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_instructions PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_synth_run PARAMS ((TextBuffer *buf, int methkey));
-static void describe_world PARAMS ((int arg, char *key, TextBuffer *buf));
-static int histogram_compare PARAMS ((const void *h1, const void *h2));
-static void describe_news PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_concepts PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_game_design PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_utype PARAMS ((int u, char *key, TextBuffer *buf));
-static void describe_mtype PARAMS ((int m, char *key, TextBuffer *buf));
-static void describe_ttype PARAMS ((int t, char *key, TextBuffer *buf));
-static void describe_atype PARAMS ((int t, char *key, TextBuffer *buf));
+static void describe_help_system(int arg, char *key, TextBuffer *buf);
+static void describe_instructions(int arg, char *key, TextBuffer *buf);
+static void describe_synth_run(TextBuffer *buf, int methkey);
+static void describe_world(int arg, char *key, TextBuffer *buf);
+static int histogram_compare(const void *h1, const void *h2);
+static void describe_news(int arg, char *key, TextBuffer *buf);
+static void describe_concepts(int arg, char *key, TextBuffer *buf);
+static void describe_game_design(int arg, char *key, TextBuffer *buf);
+static void describe_utype(int u, char *key, TextBuffer *buf);
+static void describe_mtype(int m, char *key, TextBuffer *buf);
+static void describe_ttype(int t, char *key, TextBuffer *buf);
+static void describe_atype(int t, char *key, TextBuffer *buf);
 
-static void describe_scorekeepers PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_setup PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_game_modules PARAMS ((int arg, char *key, TextBuffer *buf));
-static void describe_game_module_aux PARAMS ((TextBuffer *buf, Module *module, int level));
-static void describe_module_notes PARAMS ((TextBuffer *buf, Module *module));
+static void describe_scorekeepers(int arg, char *key, TextBuffer *buf);
+static void describe_setup(int arg, char *key, TextBuffer *buf);
+static void describe_game_modules(int arg, char *key, TextBuffer *buf);
+static void describe_game_module_aux(TextBuffer *buf, Module *module,
+				     int level);
+static void describe_module_notes(TextBuffer *buf, Module *module);
 
-static int u_property_not_default PARAMS ((int (*fn)(int i), int dflt));
-static int t_property_not_default PARAMS ((int (*fn)(int i), int dflt));
-static int m_property_not_default PARAMS ((int (*fn)(int i), int dflt));
-static int uu_table_row_not_default PARAMS ((int u, int (*fn)(int i, int j), int dflt));
-static int ut_table_row_not_default PARAMS ((int u, int (*fn)(int i, int j), int dflt));
-static int um_table_row_not_default PARAMS ((int u, int (*fn)(int i, int j), int dflt));
-static int tt_table_row_not_default PARAMS ((int t, int (*fn)(int i, int j), int dflt));
-static int tm_table_row_not_default PARAMS ((int t, int (*fn)(int i, int j), int dflt));
-static int aa_table_row_not_default PARAMS ((int a1, int (*fn)(int i, int j), int dflt));
-static int uu_table_column_not_default PARAMS ((int u, int (*fn)(int i, int j), int dflt));
-static int aa_table_column_not_default PARAMS ((int a1, int (*fn)(int i, int j), int dflt));
-static void u_property_desc PARAMS ((TextBuffer *buf, int (*fn)(int), void (*formatter)(TextBuffer *, int)));
-static void t_property_desc PARAMS ((TextBuffer *buf, int (*fn)(int), void (*formatter)(TextBuffer *, int)));
-static void m_property_desc PARAMS ((TextBuffer *buf, int (*fn)(int), void (*formatter)(TextBuffer *, int)));
-static void uu_table_row_desc PARAMS ((TextBuffer *buf, int u, int (*fn)(int, int), void (*formatter)(TextBuffer *, int), char *connect));
-static void uu_table_column_desc PARAMS ((TextBuffer *buf, int u, int (*fn)(int, int), void (*formatter)(TextBuffer *, int), char *connect));
-static void uu_table_rowcol_desc PARAMS ((TextBuffer *buf, int u, int (*fn)(int, int), void (*formatter)(TextBuffer *, int), char *connect, int rowcol));
-static void ut_table_row_desc PARAMS ((TextBuffer *buf, int u, int (*fn)(int, int), void (*formatter)(TextBuffer *, int), char *connect));
-static void um_table_row_desc PARAMS ((TextBuffer *buf, int u, int (*fn)(int, int), void (*formatter)(TextBuffer *, int)));
-static void tt_table_row_desc PARAMS ((TextBuffer *buf, int t1, int (*fn)(int, int), void (*formatter)(TextBuffer *, int)));
-static void tm_table_row_desc PARAMS ((TextBuffer *buf, int t, int (*fn)(int, int), void (*formatter)(TextBuffer *, int)));
-static void aa_table_row_desc PARAMS ((TextBuffer *buf, int a1, int (*fn)(int, int), void (*formatter)(TextBuffer *, int)));
-static void aa_table_column_desc PARAMS ((TextBuffer *buf, int a1, int (*fn)(int, int), void (*formatter)(TextBuffer *, int)));
-static void aa_table_rowcol_desc PARAMS ((TextBuffer *buf, int a1, int (*fn)(int, int), void (*formatter)(TextBuffer *, int), int rowcol));
-static void tb_value_desc PARAMS ((TextBuffer *buf, int val));
-static void tb_fraction_desc PARAMS ((TextBuffer *buf, int val));
-static void tb_percent_desc PARAMS ((TextBuffer *buf, int val));
-static void tb_percent_100th_desc PARAMS ((TextBuffer *buf, int val));
-static void tb_dice_desc PARAMS ((TextBuffer *buf, int val));
-static void tb_mult_desc PARAMS ((TextBuffer *buf, int val));
-static void tb_bool_desc PARAMS ((TextBuffer *buf, int val));
-static void append_number PARAMS ((TextBuffer *buf, int value, int dflt));
-static void append_help_phrase PARAMS ((TextBuffer *buf, char *phrase));
-static void append_notes PARAMS ((TextBuffer *buf, Obj *notes));
+static int u_property_not_default(int (*fn)(int i), int dflt);
+static int t_property_not_default(int (*fn)(int i), int dflt);
+static int uu_table_row_not_default(int u, int (*fn)(int i, int j), int dflt);
+static int ut_table_row_not_default(int u, int (*fn)(int i, int j), int dflt);
+static int um_table_row_not_default(int u, int (*fn)(int i, int j), int dflt);
+static int tt_table_row_not_default(int t, int (*fn)(int i, int j), int dflt);
+#if 0
+static int tm_table_row_not_default(int t, int (*fn)(int i, int j), int dflt);
+#endif
+static int aa_table_row_not_default(int a1, int (*fn)(int i, int j), int dflt);
+static int uu_table_column_not_default(int u, int (*fn)(int i, int j),
+				       int dflt);
+static int aa_table_column_not_default(int a1, int (*fn)(int i, int j),
+				       int dflt);
+static void u_property_desc(TextBuffer *buf, int (*fn)(int),
+			    void (*formatter)(TextBuffer *, int));
+static void t_property_desc(TextBuffer *buf, int (*fn)(int),
+			    void (*formatter)(TextBuffer *, int));
+static void uu_table_row_desc(TextBuffer *buf, int u, int (*fn)(int, int),
+			      void (*formatter)(TextBuffer *, int),
+			      char *connect);
+static void uu_table_column_desc(TextBuffer *buf, int u, int (*fn)(int, int),
+				 void (*formatter)(TextBuffer *, int),
+				 char *connect);
+static void uu_table_rowcol_desc(TextBuffer *buf, int u, int (*fn)(int, int),
+				 void (*formatter)(TextBuffer *, int),
+				 char *connect, int rowcol);
+static void ut_table_row_desc(TextBuffer *buf, int u, int (*fn)(int, int),
+			      void (*formatter)(TextBuffer *, int),
+			      char *connect);
+static void um_table_row_desc(TextBuffer *buf, int u, int (*fn)(int, int),
+			      void (*formatter)(TextBuffer *, int));
+static void tt_table_row_desc(TextBuffer *buf, int t1, int (*fn)(int, int),
+			      void (*formatter)(TextBuffer *, int));
+#if 0
+static void tm_table_row_desc(TextBuffer *buf, int t, int (*fn)(int, int),
+			      void (*formatter)(TextBuffer *, int));
+#endif
+static void aa_table_row_desc(TextBuffer *buf, int a1, int (*fn)(int, int),
+			      void (*formatter)(TextBuffer *, int));
+static void aa_table_column_desc(TextBuffer *buf, int a1, int (*fn)(int, int),
+				 void (*formatter)(TextBuffer *, int));
+static void aa_table_rowcol_desc(TextBuffer *buf, int a1, int (*fn)(int, int),
+				 void (*formatter)(TextBuffer *, int),
+				 int rowcol);
+static void tb_value_desc(TextBuffer *buf, int val);
+static void tb_fraction_desc(TextBuffer *buf, int val);
+static void tb_percent_desc(TextBuffer *buf, int val);
+static void tb_percent_100th_desc(TextBuffer *buf, int val);
+static void tb_dice_desc(TextBuffer *buf, int val);
+static void tb_mult_desc(TextBuffer *buf, int val);
+static void tb_bool_desc(TextBuffer *buf, int val);
+#if 0
+static void append_number(TextBuffer *buf, int value, int dflt);
+#endif
+static void append_help_phrase(TextBuffer *buf, char *phrase);
+static void append_notes(TextBuffer *buf, Obj *notes);
 
 /* The first help node in the chain. */
 
@@ -182,11 +207,8 @@ create_help_node()
 /* Add a help node after the given node. */
 
 HelpNode *
-add_help_node(key, fn, arg, prevnode)
-char *key;
-void (*fn)();
-int arg;
-HelpNode *prevnode;
+add_help_node(char *key, void (*fn)(int t, char *key, TextBuffer *buf),
+	      int arg, HelpNode *prevnode)
 {
     HelpNode *node, *nextnode;
 
@@ -310,7 +332,7 @@ TextBuffer *buf;
 {
     FILE *fp;
 
-    fp = fopen(news_filename(), "r");
+    fp = open_file(news_filename(), "r");
     if (fp != NULL) {
 	tbcat(buf, "XCONQ NEWS\n\n");
 	while (fgets(spbuf, BUFSIZE-1, fp) != NULL) {
@@ -1408,7 +1430,8 @@ TextBuffer *buf;
 	ut_table_row_desc(buf, u, ut_attrition, tb_percent_100th_desc, "in");
 	tbcat(buf, ".\n");
     }
-    if (nummtypes > 0) {
+    /* Don't bother with this if standard economy has been disabled. */
+    if (nummtypes > 0 && !g_disable_standard_economy()) {
 	tbcat(buf, "\nMaterial Handling:\n");
 	for_all_material_types(m) {
 	    usesm = FALSE;
@@ -1515,17 +1538,21 @@ TextBuffer *buf;
 	}
     }
     if (numatypes > 0) {
-	tbcat(buf, "\nAdvances:\n");
+	/* Now also prints "None" if no advances are required. */
+	int	found = FALSE;
+
+	tbcat(buf, "\nRequired advances to build: ");
 	for_all_advance_types(a) {
 	    if (ua_needed_to_build(u, a) > 0) {
-		tbprintf(buf, "  %s", a_type_name(a));
-		if (ua_needed_to_build(u, a) > 0) {
-		    tbprintf(buf, ", needed to build unit");
-		}
-		/* (should add other unit-advance tables here) */
-		tbcat(buf, "\n");
+		if (found)
+			tbprintf(buf, ", ");
+		tbprintf(buf, "%s", a_type_name(a));
+		found = TRUE;
 	    }
 	}
+	if (found)
+		tbprintf(buf, ".\n");		
+	else	tbprintf(buf, "None.\n");
     }
     tbcat(buf, "\n");
     /* (should display weather interaction here) */
@@ -1792,8 +1819,6 @@ int a;
 char *key;
 TextBuffer *buf;
 {
-    int a2;
-
     append_help_phrase(buf, a_help(a));
     tbprintf(buf, "Research points (RP) to discover: %d.\n", a_rp(a));
     tbcat(buf, "\n");
@@ -2284,7 +2309,7 @@ TextBuffer *buf;
 
 static int
 u_property_not_default(fn, dflt)
-int (*fn) PARAMS ((int i));
+int (*fn)(int i);
 int dflt;
 {
     int u, val;
@@ -2299,7 +2324,7 @@ int dflt;
 
 static int
 t_property_not_default(fn, dflt)
-int (*fn) PARAMS ((int i));
+int (*fn)(int i);
 int dflt;
 {
     int t, val;
@@ -2315,7 +2340,7 @@ int dflt;
 static int
 uu_table_row_not_default(u, fn, dflt)
 int u, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int u2, val2;
 
@@ -2330,7 +2355,7 @@ int (*fn) PARAMS ((int i, int j));
 static int
 ut_table_row_not_default(u, fn, dflt)
 int u, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int t, val2;
 
@@ -2345,7 +2370,7 @@ int (*fn) PARAMS ((int i, int j));
 static int
 um_table_row_not_default(u, fn, dflt)
 int u, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int m, val2;
 
@@ -2360,7 +2385,7 @@ int (*fn) PARAMS ((int i, int j));
 static int
 tt_table_row_not_default(t1, fn, dflt)
 int t1, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int t2, val2;
 
@@ -2372,10 +2397,11 @@ int (*fn) PARAMS ((int i, int j));
     return FALSE;
 }
 
+#if 0 /* not used currently */
 static int
 tm_table_row_not_default(t, fn, dflt)
 int t, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int m, val2;
 
@@ -2386,11 +2412,12 @@ int (*fn) PARAMS ((int i, int j));
     }
     return FALSE;
 }
+#endif
 
 static int
 aa_table_row_not_default(a1, fn, dflt)
 int a1, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int a2, val2;
 
@@ -2405,7 +2432,7 @@ int (*fn) PARAMS ((int i, int j));
 static int
 uu_table_column_not_default(u, fn, dflt)
 int u, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int u2, val2;
 
@@ -2420,7 +2447,7 @@ int (*fn) PARAMS ((int i, int j));
 static int
 aa_table_column_not_default(a1, fn, dflt)
 int a1, dflt;
-int (*fn) PARAMS ((int i, int j));
+int (*fn)(int i, int j);
 {
     int a2, val2;
 
@@ -2453,8 +2480,8 @@ CONST void *h1, *h2;
 static void
 u_property_desc(buf, fn, formatter)
 TextBuffer *buf;
-int (*fn) PARAMS ((int i));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i);
+void (*formatter)(TextBuffer *buf, int val);
 {
     int val, u, val2, constant = TRUE, found;
     int i, numentries, first;
@@ -2515,7 +2542,11 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 	for_all_unit_types(u) {
 	    if ((*fn)(u) == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  /* For this and similar situations, we need a space
+		     in addition to the comma, so interface elements
+		     (such as the Mac's text display) can decide to
+		     add line breaks. */
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, u_type_name(u));
@@ -2527,8 +2558,8 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 static void
 t_property_desc(buf, fn, formatter)
 TextBuffer *buf;
-int (*fn) PARAMS ((int i));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i);
+void (*formatter)(TextBuffer *buf, int val);
 {
     int val, t, val2, constant = TRUE, found;
     int i, numentries, first;
@@ -2589,7 +2620,7 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 	for_all_terrain_types(t) {
 	    if ((*fn)(t) == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, t_type_name(t));
@@ -2605,8 +2636,8 @@ static void
 uu_table_row_desc(buf, u, fn, formatter, connect)
 TextBuffer *buf;
 int u;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 char *connect;
 {
     uu_table_rowcol_desc(buf, u, fn, formatter, connect, 0);
@@ -2616,8 +2647,8 @@ static void
 uu_table_column_desc(buf, u, fn, formatter, connect)
 TextBuffer *buf;
 int u;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 char *connect;
 {
     uu_table_rowcol_desc(buf, u, fn, formatter, connect, 1);
@@ -2627,8 +2658,8 @@ static void
 uu_table_rowcol_desc(buf, u, fn, formatter, connect, rowcol)
 TextBuffer *buf;
 int u;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 char *connect;
 int rowcol;
 {
@@ -2694,7 +2725,7 @@ int rowcol;
 	    val2 = (rowcol ? (*fn)(u2, u) : (*fn)(u, u2));
 	    if (val2 == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, u_type_name(u2));
@@ -2710,8 +2741,8 @@ static void
 ut_table_row_desc(buf, u, fn, formatter, connect)
 TextBuffer *buf;
 int u;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 char *connect;
 {
     int val = (*fn)(u, 0), val2, t, constant = TRUE, found;
@@ -2774,7 +2805,7 @@ char *connect;
 	for_all_terrain_types(t) {
 	    if ((*fn)(u, t) == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, t_type_name(t));
@@ -2787,8 +2818,8 @@ static void
 um_table_row_desc(buf, u, fn, formatter)
 TextBuffer *buf;
 int u;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 {
     int val = (*fn)(u, 0), val2, m, constant = TRUE, found;
     int i, numentries, first;
@@ -2849,7 +2880,7 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 	for_all_material_types(m) {
 	    if ((*fn)(u, m) == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, m_type_name(m));
@@ -2862,8 +2893,8 @@ static void
 tt_table_row_desc(buf, t0, fn, formatter)
 TextBuffer *buf;
 int t0;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 {
     int val = (*fn)(t0, 0), val2, t, constant = TRUE, found;
     int i, numentries, first;
@@ -2923,7 +2954,7 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 	for_all_terrain_types(t) {
 	    if ((*fn)(t0, t) == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, t_type_name(t));
@@ -2932,12 +2963,13 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
     }
 }
 
+#if 0 /* not currently used */
 static void
 tm_table_row_desc(buf, t0, fn, formatter)
 TextBuffer *buf;
 int t0;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 {
     int val = (*fn)(t0, 0), val2, m, constant = TRUE, found;
     int i, numentries, first;
@@ -2997,7 +3029,7 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 	for_all_material_types(m) {
 	    if ((*fn)(t0, m) == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, m_type_name(m));
@@ -3005,13 +3037,14 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 	}
     }
 }
+#endif
 
 static void
 aa_table_row_desc(buf, a0, fn, formatter)
 TextBuffer *buf;
 int a0;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 {
     aa_table_rowcol_desc(buf, a0, fn, formatter, 0);
 }
@@ -3020,8 +3053,8 @@ static void
 aa_table_column_desc(buf, a1, fn, formatter)
 TextBuffer *buf;
 int a1;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 {
     aa_table_rowcol_desc(buf, a1, fn, formatter, 1);
 }
@@ -3030,8 +3063,8 @@ static void
 aa_table_rowcol_desc(buf, a1, fn, formatter, rowcol)
 TextBuffer *buf;
 int a1;
-int (*fn) PARAMS ((int i, int j));
-void (*formatter) PARAMS ((TextBuffer *buf, int val));
+int (*fn)(int i, int j);
+void (*formatter)(TextBuffer *buf, int val);
 {
     int val = (*fn)(0, a1), val2, a, constant = TRUE, found;
     int i, numentries, first;
@@ -3093,7 +3126,7 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
 	    val2 = (rowcol ? (*fn)(a, a1) : (*fn)(a1, a));
 	    if (val2 == histogram[i].val) {
 		if (!first)
-		  tbcat(buf, ",");
+		  tbcat(buf, ", ");
 		else
 		  first = FALSE;
 		tbcat(buf, a_type_name(a));
@@ -3102,9 +3135,9 @@ void (*formatter) PARAMS ((TextBuffer *buf, int val));
     }
 }
 
+#if 0 /* not currently used */
 /* A simple table-printing utility. Blanks out default values so they don't
    clutter the table. */
-/* (not currently used anywhere?) */
 
 static void
 append_number(buf, value, dflt)
@@ -3117,6 +3150,7 @@ int value, dflt;
 	tbprintf(buf, "      ");
     }
 }
+#endif
 
 static void
 append_help_phrase(buf, phrase)
@@ -3125,9 +3159,11 @@ char *phrase;
 {
     if (empty_string(phrase))
       return;
-    tbcat(buf, "-- ");
+
+    /* Extra new line makes display less cluttered. */
+    tbcat(buf, "----- ");
     tbcat(buf, phrase);
-    tbcat(buf, " --\n");
+    tbcat(buf, " -----\n\n");
 }
 
 static void
