@@ -1,5 +1,5 @@
 /* Management of historical information about an Xconq game.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,20 +7,19 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int find_event_type PARAMS ((Obj *sym));
-extern int update_total_hist_lines PARAMS ((Side *side));
-extern int build_hist_contents PARAMS ((Side *side, int n, HistEvent **histcontents,
-					int numvishistlines));
+extern int find_event_type(Obj *sym);
+extern int update_total_hist_lines(Side *side);
+extern int build_hist_contents(Side *side, int n, HistEvent **histcontents,
+			       int numvishistlines);
 
-int pattern_matches_event PARAMS ((Obj *pattern, HistEvent *hevt));
-void event_desc_from_list PARAMS ((Side *side, Obj *lis, HistEvent *hevt,
-				   char *buf));
+int pattern_matches_event(Obj *pattern, HistEvent *hevt);
+void event_desc_from_list(Side *side, Obj *lis, HistEvent *hevt, char *buf);
 
-static void notify_event PARAMS ((Side *side, HistEvent *hevt));
-static void play_event_movies PARAMS ((Side *side, HistEvent *hevt));
-static void rewrite_unit_references PARAMS ((int oldid, int newid));
-static void rewrite_unit_references_in_event PARAMS ((HistEvent *hevt,
-						      int oldid, int newid));
+static void notify_event(Side *side, HistEvent *hevt);
+static void play_event_movies(Side *side, HistEvent *hevt);
+static void rewrite_unit_references(int oldid, int newid);
+static void rewrite_unit_references_in_event(HistEvent *hevt, int oldid,
+					     int newid);
 
 extern int any_post_event_scores;
 extern int need_post_event_scores;
@@ -198,9 +197,8 @@ notify_event(side, hevt)
 Side *side;
 HistEvent *hevt;
 {
-    int found = FALSE;
     char abuf[BUFSIZE];
-    Obj *rest, *head, *pattern, *parms, *msgdesc, *text;
+    Obj *rest, *head, *pattern, *text;
     Side *side2;
     Unit *unit;
 
@@ -281,6 +279,8 @@ HistEvent *hevt;
 	    }
 	}
 	break;
+      default:
+	break;
     }
 }
 
@@ -673,7 +673,7 @@ dump_statistics()
        (for instance, the user might have cancelled). */
     if (empty_string(fname))
       return;
-    fp = fopen(fname, "w");
+    fp = open_file(fname, "w");
     if (fp != NULL) {
 	if (1 /* records exist */) {
 	    write_side_results(fp, NULL);
