@@ -1,5 +1,5 @@
 /* Interpretation of generic GDL images for Xconq.
-   Copyright (C) 1994-1998 Stanley T. Shebs.
+   Copyright (C) 1994-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,9 +13,16 @@ any later version.  See the file COPYING.  */
 #include "misc.h"
 #include "lisp.h"
 #include "imf.h"
-extern Image *largest_image PARAMS ((ImageFamily *imf));
+extern Image *get_subimg(ImageFamily *imf, int w, int h);
+extern Image *largest_image(ImageFamily *imf);
+extern int image_pixel_at(Image *img, int imtype, int x, int y);
+extern void set_image_pixel_at(Image *img, int imtype, int x, int y, int val);
+extern void make_image_from_file_image(ImageFamily *imf, Image *img,
+				       Image *subimg, int subi);
+#include "module.h"
+#include "system.h"
 
-extern void syntax_error PARAMS ((Obj *x, char *msg));
+extern void syntax_error(Obj *x, char *msg);
 
 /* RGB above this value should be considered white. */
 
@@ -35,23 +42,24 @@ enum {
     K_MONO_,
     K_MASK_,
     K_COLR_,
+    K_FILE_,
     K_OTHER_
 };
 
-static ImageFamily *new_imf PARAMS ((char *name));
-static Image *add_shrunken_image PARAMS ((ImageFamily *imf));
-static Image *add_magnified_image PARAMS ((ImageFamily *imf));
-static int bitmaps_match PARAMS ((int w, int h, Obj *lispdata, char *rawdata));
-static int color_matches_mono PARAMS ((Image *img));
-static void write_pixmap PARAMS ((FILE *fp, int w, int h, int aw, int ah,
+static ImageFamily *new_imf(char *name);
+static Image *add_shrunken_image(ImageFamily *imf);
+static Image *add_magnified_image(ImageFamily *imf);
+static int bitmaps_match(int w, int h, Obj *lispdata, char *rawdata);
+static int color_matches_mono(Image *img);
+static void write_pixmap(FILE *fp, int w, int h, int aw, int ah,
 				 int pixelsize,
 				 Obj *palette, int *rawpalette, int numcolors,
-				 Obj *lispdata, char *rawdata));
-static void write_bitmap PARAMS ((FILE *fp, char *subtyp, int w, int h,
-				 Obj *data, char *rawdata));
-static void write_palette_contents PARAMS ((FILE *fp, Obj *palette,
-					    int *rawpalette, int numcolors));
-static void write_color PARAMS ((FILE *fp, int n, int r, int g, int b));
+				 Obj *lispdata, char *rawdata);
+static void write_bitmap(FILE *fp, char *subtyp, int w, int h,
+				 Obj *data, char *rawdata);
+static void write_palette_contents(FILE *fp, Obj *palette,
+					    int *rawpalette, int numcolors);
+static void write_color(FILE *fp, int n, int r, int g, int b);
 
 /* This is the array and count of known image families. */
 
@@ -69,9 +77,13 @@ int numpalettes = 0;
 
 ImageFile *image_files;
 
-ImageFamily *(*imf_load_hook) PARAMS ((ImageFamily *imf));
+/* Head of the linked list of "file" images used by several image families. */
 
-ImageFamily *(*imf_interp_hook) PARAMS ((ImageFamily *imf, Image *img, int force));
+FileImage *file_images;
+
+ImageFamily *(*imf_load_hook)(ImageFamily *imf);
+
+ImageFamily *(*imf_interp_hook)(ImageFamily *imf, Image *img, int force);
 
 short write_synthetic_also;
 
@@ -105,7 +117,7 @@ ImageFamily *imf;
     imf2->numsizes = 0;
     /* Clone the images. */
     for_all_images(imf, img) {
-	img2 = get_img(imf2, img->w, img->h, img->style);
+	img2 = get_img(imf2, img->w, img->h, 0);
 	truenext = img2->next;
 	memcpy(img2, img, sizeof(Image));
 	/* Clear the hook, we expect that the caller of this routine
@@ -209,9 +221,9 @@ void
 load_image_families(fp, loadnow, callback)
 FILE *fp;
 int loadnow;
-void (*callback) PARAMS ((ImageFamily *imf, int loadnow));
+void (*callback)(ImageFamily *imf, int loadnow);
 {
-    int done = FALSE, rslt, first = TRUE;
+    int done = FALSE, first = TRUE;
     char buf[BUFSIZE], *buf1, *buf2, *tmp;
     ImageFamily *imf = NULL;
     ImageFile *imfile;
@@ -271,13 +283,13 @@ void (*callback) PARAMS ((ImageFamily *imf, int loadnow));
 int
 load_imf_file(filename, callback)
 char *filename;
-void (*callback) PARAMS ((ImageFamily *imf, int loadnow));
+void (*callback)(ImageFamily *imf, int loadnow);
 {
     int startlineno = 1, endlineno = 1;
     Obj *form;
     FILE *fp;
 
-    fp = fopen(filename, "r");
+    fp = open_file(filename, "r");
     if (fp != NULL) {
 	/* Read everything in the file. */
 	while ((form = read_form(fp, &startlineno, &endlineno)) != lispeof) {
@@ -294,7 +306,7 @@ void (*callback) PARAMS ((ImageFamily *imf, int loadnow));
 void
 interp_imf_form(form, imf_callback)
 Obj *form;
-void (*imf_callback) PARAMS ((ImageFamily *imf, int loadnow));
+void (*imf_callback)(ImageFamily *imf, int loadnow);
 {
     Obj *head;
     ImageFamily *imf;
@@ -318,8 +330,7 @@ void (*imf_callback) PARAMS ((ImageFamily *imf, int loadnow));
 /* Find the image family of the given name, if it exists. */
 
 ImageFamily *
-find_imf(name)
-char *name;
+find_imf(char *name)
 {
     int i;
 
@@ -334,36 +345,35 @@ char *name;
    if necessary. */
 
 Image *
-get_img(imf, w, h, style)
-ImageFamily *imf;
-int w, h;
+get_img(ImageFamily *imf, int w, int h, int subimg)
 {
     Image *img, *nimg, *previmg;
 
     for_all_images(imf, img) {
-	if (w == img->w && h == img->h && style == img->style)
+	if (w == img->w && h == img->h)
 	  return img;
     }
     /* Not found; create a new image and add it to the family. */
     nimg = (Image *) xmalloc(sizeof(Image));
     nimg->w = w;  nimg->h = h;
-    nimg->style = style;
     nimg->embedx = nimg->embedy = -1;
     nimg->embedw = nimg->embedh = -1;
     nimg->monodata = nimg->colrdata = nimg->maskdata = lispnil;
+    nimg->filedata = lispnil;
     nimg->palette = lispnil;
     nimg->actualw = w;  nimg->actualh = h;
     nimg->notes = lispnil;
     nimg->bboxw = w;  nimg->bboxh = h;
+    /* Subimages get handled separately, we don't want to link into the
+       main list of images. */
+    if (subimg)
+      return nimg;
     /* Rely on zeroing of xmalloc blocks to avoid clearing other fields. */
-    /* Link in order by size/style, smallest and lowest-number style
-       first. */
+    /* Link in order by size, smallest first. */
     previmg = NULL;
     for_all_images(imf, img) {
-	if ((nimg->style < img->style)
-	    || (nimg->style == img->style && nimg->w < img->w)
-	    || (nimg->style == img->style && nimg->w == img->w
-		&& nimg->h < img->h))
+	if ((nimg->w < img->w)
+	    || (nimg->w == img->w && nimg->h < img->h))
 	  break;
 	previmg = img;
     }
@@ -378,10 +388,30 @@ int w, h;
     return nimg;
 }
 
+/* Get an image of the given size from the family, creating a new one
+   if necessary. */
+
 Image *
-find_img(imf, w, h)
-ImageFamily *imf;
-int w, h;
+get_subimg(ImageFamily *imf, int w, int h)
+{
+    Image *nimg;
+
+    /* Not found; create a new image and add it to the family. */
+    nimg = (Image *) xmalloc(sizeof(Image));
+    nimg->w = w;  nimg->h = h;
+    nimg->embedx = nimg->embedy = -1;
+    nimg->embedw = nimg->embedh = -1;
+    nimg->monodata = nimg->colrdata = nimg->maskdata = lispnil;
+    nimg->filedata = lispnil;
+    nimg->palette = lispnil;
+    nimg->actualw = w;  nimg->actualh = h;
+    nimg->notes = lispnil;
+    nimg->bboxw = w;  nimg->bboxh = h;
+    return nimg;
+}
+
+Image *
+find_img(ImageFamily *imf, int w, int h)
 {
     Image *img;
 	
@@ -439,21 +469,24 @@ Obj *clauses;
     compute_image_bboxes(imf);
 }
 
+/* Given an image family, a size, and a list describing the elements
+   of a single image, parse the size and elements, put those into the
+   right slots of an image object.  Also detect and warn about changes
+   to an existing image, since this usually indicates some kind of
+   problem. */
+
 void
 interp_image(imf, size, parts)
 ImageFamily *imf;
 Obj *size, *parts;
 {
-    int w, h, style, imtype, emx, emy, emw, emh;
+    int w, h, imtype, emx, emy, emw, emh, subi;
     char *name;
-    Image *img;
+    Image *img, *subimg;
     Obj *head, *rest, *typ, *prop, *proptype, *datalist;
     
     w = c_number(car(size));  h = c_number(cadr(size));
-    style = 0;
-    if (match_keyword(car(cddr(size)), K_SILHOUETTE))
-      style = 1;
-    img = get_img(imf, w, h, style);
+    img = get_img(imf, w, h, 0);
     if (img == NULL)
       run_error("no image?");
     if (img->w == 1 && img->h == 1) {
@@ -464,6 +497,12 @@ Obj *size, *parts;
     }
     if (match_keyword(car(cddr(size)), K_TILE))
       img->istile = TRUE;
+    if (match_keyword(car(cddr(size)), K_CONNECTION))
+      img->isconnection = TRUE;
+    if (match_keyword(car(cddr(size)), K_BORDER))
+      img->isborder = TRUE;
+    if (symbolp(car(cddr(size))) && strcmp(c_string(car(cddr(size))), "transition") == 0)
+      img->istransition = TRUE;
     for_all_list(parts, rest) {
 	head = car(rest);
 	typ = car(head);
@@ -474,6 +513,8 @@ Obj *size, *parts;
 	    imtype = K_MASK_;
 	} else if (match_keyword(typ, K_COLOR)) {
 	    imtype = K_COLR_;
+	} else if (match_keyword(typ, K_FILE)) {
+	    imtype = K_FILE_;
 	} else if (match_keyword(typ, K_EMBED)) {
 	    name = c_string(cadr(head));
 	    if (img->embedname != NULL
@@ -498,9 +539,17 @@ Obj *size, *parts;
 	} else if (match_keyword(typ, K_NOTES)) {
 	    img->notes = cadr(head);
 	    syntax_error(head, "extra junk after image notes property");
+	} else if (match_keyword(typ, K_X)) {
+	    img->numsubimages = c_number(cadr(head));
+	    if (cddr(head) != lispnil) {
+		img->subx = c_number(caddr(head));
+		img->suby = c_number(car(cdddr(head)));
+	    }
 	} else {
 	    run_warning("unknown image property in \"%s\"", imf->name);
 	}
+	/* If there is no actual image data to process, skip to the next
+	   clause in the form. */
 	if (imtype == K_OTHER_)
 	  continue;
 	datalist = cdr(head);
@@ -514,9 +563,6 @@ Obj *size, *parts;
 	    } else if (match_keyword(proptype, K_PIXEL_SIZE)) {
 		img->pixelsize = c_number(cadr(prop));
 	    } else if (match_keyword(proptype, K_PALETTE)) {
-		if (img->palette != lispnil && !equal(cdr(prop), img->palette))
-		  run_warning("Changing palette in %dx%d image of \"%s\"",
-			      w, h, imf->name);
 		img->palette = cdr(prop);
 	    } else {
 		run_warning("unknown image subproperty in \"%s\"", imf->name);
@@ -542,10 +588,47 @@ Obj *size, *parts;
 			  w, h, imf->name);
 	    img->maskdata = datalist;
 	    break;
+	  case K_FILE_:
+	    if (img->filedata != lispnil && !equal(datalist, img->filedata))
+	      run_warning("Changing file data in %dx%d image of \"%s\"",
+			  w, h, imf->name);
+	    img->filedata = datalist;
+	    break;
 	  default:
 	    break;
 	}
     }
+    /* Allocate space for any subimages that might be needed. */
+    /* (should simplify?) */
+    if (img->isborder) {
+	img->subimages = (Image **) xmalloc (16 * sizeof(Image *));
+	for (subi = 0; subi < 16; ++subi) {
+	    subimg = get_subimg(imf, img->w, img->h);
+	    img->subimages[subi] = subimg;
+	}
+    } else if (img->isconnection) {
+	img->subimages = (Image **) xmalloc (64 * sizeof(Image *));
+	for (subi = 0; subi < 64; ++subi) {
+	    subimg = get_subimg(imf, img->w, img->h);
+	    img->subimages[subi] = subimg;
+	}
+    } else if (img->istransition) {
+	/* Leave the main image mostly alone, iterate over the subimages. */
+	img->subimages = (Image **) xmalloc (4 * 4 * sizeof(Image *));
+	for (subi = 0; subi < 4 * 4; ++subi) {
+	    subimg = get_subimg(imf, img->w, img->h);
+	    img->subimages[subi] = subimg;
+	}
+    } else {
+	if (img->numsubimages > 0) {
+	    img->subimages =
+	      (Image **) xmalloc (img->numsubimages * sizeof(Image *));
+	    for (subi = 0; subi < img->numsubimages; ++subi) {
+		subimg = get_subimg(imf, img->w, img->h);
+		img->subimages[subi] = subimg;
+	    }
+	}
+    }
 }
 
 void
@@ -626,14 +709,10 @@ Image *img;
     }
 }
 
-extern int image_pixel_at PARAMS ((Image *img, int imtype, int x, int y));
-
-extern void set_image_pixel_at PARAMS ((Image *img, int imtype, int x, int y, int val));
+/* Get a single pixel from the given type of data of an image. */
 
 int
-image_pixel_at(img, imtype, x, y)
-Image *img;
-int imtype, x, y;
+image_pixel_at(Image *img, int imtype, int x, int y)
 {
     int rowbytes, psize, i, byte, rslt;
     char *rawdata = NULL;
@@ -644,23 +723,23 @@ int imtype, x, y;
     } else if (imtype == K_MASK_) {
 	rawdata = img->rawmaskdata;
 	psize = 1;
-    } else {
+    } else if (imtype == K_COLR_) {
 	rawdata = img->rawcolrdata;
 	psize = img->pixelsize;
     }
-    if (rawdata != NULL) {
-	rowbytes = computed_rowbytes(img->w, psize);
-	i = y * rowbytes + ((x * psize) >> 3);
-	byte = rawdata[i];
-	rslt = (byte >> ((8 - psize) - ((x * psize) & 0x7))) & ((1 << psize) - 1);
-	return rslt;
-    }
+    if (rawdata == NULL)
+      return 0;
+    rowbytes = computed_rowbytes(img->w, psize);
+    i = y * rowbytes + ((x * psize) >> 3);
+    byte = rawdata[i];
+    rslt = (byte >> ((8 - psize) - ((x * psize) & 0x7))) & ((1 << psize) - 1);
+    return rslt;
 }
 
+/* Set a single pixel in the given type of data of an image. */
+
 void
-set_image_pixel_at(img, imtype, x, y, val)
-Image *img;
-int imtype, x, y, val;
+set_image_pixel_at(Image *img, int imtype, int x, int y, int val)
 {
     int rowbytes, psize, i, byte;
     char *rawdata = NULL;
@@ -671,7 +750,7 @@ int imtype, x, y, val;
     } else if (imtype == K_MASK_) {
 	rawdata = img->rawmaskdata;
 	psize = 1;
-    } else {
+    } else if (imtype == K_COLR_) {
 	rawdata = img->rawcolrdata;
 	psize = img->pixelsize;
     }
@@ -687,7 +766,6 @@ int imtype, x, y, val;
 /* If an image family includes no small images (16x16 or less), then
    add an image that is half the size of the smallest image in the
    family. */
-/* (should shrink all styles of smallest images) */
 
 static Image *
 add_shrunken_image(imf)
@@ -703,7 +781,7 @@ ImageFamily *imf;
        small images. */
     if (img->istile || img->synthetic || img->w <= 8 || img->h <= 8)
       return NULL;
-    img2 = get_img(imf, img->w / 2, img->h / 2, img->style);
+    img2 = get_img(imf, img->w / 2, img->h / 2, 0);
     /* Chances are that any embedded subimage will become unrecognizable,
        so leave embedname empty. */
     if (img->embedx > 0)
@@ -719,10 +797,15 @@ ImageFamily *imf;
     img2->rawpalette = img->rawpalette;
     img2->numcolors = img->numcolors;
     img2->synthetic = TRUE;
-    if (img->rawcolrdata == NULL && img->colrdata != lispnil) {
-	numbytes = img->h * computed_rowbytes(img->w, img->pixelsize);
-	img->rawcolrdata = xmalloc(numbytes);
-	interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+    if (img->rawcolrdata == NULL) {
+	/* Try different ways to get some image data. */
+	if (img->colrdata != lispnil) {
+	    numbytes = img->h * computed_rowbytes(img->w, img->pixelsize);
+	    img->rawcolrdata = xmalloc(numbytes);
+	    interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+	} else if (img->filedata != lispnil) {
+	    make_image_from_file_image(imf, img, img, 0);
+	}
     }
     if (img->rawcolrdata != NULL) {
 	numbytes2 = img2->h * computed_rowbytes(img2->w, img2->pixelsize);
@@ -731,11 +814,7 @@ ImageFamily *imf;
     numbytes = img->h * computed_rowbytes(img->w, 1);
     numbytes2 = img2->h * computed_rowbytes(img2->w, 1);
     /* Ensure that binary version of mono image exists. */
-    /* (should make into sep. routine) */
-    if (img->rawmonodata == NULL && img->monodata != lispnil) {
-	img->rawmonodata = xmalloc(numbytes);
-	interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
-    }
+    make_raw_mono_data(img, FALSE);
     if (img->rawmonodata != NULL)
       img2->rawmonodata = xmalloc(numbytes2);
     /* Ensure that binary version of mask exists. */
@@ -791,7 +870,6 @@ ImageFamily *imf;
 
 /* Add an image twice the size of the largest image in the family, if
    the family does not already include a large image. */
-/* (should magnify all styles of smallest images) */
 
 static Image *
 add_magnified_image(imf)
@@ -805,7 +883,7 @@ ImageFamily *imf;
     img = largest_image(imf);
     if (img->istile || img->synthetic || img->w >= 64 || img->h >= 64)
       return NULL;
-    img2 = get_img(imf, img->w * 2, img->h * 2, img->style);
+    img2 = get_img(imf, img->w * 2, img->h * 2, 0);
     img2->embedname = img->embedname;
     if (img->embedx > 0)
       img2->embedx = img->embedx * 2;
@@ -820,10 +898,15 @@ ImageFamily *imf;
     img2->rawpalette = img->rawpalette;
     img2->numcolors = img->numcolors;
     img2->synthetic = TRUE;
-    if (img->rawcolrdata == NULL && img->colrdata != lispnil) {
-	numbytes = img->h * computed_rowbytes(img->w, img->pixelsize);
-	img->rawcolrdata = xmalloc(numbytes);
-	interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+    if (img->rawcolrdata == NULL) {
+	/* Try different ways to get some image data. */
+	if (img->colrdata != lispnil) {
+	    numbytes = img->h * computed_rowbytes(img->w, img->pixelsize);
+	    img->rawcolrdata = xmalloc(numbytes);
+	    interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+	} else if (img->filedata != lispnil) {
+	    make_image_from_file_image(imf, img, img, 0);
+	}
     }
     if (img->rawcolrdata != NULL) {
 	numbytes2 = img2->h * computed_rowbytes(img2->w, img2->pixelsize);
@@ -831,10 +914,7 @@ ImageFamily *imf;
     }
     numbytes = img->h * computed_rowbytes(img->w, 1);
     numbytes2 = img2->h * computed_rowbytes(img2->w, 1);
-    if (img->rawmonodata == NULL && img->monodata != lispnil) {
-	img->rawmonodata = xmalloc(numbytes);
-	interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
-    }
+    make_raw_mono_data(img, FALSE);
     if (img->rawmonodata != NULL)
       img2->rawmonodata = xmalloc(numbytes2);
     if (img->rawmaskdata == NULL && img->maskdata != lispnil ) {
@@ -866,6 +946,17 @@ ImageFamily *imf;
     return img2;
 }
 
+void
+make_raw_mono_data(Image *img, int force)
+{
+    int numbytes = img->h * computed_rowbytes(img->w, 1);
+
+    if ((img->rawmonodata == NULL || force) && img->monodata != lispnil) {
+	img->rawmonodata = xmalloc(numbytes);
+	interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
+    }
+}
+
 /* Given a list of strings, interpret the hex digits and put the
    results at the given address. */
 
@@ -997,48 +1088,39 @@ best_image(imf, w, h)
 ImageFamily *imf;
 int w, h;
 {
-    Image *img, *best = NULL, *fallback = NULL;
+    Image *img, *best = NULL, *fallback = NULL, *best_tile = NULL;
 
     if (imf == NULL || imf->images == NULL)
       return NULL;
-    if (imf->images->istile) {
-	/* For tiles, always use the largest available. */
-	/* (should test depth also...) */
-	for_all_images(imf, img) {
-	    if (best == NULL || (img->w > best->w && img->h > best->h))
-	      best = img;
+    for_all_images(imf, img) {
+	/* Exact matches need no further searching. */
+	if (w == img->w && h == img->h && !img->istile)
+	  return img;
+	/* Find the best size of image. */
+	if (best == NULL
+	    || (img->w <= w && img->h <= h
+		&& img->w > best->w && img->h > best->h))
+	  best = img;
+	/* For tiles, pick out the largest tile. */
+	if (img->istile) {
+	    if (best_tile == NULL
+		|| (img->w > best_tile->w && img->h > best_tile->h))
+	      best_tile = img;
 	}
-    } else {
-	for_all_images(imf, img) {
-	    /* Exact matches need no further searching. */
-	    if (w == img->w && h == img->h
-		&& imf->preferred_style == img->style)
-	      return img;
-	    if (imf->preferred_style == img->style) {
-		/* Find the best image with a matching style. */
-		if (best == NULL
-		    || (img->w <= w && img->h <= h
-			&& img->w > best->w && img->h > best->h))
-		  best = img;
-	    } else {
-		/* Find the best match in a non-matching style. */
-		if (fallback == NULL
-		    || (img->w <= w && img->h <= h
-			&& img->w > fallback->w && img->h > fallback->h))
-		  fallback = img;
-	    }
-	}
-	/* If no image in our preferred style, fall back to images in
-	   other styles. */
-	if (best == NULL)
+    }
+    /* If there were any tiles at all for this image, we're probably
+       here because it's a terrain image for which no exact matches
+       were found; so return the largest tile instead. */
+    if (best_tile != NULL)
+      return best_tile;
+    /* If the best image is too large or too small (at this point we
+       know it's not a tiling pattern), scale the image and return that. */
+    if (best->w > w && best->h > h) {
+	best = add_shrunken_image(imf);
+    } else if (best->w <= (w >> 1) && best->h <= (h >> 1)) {
+	fallback = add_magnified_image(imf);
+	if (fallback != NULL)
 	  best = fallback;
-	if (best->w > w && best->h > h) {
-	    best = add_shrunken_image(imf);
-	} else if (best->w <= (w >> 1) && best->h <= (h >> 1)) {
-	    fallback = add_magnified_image(imf);
-	    if (fallback != NULL)
-	      best = fallback;
-	}
     }
     return best;
 }
@@ -1164,18 +1246,14 @@ int sw, sh, *exxp, *eyyp, *ewp, *ehp;
    and *requires* that all image families be named and named uniquely. */
 
 static int
-image_name_compare(imf1, imf2)
-#ifdef THINK_C
-const
-#endif
-void *imf1, *imf2;
+image_name_compare(CONST void *imf1, CONST void *imf2)
 {
     return strcmp((*((ImageFamily **) imf1))->name,
 		  (*((ImageFamily **) imf2))->name);
 }
 
 void
-sort_all_images()
+sort_all_images(void)
 {
     qsort(&(images[0]), numimages, sizeof(ImageFamily *), image_name_compare);
 }
@@ -1184,18 +1262,14 @@ sort_all_images()
    and *requires* that all image palettes be named and named uniquely. */
 
 static int
-palette_name_compare(imp1, imp2)
-#ifdef THINK_C
-const
-#endif
-void *imp1, *imp2;
+palette_name_compare(CONST void *imp1, CONST void *imp2)
 {
     return strcmp((*((ImagePalette **) imp1))->name,
 		  (*((ImagePalette **) imp2))->name);
 }
 
 void
-sort_all_palettes()
+sort_all_palettes(void)
 {
     qsort(&(palettes[0]), numpalettes, sizeof(ImagePalette *), palette_name_compare);
 }
@@ -1232,12 +1306,9 @@ ImageFamily *imf;
 }
 
 static int
-bitmaps_match(w, h, lispdata, rawdata)
-int w, h;
-Obj *lispdata;
-char *rawdata;
+bitmaps_match(int w, int h, Obj *lispdata, char *rawdata)
 {
-    int i, j = 0, rowbytes, numbytes, byte, jump = 0;
+    int i, j = 0, rowbytes, numbytes, byte;
     char *datastr = NULL;
 
     rowbytes = computed_rowbytes(w, 1);
@@ -1263,17 +1334,14 @@ char *rawdata;
 /* Write the imf directory for the given set of images. */
 
 void
-write_imf_dir(filename, images, num)
-char *filename;
-ImageFamily **images;
-int num;
+write_imf_dir(char *filename, ImageFamily **images, int num)
 {
     int i;
     char *loc;
     ImageFamily *imf;
     FILE *fp;
 
-    fp = fopen(filename, "w");
+    fp = open_file(filename, "w");
     if (fp != NULL) {
 	fprintf(fp, "ImageFamilyName FileName\n");
 	for (i = 0; i < num; ++i) {
@@ -1330,9 +1398,19 @@ ImageFamily *imf;
 	    fprintf(fp, "(%d %d", img->w, img->h);
 	    if (img->istile && !(img->w == 1 && img->h == 1))
 	      fprintf(fp, " %s", keyword_name(K_TILE));
-	    if (img->style == 1)
-	      fprintf(fp, " %s", keyword_name(K_SILHOUETTE));
+	    if (img->isconnection)
+	      fprintf(fp, " %s", keyword_name(K_CONNECTION));
+	    if (img->isborder)
+	      fprintf(fp, " %s", keyword_name(K_BORDER));
+	    if (img->istransition)
+	      fprintf(fp, " %s", "transition");
 	    fprintf(fp, ")");
+	    if (img->numsubimages > 0) {
+	      fprintf(fp, " (%s %d", keyword_name(K_X), img->numsubimages);
+	      if (img->subx > 0 || img->suby > 0)
+		fprintf(fp, " %d %d", img->subx, img->suby);
+	      fprintf(fp, ")");
+	    }
 	    if (img->embedname) {
 		fprintf(fp, " (%s \"%s\")",
 			keyword_name(K_EMBED), img->embedname);
@@ -1366,6 +1444,14 @@ ImageFamily *imf;
 				    c_number(caddr(palent)));
 		    }
 		}
+	    } else if (img->filedata != lispnil) {
+		fprintf(fp, " (%s ", keyword_name(K_FILE));
+		fprintf(fp, " \"%s\"", c_string(car(img->filedata)));
+		if (cdr(img->filedata) != lispnil)
+		  fprintf(fp, " %d %d",
+			  c_number(cadr(img->filedata)),
+			  c_number(caddr(img->filedata)));
+		fprintf(fp, ")");
 	    } else if ((img->colrdata != lispnil || img->rawcolrdata)
 			&& !color_matches_mono(img)) {
 		fprintf(fp, "\n  ");
@@ -1394,10 +1480,9 @@ ImageFamily *imf;
    the same as the mono version of the image. */
 
 static int
-color_matches_mono(img)
-Image *img;
+color_matches_mono(Image *img)
 {
-    int i, cj, mj, rowbytes, numbytes, cbyte, mbyte, jump = 0;
+    int i, cj, mj, rowbytes, numbytes, cbyte, mbyte;
     int col[2], red[2], grn[2], blu[2];
     char *cdatastr = NULL, *mdatastr = NULL;
     Obj *clispdata = img->colrdata, *mlispdata = img->monodata, *palette;
@@ -1672,7 +1757,8 @@ ImagePalette *imp;
     fprintf(fp, ")\n");
 }
 
-/* Given rgb components, return names of standard colors if the match is close. */
+/* Given rgb components, return names of standard colors if the match
+   is close. */
 
 char *
 find_color_name(r, g, b)
@@ -1738,3 +1824,210 @@ int *col, *red, *grn, *blu;
 	init_warning("palette color info is not a name or set of numbers, ignoring");
     }
 }
+
+/* Given a filename, find or create a file image structure for it. */
+
+FileImage *get_file_image(char *fname)
+{
+    FileImage *fimg, *newfimg;
+
+    for (fimg = file_images; fimg != NULL; fimg = fimg->next) {
+	if (strcmp(fimg->name, fname) == 0)
+	  return fimg;
+    }
+    newfimg = (FileImage *) xmalloc(sizeof(FileImage));
+    newfimg->name = fname;
+    newfimg->next = file_images;
+    file_images = newfimg;
+    return newfimg;
+}
+
+/* Collect the file image for the given image and use it to generate
+   the image's (or subimage's) raw data. */
+
+void
+make_image_from_file_image(ImageFamily *imf, Image *img, Image *subimg,
+			   int subi)
+{
+    int hch, pad = 2;
+    int xoffset, yoffset, xoff, yoff;
+    FileImage *fimg;
+
+    if (img->filedata != lispnil) {
+	if (img->file_image == NULL) {
+	    img->file_image = get_file_image(c_string(car(img->filedata)));
+	}
+    }
+    fimg = img->file_image;
+    load_file_image(fimg);
+    if (!fimg->loaded) {
+	init_warning("Could not load file \"%s\" for image family \"%s\", ignoring",
+		     fimg->name, imf->name);
+	return;
+    }
+    xoffset = yoffset = 0;
+    if (cdr(img->filedata) != lispnil) {
+	xoffset = c_number(cadr(img->filedata));
+	yoffset = c_number(caddr(img->filedata));
+    }
+    /* All subimages share color data. */
+    if (subimg == img ||
+	(img->subimages != NULL && subimg == img->subimages[0])) {
+	subimg->pixelsize = 8;
+	/* Copy the palette over verbatim from the file image. */
+	subimg->numcolors = fimg->numcolors;
+	subimg->rawpalette =
+	  (int *) xmalloc(subimg->numcolors * 4 * sizeof(int));
+	memcpy(subimg->rawpalette, fimg->palette,
+	       subimg->numcolors * 4 * sizeof(int));
+    } else {
+	/* Inherit color data from the first subimage. */
+	subimg->pixelsize = img->subimages[0]->pixelsize;
+	subimg->numcolors = img->subimages[0]->numcolors;
+	subimg->rawpalette = img->subimages[0]->rawpalette;
+    }
+    if (img->isborder) {
+	/* a hack since this code doesn't know true hch */
+	hch = img->h;
+	if (img->h == 26)
+	  hch = 20;
+	else if (img->h == 48)
+	  hch = 37;
+	else if (img->h == 96)
+	  hch = 74;
+	xoff = xoffset + (subi % 4) * img->w;
+	yoff = yoffset + (subi / 4) * hch;
+	copy_from_file_image(subimg, fimg, xoff, yoff);
+    } else if (img->isconnection) {
+	xoff = xoffset + (subi % 8) * (img->w + pad);
+	yoff = yoffset + (subi / 8) * (img->h + pad);
+	copy_from_file_image(subimg, fimg, xoff, yoff);
+    } else if (img->istransition) {
+	xoff = xoffset + (subi % 4) * (img->w + pad);
+	yoff = yoffset + (subi / 4) * (img->h + pad);
+	copy_from_file_image(subimg, fimg, xoff, yoff);
+    } else if (img->numsubimages > 0) {
+	/* Figure where the next subimage is at. */
+	if (img->subx > 0 || img->suby > 0) {
+	    xoff = xoffset + subi * img->subx;
+	    yoff = yoffset + subi * img->suby;
+	} else {
+	    xoff = xoffset + subi * img->w;
+	    yoff = yoffset;
+	}
+	copy_from_file_image(subimg, fimg, xoff, yoff);
+    } else {
+	copy_from_file_image(img, fimg, xoffset, yoffset);
+    }
+}
+
+/* Extract a single color image from a file image.  Also create mask
+   data for the image, if the file image has any transparent colors.  */
+
+void
+copy_from_file_image(Image *img, FileImage *fimg, int xoffset, int yoffset)
+{
+    char pix;
+    int i, j, k, ii, val, kk, kkb;
+
+    /* Make space for the color data (assuming 1 byte/pixel). */
+    img->rawcolrdata = (char *) xmalloc(img->w * img->h);
+    if (fimg->numtransparent > 0)
+      img->rawmaskdata =
+	(char *) xmalloc(img->h * computed_rowbytes(img->w, 1));
+    /* Scan through all the pixels of the subimage we're building. */
+    for (i = 0; i < img->h; ++i) {
+	for (j = 0; j < img->w; ++j) {
+	    k = (yoffset + i) * fimg->width + xoffset + j;
+	    pix = fimg->data[k];
+	    kk = i * img->w + j;
+	    img->rawcolrdata[kk] = pix;
+	    /* If there are transparent colors in the image, modify
+	       the mask bitmap. */
+	    if (fimg->numtransparent > 0) {
+		val = 1;
+		for (ii = 0; ii < fimg->numtransparent; ++ii) {
+		    if (pix == fimg->transparent[ii]) {
+			val = 0;
+			break;
+		    }
+		}
+		if (val) {
+		    kkb = i * computed_rowbytes(img->w, 1) + j / 8;
+		    img->rawmaskdata[kkb] |= 1 << (7 - j % 8);
+		}
+	    }
+	}
+    }
+}
+
+/* Give a file image, attempt to load it into memory. */
+
+void
+load_file_image(FileImage *fimg)
+{
+    int rslt;
+
+    if (fimg->loaded)
+      return;
+    /* Only doing GIFs right now. */
+    rslt = get_gif(fimg);
+    if (rslt)
+      fimg->loaded = TRUE;
+}
+
+/* Generic image setup. */
+
+static short imf_dir_loaded;
+
+ImageFamily *
+get_generic_images(char *name)
+{
+    FILE *fp;
+    int found = FALSE;
+    ImageFamily *imf;
+    LibraryPath *path;
+
+    imf = get_imf(name);
+    if (imf == NULL)
+      return NULL;
+    if (imf->numsizes > 0 && imf_interp_hook != NULL)
+      imf = (*imf_interp_hook)(imf, NULL, FALSE);
+    if (imf_load_hook != NULL)
+      imf = (*imf_load_hook)(imf);
+    if (imf->numsizes == 0) {
+	/* Maybe collect the names/locations of all image families. */
+	if (!imf_dir_loaded) {
+	    /* (should let name be decided by platform?) */
+	    fp = open_library_file("imf.dir");
+	    if (fp != NULL) {
+		load_image_families(fp, FALSE, NULL);
+		fclose(fp);
+	    } else {
+		init_warning("Cannot open \"%s\", will not use it", "imf.dir");
+	    }
+	    imf_dir_loaded = TRUE;
+	}
+	/* Get a (possibly empty) family. */
+	imf = get_imf(name);
+	if (imf == NULL)
+	  return NULL;
+	if (imf->location != NULL) {
+	    /* Load data filling in the family. */
+	    for_all_library_paths(path) {
+		make_pathname(path->path, imf->location->name, "", spbuf);
+		if (load_imf_file(spbuf, NULL))
+		  found = TRUE;
+		  break;
+	    }
+	    /* Maybe try the plain filename, just in case. */
+	    if (!found)
+	      load_imf_file(imf->location->name, NULL);
+	    /* We don't complain here about not finding the file, because
+	       we'll get a more useful warning alert later on. */
+	    if (imf_interp_hook != NULL)
+	      imf = (*imf_interp_hook)(imf, NULL, FALSE);
+	}
+    }
+    return imf;
+}
