@@ -18,6 +18,7 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kernel.h"
+#include "ai.h"
 #include "version.h"
 extern int gameinited;
 
@@ -96,30 +97,6 @@ int max_zoc_range;
 
 short *u_possible;
 
-// --- ADVANCED UNIT AI SUPPORT ------------------------
-
-int *advanced_utypes; 
-int *colonizer_utypes; 
-int *offensive_utypes; 
-int *defensive_utypes; 
-int *improver_utypes; 
-int *facility_utypes; 
-int *explorer_utypes; 
-int *mobile_utypes; 
-
-int numadv = 0;
-int numdef = 0;
-int numoff = 0;
-int numimp = 0;
-int numcol = 0;
-int numexp = 0;
-int nummob = 0;
-int numfac = 0;
-
-void setup_utype_vectors(void);
-
-// ------------------------------------------------
-
 /* The table of all available synthesis methods. */
 
 struct synthtype {
@@ -152,17 +129,23 @@ init_library_path(path)
 char *path;
 {
     char *xconqlib;
+    LibraryPath *image_path;
 
     xconqlib = getenv("XCONQLIB");
     if (empty_string(xconqlib)) {
 	if (!empty_string(path)) {
 	    xconqlib = path;
 	} else {
-	    xconqlib = default_library_filename();
+	    xconqlib = default_library_pathname();
 	}
     }
     xconq_libs = (LibraryPath *) xmalloc(sizeof(LibraryPath));
     xconq_libs->path = copy_string(xconqlib);
+    /* Add the pathname to the binary images directory, as an
+       alternate place to look. */
+    image_path = (LibraryPath *) xmalloc(sizeof(LibraryPath));
+    image_path->path = copy_string(default_images_pathname());
+    xconq_libs->next = image_path;
     last_user_xconq_lib = NULL;
     standard_game_name = getenv("XCONQSTANDARDGAME");
     if (empty_string(standard_game_name))
@@ -245,10 +228,11 @@ set_g_synth_methods_default()
 }
 
 /* Run a doublecheck on plausibility of game parameters.  Additional
-   checks are performed elsewhere as needed, for instance during random
-   generation.  Serious mistakes exit now, since they can cause all sorts
-   of strange behavior and core dumps.  It's a little more friendly to only
-   exit at the end of the tests, so all the mistakes can be found at once. */
+   checks are performed elsewhere as needed, for instance during
+   random generation.  Serious mistakes exit now, since they can cause
+   all sorts of strange behavior and core dumps.  It's a little more
+   friendly to only exit at the end of the tests, so all the mistakes
+   can be found at once. */
 
 /* In theory, if a game passes these tests, then Xconq will never crash. */
 
@@ -256,7 +240,7 @@ void
 check_game_validity()
 {
     int failed = FALSE, movers = FALSE, actors = FALSE;
-    int u1, u2, m1, t1, t2, a1, i, methkey, found;
+    int u1, u2, t1, t2, i, methkey, found;
     Obj *synthlist = g_synth_methods(), *methods, *method;
 
     /* We must have at least one kind of unit. */
@@ -361,28 +345,8 @@ check_game_validity()
 	if (u_acp(u1) > 0) {
 	    actors = TRUE;
 	}
-    }
-    /* Check various material type properties. */
-    for_all_material_types(m1) {
-	/* Can't make use of this yet, so error out if anybody tries. */
-	if (m_available(m1) != 1) {
-	    init_error("material type %d not always available", m1);
-	    failed = TRUE;
-	}
-    }
-    /* Check various terrain type properties. */
-    for_all_terrain_types(t1) {
-	/* Can't make use of this yet, so error out if anybody tries. */
-	if (t_available(t1) != 1) {
-	    init_error("terrain type %d not always available", t1);
-	    failed = TRUE;
-	}
-    }
-    /* Check various advance type properties. */
-    for_all_advance_types(a1) {
-	/* Can't make use of this yet, so error out if anybody tries. */
-	if (a_available(a1) != 1) {
-	    init_error("advance type %d not always available", a1);
+	if (u_wrecked_type(u1) == u1) {
+	    init_error("unit type %d wrecks to same type.", u1);
 	    failed = TRUE;
 	}
     }
@@ -399,15 +363,16 @@ check_game_validity()
 	init_error("no terrain type has been allowed for cells");
 	failed = TRUE;
     }
-    /* This is a really bad game definition, leave before we crash.  This would
-       only be executed on systems where init_error doesn't exit immediately. */
+    /* This is a really bad game definition, leave before we crash.
+       This would only be executed on systems where init_error doesn't
+       exit immediately. */
     if (failed) {
 	exit(0);
     }
     Dprintf("Finished checking game design.\n");
-
-    Dprintf("It defines %d unit types, %d material types, %d terrain types, %d advance types.\n",
-	    numutypes, nummtypes, numttypes, numatypes);
+    Dprintf("It defines %d unit types, %d material types, %d terrain types",
+	    numutypes, nummtypes, numttypes);
+    Dprintf(", %d advance types.\n", numatypes);
 }
 
 /* Calculate the values of global variables that are used everywhere. */
@@ -440,10 +405,10 @@ calculate_globals()
    if called more than once. */
 
 void
-patch_object_references()
+patch_object_references(void)
 {
     int numhere, numoccs, t, id;
-    Unit *unit, *transport, *unit2;
+    Unit *unit, *transport;
     Side *side;
     Obj *utref, *utorig;
 
@@ -882,10 +847,10 @@ remove_side_and_player()
    the player(s). */
 
 void
-run_synth_methods()
+run_synth_methods(void)
 {
     int i, methkey, found, rslt;
-    Obj *restlist, *done, *synthlist, *methods, *method, *parms, *donelist;
+    Obj *restlist, *done, *synthlist, *methods, *method, *donelist;
 
     /* Restore any saved state for synthmethods. */
     if (!gameinited) {
@@ -1057,13 +1022,6 @@ final_init()
     check_consistency();
     cache_possible_types();
     kill_excess_units();
-
-// --- ADVANCED UNIT AI SUPPORT --------------------------------
-
-    setup_utype_vectors();
-
-// --------------------------------------------------------
-
     /* Count the units initially present. */
     init_side_balance();
     /* Fix up garbled view data. */
@@ -1411,7 +1369,7 @@ saved_game()
 {
     FILE *fp;
 
-    fp = fopen(saved_game_filename(), "r");
+    fp = open_file(saved_game_filename(), "r");
     if (fp != NULL) {
 	fclose(fp);
 	return TRUE;
@@ -1424,9 +1382,9 @@ saved_game()
    if all the balance sheets are completely blank. */
 
 void
-init_side_balance()
+init_side_balance(void)
 {
-    int u, anything = FALSE;
+    int u;
     Unit *unit;
     Side *side;
 
@@ -1498,49 +1456,6 @@ cache_possible_types()
     }
 }
 
-
-// --- ADVANCED UNIT AI SUPPORT ----------------------------------------
-
-/* The idea is to let the game designer specify which units are best for different plans instead 
-of having the AI do complex calculations each turn (which seldom give the right answer). */
-
-void
-setup_utype_vectors()
-{
-	int	u;
-
-	advanced_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-	colonizer_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-	offensive_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-	defensive_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-	improver_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-	facility_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-	explorer_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-	mobile_utypes = (int *) xmalloc(sizeof(int) * numutypes);
-
-	/* Note: one utype can belong to several different vectors. */
-	for_all_unit_types(u) {
-		if (u_colonizer(u))
-			colonizer_utypes[numcol++] = u;			
-		if (u_offensive(u))
-			offensive_utypes[numoff++] = u;			
-		if (u_defensive(u))
-			defensive_utypes[numdef++] = u;			
-		if (u_improver(u))
-			improver_utypes[numimp++] = u;			
-		if (u_explorer(u))
-			explorer_utypes[numexp++] = u;			
-		if (u_advanced(u))
-			advanced_utypes[numadv++] = u;			
-		if (u_facility(u))
-			facility_utypes[numfac++] = u;			
-		if (mobile(u))
-			mobile_utypes[nummob++] = u;			
-	}
-}
-    
-// ------------------------------------------------------------
-
 /* This does the actual assignment of players to sides, and initializes the
    side structures appropriately. */
 
