@@ -1,5 +1,5 @@
 /* Support for Lisp-style data.
-   Copyright (C) 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -12,21 +12,21 @@ any later version.  See the file COPYING.  */
 #include "config.h"
 #include "misc.h"
 #include "lisp.h"
-extern char *escaped_symbol PARAMS ((char *str));
-extern char *escaped_string PARAMS ((char *str));
-extern void interp_short_array PARAMS ((short *arr, Obj *lis, int n));
-extern void interp_long_array PARAMS ((long *arr, Obj *lis, int n));
-extern int eval_number PARAMS ((Obj *val, int *isnumber));
 
-static Symentry *lookup_string PARAMS ((char *str));
-static int hash_name PARAMS ((char *str));
+/* Declarations of local functions. */
 
-static int strmgetc PARAMS ((Strm *strm));
-static void strmungetc PARAMS ((int ch, Strm *strm));
-static void sprintf_context PARAMS ((char *buf, int n, int *start, int *end, Strm *strm));
-static Obj *read_form_aux PARAMS ((Strm *strm));
-static Obj *read_list PARAMS ((Strm *strm));
-static int read_delimited_text PARAMS ((Strm *strm, char *delim, int spacedelimits, int eofdelimits));
+static Obj *newobj(void);
+static Symentry *lookup_string(char *str);
+static int hash_name(char *str);
+
+static int strmgetc(Strm *strm);
+static void strmungetc(int ch, Strm *strm);
+static void sprintf_context(char *buf, int n, int *start, int *end,
+			    Strm *strm);
+static Obj *read_form_aux(Strm *strm);
+static Obj *read_list(Strm *strm);
+static int read_delimited_text(Strm *strm, char *delim, int spacedelimits,
+			       int eofdelimits);
 
 /* Pointer to "nil", the empty list. */
 
@@ -75,7 +75,7 @@ static char *escapedthingbuf;
 /* Allocate a new Lisp object, count it as such. */
 
 static Obj *
-newobj()
+newobj(void)
 {
     lispmalloc += sizeof(Obj);
     return ((Obj *) xmalloc(sizeof(Obj)));
@@ -1413,8 +1413,7 @@ Obj *sym;
 /* List evaluation just blasts straight through the list. */
 
 Obj *
-eval_list(lis)
-Obj *lis;
+eval_list(Obj *lis)
 {
     if (lis == lispnil) {
 	return lispnil;
@@ -1424,9 +1423,7 @@ Obj *lis;
 }
 
 int
-eval_boolean_expression(expr, fn, dflt)
-Obj *expr;
-int (*fn) PARAMS ((Obj *)), dflt;
+eval_boolean_expression(Obj *expr, int (*fn)(Obj *), int dflt)
 {
     char *opname;
 
@@ -1452,12 +1449,8 @@ int (*fn) PARAMS ((Obj *)), dflt;
 }
 
 int
-eval_number(val, isnumber)
-Obj *val;
-int *isnumber;
+eval_number(Obj *val, int *isnumber)
 {
-    Obj *rslt;
-
     /* (should have a non-complaining eval for this) */
     if (numberp(val)) {
 	*isnumber = TRUE;
