@@ -1,5 +1,5 @@
 /* Mac-specific code for Xconq kernel.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -45,21 +45,29 @@ static void close_appletalk(void);
 #endif /* THINK_C */
 
 #ifdef MPW
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#else
 #include <Types.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
 #include <Events.h>  /* for TickCount */
 #include <TextUtils.h>  /* for GetIndString */
 #endif /* MPW */
 
 #ifdef __MWERKS__
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#else
 #include <Types.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
 #include <Events.h>  /* for TickCount */
 #include <TextUtils.h>  /* for GetIndString */
 #include <Quickdraw.h>
 #include <StandardFile.h>
 #include <Devices.h>
-#endif
+#endif /* __MWERKS__ */
 
 #include <signal.h>
 
@@ -119,16 +127,29 @@ static char *checkpoint_fname;
 static char *error_save_fname;
 static char *statistics_fname;
 
+/* We normally keep the library and image folders in the same folder
+   as the app. */
+
 #ifndef XCONQLIB
 #define XCONQLIB ":lib"
 #endif
 
+#ifndef XCONQIMAGES
+#define XCONQIMAGES ":images"
+#endif
+
 char *
-default_library_filename()
+default_library_pathname()
 {
     return XCONQLIB;
 }
 
+char *
+default_images_pathname()
+{
+    return XCONQIMAGES;
+}
+
 char *
 news_filename()
 {
@@ -226,6 +247,48 @@ statistics_filename()
     return statistics_fname;
 }
 
+/* The purpose of this wrapper for fopen is to handle cases where the 
+mac is opening a unix file and automatic linefeed conversion therefore
+should be disabled. */
+
+FILE *
+open_file(char *filename, char *mode)
+{
+	FILE 	*fp;
+	int 	mac_linefeeds = 0;
+	int 	unix_linefeeds = 0;
+	int	ch;
+
+	/* First open in binary read mode (no conversion). */
+	fp = fopen(filename, "rb");
+	/* Don't proceed if we failed. */
+	if (fp == NULL)
+		return NULL;
+	if (strcmp(mode, "rb") == 0)
+	  return fp;
+	/* Count the total number of linefeeds of each kind. */
+	while ((ch = getc(fp)) != EOF) {
+		if (ch == '\r')
+			++mac_linefeeds;
+		if (ch == '\n')
+			++unix_linefeeds;
+	}
+	/* Close the file in case we wish to append or write. */
+	fclose(fp);
+	/* Assume that a mac file has more mac than unix linefeeds. */
+	if (mac_linefeeds > unix_linefeeds) {
+		/* Reopen mac files in text mode (with conversion). */
+		fp = fopen(filename, mode);
+	} else	{
+		/* Reopen unix files in binary mode (no conversion). */
+		fp = fopen(filename, strcat(mode, "b"));
+		/* Remove the "b" when done, or mode will be "rbbbbbb" 
+		after a few unix files have been opened. */
+		mode[1] = '\0';
+	}
+	return fp;		
+}
+
 /* Attempt to open a library file. */
 
 FILE *
@@ -243,7 +306,7 @@ open_module_library_file(Module *module)
     for_all_library_paths(p) {
 	make_pathname(p->path, module->name, "g", fullnamebuf);
 	/* Now try to open the file. */
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL) {
 	    /* Remember the filename where we found it. */
 	    module->filename = copy_string(fullnamebuf);
@@ -252,7 +315,7 @@ open_module_library_file(Module *module)
 	/* Try the same name in a different directory. */
 	GetVol(NULL, &curvrefnum);
 	SetVol(NULL, initialvrefnum);
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	SetVol(NULL, curvrefnum);
 	if (fp != NULL) {
 	    /* Remember the filename (what about volume?) where we found it. */
@@ -277,12 +340,12 @@ open_module_explicit_file(Module *module)
 	    for_all_library_paths(p) {
 		make_pathname(p->path, module->name, "g", fullnamebuf);
 		/* Now try to open the file. */
-		fp = fopen(fullnamebuf, "r");
+		fp = open_file(fullnamebuf, "r");
 		if (fp != NULL)
 		  return fp;
 		GetVol(NULL, &curvrefnum);
 		SetVol(NULL, initialvrefnum);
-		fp = fopen(fullnamebuf, "r");
+		fp = open_file(fullnamebuf, "r");
 		SetVol(NULL, curvrefnum);
 		if (fp != NULL)
 		  return fp;
@@ -291,23 +354,23 @@ open_module_explicit_file(Module *module)
     } else {
 	/* Try some other random ideas. */
 	sprintf(fullnamebuf, "%s", module->filename);
-	fp = fopen(module->filename, "r");
+	fp = open_file(module->filename, "r");
 	if (fp != NULL) {
 	    add_library_path("");
 	    return fp;
 	}
 	sprintf(fullnamebuf, ":%s", module->filename);
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL)
 	  return fp;
 	sprintf(fullnamebuf, "%s%s", ":lib:", module->filename);
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL)
 	  return fp;
 	/* Try opening a library module under where the program started. */
 	GetVol(NULL, &curvrefnum);
 	SetVol(NULL, initialvrefnum);
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	SetVol(NULL, curvrefnum);
     }
     return fp;
@@ -322,20 +385,20 @@ open_library_file(char *filename)
     FILE *fp;
 	
     /* Now try to open the file. */
-    fp = fopen(filename, "r");
+    fp = open_file(filename, "r");
     if (fp != NULL)
       return fp;
     /* Generate library pathname. */
     for_all_library_paths(p) {
 	make_pathname(p->path, filename, NULL, fullnamebuf);
 	/* Now try to open the file. */
-	fp = fopen(fullnamebuf, "r");
+ 	fp = open_file(fullnamebuf, "r");
 	if (fp != NULL)
 	  return fp;
 	/* Change to volume where the program started. */
 	GetVol(NULL, &curvrefnum);
 	SetVol(NULL, initialvrefnum);
-	fp = fopen(fullnamebuf, "r");
+	fp = open_file(fullnamebuf, "r");
 	SetVol(NULL, curvrefnum);
 	if (fp != NULL)
 	  return fp;
@@ -354,7 +417,7 @@ open_scorefile_for_reading(char *name)
     SetVol(NULL, initialvrefnum);
     /* Now try to open the file. */
     sprintf(buf, ":Scores:%s", name);
-    fp = fopen(buf, "r");
+    fp = open_file(buf, "r");
     SetVol(NULL, curvrefnum);
     return fp;
 }
@@ -370,7 +433,7 @@ open_scorefile_for_writing(char *name)
     SetVol(NULL, initialvrefnum);
     /* Now try to open the file. */
     sprintf(buf, ":Scores:%s", name);
-    fp = fopen(buf, "a");
+    fp = open_file(buf, "a");
     SetVol(NULL, curvrefnum);
     return fp;
 }
