@@ -1,5 +1,5 @@
 /* Name generation for Xconq.
-   Copyright (C) 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 
-Obj *choose_from_weighted_list PARAMS ((Obj *lis, int *totalweightp));
+Obj *choose_from_weighted_list(Obj *lis, int *totalweightp);
 
 Obj *namerlist;
 
@@ -64,10 +64,9 @@ int *totalweightp;
 /* Pick a side name that is not already being used. */
 
 void
-make_up_side_name(side)
-Side *side;
+make_up_side_name(Side *side)
 {
-    int uniq = FALSE, tries = 0, n, sofar;
+    int uniq = FALSE, tries = 0, n;
     Obj *sidelib, *subobj, *subelts, *head, *filler;
 
     sidelib = g_side_lib();
@@ -199,7 +198,7 @@ propose_unit_name(unit)
 Unit *unit;
 {
     int u;
-    char *method;
+    char *method, *sidename;
 
     if (unit == NULL)
       return NULL;
@@ -214,12 +213,14 @@ Unit *unit;
 	switch (keyword_code(method)) {
 	  case K_JUNKY:
 	    /* Kind of a bizarre thing, but flavorful sometimes. */
-	    if (unit->side) {
-		sprintf(spbuf, "%c%c-%s-%02ld",
-			uppercase(unit->side->name[0]),
-			uppercase(unit->side->name[1]),
+	    sidename = side_name(unit->side);
+	    if (!empty_string(sidename) && strlen(sidename) >= 2) {
+		sprintf(spbuf, "%c%c-%s-%02d",
+			uppercase(sidename[0]), uppercase(sidename[1]),
 			utype_name_n(u, 3), unit->number);
 	    } else {
+		/* If no side name, use unit id to get better chance
+		   of uniqueness. */
 		sprintf(spbuf, "%s-%d", utype_name_n(u, 3), unit->id);
 	    }
 	    return copy_string(spbuf);
