@@ -1,5 +1,5 @@
 /* Unit generation for Xconq.
-   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,26 +7,25 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-Obj *choose_from_weighted_list PARAMS ((Obj *lis, int *totalweightp));
+Obj *choose_from_weighted_list(Obj *lis, int *totalweightp);
 #include "kernel.h"
 
-static void add_preexisting_units PARAMS ((Side *side, int runs));
-static int country_is_complete PARAMS ((Side *side, int runs));
-static int average_numunits PARAMS ((int u));
-static void count_cells PARAMS ((int x, int y));
-static int good_place PARAMS ((int cx, int cy));
-static void find_a_place PARAMS ((Side *side));
-static int valid_unit_place PARAMS ((int x, int y));
-static int possible_unit_place PARAMS ((int x, int y));
-static int find_unit_place PARAMS ((int u, int cx, int cy, int *xp, int *yp));
-static void mung_terrain PARAMS ((int x, int y, int u));
-static Unit *find_occupant_place PARAMS ((Side *side, int u, int x0, int y0));
-static int at_country_units_max PARAMS ((Side *side, int u));
-static void place_unit PARAMS ((int u, Side *side, int x, int y,
-				Unit *transport));
-static void expand_countries PARAMS ((void));
-static void expand_country PARAMS ((int x, int y));
-static void set_people_on_side PARAMS ((int x, int y));
+static void add_preexisting_units(Side *side, int runs);
+static int country_is_complete(Side *side, int runs);
+static int average_numunits(int u);
+static void count_cells(int x, int y);
+static int good_place(int cx, int cy);
+static void find_a_place(Side *side);
+static int valid_unit_place(int x, int y);
+static int possible_unit_place(int x, int y);
+static int find_unit_place(int u, int cx, int cy, int *xp, int *yp);
+static void mung_terrain(int x, int y, int u);
+static Unit *find_occupant_place(Side *side, int u, int x0, int y0);
+static int at_country_units_max(Side *side, int u);
+static void place_unit(int u, Side *side, int x, int y, Unit *transport);
+static void expand_countries(void);
+static void expand_country(int x, int y);
+static void set_people_on_side(int x, int y);
 
 /* Tmp array for counting terrain. */
 
@@ -61,22 +60,19 @@ static int growth;
 static int totnumtodo;
 static int totnumdone;
 
-#define announce_unit_progress(n) \
-  
-
 /* Place all the units belonging to countries. */
 
 int any_side_possible_units = -1;
 
 int
-make_countries(calls, runs)
-int calls, runs;
+make_countries(int calls, int runs)
 {
     int alldone, x0, y0, u, t, i, tot, x, y;
-    int advantage, sideadvantage, favor, x1, y1, dir;
+    int advantage, sideadvantage, favor;
     int canbeinopen[MAXUTYPES];  /* true if type need not be occ at start */
     int numtodo[MAXUTYPES], numindeptodo[MAXUTYPES];
-    int numleft[MAXUTYPES], numindepleft[MAXUTYPES], totleft, numlisted, numfails;
+    int numleft[MAXUTYPES], numindepleft[MAXUTYPES], totleft;
+    int numlisted, numfails;
     int checkmins = FALSE;
     int dopeoplesides = FALSE, dopeopleindeps = FALSE;
     int numdone = 0;
@@ -110,7 +106,7 @@ int calls, runs;
     if (checkmins) {
     	/* Edge cells are useless for country placement. */
     	for_all_interior_cells(x, y) {
-    	    ++totnumcells[terrain_at(x, y)];
+    	    ++totnumcells[(int) terrain_at(x, y)];
     	}
     	for_all_terrain_types(t) {
     	    if (t_country_min(t) * numsides > totnumcells[t]) {
@@ -183,7 +179,9 @@ int calls, runs;
 	       that belong to the side, and the independents nearby. */
 	    for_all_unit_types(u) {
 		numtodo[u] = numindeptodo[u] = 0;
-		if (type_allowed_on_side(u, side)) {
+		   /* Seems logical that side does not get units at start it can't build. */
+		if (type_allowed_on_side(u, side)
+		    && has_advance_to_build(side, u)) {
 		    if (runs > 0) {
 			/* If the side is just coming into the game,
 			   give it an average number of each type that
@@ -196,9 +194,11 @@ int calls, runs;
 		    }
 		    totnumtodo += numtodo[u];
 		}
-		if (type_allowed_on_side(u, NULL)) {
-		    numindeptodo[u] = (u_indep_near_start(u) * advantage) / sideadvantage;
-		    totnumtodo += numindeptodo[u];
+		/* Seems logical that null side does not get units at start it can't build. */
+		if (type_allowed_on_side(u, NULL)
+		    && has_advance_to_build(NULL, u)) {
+			numindeptodo[u] = (u_indep_near_start(u) * advantage) / sideadvantage;
+			totnumtodo += numindeptodo[u];
 		}
 		numleft[u] = numindepleft[u] = 0;
 	    }
@@ -430,19 +430,17 @@ int u;
 /* Count the cell as being of a particular type. */
 
 static void
-count_cells(x, y)
-int x, y;
+count_cells(int x, int y)
 {
-    ++numcells[terrain_at(x, y)];
+    ++numcells[(int) terrain_at(x, y)];
 }
 
-/* Test whether a given location is desirable for a country.  It should be
-   in the right distance range from other countries, and have enough of
-   the right sorts of terrain. */
+/* Test whether a given location is desirable for a country.  It
+   should be in the right distance range from other countries, and
+   have enough of the right sorts of terrain. */
 
 static int
-good_place(cx, cy)
-int cx, cy;
+good_place(int cx, int cy)
 {
     int toofar = TRUE, notfirst = FALSE, px, py, dist, t;
     Side *side;
@@ -663,15 +661,12 @@ Unit *transport;
 {
     Unit *unit;
 
-// --- FIX OF INDEPENDENT GHOST UNIT BUG --------------------------------------
-
     /* Check for this BEFORE creating the unit, or it will end up as an independent ghost! */
-
-    if (!new_unit_allowed_on_side(u, side))
+    /* has_advance_to_build no longer included by type_allowed_on_side. */
+    /* Reasonable to limit start units to those we can build ourselves. */
+    if (!new_unit_allowed_on_side(u, side)
+       || !has_advance_to_build(side, u))
 	    return;
-
-//---------------------------------------------------------------------
-
     unit = create_unit(u, TRUE);
     if (unit != NULL) {
 	if (side != NULL) {
