@@ -314,7 +314,9 @@ Side *side;
     /* (should incorp config name somehow, also be sys-dependent) */
     module = create_game_module(side->player->name);
 
-    if ((module->fp = fopen(module->filename, "r")) != NULL) {
+    /* Don't use fopen directly, use open_module which indirectly calls 
+    open_file to ensure mac/unix cross-platform compatibility. */    
+    if (open_module(module, FALSE)) {
 	if ((config = read_form(module->fp,
 				&(module->startlineno),
 				&(module->endlineno)))
