@@ -1,5 +1,5 @@
 /* The movement-related actions of Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,10 +7,10 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int carryable PARAMS ((int u));
-extern int accelerable PARAMS ((int u));
-extern int accelerator PARAMS ((int u1, int u2));
-extern int border_slide_possible PARAMS ((int u2, int ox, int oy, int nx, int ny)); 
+extern int carryable(int u);
+extern int accelerable(int u);
+extern int accelerator(int u1, int u2);
+extern int border_slide_possible(int u2, int ox, int oy, int nx, int ny); 
 #include "kernel.h"
 #include "kpublic.h"
 
@@ -26,7 +26,7 @@ enum {
 
 #undef  DEF_ACTION
 #define DEF_ACTION(name,code,args,prepfn,netprepfn,DOFN,checkfn,ARGDECL,doc)  \
-  extern int DOFN PARAMS (ARGDECL);
+  extern int DOFN ARGDECL;
 
 #include "action.def"
 
@@ -74,7 +74,7 @@ do_move_action(unit, unit2, nx, ny, nz)
 Unit *unit, *unit2;
 int nx, ny, nz;
 {
-    int u, u2, nu2, t, rslt, speed, mpcost, acpcost, ox, oy, oz, ot, dist;
+    int u, u2, t, rslt, speed, mpcost, acpcost, ox, oy, oz, ot, dist;
     int nummoves, dirs[NUMDIRS], numdirs, i, ix, iy, ndist;
 
     u = unit->type;  u2 = unit2->type;
@@ -118,7 +118,7 @@ int nx, ny, nz;
 		}
 		wreck_unit(unit2);
 		/* Now make it go away, taking unlucky occupants with. */
-		if (ut_vanishes_on(nu2, t)) {
+		if (ut_vanishes_on(u2, t)) {
 		    kill_unit(unit2, H_UNIT_VANISHED);
 		}
 	    }
@@ -146,7 +146,7 @@ int nx, ny, nz;
 		   allowed for. */
 		wreck_unit(unit2);
 		/* Now make it go away, taking unlucky occupants with. */
-		if (ut_vanishes_on(nu2, ot)) {
+		if (ut_vanishes_on(u2, t)) {
 		    kill_unit(unit2, H_UNIT_VANISHED);
 		}
 	    }
@@ -494,9 +494,7 @@ Obj *lis;
    in the same cell. */
 
 int
-move_unit(unit, nx, ny)
-Unit *unit;
-int nx, ny;
+move_unit(Unit *unit, int nx, int ny)
 {
     int u = unit->type, u3, ox = unit->x, oy = unit->y, oz = unit->z;
     int nz = oz;
@@ -537,21 +535,8 @@ int nx, ny;
 	    change_control_side_around(nx, ny, u, unit->side);
 	}
     }
-
-
-// --- USER LAYER FIX --------------------------------------------------
-
     kick_out_enemy_users(unit->side, nx, ny);
-
-//    /* Kick out enemy users if we control the cell */
-//    if (user_at(nx, ny) != NULL 
-//    	&& (!user_at(nx, ny)->side || !trusted_side(user_at(nx, ny)->side, unit->side))) {
-//		user_at(nx, ny)->usedcells -=1;
-//    		set_user_at(nx, ny, NULL);
-//    }
-         
-// -----------------------------------------------------------------
-
+    
     /* Use up supplies as directed. */
     consume_move_supplies(unit);
     /* a hack */
@@ -561,11 +546,9 @@ int nx, ny;
 }
 
 void
-change_control_side_around(x, y, u, side)
-int x, y, u;
-Side *side;
+change_control_side_around(int x, int y, int u, Side *side)
 {
-    int con = control_side_at(x, y), s = side_number(side), dir, x1, y1;
+    int con = control_side_at(x, y), s = side_number(side);
     Side *oldside, *side2;
 
     if (con != NOCONTROL
@@ -595,8 +578,7 @@ Side *side;
 }
 
 int
-can_move_at_all(unit)
-Unit *unit;
+can_move_at_all(Unit *unit)
 {
     return u_speed(unit->type) > 0;
 }
@@ -605,12 +587,10 @@ Unit *unit;
 
 static int tmprslt;
 
-static void test_blocking_zoc PARAMS ((int x, int y));
+static void test_blocking_zoc(int x, int y);
 
 int
-in_blocking_zoc(unit, x, y, z)
-Unit *unit;
-int x, y, z;
+in_blocking_zoc(Unit *unit, int x, int y, int z)
 {
     int u = unit->type, t = terrain_at(x, y), dir, x1, y1, u2, range;
     Unit *unit2;
@@ -784,12 +764,10 @@ int u2;
 
 static int tmpmpcost, tmpox, tmpoy;
 
-static void zoc_cost_fn PARAMS ((int x, int y));
+static void zoc_cost_fn(int x, int y);
 
 int
-zoc_move_cost(unit, ox, oy, oz)
-Unit *unit;
-int ox, oy, oz;
+zoc_move_cost(Unit *unit, int ox, int oy, int oz)
 {
     int u = unit->type, u2, t1, t2, cost, mpcost, x, y, dir, x1, y1, range;
     Unit *unit2;
@@ -889,16 +867,16 @@ int ox, oy, oz;
 }
 
 static int
-allowed_in_zoc(unit, unit2)
-Unit *unit, *unit2;
+allowed_in_zoc(Unit *unit, Unit *unit2)
 {
     /* should make a better test */
     return trusted_side(unit->side, unit2->side);
 }
 
+static void zoc_cost_fn(int x, int y);
+
 static void
-zoc_cost_fn(x, y)
-int x, y;
+zoc_cost_fn(int x, int y)
 {
     int u, u2, x0, y0, dist, odist, range, cost;
     Unit *unit2;
@@ -944,16 +922,15 @@ int x, y;
 /* This is a hook to handle any reactions to the unit's successful move. */
 
 int
-maybe_react_to_move(unit, ox, oy)
-Unit *unit;
-int ox, oy;
+maybe_react_to_move(Unit *unit, int ox, int oy)
 {
     return 0;
 }
 
+static void try_detonate_on_approach(int x, int y);
+
 static void
-try_detonate_on_approach(x, y)
-int x, y;
+try_detonate_on_approach(int x, int y)
 {
     int dist;
     Unit *unit;
@@ -971,8 +948,7 @@ int x, y;
 }
 
 void
-detonate_on_approach_around(unit)
-Unit *unit;
+detonate_on_approach_around(Unit *unit)
 {
     int maxrange;
 
@@ -1046,7 +1022,7 @@ int
 do_enter_action(unit, unit2, unit3)
 Unit *unit, *unit2, *unit3;
 {
-    int u2, u3, u4, ox, oy, oz, ot, nx, ny, nz, nu2, speed, acpcost, mpcost, rslt;
+    int u2, u3, u4, ox, oy, oz, ot, nx, ny, nz, speed, acpcost, mpcost, rslt;
 
     u2 = unit2->type;
     ox = unit2->x;  oy = unit2->y;  oz = unit2->z;
@@ -1076,7 +1052,7 @@ Unit *unit, *unit2, *unit3;
 	       allowed for. */
 	    wreck_unit(unit);
 	    /* Now make it go away, taking unlucky occupants with. */
-	    if (ut_vanishes_on(nu2, ot)) {
+	    if (ut_vanishes_on(u2, ot)) {
 		kill_unit(unit2, H_UNIT_VANISHED);
 	    }
 	}
