@@ -17,13 +17,16 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 
-static void notify_combat PARAMS ((Unit *unit, Unit *atker, char *str));
-static void init_calendar PARAMS ((void));
-static void maybe_mention_date PARAMS ((Side *side));
-static int gain_count PARAMS ((Side *side, int u, int r));
-static int loss_count PARAMS ((Side *side, int u, int r));
-static int atkstats PARAMS ((Side *side, int a, int d));
-static int hitstats PARAMS ((Side *side, int a, int d));
+static void notify_combat(Unit *unit, Unit *atker, char *str);
+static int pattern_matches_combat(Obj *pattern, Unit *unit, Unit *unit2);
+static void combat_desc_from_list(Side *side, Obj *lis, Unit *unit,
+				  Unit *unit2, char *str, char *buf);
+static void init_calendar(void);
+static void maybe_mention_date(Side *side);
+static int gain_count(Side *side, int u, int r);
+static int loss_count(Side *side, int u, int r);
+static int atkstats(Side *side, int a, int d);
+static int hitstats(Side *side, int a, int d);
 
 static char *tmpnbuf;
 
@@ -75,8 +78,8 @@ typedef struct a_usualdate {
     int year;
 } UsualDate;
 
-static char *usual_date_string PARAMS ((int date));
-static void parse_usual_date PARAMS ((char *datestr, UsualDate *udate));
+static char *usual_date_string(int date);
+static void parse_usual_date(char *datestr, UsualDate *udate);
 
 static int turn_initial;
 
@@ -875,8 +878,8 @@ Obj *sym;
     return -1;
 }
 
-int pattern_matches_event PARAMS ((Obj *pattern, HistEvent *hevt));
-void event_desc_from_list PARAMS ((Side *side, Obj *lis, HistEvent *hevt, char *buf));
+int pattern_matches_event(Obj *pattern, HistEvent *hevt);
+void event_desc_from_list(Side *side, Obj *lis, HistEvent *hevt, char *buf);
 
 /* (should abstract out evt arg -> unit/pastunit description code) */
 
@@ -1887,7 +1890,7 @@ Unit *unit;
     }
 }
 
-extern char *goal_desc PARAMS ((char *buf, Goal *goal));
+extern char *goal_desc(char *buf, Goal *goal);
 
 void
 plan_desc(buf, unit)
@@ -1918,13 +1921,15 @@ Unit *unit;
       strcat(buf, " NoAI");
     if (plan->supply_is_low)
       strcat(buf, " SupplyLow");
+    if (plan->waitingfortransport)
+      strcat(buf, " WaitingForTransport");
     if (plan->maingoal) {
 	strcat(buf, " goal ");
 	strcat(buf, goal_desc(goalbuf, plan->maingoal));
     }
     if (plan->formation) {
 	strcat(buf, " formation ");
-	goal_desc(buf+strlen(buf), unit->plan->formation);
+	strcat(buf, goal_desc(goalbuf, plan->formation));
     }
     if (plan->tasks) {
 	i = 0;
@@ -2012,6 +2017,15 @@ task_desc(char *buf, Side *side, Unit *unit, Task *task)
 	    tprintf(buf, "unknown unit #%d", arg0);
 	}
 	break;
+      case TASK_PICKUP:
+	sprintf(buf, "pick up ");
+	unit2 = find_unit(arg0);
+	if (unit2 != NULL) {
+	    tprintf(buf, "%s", unit_handle(side, unit2));
+	} else {
+	    tprintf(buf, "unknown unit #%d", arg0);
+	}
+	break;
       case TASK_RESUPPLY:
 	sprintf(buf, "%s", taskdefns[task->type].name);
 	if (arg0 != NONMTYPE)
@@ -2067,6 +2081,10 @@ Goal *goal;
 		unit_handle(NULL, find_unit(goal->args[0])),
 		goal->args[3]);
 	break;
+      case GOAL_VICINITY_HELD:
+	sprintf(buf, "hold %dx%d area around %d,%d",
+		goal->args[2], goal->args[3], goal->args[0], goal->args[1]);
+	break;
 	/* (should add more cases specific to common types of goals) */
       default:
 	sprintf(buf, "<goal s%d %s%s",
@@ -2161,8 +2179,8 @@ int *ucnts;
     return buf;
 }
 
-char *get_word PARAMS ((char *str));
-void notify_doctrine_1 PARAMS ((Side *side, Doctrine *doctrine));
+char *get_word(char *str);
+void notify_doctrine_1(Side *side, Doctrine *doctrine);
 
 void
 notify_doctrine(side, spec)
@@ -2226,8 +2244,8 @@ Doctrine *doctrine;
 
 #define first_person(unit) ((unit)->side && (unit)->side->self_unit == (unit))
 
-int report_combat_special PARAMS ((Unit *unit1, Unit *unit2, char *str));
-int type_matches_symbol PARAMS ((Obj *sym, int u));
+int report_combat_special(Unit *unit1, Unit *unit2, char *str);
+int type_matches_symbol(Obj *sym, int u);
 
 void
 report_combat(atker, other, str)
@@ -2293,17 +2311,12 @@ char *str;
     }
 }
 
-static int pattern_matches_combat PARAMS ((Obj *pattern, Unit *unit, Unit *unit2));
-static void combat_desc_from_list PARAMS ((Side *side, Obj *lis, Unit *unit, Unit *unit2, char *str, char *buf));
-
 int
-report_combat_special(unit1, unit2, str)
-Unit *unit1, *unit2;
-char *str;
+report_combat_special(Unit *unit1, Unit *unit2, char *str)
 {
     int found = FALSE;
-    char *soundname, abuf[BUFSIZE];
-    Obj *rest, *head, *pat, *parms, *msgdesc;
+    char abuf[BUFSIZE];
+    Obj *rest, *head, *pat, *msgdesc;
 
     for_all_list(g_action_notices(), rest) {
 	head = car(rest);
@@ -2505,9 +2518,11 @@ Side *side;
     } else if (numlost == numsides) {
 	sprintf(spbuf, "Everybody lost!");
     } else if (numwon == 1) {
-	sprintf(spbuf, "%s won!", (side == winner ? "You" : side_desig(winner)));
+	sprintf(spbuf, "%s won!",
+		(side == winner ? "You" : side_desig(winner)));
     } else if (numlost == 1) {
-	sprintf(spbuf, "%s lost!", (side == loser ? "You" : side_desig(loser)));
+	sprintf(spbuf, "%s lost!",
+		(side == loser ? "You" : side_desig(loser)));
     } else {
 	sprintf(spbuf, "Some won and some lost.");
     }
@@ -2608,8 +2623,7 @@ long a1, a2, a3, a4;
 /* Compose a readable form of the given date. */
 
 char *
-absolute_date_string(date)
-int date;
+absolute_date_string(int date)
 {
     /* The first time we ask for a date, interpret the calendar. */
     if (calendar_type == cal_unknown)
@@ -2629,7 +2643,7 @@ int date;
 /* Interpret the calendar definition. */
 
 static void
-init_calendar()
+init_calendar(void)
 {
     int caltypeunknown = FALSE;
     char *stepnamestr;
