@@ -1,5 +1,5 @@
 /* Unit plan handling for Xconq.
-   Copyright (C) 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@ extern int repair_if_damaged(Unit *unit);
 
 extern short any_auto_repair;
 
-static void plan_passive(Unit *unit, int try);
+static void plan_passive(Unit *unit);
 static void plan_offense(Unit *unit);
 static void plan_offense_support(Unit *unit);
 static void plan_defense(Unit *unit);
@@ -21,7 +21,9 @@ static void plan_exploration(Unit *unit);
 static void plan_explorer_support(Unit *unit);
 static void plan_improve(Unit *unit);
 static void plan_colonize(Unit *unit);
+#if 0 /* not used currently */
 static void plan_colonizer_support(Unit *unit);
+#endif
 static int good_cell_to_colonize(int u, Unit *unit);
 static void plan_random(Unit *unit);
 static void wake_at(int x, int y);
@@ -198,37 +200,29 @@ int flag;
 
 /* Execute the plan. */
 
-void
-execute_plan(unit, try)
-Unit *unit;
-int try;
+int
+execute_plan(Unit *unit)
 {
     Plan *plan = unit->plan;
-    extern int planexecs;
 
+    /* Plan execution for advanced units is (for now) handled in run.c. */
+    if (u_advanced(unit->type)) {
+	unit->act->acp = unit->act->initacp = 0;
+	return 0;
+    }
     if (!in_play(unit) || !completed(unit)) {
 	DMprintf("%s shouldn't be planning yet\n", unit_desig(unit));
-	return; 
+	return 0; 
     }
-    DMprintf("%s using plan %s", unit_desig(unit), plan_desig(plan));
-    if (try > 1)
-      DMprintf(" (try #%d)", try);
-    DMprintf("\n");
+    DMprintf("%s using plan %s\n", unit_desig(unit), plan_desig(plan));
     /* Units that are asleep or in reserve do nothing. */
     if (plan->asleep || plan->reserve)
-      return;
-    ++planexecs;
-    if (try > 5) {
-	DMprintf("%s redoing plan too often, going into reserve\n",
-		 unit_desig(unit));
-	plan->reserve = TRUE;
-	return;
-    }
-    if (plan->execs_this_turn > 1000) {
+      return 0;
+    if (plan->execs_this_turn > 1000 && g_units_may_go_into_reserve()) {
 	DMprintf("%s executed plan 1000 times this turn, going into reserve\n",
 		 unit_desig(unit));
 	plan->reserve = TRUE;
-	return;
+	return 1;
     }
     /* Unit actually has a plan, dispatch on its type. */
     switch (plan->type) {
@@ -236,7 +230,7 @@ int try;
 	/* Unit has not gotten a plan yet, leave it alone. */
 	break;
       case PLAN_PASSIVE:
-	plan_passive(unit, try);
+	plan_passive(unit);
 	break;
       case PLAN_OFFENSIVE:
 	plan_offense(unit);
@@ -261,6 +255,7 @@ int try;
 	break;
     }
     ++(plan->execs_this_turn);
+    return 1;
 }
 
 /* See if we're too far away from an assigned position, set a task
@@ -398,9 +393,7 @@ Task *task1, *task2;
    what to do. */
 
 static void
-plan_passive(unit, try)
-Unit *unit;
-int try;
+plan_passive(Unit *unit)
 {
     Plan *plan = unit->plan;
 
@@ -459,6 +452,7 @@ Unit *unit;
     int x, y, w, h, range, x1, y1, nx, ny;
     Plan *plan = unit->plan;
     Task *lasttask;
+    Unit *unit2;
 
     if (resupply_if_low(unit)) {
 	if (plan->tasks)
@@ -508,6 +502,21 @@ Unit *unit;
     }
     if (plan->maingoal && mobile(u)) {
 	switch (plan->maingoal->type) {
+	  case GOAL_UNIT_OCCUPIED:
+	    /* Move to occupy our goal if necessary. */
+	    unit2 = find_unit(plan->maingoal->args[0]);
+	    if (unit->transport != unit2)
+	      set_occupy_task(unit, unit2);
+	    break;
+
+	  case GOAL_CELL_OCCUPIED:
+	    /* Move to occupy our goal if necessary. */
+	    x = plan->maingoal->args[0];  
+	    y = plan->maingoal->args[1];
+	    if (unit->x != x || unit->y != y)
+	      set_move_to_task(unit, x, y, 0);
+	    break;
+
 	  case GOAL_VICINITY_HELD:
 	    x = plan->maingoal->args[0];  y = plan->maingoal->args[1];
 	    w = plan->maingoal->args[2];  h = plan->maingoal->args[3];
@@ -571,10 +580,12 @@ Unit *unit;
     	execute_task(unit);
     } else {
     	DMprintf("%s found nothing to do offensively", unit_desig(unit));
-    	if (flip_coin()) {
+	/* Only do this if units may go into reserve. */
+    	if (flip_coin() && g_units_may_go_into_reserve()) {
     	    DMprintf("- going into reserve");
     	    plan->reserve = TRUE;
-    	} else if (probability(5)) {
+	/* Only do this if units may go to sleep. */
+    	} else if (probability(5) && g_units_may_go_to_sleep()) {
     	    DMprintf("- going to sleep");
     	    plan->asleep = TRUE;
     	}
@@ -637,10 +648,12 @@ Unit *unit;
 
     for_all_unit_types(u2) {
 	if (uu_acp_to_create(u, u2) > 0
-	    && (uu_creation_cp(u, u2) >= u_cp(u2)
-	        || uu_acp_to_build(u, u2) > 0)
+	    && ((uu_creation_cp(u, u2) >= u_cp(u2)
+		 && type_allowed_on_side(u2, unit->side)
+		 && has_advance_to_build(unit->side, u2))
+		|| uu_acp_to_build(u, u2) > 0)
 	    /* (should check if any advantage to building) */
-	   ) {
+	    ) {
 	   cando = TRUE;
 	   break;
 	}
@@ -664,6 +677,8 @@ Unit *unit;
     for_all_unit_types(u2) {
 	    if (mobile(u2)
 		&& (type_can_attack(u2) || type_can_fire(u2))
+		&& type_allowed_on_side(u2, unit->side) 
+		&& has_advance_to_build(unit->side, u2)
 		&& uu_acp_to_create(u, u2) > 0) {
 		backup = u2;
 		if (flip_coin()) {
@@ -736,6 +751,7 @@ Unit *unit;
 {
     int u = unit->type, range, x, y, w, h, x1, y1;
     Plan *plan = unit->plan;
+    Unit *unit2;
 
     if (resupply_if_low(unit)) {
 	if (plan->tasks)
@@ -759,6 +775,20 @@ Unit *unit;
     }
     if (plan->maingoal) {
 	switch (plan->maingoal->type) {
+  	  case GOAL_UNIT_OCCUPIED:
+	    /* Move to occupy our goal if necessary. */
+	    unit2 = find_unit(plan->maingoal->args[0]);
+	    if (unit->transport != unit2)
+	      set_occupy_task(unit, unit2);
+	    break;
+
+  	  case GOAL_CELL_OCCUPIED:
+	    /* Move to occupy our goal if necessary. */
+	    x = plan->maingoal->args[0];  y = plan->maingoal->args[1];
+	    if (unit->x != x || unit->y != y)
+	      set_move_to_task(unit, x, y, 0);
+	    break;
+
 	  case GOAL_VICINITY_HELD:
 	    x = plan->maingoal->args[0];  y = plan->maingoal->args[1];
 	    w = plan->maingoal->args[2];  h = plan->maingoal->args[3];
@@ -815,14 +845,16 @@ Unit *unit;
 	}
 	/* Nothing to shoot at, so just hang out. */
 	/* (should consider moving to better shooting position if possible) */
-    } else if (can_attack(unit)) {
-	/* Consider moving a short ways to attack an interloper. */
-	range = 3 * u_acp(u);
-	if (go_after_victim(unit, range)) {
+
+    /* Also go for out-of-range victim that we can fire at. */
+    } else if (can_attack(unit) || can_fire(unit)) {
+	/* Use the tactical range. */ 
+	if (go_after_victim(unit, u_ai_tactical_range(unit->type))) {
 	    execute_task(unit);
 	    return;
 	}
-	/* Nobody close by, just hang out, shifting around a bit occasionally. */
+	/* Nobody close by, just hang out, shifting around a bit
+           occasionally. */
 	if (mobile(unit->type) && probability(10)) {
 	    if (random_point_near(unit->x, unit->y, u_acp(u), &x1, &y1)) {
 		DMprintf("%s to shift defensive position to %d,%d\n",
@@ -838,7 +870,7 @@ Unit *unit;
     }
     if (plan->tasks) {
 	execute_task(unit);
-    } else if (!plan->reserve) {
+    } else if (!plan->reserve && g_units_may_go_into_reserve()) {
 	/* Just stay in reserve for now. */
 	DMprintf("%s going into defensive reserve\n", unit_desig(unit));
 	plan->reserve = TRUE;
@@ -850,67 +882,80 @@ Unit *unit;
 static void
 plan_colonize(Unit *unit)
 {
-    Task *task = unit->plan->tasks;
-    Plan *plan = unit->plan;
-    int 	type[MAXUTYPES];
-    int 	numtypes = 0;
-    int 	u = unit->type;
-    int 	u2;
+	Plan 	*plan = unit->plan;
+	Unit	*unit2;
+	int 	x, y, u2;
 
-    /* Maybe set building task for colonizers. */
-    if (u_colonizer(u)) {
-	/* Pick an advanced unit type to build. */
-	for_all_unit_types(u2) {
-	    if (u_advanced(u2) 
-		&& could_create(u, u2)
-		&& type_allowed_on_side(u2, unit->side)) {
-		type[numtypes] = u2;
-		numtypes++;
-	    }
+	if (resupply_if_low(unit)) {
+		if (plan->tasks)
+			execute_task(unit);
+		return;
 	}
-	if (numtypes) {
-	    u2 = type[xrandom(numtypes)];
-	    if (task == NULL || task->type != TASK_BUILD) {
-		/* First test if we have found a good spot to settle. */
-		if(good_cell_to_colonize(u2, unit)) {
-		    /* Initiate construction if spot is OK. */
-		    set_build_task(unit, u2, 1);
-		    DMprintf("%s colonizing by building %s\n", 
-			     unit_desig(unit), u_type_name(u2));
+	if (repair_if_damaged(unit)) {
+		if (plan->tasks)
+	  		execute_task(unit);
+		return;
+	}
+	/* Set build task if we are colonizing and have found a good spot. */
+	if (plan->maingoal && plan->maingoal->type == GOAL_COLONIZE) {
+	     	u2 = plan->maingoal->args[0];
+	     	if (good_cell_to_colonize(u2, unit)) {
+			set_build_task(unit, u2, 1);	
+			DMprintf("%s colonizing by building %s\n", 
+		     			unit_desig(unit), u_type_name(u2));
 		}
-	    } else DMprintf("%s already building, leaving alone\n", 
-			    unit_desig(unit));
-	} else DMprintf("%s has no way to colonize\n", 
-			unit_desig(unit));
-    }
-    /* Then execute task. */
-    if (unit->plan->tasks) {
-	execute_task(unit);
-	return;
-	/* No pending tasks. */
-    } else if (mobile(u)) {
-	/* Let colonizer move in a random dir along straight line. */
-	set_move_dir_task(unit, random_dir(), xrandom(20));
-	return;
-	/* Set a build task for non-colonizers. */
-    } else {
-	plan_colonizer_support(unit);
-	return;
-    }
-    /* Check again if we now have a task to execute. */
-    if (unit->plan->tasks) {
-	execute_task(unit);
-	/* Let colonizer move in a random dir along straight line if task failed. */
-	if (mobile(u) && unit->plan->last_task_outcome == TASK_FAILED)
-	  set_move_dir_task(unit, random_dir(), xrandom(20));
+	}
+	/* Then proceed with the tasks. */
+	if (plan->tasks) {
+		execute_task(unit);
+		return;
+	}
+	/* The task queue was empty. Get a new task. */
+	if (plan->maingoal) {
+		switch (plan->maingoal->type) {
 
-	/* Else maybe go to sleep. */
-    } else if (probability(10)) {
-	DMprintf("no tasks, going to sleep (dunno why)\n");
-	plan->asleep = TRUE;
-    }
+		  case GOAL_UNIT_OCCUPIED:
+			/* Move to occupy our goal if necessary. */
+			unit2 = find_unit(plan->maingoal->args[0]);
+			if (unit->transport != unit2)
+				set_occupy_task(unit, unit2);
+		  break;
+
+		  case GOAL_CELL_OCCUPIED:
+			/* Move to our goal if necessary. */
+		    	x = plan->maingoal->args[0];  
+		    	y = plan->maingoal->args[1];
+			if (unit->x != x || unit->y != y)
+				set_move_to_task(unit, x, y, 0);
+		  break;
+
+		  case GOAL_COLONIZE:
+			/* Move in a random dir along a straight line. */
+			set_move_dir_task(unit, random_dir(), xrandom(20));
+		  break;
+
+		  default:
+			/* Should never happen! */
+			DMprintf("Colonizer %s has weird goal %s\n",
+					unit_desig(unit), goal_desig(plan->maingoal));
+		    break;
+		}
+	} else {
+		/* Should never happen! */
+		DMprintf("Colonizer %s has no goal\n", unit_desig(unit));
+	}
+		/* Execute the new task. */
+	if (plan->tasks) {
+		execute_task(unit);
+		return;
+	} else {
+		/* Holding occupied unit and waiting for something to happen. */
+		DMprintf("Colonizer %s with goal %s unable to find new task\n",
+				unit_desig(unit), goal_desig(plan->maingoal));
+	}	
 }
 
+#if 0 /* not used currently */
 static void
 plan_colonizer_support(Unit *unit)
 {
@@ -921,10 +966,12 @@ plan_colonizer_support(Unit *unit)
 	/* Load all colonizing types into type vector. */
     for_all_unit_types(u) {
 	if (type_allowed_on_side(u, unit->side) 
+	    /* No longer included in type_allowed_on_side. */
+	    && has_advance_to_build(unit->side, u)
 	    && could_create(unit->type, u)
-	    && u_colonizer(u)) {
-	    type[numtypes] = u;
-	    numtypes++;
+	    && u_colonizer_worth(u) > 0) {
+	    	type[numtypes] = u;
+	    	numtypes++;
 	} 
     }
     /* Then pick one of them at random. */
@@ -937,9 +984,11 @@ plan_colonizer_support(Unit *unit)
 	/* Load all possible types instead if we failed. */
     } else for_all_unit_types(u) {
 	if (type_allowed_on_side(u, unit->side) 
+	    /* No longer included in type_allowed_on_side. */
+	    && has_advance_to_build(unit->side, u)
 	    && could_create(unit->type, u)) {
-	    type[numtypes] = u;
-	    numtypes++;
+	    	type[numtypes] = u;
+	    	numtypes++;
 	} 
     }
     /* Then pick one of them at random. */
@@ -952,6 +1001,7 @@ plan_colonizer_support(Unit *unit)
 	/* Nothing can be built. */
     } else	DMprintf("Sorry, %s can't build anything!\n", unit_desig(unit));
 }
+#endif
 
 static void
 plan_improve(Unit *unit)
@@ -963,10 +1013,12 @@ plan_improve(Unit *unit)
 	/* Load all immobile types (facilities) into type vector. */
     for_all_unit_types(u) {
 	if (type_allowed_on_side(u, unit->side) 
+	    /* No longer included in type_allowed_on_side. */
+	    && has_advance_to_build(unit->side, u)
 	    && could_create(unit->type, u)
-	    && !mobile(u)) {
-	    type[numtypes] = u;
-	    numtypes++;
+	    && u_facility(u)) {
+		type[numtypes] = u;
+	    	numtypes++;
 	} 
     }
     /* Then pick one of them at random. */
@@ -979,9 +1031,11 @@ plan_improve(Unit *unit)
 	/* Load all possible types instead if we failed. */
     } else for_all_unit_types(u) {
 	if (type_allowed_on_side(u, unit->side) 
+	    /* No longer included in type_allowed_on_side. */
+	    && has_advance_to_build(unit->side, u)
 	    && could_create(unit->type, u)) {
-	    type[numtypes] = u;
-	    numtypes++;
+	    	type[numtypes] = u;
+	    	numtypes++;
 	} 
     }
     /* Then pick one of them at random. */
@@ -998,52 +1052,69 @@ plan_improve(Unit *unit)
 static int 
 good_cell_to_colonize(int u, Unit *unit)
 {
-    int 	x1, y1, m, supply[MAXMTYPES], maxsize = PROPHI;
-    Unit	*unit2;
-			
-    /* First zero the supply vector. */
-    for_all_material_types (m)
-      supply[m] = 0;
-    /* Go through all cells within reach and compute supply of materials. */
-    for_all_cells_within_n_steps(unit->x, unit->y, u_reach(u), x1, y1) {
-	if (!inside_area(x1, y1))
-	  continue;
-	/* Return FALSE if other city or enemy unit is nearby. */
-	for_all_stack(x1, y1, unit2) {
-	    if (u_advanced(unit2->type) 
-		|| !trusted_side(unit2->side, unit->side))
-	      return FALSE; 
+	int 	x1, y1, m, supply[MAXMTYPES], maxsize;
+	int	x = unit->x;
+	int	y = unit->y;
+	Unit	*unit2;
+
+	/* Check for nexthere prevents two colonizers from trying to colonize the same 
+	cell and blocking each other from doing so again and again due to terrain capacity 
+	restrictions. */
+	if (unit->nexthere != NULL)
+		return FALSE;
+
+	/* Then check if another advanced unit is too close to this cell. */
+	for_all_units(unit2) {
+		if (u_advanced(unit2->type)
+		    && distance(x, y, unit2->x, unit2->y) 
+		          < g_ai_advanced_unit_separation())
+		    	return FALSE;
 	}
-	/* Don't count supply in cells already used by others. */
-	if (user_at(x1, y1) != NOUSER)
-	  continue;
-	/* Add the supply of materials from this cell. */
-	for_all_material_types (m)
-	  supply[m] += tm_production(terrain_at(x1, y1), m); 
-    }
-    /* Calculate max sustainable city size at this location. */
-    for_all_material_types (m) {
-	if (um_consumption_per_size(u, m) > 0)
-	  maxsize =
-	    min(maxsize, (float) supply[m] / um_consumption_per_size(u, m));
-    }
-    /* Return FALSE if there is not enough food around 
-	to sustain the minimal size goal for this unit type. */
-    if (maxsize < u_minimal_size_goal(u))
-      return FALSE;
-    return TRUE;
+	/* Then check that the minimal size goal can be attained for all possible advanced 
+	units (even if we start with a village it may eventually grow into a metropolis). */
+	for_all_unit_types(u) {
+		if (!u_advanced(u))
+			continue;
+		maxsize = PROPHI;
+
+		/* First zero the supply vector. */
+		for_all_material_types (m)
+			supply[m] = 0;
+		/* Go through all cells within reach and compute supply of materials. */
+		for_all_cells_within_range(x, y, u_reach(u), x1, y1) {
+			if (!inside_area(x1, y1))
+				continue;
+			/* Don't count supply in cells already used by others. */
+			if (user_at(x1, y1) != NOUSER)
+				continue;
+			/* Add the supply of materials from this cell. */
+			for_all_material_types (m)
+				supply[m] += tm_production(terrain_at(x1, y1), m); 
+		}
+		/* Calculate max sustainable city size at this location. */
+		for_all_material_types (m) {
+			if (um_consumption_per_size(u, m) > 0)
+				maxsize = min(maxsize, supply[m] / um_consumption_per_size(u, m));
+		}
+		/* Return FALSE if there is not enough food around 
+		to sustain the minimal size goal for this unit type. */
+		if (maxsize < u_minimal_size_goal(u))
+			return FALSE;
+	}
+	return TRUE;
 }
 
 static void
 plan_exploration(unit)
 Unit *unit;
 {
+    Unit *unit2;
     Plan *plan = unit->plan;
     int u = unit->type;
     int x, y, w, h, range, x1, y1;
 
     /* If the world has no secrets, exploration is sort of pointless. */
-    if (all_see_all) {
+    if (all_see_all && g_units_may_go_into_reserve()) {
     	plan->reserve = TRUE;
     	return;
     }
@@ -1068,6 +1139,20 @@ Unit *unit;
     }
     if (plan->maingoal) {
 	switch (plan->maingoal->type) {
+  	  case GOAL_UNIT_OCCUPIED:
+	    /* Move to occupy our goal if necessary. */
+	    unit2 = find_unit(plan->maingoal->args[0]);
+	    if (unit->transport != unit2)
+	      set_occupy_task(unit, unit2);
+	    break;
+
+	  case GOAL_CELL_OCCUPIED:
+	    /* Move to occupy our goal if necessary. */
+	    x = plan->maingoal->args[0];  y = plan->maingoal->args[1];
+	    if (unit->x != x || unit->y != y)
+	      set_move_to_task(unit, x, y, 0);
+	    break;
+
 	  case GOAL_VICINITY_KNOWN:
 	  case GOAL_VICINITY_HELD:
 	    if (mobile(u)) {
@@ -1133,8 +1218,8 @@ Unit *unit;
     if (plan->tasks) {
         execute_task(unit);
     } else {
-        if (probability(10)) {
-    	    DMprintf("no tasks, going to sleep (dunno why)\n");
+	    if (probability(10) && g_units_may_go_to_sleep()) {
+   	    DMprintf("no tasks, going to sleep (dunno why)\n");
     	    plan->asleep = TRUE;
         }
     }
@@ -1149,6 +1234,8 @@ Unit *unit;
 
     for_all_unit_types(u2) {
 	    if (mobile(u2)
+		&& type_allowed_on_side(u2, unit->side) 
+		&& has_advance_to_build(unit->side, u2)
 		&& 1 /* better on more kinds of terrain? */
 		&& uu_acp_to_create(u, u2) > 0) {
 		backup = u2;
@@ -1175,7 +1262,7 @@ Unit *unit;
     }
 }
 
-static int victimx, victimy, victimrating, victimutype, victimsidenum;
+int victimx, victimy, victimrating, victimutype, victimsidenum;
 
 int
 victim_here(x, y)
@@ -1184,6 +1271,7 @@ int x, y;
     int u2 = NONUTYPE, uview, rating, dist;
     Unit *unit2;
     Side *side = tmpunit->side, *oside = NULL;
+    Unit *unit3;
 
     if (units_visible(side, x, y)) {
 	if ((unit2 = unit_at(x, y)) != NULL) {
@@ -1202,7 +1290,12 @@ int x, y;
     if (is_unit_type(u2)
 	&& enemy_side(side, oside)
 	&& ((could_hit(tmpunit->type, u2)
-	     && uu_damage(tmpunit->type, u2) > 0
+	/* Also consider damage by fire. Moreover, if the unit can
+	   carry occupants they may be vulnerable even though the unit
+	   itself is not. */
+	&& (uu_damage(tmpunit->type, u2) > 0
+		|| uu_fire_damage(tmpunit->type, u2) > 0
+		|| type_can_carry(u2))		
 	     && (!worth_capturing(side, u2, oside, x, y)
 	         || capture_chance(tmpunit->type, u2, oside) > 0))
 	    || capture_chance(tmpunit->type, u2, oside) > 0)
@@ -1233,6 +1326,18 @@ int x, y;
 	if (dist > u_acp(tmpunit->type)) {
 	    rating /= max(1, isqrt(dist - u_acp(tmpunit->type)));
 	}
+	/* A large city is more worth capturing. */
+    	if (units_visible(side, x, y))
+	  rating *= unit_at(x,y)->size;
+	/* Real enemies are more important targets. */
+	if (oside != NULL)
+	  rating *= 2;
+	/* Always attack units that threaten one of our own cities. */
+	for_all_side_units(side, unit3) {
+	    if (u_advanced(unit3->type)
+		&& distance(unit3->x, unit3->y, x, y) < 5)
+	      rating *= 5 - distance(unit3->x, unit3->y, x, y);
+	}
 	if (rating > victimrating || (rating == victimrating && flip_coin())) {
 	    victimx = x;  victimy = y;
 	    victimrating = rating;
@@ -1263,6 +1368,9 @@ int u2, x, y;
     return TRUE;
 }
 
+/* Note: go_after_victim can be used for both attacking and firing units since
+victim_here only tests for could_hit.  */
+
 /* This routine looks for somebody, anybody to attack. */
 
 int
@@ -1306,7 +1414,37 @@ int range;
     return rslt;
 }
 
-static int targetx, targety, targetrating, targetutype, targetsidenum;
+/* The point of this new function is to limit the search for captives
+   to a given range. */
+
+int
+go_after_captive(Unit *unit, int range)
+{
+    int x, y, rslt;
+
+    tmpunit = unit;
+
+    DMprintf("%s searching for useful capture within %d; found ",
+	     unit_desig(unit), range);
+    rslt = search_around(unit->x, unit->y, range, 
+			 useful_captureable_here, &x, &y, 1);
+    if (rslt) {
+	DMprintf("one at %d,%d\n", x, y);
+	/* Set up a task to go after the unit found. */
+	set_capture_task(unit, x, y);
+	if (unit->transport
+	    && mobile(unit->transport->type)
+	    && unit->transport->plan) {
+	    set_move_to_task(unit->transport, x, y, 1);
+	}
+	return (execute_task(unit) != TASK_FAILED);
+    } else {
+	DMprintf("nothing\n");
+	return FALSE;
+    }
+}
+
+int targetx, targety, targetrating, targetutype, targetsidenum;
 
 int
 target_here(x, y)
@@ -1315,6 +1453,7 @@ int x, y;
     int u2 = NONUTYPE, uview, rating, dist;
     Unit *unit2;
     Side *side = tmpunit->side, *oside = NULL;
+    Unit *unit3;
 
     if (units_visible(side, x, y)) {
 	if ((unit2 = unit_at(x, y)) != NULL) {
@@ -1335,7 +1474,11 @@ int x, y;
     if (is_unit_type(u2)
 	&& enemy_side(side, oside)
 	&& could_hit(tmpunit->type, u2)
-	&& uu_damage(tmpunit->type, u2) > 0
+	/* Also consider damage by fire. Moreover, if the unit can carry occupants
+	they may be vulnerable even though the unit itself is not. */
+         	&& (uu_damage(tmpunit->type, u2) > 0
+	    || uu_fire_damage(tmpunit->type, u2) > 0
+	    || type_can_carry(u2))		
 	/* and have correct ammo */
 	&& !side_planning_to_capture(side, u2, x, y)
 	) {
@@ -1344,6 +1487,18 @@ int x, y;
 	dist = distance(tmpunit->x, tmpunit->y, x, y);
 	if (dist > 0)
 	  rating /= dist;
+	/* A large city is more worth capturing. */
+    	if (units_visible(side, x, y))
+		rating *= unit_at(x,y)->size;
+	/* Real enemies are more important targets. */
+	if (oside != NULL)
+		rating *= 2;
+	/* Always attack units that threaten one of our own cities. */
+	for_all_side_units(side, unit3) {
+		if (u_advanced(unit3->type)
+		    && distance(unit3->x, unit3->y, x, y) < 5)
+		    	rating *= 5 - distance(unit3->x, unit3->y, x, y);
+	}
 	if (rating > targetrating || (rating == targetrating && flip_coin())) {
 	    targetx = x;  targety = y;
 	    targetrating = rating;
@@ -1586,7 +1741,9 @@ int
 capture_indep_if_nearby(unit)
 Unit *unit;
 {
-    int u = unit->type, range = 5;
+    /* Use the unit's tactical range instead. */
+    int	u = unit->type;
+    int	range = u_ai_tactical_range(u);
     int x, y, rslt;
     Task *curtask = unit->plan->tasks;
 
@@ -1657,7 +1814,9 @@ int
 capture_useful_if_nearby(unit)
 Unit *unit;
 {
-    int u = unit->type, range = 2;
+    /* Use the unit's tactical range instead. */
+    int u = unit->type;
+    int	range = u_ai_tactical_range(u);
     int x, y, rslt;
     Task *curtask = unit->plan->tasks;
 
@@ -1798,7 +1957,7 @@ plan_random(Unit *unit)
 	    break;
 	  case TASK_COLLECT:
 	    if (nummtypes > 0) {
-		randmtype = (flip_coin() ? xrandom(nummtypes) : NONMTYPE);
+		randmtype = xrandom(nummtypes);
 		random_point_near(unit->x, unit->y, 4, &x1, &y1);
 	    	set_collect_task(unit, randmtype, x1, y1);
 	    } else {
@@ -1845,7 +2004,7 @@ plan_random(Unit *unit)
 	    break;
 	  case TASK_PRODUCE:
 	    if (nummtypes > 0) {
-		randmtype = (flip_coin() ? xrandom(nummtypes) : NONMTYPE);
+		randmtype = xrandom(nummtypes);
 		task = create_produce_task(unit, randmtype, xrandom(100));
 		add_task(unit, CLEAR_AGENDA, task);
 	    } else {
@@ -1861,7 +2020,7 @@ plan_random(Unit *unit)
 	    break;
 	  case TASK_RESUPPLY:
 	    if (nummtypes > 0) {
-		randmtype = (flip_coin() ? xrandom(nummtypes) : NONMTYPE);
+		randmtype = xrandom(nummtypes);
 		set_resupply_task(unit, randmtype);
 	    } else {
 		set_sentry_task(unit, xrandom(5));
@@ -2476,7 +2635,7 @@ Unit *unit;
 int
 find_closest_unit(side, x0, y0, maxdist, pred, rxp, ryp)
 Side *side;
-int x0, y0, maxdist, (*pred)(), *rxp, *ryp;
+int x0, y0, maxdist, (*pred)(int x, int y), *rxp, *ryp;
 {
 #if 0    
     Unit *unit;
