@@ -1018,7 +1018,7 @@ char *filename;
     }
     
     /*  open view file  */
-    fp = fopen(filename, "w");
+    fp = open_file(filename, "w");
     if (fp == NULL) {
 	notify(ps_side, "Cannot open \"%s\", dump skipped", filename);
 	return;
