@@ -11,6 +11,7 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 #include "kpublic.h"
+#include "ai.h"
 
 extern TaskOutcome execute_task_aux PARAMS ((Unit *unit, Task *task));
 
@@ -21,13 +22,11 @@ extern void run_restored_turn_start PARAMS ((void));
 extern void run_turn_end PARAMS ((void));
 static void compose_actionvectors PARAMS ((void));
 static void init_movement PARAMS ((void));
-static void init_actionvectors PARAMS ((void));
 static int move_some_units PARAMS ((int lim));
 static int side_move_some_units PARAMS ((Side *side, int lim));
 static int unit_still_acting PARAMS ((Unit *unit));
 static int unit_still_acting_no_plan PARAMS ((Unit *unit));
 static int move_one_unit_multiple PARAMS ((Unit *unit, int lim));
-static void finish_movement PARAMS ((void));
 
 /* Priority of sides that are now moving. */
 
@@ -113,8 +112,6 @@ int current_play_rate;
 
 static int nothing_count;
 
-static int nothing_timeout;
-
 /* True when an event occurs and we need to check scorekeepers. */
 
 int need_post_event_scores;
@@ -183,8 +180,10 @@ int maxactions;
     time_t rungamestart, rungameend;
     Side *side;
     extern long randstate;
+    char *activity = "run_game";
 
     in_run_game = TRUE;
+    record_activity_start(activity, maxactions);
     if (Debug && numremotes > 0 && 0) {
 	sprintf(spbuf, "states/%dstate%06da", my_rid, g_run_serial_number());
 	debugging_state_sync = TRUE;
@@ -345,6 +344,7 @@ rungamereturn:
     }
     in_run_game = FALSE;
     set_g_run_serial_number(g_run_serial_number() + 1);
+    record_activity_end(activity, numdone);
     return numdone;
 }
 
@@ -618,7 +618,8 @@ Unit *unit;
     return u_action_priority(unit->type);
 }
 
-/* Do some number of actions. */
+/* Do some number of actions, up to a total of LIM per side.  Return
+   the total number of actions performed. */
 
 static int
 move_some_units(lim)
@@ -632,7 +633,7 @@ int lim;
 	     curpriority == side->priority :
 	     TRUE)) {
 	    sidenum = side_move_some_units(side, lim);
-	    num = max(num, sidenum);
+	    num += sidenum;
 	}
     }
     return num;
@@ -671,8 +672,8 @@ int lim;
 	    continue;
 	}
 	current_play_rate = slow_play_rate;
-	/* AIs should play as fast as possible if turns are sequential or if the
-	   human players are all done. */
+	/* AIs should play as fast as possible if turns are sequential
+	   or if the human players are all done. */
 	if (g_use_side_priority() || all_human_only_sides_finished()) {
 	    current_play_rate = fast_play_rate;
 	}
@@ -823,13 +824,13 @@ int lim;
 		&& unit->side->orders
 		&& unit->plan->tasks == NULL
 		&& execute_standing_order(unit, TRUE)) {
-		execute_plan(unit, 1);
+		planexecs += execute_plan(unit);
 		gamestatesafe = FALSE;
 		++buzz;
 	    }
 	    /* Similarly for formations. */
 	    if (unit->plan->formation && move_into_formation(unit)) {
-		execute_plan(unit, 1);
+		planexecs += execute_plan(unit);
 		gamestatesafe = FALSE;
 		++buzz;
 	    }
@@ -845,7 +846,7 @@ int lim;
 		|| unit->plan->delayed)
 	      break;
 	    /* Normal plan execution. */
-	    execute_plan(unit, 1);
+	    planexecs += execute_plan(unit);
 	    record_ms();
 	    gamestatesafe = FALSE;
 	    ++buzz;
@@ -927,12 +928,16 @@ int when, maxplanning;
 {
     Side *side;
     Unit *unit;
+    char *activity = "run_local_ai";
 
+    record_activity_start(activity, when);
     /* Only do this routine at a certain rate. */
     if (current_play_rate > 0
 	&& !n_ms_elapsed(60000 / current_play_rate)
-	)
-      return 0;
+	) {
+	record_activity_end(activity, 1);
+	return 0;
+    }
     if (need_ai_for_new_side) {
 	for_all_sides(side) {
 	    if (side_wants_ai(side) && !side_has_ai(side)) {
@@ -957,6 +962,15 @@ int when, maxplanning;
 			&& unit->plan->aicontrol
 			&& !unit->plan->asleep) {
 			ai_decide_plan(side, unit);
+#if 0 /* comment out until this is figured out */
+			/* ...was in execute_plan - not ideal location, but
+			   needs to be in this function somewhere... */
+			/* Always reevaluate the plan before executing it! */
+			ai_update_plan(side, unit);
+			/* Deal with tactical emergencies such as
+			   enemy units that suddenly appear nearby. */
+			ai_handle_emergency(side, unit);
+#endif
 		    }
 		}
 	    }
@@ -995,6 +1009,7 @@ int when, maxplanning;
 	}
 	need_ai_finish_movement = FALSE;
     }
+    record_activity_end(activity, 1);
     return 0;
 }
 
@@ -1116,8 +1131,6 @@ end_the_game()
     ok_to_exit = TRUE;
 }
 
-// --- ALL ADVANCED UNIT CODE BELOW UNTIL EOF REWRITTEN 990117 -------------
-
 /* 	Advanced unit and advance code for Xconq.
      	Copyright (C) 1998 Hans Ronne. */
 
@@ -1127,20 +1140,19 @@ extern void unit_advance_dialog(Unit *unit);
 extern void global_advance_dialog(Side *side, Unit *unit, int advance);
 extern void auto_pick_new_research(Unit *unit);
 
+extern void unit_plan_dialog(Unit *unit);
+extern void net_resume_build_task(Unit *unit, Unit *unit2, int run);
 extern int has_advance_to_research PARAMS ((Side *side, int a));
-extern int has_advance_to_build(Side *side, int u);
-extern int unit_build_dialog(Unit *unit, int oldtask);
-
-extern Unit *create_selected_unit(Unit *unit, int u);
+extern int select_by_weight PARAMS ((int *arr, int numvals));
+extern int unit_build_dialog(Unit *unit);
 
 static void run_population(Unit *unit);
 static void run_construction(Unit *unit);
 static void run_research(Unit *unit);
-static void 	allocate_used_cells(Unit *unit);
+static void allocate_used_cells(Unit *unit);
 
-static int auto_pick_new_build_task(Unit *unit);
-
-static Unit *set_new_build_task(Unit *unit, int oldtask);
+extern int auto_pick_new_build_task(Unit *unit);
+extern void auto_pick_new_plan(Unit *unit);
 
 /* TRUE if side has advance to research this advance. */
 
@@ -1271,9 +1283,7 @@ run_advanced_units()
 	    }
 	}
 	/* Run building tasks. */
-#ifndef UNIX /* crashes on Unix */
 	run_construction(unit);
-#endif
 	/* Do the research. */		
 	run_research(unit);
 	/* Return any borrowed material to treasury. */
@@ -1320,7 +1330,10 @@ run_population(Unit *unit)
                treasury. */
 	    for_all_material_types(m) {
 		if (m_treasury(m)) {
-		    unit->side->treasury[m] += unit->supply[m];
+		    /* Copy of Stan's bug fix in run_advanced_units. */
+  		    Side *sidex = (unit->side ? unit->side : indepside);
+    
+		    sidex->treasury[m] += unit->supply[m];
 		    unit->supply[m] = 0; 			
 		}
 	    }
@@ -1373,82 +1386,137 @@ run_population(Unit *unit)
 
 void
 run_construction(Unit *unit)
-{
-    Unit *unit2; 
-			
-    /* Return if this is an independent unit which is not allowed to build. */
-    if (indep(unit) && !g_indep_may_build())
-      return;
-    if (unit->plan == NULL)
-      return;
-    /* First check that we are building and set build task if not. */
-    if (unit->plan->tasks == NULL || unit->plan->tasks->type != TASK_BUILD)
-      unit2 = set_new_build_task(unit, NOUNIT);
-    /* Else find the unit currently being constructed. */
-    else if (unit->plan->tasks->args[1])
-      unit2 = find_unit(unit->plan->tasks->args[1]);
-    /* If we fail: */
-    if (!is_unit(unit2) && unit->plan->tasks)
-      /* Look around for incomplete units. */
-      unit2 = find_unit_to_complete(unit, unit->plan->tasks);
-    if (!is_unit(unit2)) 
-      /* Start a new build task. */
-      unit2 = set_new_build_task(unit, NOUNIT);
-    if (!is_unit(unit2))
-      /* Give up. */
-      return;
-    
-    /* Exit loop by return. */
-    while (1) {
-	int	fallback = unit->size;
-	int 	u = unit2->type;
-	int 	limited = FALSE;
-	int 	m;
+{			
+    int builds = unit->size;
 
-	/* Check if any material limits its construction. */
-	for_all_material_types(m) {
-	    if (um_consumption_per_cp(u, m) > 0) {
-		limited = TRUE;
+    /* Skip if this is an indep which cannot build. */
+    if (indep(unit) &! g_indep_may_build())
+      return;
+    /* Make sure we have a real plan. */
+    if (unit->plan == NULL
+	|| unit->plan->type == PLAN_NONE
+	|| unit->plan->type == PLAN_PASSIVE) { 
+	/* Pick a new plan if we lack one. */
+	if (!indep(unit)
+	    && side_has_display(unit->side)
+	    && (!side_has_ai(unit->side) 
+		|| !unit->plan->aicontrol)
+	    && !unit->autoplan)
+	  unit_plan_dialog(unit);
+	else
+	  auto_pick_new_plan(unit);
+    }
+
+    while (builds) {
+	Unit *unit2 = NULL;
+	int u2 = NONUTYPE;
+	int limited = FALSE;
+	int actionresult;
+	TaskOutcome taskresult;
+	int m;
+
+	if (unit->plan
+	    && unit->plan->tasks) {
+	    unit2 = find_unit(unit->plan->tasks->args[1]);
+	    u2 = unit->plan->tasks->args[0];
+	}
+	/* Check if we lack a build task or just completed one. */
+	if (unit->plan->tasks == NULL
+	    || unit->plan->tasks->type != TASK_BUILD
+	    || unit->plan->last_task_outcome == TASK_IS_COMPLETE) {
+	    /* Find any incomplete occupant that is hanging around.
+	       This is also how we pick up old tasks after saving a
+	       game.*/
+	    unit2 = NULL;
+	    for_all_occupants(unit, unit2) {
+		if (in_play(unit2) && !fullsized(unit2))
+		  break;
+	    }
+	    /* Resume building of incomplete occupant. */
+	    if (is_unit(unit2)){
+		net_resume_build_task(unit, unit2, 1);
+		u2 = unit2->type;
+	    } else {
+		/* Else select a new unit type to build. */
+		if (!indep(unit)
+		    && side_has_display(unit->side)
+		    && (!side_has_ai(unit->side) || !unit->plan->aicontrol)
+		    && !unit->autobuild)
+		  u2 = unit_build_dialog(unit);
+		else
+		  u2 = auto_pick_new_build_task(unit);
+		/* Emergency exit. Happens if "Idle" is manually
+		   selected or if the city is full. */
+		if (!is_unit_type(u2))
+		  return;
+	    }
+	}
+	/* unit2 has either not been created or is complete. */
+	if (unit2 == NULL || fullsized(unit2)) {
+	    /* First check if there is enough supplies for creation. */	
+	    for_all_material_types(m) {
+		if (unit->supply[m] < um_consumption_on_creation(u2, m))
+		  return;
+	    }
+	    /* Then consume materials as required. */
+	    for_all_material_types(m) {
+		unit->supply[m] -= um_consumption_on_creation(u2, m);
+	    }
+	    /* Give unit just enough acps for one create action. */
+	    unit->act->acp = uu_acp_to_create(unit->type, u2); 
+
+	    /* unit2 is still in progress. */
+	} else {
+	    /* Check if any material limits its construction. */
+	    for_all_material_types(m) {
+		if (um_consumption_per_cp(u2, m) > 0)
+		  limited = TRUE;
 		break;
 	    }
+	    if (limited) {
+		/* First check if there is enough supplies for one build. */
+		for_all_material_types(m) {
+		    if (unit->supply[m] < um_consumption_per_cp(u2, m)) 
+		      return;
+		}
+		/* Then consume materials as required. */
+		for_all_material_types(m) {
+		    unit->supply[m] -= um_consumption_per_cp(u2, m);
+		}
+		/* Else limit total builds to unit size as fallback. */
+	    } else
+	      --builds;
+	    /* Give unit just enough acps for one build action. */
+	    unit->act->acp = uu_acp_to_build(unit->type, u2); 
 	}
-	if (limited) {
-	    /* First check if there is enough supplies for 1 cp. */		
-	    for_all_material_types(m)
-	      if (unit->supply[m] < um_consumption_per_cp(u, m))
-		return;
-	    /* Then consume materials as required. */
-	    for_all_material_types(m)
-	      unit->supply[m] -= um_consumption_per_cp(u, m);
-	    /* Limit cps to unit size if unlimited by materials. */
-	} else if (--fallback < 0)
-	  return;
-	/* Give it just enough acps to execute 1 build action. */
-	unit->act->acp = uu_acp_to_build(unit->type, u);
+						
 	/* Necessary to enable execute_task. */
 	need_ai_task_reaction = FALSE;
-	/* Prepare and execute the build action. */
-	execute_task(unit);
-	execute_action(unit, &(unit->act->nextaction));
-	gamestatesafe = FALSE;
-
-	if (completed(unit2)) {
-	    if (unit->plan->tasks->args[2] <= unit->plan->tasks->args[3]) {
-		/* Continue with this build run as long as possible. */
-		unit2 = create_selected_unit(unit, u);
-		if (is_unit(unit2))
-		  continue;
-	    }
-	    if (unit->plan->tasks)
-	      /* Else look around for incomplete units. */
-	      unit2 = find_unit_to_complete(unit, unit->plan->tasks);
-	    if (!is_unit(unit2))
-	      /* Else set a new build task. */
-	      unit2 = set_new_build_task(unit, u);
-	    if (!is_unit(unit2))
-	      /* Give up. */
-	      return;
+	/* Prepare the build action. */
+	taskresult = execute_task(unit);
+	/* Never happens, but check anyway. */
+	if (taskresult == TASK_FAILED || taskresult == TASK_UNKNOWN) {
+	    Dprintf("Task %s failed for %s!\n", 
+		    task_desig(unit->plan->tasks), 
+		    unit_desig(unit));
+	    return;
 	}
+	/* Do not execute pending action if we lack one or need to
+	   pick a new task!  The latter is important since we will
+	   continue to build on the old unit and get an error if a new
+	   build task has not been assigned. */
+	if (has_pending_action(unit) && taskresult != TASK_IS_COMPLETE) {
+	    actionresult = execute_action(unit, &unit->act->nextaction);
+	    /* Never happens, but check anyway. */
+	    if (actionresult == A_ANY_ERROR) {
+		Dprintf("Action %s failed for %s!\n", 
+			action_desig(&unit->act->nextaction), 
+			unit_desig(unit));
+		return;
+	    }
+	}
+	/* Things have changed. */
+	gamestatesafe = FALSE;
     }
 }
 
@@ -1682,138 +1750,240 @@ free_used_cells(Unit *unit)
     unit->usedcells = 0;
 }
 
-/* Pick a new advance to research automatically. */
+/* Now picks new advance at random rather than by number. */
 
 void
-auto_pick_new_research(unit)
-Unit *unit;
+auto_pick_new_research(Unit *unit)
 {
     int a;
+    int type[MAXUTYPES];
+    int numtypes = 0;
 
-    /* Get researchable advance types */
+    /* Get researchable advance types. */
     for_all_advance_types(a) {
 	if (has_advance_to_research(unit->side, a)
 	    && !has_advance(unit->side, a)) {
-	    /* Pick the first available advance */
-	    unit->curadvance = a;
-	    return;
+	    /* Add to list of researchable advances. */
+	    type[numtypes++] = a;
 	}
     }
-    /* No advance can be researched */
-    unit->curadvance = NOADVANCE;
-}
-
-/* Second parameter is the type of the just completed unit, NOUNIT
-   (-1) if nothing was completed. */
-
-Unit *
-set_new_build_task(Unit *unit, int oldtask)
-{
-    Unit *unit2;
-    int u;
-
-    /* Select a new unit type to build. */
-    if (!indep(unit)
-	&& side_has_display(unit->side)
-	&& !side_has_ai(unit->side)
-	&& !unit->autobuild)
-      /* Do it manually. */
-      u = unit_build_dialog(unit, oldtask);
-    /* Or automatically. */
+    if (numtypes)
+      /* Pick one of them at random. */
+      unit->curadvance = type[xrandom(numtypes)];
     else
-      u = auto_pick_new_build_task(unit);
-    set_build_task(unit, u, 1);
-    /* Return NULL if we failed. */
-    if (is_unit_type(u))
-      return create_selected_unit(unit, u);
-    else
-      return NULL;
+      unit->curadvance = NOADVANCE;
 }
 
-Unit *
-create_selected_unit(Unit *unit, int u)
-{
-	Unit *unit2;
-
-	/* Give the unit just enough acps to create one new unit. */
-	unit->act->acp = uu_acp_to_create(unit->type, u);
-	/* Necessary to enable execute_task. */
-	need_ai_task_reaction = FALSE;
-	/* Prepare and execute the creation. */
-	execute_task(unit);
-    	execute_action(unit, &(unit->act->nextaction));
-	gamestatesafe = FALSE;
-	/* Find the unit that was created and put it on build list. */
-	/* Note: find_unit(unit->plan->tasks->args[1]) fails
-	here since do_create_in_action does NOT put the created
-	unit on the build task list. */
-	if (unit->plan->tasks == NULL)
-	  return NULL;
-	unit2 = find_unit_to_complete(unit, unit->plan->tasks);
-	if (unit2) {
-		unit->plan->tasks->args[0] = unit2->type;
-		unit->plan->tasks->args[1] = unit2->id;
-	}
-	return unit2;
-}
-
-/* Pick a new unit type to build automatically. */
-
 int
 auto_pick_new_build_task(Unit *unit)
 {
-	int 	type[MAXUTYPES];
-	int 	numtypes = 0;
-	int	failed = 0;
-	int	index, u;
-	Unit	*unit2;
-	int	rslt;
-				
-	/* First make 25% colonizers. */
-	if (probability(25)) {
-		for_all_colonizer_utypes(index) {
-			u = colonizer_utypes[index];
-			if (type_allowed_on_side(u, unit->side) 
-			    	&& could_create(unit->type, u)) {
-					type[numtypes] = u;
-					numtypes++;
-			}
-		}
-	/* Then make 25% (33% of what is left) facilities. */
-	} else if (probability(33)) {
-		for_all_facility_utypes(index) {
-			u = facility_utypes[index];
-			if (type_allowed_on_side(u, unit->side) 
-			    	&& could_create(unit->type, u)) {
-					type[numtypes] = u;
-					numtypes++;
-			}
-		}
-	/* Make remaining 50% attackers. */	
-	} else for_all_offensive_utypes(index) { 
-		u = offensive_utypes[index];
-		if (type_allowed_on_side(u, unit->side) 
-		    	&& could_create(unit->type, u)) {
-				type[numtypes] = u;
-				numtypes++;
-		}	
-	}
-	/* Try all possible utypes if we found nothing to build. */
-	if (!numtypes) {
-		 for_all_unit_types(u) {	
-			if (type_allowed_on_side(u, unit->side) 
-			    	&& could_create(unit->type, u)) {
-					type[numtypes] = u;
-					numtypes++;
-			}	
-		}
-	}
+    int prefs[MAXUTYPES] = {0};
+    int enemy_city_near = FALSE;
+    int x, y, x1, y1, u; 
+    int city_has_shore = FALSE;
+    int numtypes = 0;
+    Unit *unit2;
+	
+    /* Reevaluate the current plan every time. */
+    if (!indep(unit)
+	&& side_has_display(unit->side)
+	&& (!side_has_ai(unit->side) || !unit->plan->aicontrol)
+	&& !unit->autoplan)
+      unit_plan_dialog(unit);
+    else
+      auto_pick_new_plan(unit);
 
-	if (numtypes) {
-		u = type[xrandom(numtypes)];
-		/* Set the building task. */
-		net_set_build_task(unit, u, g_ai_build_runlength()); 
-	} else u = NONUTYPE;
-	/* Return the selected utype. */
-	return u;	
+    /* Check if the unit borders on a water cell. */ 	
+    for_all_cells_within_range(unit->x, unit->y, 1, x, y) {
+	if (t_liquid(terrain_at(wrapx(x), y))) {
+	    /* Also check if that water cell borders on another water cell.
+	       (we don't want to build battleships in the city pond) */
+	    for_all_cells_within_range(x, y, 1, x1, y1) {
+		if (t_liquid(terrain_at(wrapx(x1), y1))
+		    /* Don't count the first water cell. */
+		    && (x != x1 || y != y1)) {
+		    city_has_shore = TRUE;
+		    break;
+		}
+	    }
+	    if (city_has_shore)
+	      break;				
+	}
+    }		
+
+    /* Check if we have an enemy city within our tactical range. */
+    for_all_units(unit2) {
+	if (u_advanced(unit2->type)
+	    && distance(unit->x, unit->y, unit2->x, unit2->y)
+	    >= u_ai_tactical_range(unit->type)
+	    && enemy_side(unit->side, unit2->side)) {
+	    enemy_city_near = TRUE;
+	    break;
+	}
+    }
+	
+    /* Calculate unit values. */	
+    for_all_unit_types(u) {
+	if (type_allowed_on_side(u, unit->side) 
+	    && has_advance_to_build(unit->side, u)
+	    && could_create(unit->type, u)
+	    /* Don't build immobile units for which there is no room. */
+	    && (mobile(u) || type_can_occupy(u, unit))
+	    /* Don't build ships if we don't have a shore. */
+	    && (!u_naval_mobile(u) || city_has_shore)) {
+	    
+	    switch (unit->plan->type) {
+	      case PLAN_COLONIZING:
+		prefs[u] = u_colonizer_worth(u);
+		break;					
+	      case PLAN_IMPROVING:			
+		prefs[u] = u_facility_worth(u);
+		break;					
+	      case PLAN_EXPLORATORY:
+		prefs[u] = u_explorer_worth(u);
+		break;					
+	      case PLAN_OFFENSIVE:
+		if (enemy_city_near) {
+		    /* Build siege units. */
+		    prefs[u] = u_siege_worth(u);
+		} else	{
+		    /* Build attack units. */
+		    prefs[u] = u_offensive_worth(u);
+		}
+		break;					
+	      case PLAN_DEFENSIVE:
+		prefs[u] = occ_can_defend_transport(u, unit->type) 
+		  * u_defensive_worth(u);
+		break;					
+	      case PLAN_RANDOM:	
+		prefs[u] = u_random_worth(u);
+		break;					
+	      case PLAN_NONE:
+	      case PLAN_PASSIVE:
+	      case NUMPLANTYPES:
+		prefs[u] = 0;
+		break;					
+	    }
+	}
+	if (prefs[u] <= 0)
+	  prefs[u] = 0;
+	else ++numtypes;
+    }			
+
+    /* Try again with random choice if we found nothing to build. */
+    if (numtypes == 0) {
+	for_all_unit_types(u) {
+	    if (type_allowed_on_side(u, unit->side) 
+		&& has_advance_to_build(unit->side, u)
+		&& could_create(unit->type, u)
+		/* Don't build immobile units for which there is no room. */
+		&& (mobile(u) || type_can_occupy(u, unit))
+		/* Don't build ships if we don't have a shore. */
+		&& (!u_naval_mobile(u) || city_has_shore)) {
+		
+		prefs[u] = u_random_worth(u);
+		if (prefs[u] <= 0)
+		  prefs[u] = 0;
+		else ++numtypes;
+	    }
+	}
+    }
+			
+    if (numtypes > 0) {
+	/* Use prefs weight to pick unit type to build. */
+	u = select_by_weight(prefs, numutypes);
+	if (is_unit_type(u))
+	  net_set_build_task(unit, u, 1); 
+	else
+	  u = NONUTYPE;
+    } else
+      u = NONUTYPE;
+    return u;	
+}
+
+/* This is a rudimentary AI for advanced units. Lots of room for
+   improvement! */
+
+void
+auto_pick_new_plan(Unit *unit)
+{
+    int my_advanced = 0, my_colonizers = 0, my_total = 0;
+    int enemy_advanced = 0, enemy_colonizers = 0, enemy_total = 0;
+    int defenders = 0, facilities = 0, mobiles = 0, invisible_cells = 0;
+    int u = unit->type;
+    int x, y, u2;
+    Unit *unit2;
+			
+    /* Init plan if necessary. */
+    if (!unit->plan)
+      init_unit_plan(unit);	
+
+    /* First count our facilities, defenders and mobile types. */
+    for_all_occs_with_occs(unit, unit2) {
+	if (u_facility(unit2->type))
+	  facilities += 1;
+	if (in_play(unit2)
+	    && occ_can_defend_transport(unit2->type, unit->type)) 
+	  defenders += 1;
+	if (mobile(unit2->type))
+	  mobiles += 1;
+    }		
+    /* Always build a defender first if we are undefended. */
+    if (defenders < u_ai_min_defenders(unit->type)) {
+	set_unit_plan_type(unit->side, unit, PLAN_DEFENSIVE);
+	return;
+    }	
+
+    /* Check out the tactical neighbourhood. */
+    for_all_cells_within_range(unit->x, unit->y, 
+				 u_ai_tactical_range(unit->type), x, y) {
+	if (!inside_area(x, y))
+	  continue;
+	if (!terrain_visible(unit->side, x, y))
+	  invisible_cells +=1;
+	unit2 = NULL;
+	/* Important to count also occupants here. */
+	for_all_stack_with_occs(x, y, unit2) {
+	    /* Only count real units. */
+	    if (!in_play(unit2))
+	      continue;
+	    /* Only count units that we can see. */
+	    if (!unit_actually_visible(unit->side, unit2))
+	      continue;
+	    u2 = unit2->type;
+	    /* First count own units. */
+	    if (unit->side == unit2->side) {
+		if (u_advanced(u2))
+		  my_advanced += 1;
+		else if (u_colonizer_worth(u2))
+		  my_colonizers += 1;
+		my_total += 1;
+		/* Then count hostile units. */
+	    } else if (enemy_side(unit->side, unit2->side)){
+		if (u_advanced(u2))
+		  enemy_advanced += 1;
+		else if (u_colonizer_worth(u2))
+		  enemy_colonizers += 1;
+		enemy_total += 1;
+	    }			
+	}
+    }
+
+    /* Always build attackers if any enemy unit is near. */
+    if (enemy_total > 0)
+      net_set_unit_plan_type(unit->side, unit, PLAN_OFFENSIVE);
+    /* Then build explorers if we have any invisible cells nearby. */
+    else if (invisible_cells)
+      net_set_unit_plan_type(unit->side, unit, PLAN_EXPLORATORY);
+    /* Build colonizers if we have < 7 colonizers or places nearby. */
+    else if (my_advanced + my_colonizers < 7)
+      net_set_unit_plan_type(unit->side, unit, PLAN_COLONIZING);
+    /* Build 50% facilities if there is room left. */
+    else if (flip_coin() && facilities < u_facility_total_max(u))			
+      net_set_unit_plan_type(unit->side, unit, PLAN_IMPROVING);
+    /* Else build offensive units. */	
+    else
+      net_set_unit_plan_type(unit->side, unit, PLAN_OFFENSIVE);
+
+    net_clear_task_agenda(unit->side, unit);
 }
