@@ -10,77 +10,74 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kernel.h"
+#include "ai.h"
+
 extern int stop_apply;
 
+static void vary_winds(void);
+
 /* Advanced unit support. */
 extern void run_advanced_units(void);
-
-// --- OBSOLETE ---------------------------------
-
-//	extern void compute_acp_advanced_unit(Unit *unit);
-
-// --------------------------------------------
-
-extern void run_turn_start PARAMS ((void));
-extern void run_restored_turn_start PARAMS ((void));
-extern void run_turn_end PARAMS ((void));
+extern void run_turn_start(void);
+extern void run_restored_turn_start(void);
+extern void run_turn_end(void);
 
 extern int gamestatesafe;
 
-static void add_new_sides_to_game PARAMS ((void));
-static void run_tech_leakage PARAMS ((void));
-static void cache_init_tech_levels PARAMS ((void));
-static void run_tooling_attrition PARAMS ((void));
-static void run_cp_attrition PARAMS ((void));
-static void reset_all_reserves PARAMS ((void));
-static void compute_moves PARAMS ((void));
-static void finish_movement PARAMS ((void));
-static void test_agreements PARAMS ((void));
-static void compute_sun PARAMS ((void));
-static void run_sun PARAMS ((void));
-static void compute_season PARAMS ((void));
-static void run_environment PARAMS ((void));
-static void mix_winds PARAMS ((void));
-static void all_see_cell_weather PARAMS ((int x, int y));
-static void run_spies PARAMS ((void));
-static void run_accidents PARAMS ((void));
-static void run_attrition PARAMS ((void));
-static void run_revolts PARAMS ((void));
-static void unit_revolt PARAMS ((Unit *unit, int force));
-static void run_surrenders PARAMS ((void));
-static void unit_surrender PARAMS ((Unit *unit));
-static void maybe_surrender_to PARAMS ((Unit *unit, Unit *unit2));
-static int excess_left PARAMS ((int x, int y));
-static void try_transfer_to_cell PARAMS ((int x, int y));
-static int sharable_left PARAMS ((int x, int y));
-static void try_sharing_with_cell PARAMS ((int x, int y));
-static void run_economy PARAMS ((void));
-static int base_production PARAMS ((Unit *unit, int m));
-static void try_transfer PARAMS ((Unit *from, Unit *to, int r));
-static void try_transfer_aux PARAMS ((Unit *from, Unit *to, int r));
-static int can_satisfy_need PARAMS ((Unit *unit, int r, int need));
-static void run_people_consumption PARAMS ((void));
-static void run_cell_consumption PARAMS ((void));
-static void run_unit_base_consumption PARAMS ((void));
-static void run_unit_starvation PARAMS ((void));
-static void unit_consumes PARAMS ((Unit *unit));
-static int in_supply PARAMS ((Unit *unit, int m));
-static void run_self_builds PARAMS ((void));
-static void run_environment_effects PARAMS ((void));
-static void damage_unit_with_temperature PARAMS ((Unit *unit, int n));
-static void run_people_side_changes PARAMS ((void));
-static void run_appearances PARAMS ((void));
-static void run_disappearances PARAMS ((void));
-static void run_hp_recovery PARAMS ((void));
-static void run_auto_repair PARAMS ((void));
-static void auto_repair_from_here PARAMS ((int x, int y));
-static void auto_repair_unit PARAMS ((Unit *unit, Unit *unit2));
-static void run_morale_recovery PARAMS ((void));
-static int season_effect PARAMS ((int u));
-static void run_unit_fates PARAMS ((void));
-static void run_detonation_accidents PARAMS ((void));
-static void run_people_limits PARAMS ((void));
-static void spy_on_location PARAMS ((int x, int y));
+static void add_new_sides_to_game(void);
+static void run_tech_leakage(void);
+static void cache_init_tech_levels(void);
+static void run_tooling_attrition(void);
+static void run_cp_attrition(void);
+static void reset_all_reserves(void);
+static void compute_moves(void);
+static void finish_movement(void);
+static void test_agreements(void);
+static void compute_sun(void);
+static void run_sun(void);
+static void compute_season(void);
+static void run_environment(void);
+static void mix_winds(void);
+static void all_see_cell_weather(int x, int y);
+static void run_spies(void);
+static void run_accidents(void);
+static void run_attrition(void);
+static void run_revolts(void);
+static void unit_revolt(Unit *unit, int force);
+static void run_surrenders(void);
+static void unit_surrender(Unit *unit);
+static void maybe_surrender_to(Unit *unit, Unit *unit2);
+static int excess_left(int x, int y);
+static int need_left(int x, int y);
+static void try_transfer_to_cell(int x, int y);
+static void try_sharing_with_cell(int x, int y);
+static void run_economy(void);
+static int base_production(Unit *unit, int m);
+static void try_transfer(Unit *from, Unit *to, int m);
+static void try_sharing_need(Unit *from, Unit *to, int m);
+static void try_transfer_aux(Unit *from, Unit *to, int m);
+static int can_satisfy_need(Unit *unit, int m, int need);
+static void run_people_consumption(void);
+static void run_cell_consumption(void);
+static void run_unit_base_consumption(void);
+static void run_unit_starvation(void);
+static int in_supply(Unit *unit, int m);
+static void run_self_builds(void);
+static void run_environment_effects(void);
+static void damage_unit_with_temperature(Unit *unit, int n);
+static void run_people_side_changes(void);
+static void run_appearances(void);
+static void run_disappearances(void);
+static void run_hp_recovery(void);
+static void run_auto_repair(void);
+static void auto_repair_from_here(int x, int y);
+static void auto_repair_unit(Unit *unit, Unit *unit2);
+static void run_morale_recovery(void);
+static int season_effect(int u);
+static void run_unit_fates(void);
+static void run_detonation_accidents(void);
+static void run_people_limits(void);
+static void spy_on_location(int x, int y);
 
 /* The number of the current turn within a year. */
 
@@ -194,7 +191,7 @@ static char *any_lost_surrenders;
 /* Do everything that would happen before movement in a turn. */
 
 void
-run_turn_start()
+run_turn_start(void)
 {
     int curturn;
     time_t turncalcstart, turncalcend;
@@ -204,7 +201,8 @@ run_turn_start()
     curturn = g_turn();
     ++curturn;
     set_g_turn(curturn);
-    /* Add any sides that were requested to be added during the previous turn. */
+    /* Add any sides that were requested to be added during the
+       previous turn. */
     add_new_sides_to_game();
     /* Warn players if the game is nearly over. */
     if (curturn >= g_last_turn()) {
@@ -231,14 +229,13 @@ run_turn_start()
     /* Do a variety of each-turn backdrop activities. */
     run_sun();
     run_environment();
-    run_economy();
-
-    /* The new supply system. */
-    run_supply();
-    
+    /* Optionally turn off standard economy in advanced game to avoid complications. */
+    if (g_disable_standard_economy() != TRUE) {
+	run_economy();
+	run_supply();
+    }
     /* Advanced unit support. */
     run_advanced_units();
-
     run_hp_recovery();
     run_auto_repair();
     run_self_builds();
@@ -638,8 +635,6 @@ compute_moves()
 
 		/* Advanced unit support. */
 
-// --- USE NEW UNIT PROPERTY TO TURN OFF STANDARD ACP ------------
-
 /* This keeps the option of letting advanced units do the standard acp calculation
 followed by action(s) in the action loop, in addition to the building and research 
 they do during run_advanced_units. However, disable_standard_acp should be set 
@@ -649,13 +644,7 @@ mode and once in the standard mode. */
 
 		if (u_disable_standard_acp(unit->type))
 			unit->act->initacp = unit->act->acp = 0;
-
-//			compute_acp_advanced_unit(unit);
-
-// ------------------------------------------------------
-
-		else	
-		  compute_acp(unit);
+		else   compute_acp(unit);
 
 		update_unit_acp_display(side, unit, FALSE);
 	    }
@@ -899,8 +888,8 @@ struct a_key_point {
     int temp;
 } *key_points;
 
-static void calc_key_point_temps PARAMS ((int yearpart));
-static int interpolate_temperature PARAMS ((int x, int y));
+static void calc_key_point_temps(int yearpart);
+static int interpolate_temperature(int x, int y);
 
 static void
 calc_key_point_temps(yearpart)
@@ -1166,7 +1155,8 @@ run_environment()
     }
 }
 
-vary_winds()
+static void
+vary_winds(void)
 {
     int x, y, t, wdir, wforce, newwforce, anychange;
 
@@ -1195,7 +1185,7 @@ vary_winds()
 }
 
 static void
-mix_winds()
+mix_winds(void)
 {
     int num, i, x, y, dir, x1, y1, wdir, wforce, sumx, sumy, n, t;
 
@@ -1712,6 +1702,7 @@ Unit *unit, *unit2;
 }
 
 int tmpexcess;
+int tmpneed;
 
 /* We've "found what we were searching for" when the excess to distribute
    is gone. */
@@ -1723,6 +1714,13 @@ int x, y;
     return (tmpexcess > 0);
 }
 
+static int
+need_left(x, y)
+int x, y;
+{
+    return (tmpneed > 0);
+}
+
 static void
 try_transfer_to_cell(x, y)
 int x, y;
@@ -1747,31 +1745,22 @@ int x, y;
     }
 }
 
-static int
-sharable_left(x, y)
-int x, y;
-{
-    return tmpunit->supply[tmpmtype] > (um_storage_x(tmpunit->type, tmpmtype) / 2);
-}
-
 static void
 try_sharing_with_cell(x, y)
 int x, y;
 {
     Unit *unit2, *occ;
 
-    if (!sharable_left(x, y))
-      return;
     for_all_stack(x, y, unit2) {
 	if (in_play(unit2) && unit2->side == tmpunit->side) {
-	    try_sharing(tmpunit, unit2, tmpmtype);
+	    try_sharing_need(unit2, tmpunit, tmpmtype);
 	}
     }
     for_all_stack(x, y, unit2) {
 	if (in_play(unit2) && unit2->side == tmpunit->side) {
 	    for_all_occupants(unit2, occ) {
 		if (in_play(occ) && occ->side == tmpunit->side) {
-		    try_sharing(tmpunit, occ, tmpmtype);
+		    try_sharing_need(occ, tmpunit, tmpmtype);
 		}
 	    }
 	}
@@ -1924,15 +1913,14 @@ run_economy()
 	    u = unit->type;
 	    for_all_material_types(m) {
 		stor = um_storage_x(u, m);
-		if (unit->supply[m] > stor) {
+		tmpexcess = unit->supply[m] - stor;
+		if (tmpexcess > 0) {
 		    dist = um_outlength(u, m);
 		    if (dist >= 0) {
 		    	tmpunit = unit;
 		    	tmpmtype = m;
-		    	tmpexcess = unit->supply[m] - stor;
 			search_and_apply(unit->x, unit->y, dist, excess_left,
-					 &x1, &y1, 1,
-					 try_transfer_to_cell, 999999);
+					 &x1, &y1, 1, try_transfer_to_cell, 0);
 		    }
 		}
 	    }
@@ -1970,25 +1958,31 @@ run_economy()
 	}
     }
     Dprintf("Discarded ");
-    Dprintf("  (for terrain)");
+    Dprintf("  (from terrain)");
     for_all_material_types(m)
       Dprintf(" %d", ttotals[m]);
-    Dprintf("  (for units)");
+    Dprintf("  (from units)");
     for_all_material_types(m)
       Dprintf(" %d", utotals[m]);
     Dprintf("\n");
-    /* This next phase is for sharing of scarcer supplies. */
+    /* This next phase is for sharing of scarcer supplies.  We scan
+       for units that are in need of supplies, then search for possible
+       suppliers. */
     for_all_units(unit) {
 	if (in_play(unit) && !indep(unit)) {
 	    u = unit->type;
 	    for_all_material_types(m) {
-		dist = um_outlength(u, m);
-		if (dist >= 0) {
-		    tmpunit = unit;
-		    tmpmtype = m;
-		    search_and_apply(unit->x, unit->y, dist, sharable_left,
-				     &x1, &y1, 1,
-				     try_sharing_with_cell, 999999);
+		/* (should use doctrine to set level) */
+		if (2 * unit->supply[m] < um_storage_x(u, m)) {
+		    dist = um_inlength(u, m);
+		    if (dist >= 0) {
+			tmpunit = unit;
+			tmpmtype = m;
+			tmpneed = um_storage_x(u, m) / 2 - unit->supply[m];
+			search_and_apply(unit->x, unit->y, dist, need_left,
+					 &x1, &y1, 1, try_sharing_with_cell,
+					 0);
+		    }
 		}
 	    }
 	}
@@ -2034,6 +2028,18 @@ int m;
     tmpexcess -= (oldsupply - from->supply[m]);
 }
 
+void
+try_sharing_need(from, to, m)
+Unit *from, *to;
+int m;
+{
+    int oldsupply = to->supply[m];
+
+    try_transfer_aux(from, to, m);
+    /* Decrement our need by whatever we picked up in this transfer. */
+    tmpneed -= (to->supply[m] - oldsupply);
+}
+
 /* Note that this may be called on newly-completed units during a turn. */
 
 void
@@ -2303,11 +2309,9 @@ int u, x, y;
 }
 
 void
-change_people_side_around(x, y, u, side)
-int x, y, u;
-Side *side;
+change_people_side_around(int x, int y, int u, Side *side)
 {
-    int pop = people_side_at(x, y), s = side_number(side), dir, x1, y1;
+    int pop = people_side_at(x, y), s = side_number(side);
     Side *oldside, *side2;
 
     if (pop != NOBODY
@@ -2678,9 +2682,7 @@ Unit *unit, *unit2;
 /* (should move to unit.c?) */
 
 void
-add_to_unit_hp(unit, hp)
-Unit *unit;
-int hp;
+add_to_unit_hp(Unit *unit, int hp)
 {
     int hpmax;
 
@@ -2696,9 +2698,9 @@ int hp;
 /* Some types of units recover lost morale spontaneously. */
 
 static void
-run_morale_recovery()
+run_morale_recovery(void)
 {
-    int u, moralerecovery, moralemax, oldmorale;
+    int u, moralerecovery, moralemax;
     Unit *unit;
 
     /* Precompute whether any units ever recover morale. */
@@ -2733,11 +2735,11 @@ run_morale_recovery()
    Not all units will necessarily disappear right away, so this runs
    at each turn. */
 
-static void lost_unit_surrender PARAMS ((Unit *unit));
-static void maybe_surrender_lost_to PARAMS ((Unit *unit, Unit *unit2));
+static void lost_unit_surrender(Unit *unit);
+static void maybe_surrender_lost_to(Unit *unit, Unit *unit2);
 
 static void
-run_unit_fates()
+run_unit_fates(void)
 {
     int u, u1, u2, chance, anylost;
     Side *side, *origside;
@@ -2925,11 +2927,11 @@ run_detonation_accidents()
    none that can't wait until the next turn. */
 
 static void
-finish_movement()
+finish_movement(void)
 {
     int lostacp;
     Unit *unit;
-    Side *side, *side2;
+    Side *side;
 
     for_all_sides(side) {
 	if (Debug) {
@@ -2953,7 +2955,7 @@ finish_movement()
 /* Handle consumption by people. */
 
 static void
-run_people_consumption()
+run_people_consumption(void)
 {
     int mm1, mm2, x, y, m1, m2, t, consum, oldamt, newamt, newtype;
 
@@ -3065,10 +3067,9 @@ run_cell_consumption()
 /* Handle base consumption by units. */
 
 static void
-run_unit_base_consumption()
+run_unit_base_consumption(void)
 {
-    int u, m, usedup, consump, tempeff, checkstarve;
-    Obj *effect;
+    int u, m, usedup, consump, checkstarve;
     Unit *unit;
 
     /* Precompute whether any base consumption ever happens. */
@@ -3215,13 +3216,13 @@ in_supply(unit, m)
 Unit *unit;
 int m;
 {
-    Unit *unit2;
-
     if (unit->transport != NULL) {
     	if (unit->transport->supply[m] > 0)
 	  return TRUE;
     }
 #if 0 /* how to make this work right? */
+    Unit *unit2;
+
     for_all_stack(unit->x, unit->y, unit2) {
     	if (unit2 != unit && unit2->supply[m] > 0)
 	  return TRUE;
