@@ -1,5 +1,5 @@
 /* Scores and scoring in Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -758,8 +758,7 @@ int why;
 	}
     }
     
-// --- ADVANCED UNITS (CITIES) SHOULD NOT JUST DISAPPEAR - MAKE THEM INDEPENDENT -----
-
+    /* Advanced units (cities) should not disappear - make them independent. */
     else {
 	for_all_units(unit) {
 	    if (unit->side == side && u_advanced(unit->type)) {
@@ -776,8 +775,6 @@ int why;
 	}
     }
     
-// ---------------------------------------------------------------------    
-    
     /* Add the mark of shame itself. */
     side->status = -1;
     remove_side_from_game(side);
@@ -800,7 +797,7 @@ all_sides_draw()
     	if (side->ingame) {
 	    side->status = 0;
 	    remove_side_from_game(side);
-	    add_side_to_set(side, drew);
+	    drew = add_side_to_set(side, drew);
 	}
     }
     /* Now that all sides are out, safe to record events to that effect
@@ -1037,11 +1034,10 @@ char *
 get_scores(side)
 Side *side;
 {
-    int wins, losses, draws, plays, allplays, i;
-    char *buf, buf2[BUFSIZE], sdadvbuf[20], varbuf[BUFSIZE];
+    int wins, losses, draws, plays, allplays;
+    char *buf, sdadvbuf[20], varbuf[BUFSIZE];
     char *thisgame, *playername, *sdnamestr, *sdfatestr;
-    Obj *sds, *sd, *sdname, *sdfate, *more, *more1, *restvarset, *varset;
-    Variant *variants, *tmpvar, *var;
+    Obj *sds, *sd, *sdname, *sdfate, *more, *more1;
     ScoreRecord *sr;
 
     thisgame = (mainmodule->origmodulename
@@ -1174,7 +1170,7 @@ score_variant_desc(sr, varbuf)
 ScoreRecord *sr;
 char *varbuf;
 {
-    int i, hasdflt, dfltval, num, val;
+    int i, hasdflt, dfltval, val;
     Obj *restvarset, *varset, *varsetname, *rslt;
     Variant *variants, *tmpvar, *var;
 
