@@ -12,18 +12,21 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 extern int gameinited;
+#include "ai.h"
 
 #define checked_elev_at(x, y) (elevations_defined() ? elev_at(x, y) : 0)
 
-static void init_visible_elevation PARAMS ((int x, int y));
-static void init_visible_elevation_2 PARAMS ((int x, int y));
-static void calc_visible_elevation PARAMS ((int x, int y));
-static void calc_visible_elevation_2 PARAMS ((int x, int y));
-static void cover_area_1 PARAMS ((Side *side, struct a_unit *unit,
-				  int x0, int y0, int x1, int y1));
-static int see_materials PARAMS ((Side *side, int x, int y));
-static int see_weather PARAMS ((Side *side, int x, int y));
-static int mistaken_type PARAMS ((int u2));
+extern Agreement *find_agreement(int id);
+
+static void init_visible_elevation(int x, int y);
+static void init_visible_elevation_2(int x, int y);
+static void calc_visible_elevation(int x, int y);
+static void calc_visible_elevation_2(int x, int y);
+static void cover_area_1(Side *side, struct a_unit *unit,
+				  int x0, int y0, int x1, int y1);
+static int see_materials(Side *side, int x, int y);
+static int see_weather(Side *side, int x, int y);
+static int mistaken_type(int u2);
 
 /* Head of the list of all sides. */
 
@@ -142,7 +145,7 @@ init_sides()
 Side *
 create_side()
 {
-    int u;
+    int u, m;
     Side *newside;
 
     if (numsides >= g_sides_max()) {
@@ -188,12 +191,16 @@ create_side()
     newside->losscounts = (short *) xmalloc(numutypes * num_loss_reasons * sizeof(short));
     newside->atkstats = (long **) xmalloc(numutypes * sizeof(long *));
     newside->hitstats = (long **) xmalloc(numutypes * sizeof(long *));
+    newside->prefixarg = -1;    /* Necessary to enable AI control toggling of selected units! */
     if (numatypes > 0) {
    	 newside->advance = (short *) xmalloc(numatypes * sizeof(short));
     	 newside->numadvances = 0;
     }
     if (nummtypes > 0) {
+	/* Set up the side's supply of materials. */
 	newside->treasury = (short *) xmalloc(nummtypes * sizeof(short));
+	for_all_material_types(m)
+	  newside->treasury[m] = m_initial_treasury(m);
     }
     /* Link in at the end of the list of sides. */
     newside->next = NULL;
@@ -500,7 +507,7 @@ Side *side, *side2;
 
 short *max_control_ranges;
 
-static int controller_here PARAMS ((int x, int y));
+static int controller_here(int x, int y);
 
 static int
 controller_here(x, y)
@@ -532,6 +539,9 @@ Unit *unit;
       return FALSE;
     if (is_designer(side))
       return TRUE;
+    /* Also give debuggers full control. */
+    if (Debug || DebugG || DebugM)
+      return TRUE;
     if (side_controls_side(side, unit->side)) {
 	/* The *unit* side must have the tech to use the unit, the controlling
 	   side would have to actually take over the unit if it wants to use
@@ -599,6 +609,18 @@ Unit *unit;
       return TRUE;
     if (side_controls_side(side, unit->side))
       return TRUE;
+    /* We may also examine units that trust us, */
+    if (trusted_side(unit->side, side))
+      return TRUE;
+    /* and every unit if we are debugging, */
+    if (Debug || DebugG || DebugM)
+      return TRUE;
+    /* or if enemy peeking is allowed, */
+    if (g_peek_at_enemy())
+      return TRUE;
+    /* or if the game is over. */
+    if (endofgame)
+      return TRUE;      
     return FALSE;
 }
 
@@ -2089,7 +2111,7 @@ int nx, ny;
     }
 }
 
-extern void compute_see_chances PARAMS ((void));
+extern void compute_see_chances(void);
 
 int any_see_chances = -1;
 
@@ -2153,14 +2175,13 @@ compute_see_chances()
     }
 }
 
-int test_for_viewer PARAMS ((int x, int y));
-int test_for_mistaken_viewer PARAMS ((int x, int y));
+int test_for_viewer(int x, int y);
+int test_for_mistaken_viewer(int x, int y);
 
 static Unit *tmpunittosee, *tmpunitseen, *tmpseer;
 
 int
-test_for_viewer(x, y)
-int x, y;
+test_for_viewer(int x, int y)
 {
     int u2 = tmpunittosee->type, x2 = tmpunittosee->x, y2 = tmpunittosee->y;
     int dist, chance;
@@ -2188,11 +2209,10 @@ int x, y;
 }
 
 int
-test_for_mistaken_viewer(x, y)
-int x, y;
+test_for_mistaken_viewer(int x, int y)
 {
-    int u2 = tmpunittosee->type, x2 = tmpunittosee->x, y2 = tmpunittosee->y;
-    int dist, chance;
+    int x2 = tmpunittosee->x, y2 = tmpunittosee->y;
+    int dist;
     Unit *unit;
 
     dist = distance(x, y, x2, y2);
@@ -2211,8 +2231,7 @@ int x, y;
    to many displays. */
 
 void
-all_see_cell(x, y)
-int x, y;
+all_see_cell(int x, int y)
 {
     register Side *side;
 
@@ -2729,7 +2748,7 @@ new_standing_order()
 /* Add a new standing order for the side.  This function can add to front
    or back of existing list of orders. */
 
-int order_conds_match PARAMS ((StandingOrder *sorder, StandingOrder *sorder2));
+int order_conds_match(StandingOrder *sorder, StandingOrder *sorder2);
 
 void
 add_standing_order(side, sorder, pos)
@@ -2985,19 +3004,21 @@ char *str, *buf, **rsltp;
 
 /* Agreement handling here for now. */
 
-Agreement *agreementlist = NULL;
+Agreement *agreement_list = NULL;
 
-Agreement *lastagreement;
+Agreement *last_agreement;
 
-int numagreements = 0;
+int num_agreements = 0;
 
-char *agreementdesigbuf;
+int next_agreement_id = 1;
+
+char *agreement_desig_buf;
 
 void
 init_agreements()
 {
-    agreementlist = lastagreement = NULL;
-    numagreements = 0;
+    agreement_list = last_agreement = NULL;
+    num_agreements = 0;
 }
 
 Agreement *
@@ -3006,29 +3027,45 @@ int id;
 {
     Agreement *ag = (Agreement *) xmalloc(sizeof(Agreement));
 
+    if (id == 0)
+      id = next_agreement_id++;
     ag->id = id;
     ag->terms = lispnil;
     ag->next = NULL;
-    if (agreementlist != NULL) {
-	lastagreement->next = ag;
+    if (agreement_list != NULL) {
+	last_agreement->next = ag;
     } else {
-	agreementlist = lastagreement = ag;
+	agreement_list = ag;
     }
-    ++numagreements;
+    last_agreement = ag;
+    ++num_agreements;
     return ag;
 }
 
+Agreement *
+find_agreement(id)
+int id;
+{
+    Agreement *ag;
+
+    for_all_agreements(ag) {
+	if (ag->id == id)
+	  return ag;
+    }
+    return NULL;
+}
+
 char *
 agreement_desig(ag)
 Agreement *ag;
 {
-    if (agreementdesigbuf == NULL)
-      agreementdesigbuf = xmalloc(BUFSIZE);
-    sprintf(agreementdesigbuf, "<ag %s %s %s>",
+    if (agreement_desig_buf == NULL)
+      agreement_desig_buf = xmalloc(BUFSIZE);
+    sprintf(agreement_desig_buf, "<ag %s %s %s>",
 	    (ag->typename ? ag->typename : "(null)"),
 	    (ag->name ? ag->name : "(null)"),
 	    (ag->terms == lispnil ? "(no terms)" : "...terms..."));
-    return agreementdesigbuf;
+    return agreement_desig_buf;
 }
 
 /* Helper functions to init view layers from rle encoding. */
@@ -3157,7 +3194,7 @@ int x, y, val;
 
 #ifdef DESIGNERS
 
-static void paint_view_1 PARAMS ((int x, int y));
+static void paint_view_1(int x, int y);
 
 static int tmptview;
 static int tmpuview;
