@@ -1,5 +1,5 @@
 /* Definitions for sides in Xconq.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,7 +14,8 @@ typedef int SideMask;
 #define NOSIDES (0)
 #define ALLSIDES (-1)
 #define add_side_to_set(side,mask) ((mask) | (1 << ((side) ? (side)->id : 0)))
-#define remove_side_from_set(side,mask) ((mask) & ~(1 << ((side) ? (side)->id : 0)))
+#define remove_side_from_set(side,mask) \
+  ((mask) & ~(1 << ((side) ? (side)->id : 0)))
 #define side_in_set(side,mask) ((mask) & (1 << ((side) ? (side)->id : 0)))
 #else
 not implemented yet
@@ -94,13 +95,8 @@ typedef struct a_side {
     StandingOrder *last_order;
     char *uorders;		/* bit vector of types that have orders */
     Obj *possible_units;	/* list of units to give to side */
-
-// --- NEW SIDE PROPERTY ---------------------------------------------
-
     short ai_may_resign;	/* true if AI may draw or resign on its own */
 
-// ----------------------------------------------------------------
-
     /* View layers. */
     char *terrview;		/* ptr to view of terrain */
     char **auxterrview;		/* ptr to view of aux terrain */
@@ -191,13 +187,7 @@ typedef struct a_side {
     short numadvances;		/* Number of completed advances */
     short *advance;		/* State of research (accum pts) for each */ 
     					/* advance. Set to -1 when completed. */ 
-
-// --- TREASURY SUPPORT --------------------------------------------------------
-
     short *treasury;		/* Globally available supply of each material type. */
-
-// --------------------------------------------------------------------------
-
     /* Links. */
     struct a_side *next;	/* pointer to next in list */
 } Side;
@@ -409,7 +399,8 @@ typedef struct a_side {
 #define units_visible(side, x, y)  \
   (all_see_all || (side)->see_all || (cover((side), wrapx(x), (y)) >= 1))
 
-#define has_advance(side,a) ((side)->advance[(a)] == -1)
+/* Handle crashes due to null indep side. */
+#define has_advance(side,a) (side != NULL ? ((side)->advance[(a)] == -1) : FALSE)
 
 #ifdef DESIGNERS
 #define is_designer(side) (side->designer)
@@ -460,9 +451,9 @@ enum {
 
 /* Iteration over all agreements in the game. */
 
-#define for_all_agreements(v) for (v = agreementlist; v != NULL; v = v->next)
+#define for_all_agreements(v) for (v = agreement_list; v != NULL; v = v->next)
 
-#define any_agreements() (agreementlist != NULL)
+#define any_agreements() (agreement_list != NULL)
 
 #define side_signed_agreement(side,ag) ((ag)->signers[side_n(side)])
 
@@ -472,10 +463,10 @@ enum {
 
 /* Agreement-related variables. */
 
-extern int numagreements;
+extern int num_agreements;
 
-extern Agreement *agreementlist;
-extern Agreement *lastagreement;
+extern Agreement *agreement_list;
+extern Agreement *last_agreement;
 
 /* Note: Can't use the "Unit" typedef below, must use "struct a_unit". */
 
