@@ -1,5 +1,5 @@
 /* A minimal interface to Xconq.
-   Copyright (C) 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,36 +19,41 @@ any later version.  See the file COPYING.  */
 #include "print.h"
 #include "cmdline.h"
 
-static void init_displays PARAMS ((void));
-static void get_input PARAMS ((void));
-static void interpret_command PARAMS ((Obj *cmd));
-static void list_one_unit PARAMS ((Unit *unit));
-static void interpret_help PARAMS ((Side *side, char *str));
-static int do_cmd PARAMS ((Side *side, Obj *cmd, Obj *parms));
+void unit_advance_dialog(Unit *unit);
+void global_advance_dialog(Side *side, Unit *unit, int s);
+int unit_build_dialog(Unit *unit);
+void unit_plan_dialog(Unit *unit);
+
+static void init_displays(void);
+static void get_input(void);
+static void interpret_command(Obj *cmd);
+static void list_one_unit(Unit *unit);
+static void interpret_help(Side *side, char *str);
+static int do_cmd(Side *side, Obj *cmd, Obj *parms);
 #ifdef DEBUGGING
-static void toggle_debug PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void toggle_debugg PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void toggle_debugm PARAMS ((Side *side, Obj *cmd, Obj *parms));
+static void toggle_debug(Side *side, Obj *cmd, Obj *parms);
+static void toggle_debugg(Side *side, Obj *cmd, Obj *parms);
+static void toggle_debugm(Side *side, Obj *cmd, Obj *parms);
 #endif /* DEBUGGING */
-static void list_sides PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void list_units PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void list_actors PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void list_cells PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_task_cmd PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_finish_turn PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_finish_all PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_repeat PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_multiple PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_free_run PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_save PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_eval PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_help PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_print PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_memory PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static void do_quit PARAMS ((Side *side, Obj *cmd, Obj *parms));
-static int do_action PARAMS ((Side *side, Unit *unit, Obj *cmd, Obj *args));
-static void show_help PARAMS ((Side *side, HelpNode *node));
-static void describe_commands PARAMS ((int arg, char *key, TextBuffer *buf));
+static void list_sides(Side *side, Obj *cmd, Obj *parms);
+static void list_units(Side *side, Obj *cmd, Obj *parms);
+static void list_actors(Side *side, Obj *cmd, Obj *parms);
+static void list_cells(Side *side, Obj *cmd, Obj *parms);
+static void do_task_cmd(Side *side, Obj *cmd, Obj *parms);
+static void do_finish_turn(Side *side, Obj *cmd, Obj *parms);
+static void do_finish_all(Side *side, Obj *cmd, Obj *parms);
+static void do_repeat(Side *side, Obj *cmd, Obj *parms);
+static void do_multiple(Side *side, Obj *cmd, Obj *parms);
+static void do_free_run(Side *side, Obj *cmd, Obj *parms);
+static void do_save(Side *side, Obj *cmd, Obj *parms);
+static void do_eval(Side *side, Obj *cmd, Obj *parms);
+static void do_help(Side *side, Obj *cmd, Obj *parms);
+static void do_print(Side *side, Obj *cmd, Obj *parms);
+static void do_memory(Side *side, Obj *cmd, Obj *parms);
+static void do_quit(Side *side, Obj *cmd, Obj *parms);
+static int do_action(Side *side, Unit *unit, Obj *cmd, Obj *args);
+static void show_help(Side *side, HelpNode *node);
+static void describe_commands(int arg, char *key, TextBuffer *buf);
 
 #ifdef THINK_C
 /* This is to get the command line reader in Think C on the Mac. */
@@ -82,9 +87,7 @@ int numusefulcellupdatesperturn = 0;
    loop interpreting input and running the simulation. */
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char *argv[])
 {
     extern long initrandstate, randstate;
     long currandstate;
@@ -102,15 +105,15 @@ char *argv[];
 #endif /* DEBUGGING */
     init_data_structures();
     /* Dump the random state so we can reproduce the run if necessary. */
-    printf("Random state is %d", randstate);
+    printf("Random state is %ld", randstate);
     if (initrandstate != randstate) {
-	printf(" (seed was %d)", initrandstate);
+	printf(" (seed was %ld)", initrandstate);
     }
     printf("\n");
     currandstate = randstate;
     parse_command_line(argc, argv, general_options);
     if (currandstate != randstate) {
-	printf("Random state is now %d.\n", randstate);
+	printf("Random state is now %ld.\n", randstate);
     }
     load_all_modules();
     /* See if we have something resembling a valid game.  A synth method might
@@ -145,13 +148,13 @@ char *argv[];
 	/* We stay in here until nothing more to do. */
 	run_local_ai(1, 20);
 	while (run_game(99) > 0) {
-	  run_local_ai(2, 20);
+	    run_local_ai(2, 20);
 	}
     }
 }
 
 Player *
-add_default_player()
+add_default_player(void)
 {
     Player *dflt = add_player();
 
@@ -164,7 +167,7 @@ add_default_player()
 /* (Note that skelconq may have multiple "displays" open.) */
 
 static void
-init_displays()
+init_displays(void)
 {
     Side *side;
 
@@ -179,8 +182,7 @@ init_displays()
 /* Create a user interface, but leave it turned off. */
 
 void
-init_ui(side)
-Side *side;
+init_ui(Side *side)
 {
     if (side_wants_display(side)) {
         side->ui = (UI *) xmalloc(sizeof(UI));
@@ -198,8 +200,7 @@ Side *side;
 /* This tests whether the side has a display and if it is in use. */
 
 int
-active_display(side)
-Side *side;
+active_display(Side *side)
 {
     return (side && side_has_display(side) && side->ui->active);
 }
@@ -207,7 +208,7 @@ Side *side;
 /* Input reading waits for a number of sides, possibly times out. */
 
 static void
-get_input()
+get_input(void)
 {
     int cmdlineno = 1, endlineno = 1;
     Obj *cmd;
@@ -241,8 +242,7 @@ Unit *thisunit;
 /* Do some simple command parsing, just enough to exercise the program. */
 
 static void
-interpret_command(origcmd)
-Obj *origcmd;
+interpret_command(Obj *origcmd)
 {
     char *str;
     Obj *cmd, *verb = lispnil, *parms = lispnil;
@@ -281,9 +281,7 @@ Obj *origcmd;
 /* Random commands. */
 
 static void
-toggle_debug(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+toggle_debug(Side *side, Obj *cmd, Obj *parms)
 {
     if (parms == lispnil) {
 	Debug = !Debug;
@@ -295,9 +293,7 @@ Obj *cmd, *parms;
 }
 
 static void
-toggle_debugm(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+toggle_debugm(Side *side, Obj *cmd, Obj *parms)
 {
     if (parms == lispnil) {
 	DebugM = !DebugM;
@@ -309,9 +305,7 @@ Obj *cmd, *parms;
 }
 
 static void
-toggle_debugg(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+toggle_debugg(Side *side, Obj *cmd, Obj *parms)
 {
     if (parms == lispnil) {
 	DebugG = !DebugG;
@@ -323,16 +317,15 @@ Obj *cmd, *parms;
 }
 
 static void
-list_sides(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+list_sides(Side *side, Obj *cmd, Obj *parms)
 {
     int u;
     Side *side2;
     Agreement *ag;
-    extern int numagreements;
+    extern int num_agreements;
 
-    printf("Sides: (%s)\n", (g_use_side_priority() ? "sequential" : "simultaneous"));
+    printf("Sides: (%s)\n",
+	   (g_use_side_priority() ? "sequential" : "simultaneous"));
     for_all_sides(side2) {
 	printf("%s played by %s",
 	       side_desig(side2), player_desig(side2->player));
@@ -353,7 +346,7 @@ Obj *cmd, *parms;
 	}
 	/* (should say something about mplayer goals here) */
     }
-    if (numagreements > 0) {
+    if (num_agreements > 0) {
 	printf("Agreements:\n");
 	for_all_agreements(ag) {
 	    printf("%s\n", agreement_desig(ag));
@@ -362,9 +355,7 @@ Obj *cmd, *parms;
 }
 
 static void
-list_units(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+list_units(Side *side, Obj *cmd, Obj *parms)
 {
     Unit *unit;
 
@@ -375,9 +366,7 @@ Obj *cmd, *parms;
 }
 
 static void
-list_actors(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+list_actors(Side *side, Obj *cmd, Obj *parms)
 {
     int i;
     Side *side2;
@@ -398,8 +387,7 @@ Obj *cmd, *parms;
 }
 
 static void
-list_one_unit(unit)
-Unit *unit;
+list_one_unit(Unit *unit)
 {
     char status[BUFSIZE];
 
@@ -420,9 +408,7 @@ Unit *unit;
 }
 
 static void
-list_cells(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+list_cells(Side *side, Obj *cmd, Obj *parms)
 {
     int x = c_number(car(parms)), y = c_number(cadr(parms));
 
@@ -440,9 +426,7 @@ Obj *cmd, *parms;
 }
 
 static void
-do_task_cmd(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_task_cmd(Side *side, Obj *cmd, Obj *parms)
 {
     int i, j, numargs;
     char *taskname;
@@ -475,17 +459,13 @@ Obj *cmd, *parms;
 }
 
 static void
-do_finish_turn(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_finish_turn(Side *side, Obj *cmd, Obj *parms)
 {
     net_finish_turn(side ? side : defaultside);
 }
 
 static void
-do_finish_all(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_finish_all(Side *side, Obj *cmd, Obj *parms)
 {
     Side *side2;
 
@@ -495,9 +475,7 @@ Obj *cmd, *parms;
 }
 
 static void
-do_free_run(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_free_run(Side *side, Obj *cmd, Obj *parms)
 {
     Side *side2;
 
@@ -512,25 +490,19 @@ Obj *cmd, *parms;
 }
 
 static void
-do_repeat(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_repeat(Side *side, Obj *cmd, Obj *parms)
 {
 }
 
 static void
-do_multiple(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_multiple(Side *side, Obj *cmd, Obj *parms)
 {
     repetition = c_number(car(parms));
     multicmd = cdr(parms);
 }
 
 static void
-do_save(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_save(Side *side, Obj *cmd, Obj *parms)
 {
     if (!write_entire_game_state(saved_game_filename())) {
 	fprintf(stderr, "Save failed.\n");
@@ -538,26 +510,20 @@ Obj *cmd, *parms;
 }
 
 static void
-do_eval(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_eval(Side *side, Obj *cmd, Obj *parms)
 {
     interp_form(NULL, car(parms));
 }
 
 static void
-do_help(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_help(Side *side, Obj *cmd, Obj *parms)
 {
     printf("To look at help topics, type \"?<letter>\",\n");
     printf("where 'n' and 'p' go to next and previous nodes\n");
 }
 
 static void
-do_print(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_print(Side *side, Obj *cmd, Obj *parms)
 {
     char *fname;
     FILE *fp;
@@ -580,9 +546,7 @@ Obj *cmd, *parms;
 }
 
 static void
-do_memory(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_memory(Side *side, Obj *cmd, Obj *parms)
 {
     extern int grandtotmalloc;
 
@@ -592,9 +556,7 @@ Obj *cmd, *parms;
 /* Exit immediately, no questions asked. */
 
 static void
-do_quit(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_quit(Side *side, Obj *cmd, Obj *parms)
 {
     /* A quit command of the form "(quit if-end)" only actually quits if the
        game is over. */
@@ -608,7 +570,7 @@ Obj *cmd, *parms;
 
 struct a_cmd {
     char *cmd;
-    void (*fn) PARAMS ((Side *side, Obj *cmd, Obj *parms));
+    void (*fn)(Side *side, Obj *cmd, Obj *parms);
 } cmdtable[] = {
     { "debug", toggle_debug },
     { "debugm", toggle_debugm },
@@ -635,9 +597,7 @@ struct a_cmd {
 /* Try to find and execute an arbitrary command. */
 
 static int
-do_cmd(side, cmd, parms)
-Side *side;
-Obj *cmd, *parms;
+do_cmd(Side *side, Obj *cmd, Obj *parms)
 {
     struct a_cmd *cmdentry = cmdtable;
     char *cmdstr;
@@ -656,10 +616,7 @@ Obj *cmd, *parms;
 }
 
 static int
-do_action(side, unit, cmd, args)
-Side *side;
-Unit *unit;
-Obj *cmd, *args;
+do_action(Side *side, Unit *unit, Obj *cmd, Obj *args)
 {
     int randomact = FALSE;
     ActionDefn *actdefn = actiondefns;
@@ -741,7 +698,7 @@ Obj *cmd, *args;
 /* Shut down displays - should be done before any sort of exit. */
 
 void
-close_displays()
+close_displays(void)
 {
     Side *side;
 
@@ -754,22 +711,20 @@ close_displays()
 }
 
 void
-low_notify(side, str)
-Side *side;
-char *str;
+low_notify(Side *side, char *str)
 {
     printf("To %s: %s\n", side_desig(side), str);
 }
 
-update_area_display() {}
+void
+update_area_display(Side *side)
+{
+}
 
 /* (should count # of calls to inactive and non-displayed side...) */
 
 void
-update_cell_display(side, x, y, flags)
-Side *side;
-int x, y;
-int flags;
+update_cell_display(Side *side, int x, int y, int flags)
 {
     ++numcellupdatesperturn;
     if (active_display(side)) {
@@ -783,9 +738,7 @@ int flags;
    other global info, as needed. */
 
 void
-update_turn_display(side, rightnow)
-Side *side;
-int rightnow;
+update_turn_display(Side *side, int rightnow)
 {
     long secs;
     time_t xxx;
@@ -816,9 +769,7 @@ int rightnow;
 }
 
 void
-update_action_display(side, rightnow)
-Side *side;
-int rightnow;
+update_action_display(Side *side, int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: ready to act\n", side_desig(side));
@@ -826,10 +777,7 @@ int rightnow;
 }
 
 void
-update_action_result_display(side, unit, rslt, rightnow)
-Side *side;
-Unit *unit;
-int rslt, rightnow;
+update_action_result_display(Side *side, Unit *unit, int rslt, int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: %s action result is %s\n",
@@ -840,10 +788,7 @@ int rslt, rightnow;
 /* This is for animation of fire-at actions. */
 
 void
-update_fire_at_display(side, unit, unit2, m, rightnow)
-Side *side;
-Unit *unit, *unit2;
-int m, rightnow;
+update_fire_at_display(Side *side, Unit *unit, Unit *unit2, int m, int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: %s fire at %s\n",
@@ -854,10 +799,8 @@ int m, rightnow;
 /* This is for animation of fire-into actions. */
 
 void
-update_fire_into_display(side, unit, x, y, z, m, rightnow)
-Side *side;
-Unit *unit;
-int x, y, z, m, rightnow;
+update_fire_into_display(Side *side, Unit *unit, int x, int y, int z, int m,
+			 int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: %s fire into %d,%d\n",
@@ -866,10 +809,7 @@ int x, y, z, m, rightnow;
 }
 
 void
-update_event_display(side, hevt, rightnow)
-Side *side;
-HistEvent *hevt;
-int rightnow;
+update_event_display(Side *side, HistEvent *hevt, int rightnow)
 {
     if (active_display(side)) {
 	switch (hevt->type) {
@@ -888,9 +828,7 @@ int rightnow;
 }
 
 void
-update_all_progress_displays(str, s)
-char *str;
-int s;
+update_all_progress_displays(char *str, int s)
 {
     if (DebugG) {
 	printf("Update all progress displays\n");
@@ -901,9 +839,7 @@ int s;
    who it belongs to. */
 
 void
-update_side_display(side, side2, rightnow)
-Side *side, *side2;
-int rightnow;
+update_side_display(Side *side, Side *side2, int rightnow)
 {
     char *side2desc = copy_string(side_desig(side2));
 
@@ -917,10 +853,7 @@ int rightnow;
    who it belongs to. */
 
 void
-update_unit_display(side, unit, rightnow)
-Side *side;
-Unit *unit;
-int rightnow;
+update_unit_display(Side *side, Unit *unit, int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: unit %s%s\n",
@@ -929,10 +862,7 @@ int rightnow;
 }
 
 void
-update_unit_acp_display(side, unit, rightnow)
-Side *side;
-Unit *unit;
-int rightnow;
+update_unit_acp_display(Side *side, Unit *unit, int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: unit %s acp%s\n",
@@ -944,9 +874,7 @@ int rightnow;
    have any realtime constraints, this will never be called. */
 
 void
-update_clock_display(side, rightnow)
-Side *side;
-int rightnow;
+update_clock_display(Side *side, int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: %d secs this turn, %d total\n",
@@ -956,10 +884,7 @@ int rightnow;
 }
 
 void
-update_message_display(side, sender, str, rightnow)
-Side *side, *sender;
-char *str;
-int rightnow;
+update_message_display(Side *side, Side *sender, char *str, int rightnow)
 {
     if (active_display(side) && DebugG) {
 	printf("Update %s: side %d sends \"%s\"%s\n",
@@ -968,15 +893,13 @@ int rightnow;
 }
 
 void
-update_everything()
+update_everything(void)
 {
     printf("Update everything!\n");
 }
 
 void
-action_point(side, x, y)
-Side *side;
-int x, y;
+action_point(Side *side, int x, int y)
 {
 }
 
@@ -984,10 +907,7 @@ int x, y;
 /* Generate a description of all the user input that is possible. */
 
 static void
-describe_commands(arg, key, buf)
-int arg;
-char *key;
-TextBuffer *buf;
+describe_commands(int arg, char *key, TextBuffer *buf)
 {
     struct a_cmd *cmdentry;
 
@@ -998,9 +918,7 @@ TextBuffer *buf;
 }
 
 static void
-interpret_help(side, topic)
-Side *side;
-char *topic;
+interpret_help(Side *side, char *topic)
 {
     HelpNode *node;
 
@@ -1038,9 +956,7 @@ char *topic;
 /* Spew out the entire text of the current help node. */
 
 static void
-show_help(side, helpnode)
-Side *side;
-HelpNode *helpnode;
+show_help(Side *side, HelpNode *helpnode)
 {
     int linelen, skipchar;
     char *linebegin = get_help_text(helpnode), *lineend;
@@ -1064,7 +980,7 @@ HelpNode *helpnode;
 /* This reports progress in reading GDL files. */
 
 void
-announce_read_progress()
+announce_read_progress(void)
 {
 }
 
@@ -1073,8 +989,7 @@ announce_read_progress()
 int linemiddle = FALSE;
 
 void
-announce_lengthy_process(msg)
-char *msg;
+announce_lengthy_process(char *msg)
 {
     printf("%s; ", msg);
     fflush(stdout);
@@ -1084,8 +999,7 @@ char *msg;
 /* Update the progress, expressing it as a percentage done. */
 
 void
-announce_progress(percentdone)
-int percentdone;
+announce_progress(int percentdone)
 {
     printf(" %d%%", percentdone);
     fflush(stdout);
@@ -1095,33 +1009,25 @@ int percentdone;
 /* Announce the end of the lengthy process. */
 
 void
-finish_lengthy_process()
+finish_lengthy_process(void)
 {
     printf(" done.\n");
     linemiddle = FALSE;
 }
 
 int
-#ifdef __STDC__
 schedule_movie(Side *side, char *movie, ...)
-#else
-schedule_movie(side, movie)
-Side *side;
-char *movie;
-#endif
 {
     return FALSE;
 }
 
 void
-play_movies(sidemask)
-SideMask sidemask;
+play_movies(SideMask sidemask)
 {
 }
 
 void
-flush_display_buffers(side)
-Side *side;
+flush_display_buffers(Side *side)
 {
     if (active_display(side) && DebugG) {
 	printf("To %s: flush display buffers\n", side_desig(side));
@@ -1131,8 +1037,7 @@ Side *side;
 /* An init error needs to have the command re-run. */
 
 void
-low_init_error(str)
-char *str;
+low_init_error(char *str)
 {
     if (linemiddle)
       printf("\n");
@@ -1143,8 +1048,7 @@ char *str;
 /* A warning just gets displayed, no other action is taken. */
 
 void
-low_init_warning(str)
-char *str;
+low_init_warning(char *str)
 {
     if (linemiddle)
       printf("\n");
@@ -1155,8 +1059,7 @@ char *str;
 /* A run error is fatal. */
 
 void
-low_run_error(str)
-char *str;
+low_run_error(char *str)
 {
     if (linemiddle)
       fprintf(stderr, "\n");
@@ -1169,8 +1072,7 @@ char *str;
    usually a problem with Xconq or a game design. */
 
 void
-low_run_warning(str)
-char *str;
+low_run_warning(char *str)
 {
     if (linemiddle)
       printf("\n");
@@ -1179,31 +1081,29 @@ char *str;
 }
 
 void
-print_form(form)
-Obj *form;
+print_form(Obj *form)
 {
     print_form_and_value(stdout, form);
 }
 
 void
-end_printing_forms()
+end_printing_forms(void)
 {
 }
 
 /* Note that the fake declaration below will cause problems if imf.h
    is included. */
 
+void make_generic_image_data(char *imf);
 void
-make_generic_image_data(imf)
-char *imf;
+make_generic_image_data(char *imf)
 {
 }
 
 /* Networking functions (dummies for now). */
 
 void
-add_remote_player(name)
-char *name;
+add_remote_player(char *name)
 {
 }
 
@@ -1215,8 +1115,7 @@ launch_game()
 }
 
 void
-set_you_player(id)
-int id;
+set_you_player(int id)
 {
 }
 
@@ -1234,7 +1133,31 @@ serial_port_dialog()
 
 #endif /* MAC */
 
+extern void auto_pick_new_research(Unit *unit);
+extern int auto_pick_new_build_task(Unit *unit);
+extern void auto_pick_new_plan(Unit *unit);
 
-unit_advance_dialog(Unit *unit) { auto_pick_new_research(unit); }
-global_advance_dialog(Side *side, Unit *unit, int s) { auto_pick_new_research(unit); }
-unit_build_dialog(Unit *unit, Task *task) { return 0; }
+void
+unit_advance_dialog(Unit *unit)
+{
+    auto_pick_new_research(unit);
+}
+
+void
+global_advance_dialog(Side *side, Unit *unit, int s)
+{
+    auto_pick_new_research(unit);
+}
+
+int
+unit_build_dialog(Unit *unit)
+{
+    int u = auto_pick_new_build_task(unit);
+    return u;
+}
+
+void
+unit_plan_dialog(Unit *unit)
+{
+    auto_pick_new_plan(unit);
+}
