@@ -7,11 +7,11 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int need_ai_task_reaction;
-extern int attack_can_damage_or_capture PARAMS ((Unit *unit, Unit *unit2));
-extern int target_visible PARAMS ((Unit *unit, Task *task));
 #include "kernel.h"
 
+extern int need_ai_task_reaction;
+extern int target_visible PARAMS ((Unit *unit, Task *task));
+
 /* This is the number of tasks to allocate initially.  More will always be
    allocated as needed, so this should be a "reasonable" value. */
 
@@ -167,10 +167,10 @@ Task *task;
     Unit *unit2 = NULL;
     Side *us = unit->side;
 
-    /* First see if we've already built all the units requested. */
     if (task->args[2] >= run) {
         return TASK_IS_COMPLETE;
     }
+
     /* See if our technology needs improvement in order to build this type. */
     if (is_unit_type(u2)
 	&& u_tech_to_build(u2) > 0
@@ -183,6 +183,7 @@ Task *task;
 	    return TASK_FAILED;
         }
     }
+
     /* See if we need to toolup to work on this type. */
     if (is_unit_type(u2)) {
 	tp = (unit->tooling ? unit->tooling[u2] : 0);
@@ -337,7 +338,7 @@ do_capture_task(unit, task)
 Unit *unit;
 Task *task;
 {
-    int u = unit->type, tx, ty, tu2, ts2, dist, movedist, rslt;
+    int u = unit->type, tx, ty, tu2, ts2, dist;
     Unit *unit2;
     Side *us = unit->side;
 
@@ -455,7 +456,7 @@ do_hit_unit_task(unit, task)
 Unit *unit;
 Task *task;
 {
-    int u = unit->type, tx, ty, dist, movedist, enemythere, uview, tu, ts;
+    int u = unit->type, tx, ty, dist, movedist, tu, ts;
     Unit *unit2;
     Side *us = unit->side;
 
@@ -487,7 +488,9 @@ Task *task;
 	    	/* (should check designated side to hit, if defined) */
 	    	) {
 		if (valid(check_fire_at_action(unit, unit, unit2, -1))
-		    && fire_can_damage(unit, unit2)) {
+		    /* Also count with capture when firing at
+                       pointblank range. */
+		    && fire_can_damage_or_capture(unit, unit2)) {
 		    prep_fire_at_action(unit, unit, unit2, -1);
 		    return TASK_PREPPED_ACTION;
 		}
@@ -500,20 +503,54 @@ Task *task;
 	/* should move further away */
 	return TASK_FAILED;
     }
-    if (dist < u_range(u)) {
+    /* This is why units failed to fire at max range. */
+    if (dist <= u_range(u)) {
 	if (units_visible(us, tx, ty)) {
-	  for_all_stack(tx, ty, unit2) {
-	    if (!trusted_side(us, unit2->side)
-	    	&& (tu == NONUTYPE || unit2->type == tu)
-	    	/* (should check designated side to hit, if defined) */
-	    	) {
-		if (valid(check_fire_at_action(unit, unit, unit2, -1))
-		    && fire_can_damage(unit, unit2)) {
-		    prep_fire_at_action(unit, unit, unit2, -1);
-		    return TASK_PREPPED_ACTION;
+	    /* Check if we may safely move in and attempt to capture a unit. */
+	    int danger = FALSE;
+
+	    for_all_stack_with_occs(tx, ty, unit2) {
+		int u2 = unit2->type;
+		
+		if (trusted_side(us, unit2->side))
+		  continue;
+		if (!is_active(unit2))
+		  continue;
+		/* No cheating! */
+		if (!unit_actually_visible(unit->side, unit2))
+		  continue;
+		if (u_acp(u2) + u_acp_to_fire(u2) <= 0)
+		  continue;
+		if (uu_hit(u2, u) <= 0)
+		  continue;
+		if (uu_damage(u2, u) <= 0)
+		  continue;
+		/* Someone over there can still hit and damage us. */
+		danger = TRUE;
+		break;
+	    }
+	    if (!danger) {
+		for_all_stack(tx, ty, unit2) {				
+		    if (!trusted_side(us, unit2->side)
+			&& (tu == NONUTYPE || unit2->type == tu)
+			&& real_capture_chance(unit, unit2) > 0) {
+			push_move_to_task(unit, tx, ty, dist);
+			return TASK_IS_INCOMPLETE;
+		    }
+		}
+	    }
+	    for_all_stack(tx, ty, unit2) {
+		if (!trusted_side(us, unit2->side)
+		    && (tu == NONUTYPE || unit2->type == tu)
+		    /* (should check designated side to hit, if defined) */
+		    ) {
+		    if (valid(check_fire_at_action(unit, unit, unit2, -1))
+			&& fire_can_damage(unit, unit2)) {
+			prep_fire_at_action(unit, unit, unit2, -1);
+			return TASK_PREPPED_ACTION;
+		    }
 		}
 	    }
-	  }
 	}
 	return TASK_FAILED;	    
     }
@@ -544,6 +581,8 @@ int
 attack_can_damage_or_capture(unit, unit2)
 Unit *unit, *unit2;
 {
+    Unit *occ;
+
     if (!alive(unit) || !alive(unit2))
       return FALSE;
     if (capture_chance(unit->type, unit2->type, unit2->side) > 0)
@@ -551,11 +590,38 @@ Unit *unit, *unit2;
     /* (should check for right kind of ammo) */
     if (uu_hit(unit->type, unit2->type) <= 0)
       return FALSE;
-    /* (this is dubious - a no-damage attack could still consume acp) */
-    if (uu_damage(unit->type, unit2->type) <= 0)
-      return FALSE;
-    /* (should check if victim hp might be under damage low bounds) */
-    return TRUE;
+    /* Also take into account if we can damage any occs. Differs from
+       attack code in that only visible occs are considered. */
+    /* Return TRUE if we can damage the unit itself. */
+    if (uu_damage(unit->type, unit2->type) > 0)
+      return TRUE;
+    /* Else check if we can hit and damage any of its occs. */
+    for_all_occs_with_occs(unit2, occ) {
+	int immune = FALSE;
+	Unit *transport = occ->transport;
+
+	/* Skip occs that we can't hit or damage. */
+	if (uu_hit(unit->type, occ->type <= 0)
+	    || uu_damage(unit->type, occ->type <= 0))
+	  continue;
+	/* Skip occs that we cannot see (no cheating - not even for the AI!) */
+	if (!unit_actually_visible(unit->side, occ))
+	  continue;		    	
+	/* For vulnerable occs, check recursively that we also can hit
+	   all transports in the chain leading down to the occ. */
+	while (transport) {
+	    if (uu_hit(unit->type, transport->type) <= 0) {
+		immune = TRUE;
+		break;
+	    }
+	    transport = transport->transport;
+	}
+	/* If we found an occ that is not immune we are done. */
+	if (!immune)
+	  return TRUE;
+    }    	
+    /* We can't capture or hit anything. */
+    return FALSE;
 }
 
 /* Return true if the unit can actually damage the other unit. */
@@ -564,24 +630,155 @@ int
 fire_can_damage(unit, unit2)
 Unit *unit, *unit2;
 {
+    Unit *occ;
+
     if (!alive(unit) || !alive(unit2))
       return FALSE;
     /* (should check for right kind of ammo) */
     if (uu_hit(unit->type, unit2->type) <= 0)
       return FALSE;
-    /* (this is dubious - a no-damage attack could still consume acp) */
-    if (uu_damage(unit->type, unit2->type) <= 0)
-      return FALSE;
-    /* (should check if victim hp might be under damage low bounds) */
-    return TRUE;
+    /* Also take into account if we can damage any occs. Differs from fire code
+    in that only visible occs are considered. */
+    /* Return TRUE if we can damage the unit itself. */
+    if (uu_damage(unit->type, unit2->type) > 0)
+	return TRUE;
+    /* Else check if we can hit and damage any of its occs. */
+    for_all_occs_with_occs(unit2, occ) {
+
+	int 	immune = FALSE;
+	Unit	*transport = occ->transport;
+
+	/* Skip occs that we can't hit or damage. */
+	if (uu_hit(unit->type, occ->type <= 0)
+	    || uu_damage(unit->type, occ->type <= 0))
+	    	continue;
+	/* Skip occs that we cannot see (no cheating - not even for the AI!) */
+	if (!unit_actually_visible(unit->side, occ))
+		continue;		    	
+	/* For vulnerable occs, check recursively that we also can 
+	hit all transports in the chain leading down to the occ. */
+	while (transport) {
+		if (uu_hit(unit->type, transport->type) <= 0) {
+			immune = TRUE;
+			break;
+		}
+		transport = transport->transport;
+	}
+	/* If we found an occ that is not immune we are done. */
+	if (!immune)
+		return TRUE;
+    }    	
+    /* We can't damage anything. */
+    return FALSE;
 }
 
 int
-target_visible(unit, task)
-Unit *unit;
-Task *task;
+fire_can_damage_or_capture(Unit *unit, Unit *unit2)
 {
-    int u = unit->type, tx, ty, uview, tu, ts;
+    Unit *occ;
+
+    if (!alive(unit) || !alive(unit2))
+	return FALSE;
+    /* Always return TRUE if we can capture the unit. */
+    if (capture_chance(unit->type, unit2->type, unit2->side) > 0)
+	return TRUE;
+    /* If we can't hit the unit we can't hit any of its occs. */
+    if (uu_hit(unit->type, unit2->type) <= 0)
+	return FALSE;
+    /* Return TRUE if we can damage the unit itself. */
+    if (uu_damage(unit->type, unit2->type) > 0)
+      return TRUE;
+    /* Else check if we can hit and damage any of its occs. */
+    for_all_occs_with_occs(unit2, occ) {
+	int immune = FALSE;
+	Unit *transport = occ->transport;
+
+	/* Skip occs that we can't hit or damage. */
+	if (uu_hit(unit->type, occ->type <= 0)
+	    || uu_damage(unit->type, occ->type <= 0))
+	  continue;
+	/* Skip occs that we cannot see (no cheating - not even for the AI!) */
+	if (!unit_actually_visible(unit->side, occ))
+	  continue;		    	
+	/* For vulnerable occs, check recursively that we also can 
+	hit all transports in the chain leading down to the occ. */
+	while (transport) {
+	    if (uu_hit(unit->type, transport->type) <= 0) {
+		immune = TRUE;
+		break;
+	    }
+	    transport = transport->transport;
+	}
+	/* If we found an occ that is not immune we are done. */
+	if (!immune)
+	  return TRUE;
+    }    	
+    /* We can't capture or hit anything. */
+    return FALSE;
+}
+
+/* Probability that one real unit will capture another real unit with
+   protection taken into account.  Differs from real capture code in
+   that only visible occupants are considered (no cheating!) */
+
+int
+real_capture_chance(Unit *unit, Unit *unit2)
+{
+    int chance, prot;
+    int u2 = unit2->type;
+    int u = unit->type;
+    Unit *unit3;
+
+    chance = capture_chance(unit->type, unit2->type, unit2->side);
+
+    /* Occupants can protect the unit. */
+    for_all_occupants(unit2, unit3) {
+	if (is_active(unit3)
+	    && unit_actually_visible(unit->side, unit3)) {
+	    prot = uu_protection(unit3->type, u2);
+	    if (prot != 100)
+	      chance = (chance * prot) / 100;
+	}
+    }
+    /* And so can its neighbours. */
+    for_all_stack(unit2->x, unit2->y, unit3) {
+	if (unit3 != unit2
+	    && is_active(unit3)
+	    && unit_actually_visible(unit->side, unit3)
+	    && unit3->side == unit2->side) {
+	    prot = uu_stack_protection(unit3->type, u2);
+	    if (prot != 100)
+	      chance = (chance * prot) / 100;
+	}
+    }
+    for_all_stack_with_occs(unit2->x, unit2->y, unit3) {
+	if (is_active(unit3)
+	    && unit_actually_visible(unit->side, unit3)
+	    /* We also extend protection to our buddies! */
+	    && trusted_side(unit3->side, unit2->side)) {
+	    /* This is when a unit, such as triple-A, provides unique
+	       protection against a specific attacker, such as
+	       bombers, to all other units in the cell. */
+	    prot = uu_cellwide_protection_against(unit3->type, u);
+	    if (prot != 100)
+	      chance = (chance * prot) / 100;
+	    /* This is when a unit (such as a garrison) specifically
+	       protects a second unit, such as a fort (but not other
+	       nearby units), against all forms of attack. It thus
+	       works the same way as uu_protection and
+	       uu_stack_protection. */
+	    prot = uu_cellwide_protection_for(unit3->type, u2);	    
+	    if (prot != 100)
+	      chance = (chance * prot) / 100;
+	}
+    }
+    return chance;
+}
+
+int
+target_visible(Unit *unit, Task *task)
+{
+    int tx, ty, uview, tu, ts;
     Unit *unit2;
     Side *us = unit->side;
 
@@ -1013,18 +1210,18 @@ Task *task;
     return TASK_FAILED;
 }
 
+/* The collection task runs a series of extraction actions,
+   transferring the collected material to another, given, unit. */
+
 static int tmpm;
 
-Unit *collect_here PARAMS ((int x, int y));
-Unit *aux_collect_here PARAMS ((Unit *unit));
+Unit *collect_here(int x, int y);
+Unit *aux_collect_here(Unit *unit);
 
 static TaskOutcome
-do_collect_task(unit, task)
-Unit *unit;
-Task *task;
+do_collect_task(Unit *unit, Task *task)
 {
-    int m, tot, sofar, amt;
-    int x, y, dir, x1, y1, x2, y2, range;
+    int m, x, y, dir, x1, y1, x2, y2, range;
     Unit *collector, *unit3;
 
     m = task->args[0];
@@ -1052,8 +1249,9 @@ Task *task;
 	/* Compute how far out to look for a delivery point; be a little
 	   optimistic. */
 	range = operating_range_best(unit->type);
-	if (search_around(unit->x, unit->y, range, collect_test, &x2, &y2, 1)) {
-    	    /* (should collect actual unit and chase it directly) */
+	if (search_around(unit->x, unit->y, range, collect_test,
+			  &x2, &y2, 1)) {
+    	    /* (should find actual unit and chase it directly) */
 	    push_move_to_task(unit, x2, y2, 1);
 	    return TASK_IS_INCOMPLETE;
 	} else {
@@ -1095,15 +1293,13 @@ Task *task;
 }
 
 static int
-collect_test(x, y)
-int x, y;
+collect_test(int x, int y)
 {
     return (collect_here(x, y) != NULL);
 }
 
 Unit *
-collect_here(x, y)
-int x, y;
+collect_here(int x, int y)
 {
     Unit *unit, *collector;
 
@@ -1116,17 +1312,16 @@ int x, y;
 }
 
 Unit *
-aux_collect_here(unit)
-Unit *unit;
+aux_collect_here(Unit *unit)
 {
-    int i, enough = TRUE;
     Unit *occ;
 
     /* what about allies? */
     if (unit != tmpunit && unit_trusts_unit(unit, tmpunit)) {
 	if ((unit->supply[tmpm] < um_storage_x(unit->type, tmpm)
 	     || um_gives_to_treasury(unit->type, tmpm))
-	    /* this is a hack to prevent foragers from trying to deliver to each other */
+	    /* this is a hack to prevent foragers from trying to
+               deliver to each other */
 	    && !mobile(unit->type))
 	  return unit;
 	/* (should also test for ability to transfer to side's storage) */
@@ -1140,8 +1335,7 @@ Unit *unit;
 }
 
 static int
-extractable_test(x, y)
-int x, y;
+extractable_test(int x, int y)
 {
     Unit *unit;
 
@@ -1434,12 +1628,20 @@ Unit *unit;
       case TASK_FAILED:
         ++task->retrynum;
 	DMprintf("failed try %d, ", task->retrynum);
-	if (probability(20) || task->retrynum > 5) {
+	/* If a task fails, it might be because the task cannot be completed,
+	   or just because conditions are temporarily unfavorable, such as
+	   a passing unit blocking the way while moving through.  So we need
+	   to retry a couple times at least; the variables here control how
+	   hard to keep trying. */
+	/* (should be doctrine, since these affect human-run units too) */
+	if (probability(g_ai_badtask_remove_chance()) 
+	    || task->retrynum >= g_ai_badtask_max_retries()) {
 	    pop_task(plan);
 	    DMprintf("removed it");
 	    /* We might be buzzing, so maybe go into reserve. */
-	    if (probability(20)) {
-	    	plan->reserve = TRUE;
+	    if (probability(g_ai_badtask_reserve_chance())
+	         && g_units_may_go_into_reserve()) {
+		plan->reserve = TRUE;
 	    	DMprintf(" and went into reserve");
 	    }
 	} else {
@@ -1771,7 +1973,38 @@ set_build_task(unit, u2, run)
 Unit *unit;
 int u2, run;
 {
-    add_task(unit, CLEAR_AGENDA, create_build_task(unit, u2, run));
+	/* First check if we are building something. */
+	if (unit->plan && unit->plan->tasks
+	 && unit->plan->tasks->type == TASK_BUILD) {
+
+		Unit *unit2 = find_unit(unit->plan->tasks->args[1]);
+
+		/* Return if we already are building the desired unit. */
+		if (is_unit(unit2) && !completed(unit2) && unit2->type == u2) {
+			return;
+		/* Maybe disband incomplete unit of wrong type. */
+		} else if (is_unit(unit2) && !completed(unit2)
+			     && side_can_disband(unit->side, unit2)
+			     && g_disband_unfinished_units()) {
+			/* But first salvage either its cps or materials if permitted to do so. */
+			if (g_salvage_unfinished_cps())
+				unit->cp_stash += unit2->cp;	   			
+			else if (g_salvage_unfinished_materials()) {
+
+				int	m, u = unit2->type;
+
+				for_all_material_types(m) {
+					if (um_consumption_on_creation(u, m) > 0)
+						unit->supply[m] += um_consumption_on_creation(u, m);
+					if (um_consumption_per_cp(u2, m) > 0)
+						unit->supply[m] += (unit2->cp - uu_creation_cp(unit->type, u))
+									      * um_consumption_per_cp(u2, m);
+				}						
+			}
+		kill_unit(unit2, H_UNIT_DISBANDED);
+		}		
+	}	    		
+	add_task(unit, CLEAR_AGENDA, create_build_task(unit, u2, run));
 }
 
 void
@@ -1779,7 +2012,38 @@ push_build_task(unit, u2, run)
 Unit *unit;
 int u2, run;
 {
-    add_task(unit, 0, create_build_task(unit, u2, run));
+	/* First check if we are building something. */
+	if (unit->plan && unit->plan->tasks
+	 && unit->plan->tasks->type == TASK_BUILD) {
+
+		Unit *unit2 = find_unit(unit->plan->tasks->args[1]);
+
+		/* Return if we already are building the desired unit. */
+		if (is_unit(unit2) && !completed(unit2) && unit2->type == u2) {
+			return;
+		/* Maybe disband incomplete unit of wrong type. */
+		} else if (is_unit(unit2) && !completed(unit2)
+			     && side_can_disband(unit->side, unit2)
+			     && g_disband_unfinished_units()) {
+			/* But first salvage either its cps or materials if permitted to do so. */
+			if (g_salvage_unfinished_cps())
+				unit->cp_stash += unit2->cp;	   			
+			else if (g_salvage_unfinished_materials()) {
+
+				int	m, u = unit2->type;
+
+				for_all_material_types(m) {
+					if (um_consumption_on_creation(u, m) > 0)
+						unit->supply[m] += um_consumption_on_creation(u, m);
+					if (um_consumption_per_cp(u2, m) > 0)
+						unit->supply[m] += (unit2->cp - uu_creation_cp(unit->type, u))
+									      * um_consumption_per_cp(u2, m);
+				}						
+			}
+		kill_unit(unit2, H_UNIT_DISBANDED);
+		}		
+	}	    		
+	 add_task(unit, 0, create_build_task(unit, u2, run));
 }
 
 Task *
@@ -1947,6 +2211,13 @@ Unit *transport;
     return task;
 }
 
+void
+set_occupy_task(unit, transport)
+Unit *unit, *transport;
+{
+    add_task(unit, CLEAR_AGENDA, create_occupy_task(unit, transport));
+}
+
 void
 push_occupy_task(unit, transport)
 Unit *unit, *transport;
