@@ -12,8 +12,8 @@ any later version.  See the file COPYING. */
 #include "kernel.h"
 #include "kpublic.h"
 
-void flush_outgoing_queue PARAMS ((void));
-void flush_incoming_queue PARAMS ((void));
+void flush_outgoing_queue(void);
+void flush_incoming_queue(void);
 
 extern long randstate;
 
@@ -29,46 +29,43 @@ extern long randstate;
 #define ENDPKTESC '&'
 #define ESCAPEPKTESC '@'
 
-static void broadcast_message PARAMS ((Side *side, SideMask sidemask,
-				       char *str));
-static void broadcast_command_5 PARAMS ((char *cmd, int val, int val2,
-					 int val3, int val4, int val5));
-static void broadcast_side_property PARAMS ((Side *side, char *prop, int val));
-static void broadcast_side_property_2 PARAMS ((Side *side, char *prop,
-					       int val, int val2));
-static void broadcast_side_str_property PARAMS ((Side *side, char *prop,
-						 char *val));
-static void broadcast_unit_property PARAMS ((Side *side, Unit *unit,
-					     char *prop, int val));
-static void broadcast_unit_property_2 PARAMS ((Side *side, Unit *unit,
-					       char *prop, int val, int val2));
-static void broadcast_unit_property_5 PARAMS ((Side *side, Unit *unit,
-					       char *prop, int val, int val2,
-					       int val3, int val4, int val5));
-static void broadcast_unit_str_property PARAMS ((Side *side, Unit *unit,
-						 char *prop, char *val));
-static void broadcast_add_task PARAMS ((Unit *unit, int pos, Task *task));
-static void broadcast_layer_change PARAMS ((char *layername, Side *side,
-					    int x, int y,
-					    int a1, int a2, int a3));
-static void broadcast_packet PARAMS ((char *buf));
-static void save_outgoing_packet PARAMS ((int id, char *inbuf));
-static void save_incoming_packet PARAMS ((int id, char *inbuf));
-static void remove_chars PARAMS ((char *buf, int n));
-static void receive_net_message PARAMS ((char *str));
-static void receive_command PARAMS ((char *str));
-static void receive_action PARAMS ((char *str));
-static void receive_player_prop PARAMS ((char *str));
-static void receive_quit PARAMS ((char *str));
-static void receive_side_prop PARAMS ((char *str));
-static void receive_task PARAMS ((char *str));
-static void receive_unit_prop PARAMS ((char *str));
-static void receive_world_prop PARAMS ((char *str));
-static void receive_run_game PARAMS ((char *str));
-static void receive_game_checksum PARAMS ((char *str));
-static int game_checksum PARAMS ((void));
-static int tohex PARAMS ((int x));
-static int fromhex PARAMS ((int x));
+static void broadcast_message(Side *side, SideMask sidemask, char *str);
+static void broadcast_command_5(char *cmd, int val, int val2, int val3,
+				int val4, int val5);
+static void broadcast_side_property(Side *side, char *prop, int val);
+static void broadcast_side_property_2(Side *side, char *prop, int val,
+				      int val2);
+static void broadcast_side_str_property(Side *side, char *prop, char *val);
+static void broadcast_unit_property(Side *side, Unit *unit, char *prop,
+				    int val);
+static void broadcast_unit_property_2(Side *side, Unit *unit, char *prop,
+				      int val, int val2);
+static void broadcast_unit_property_5(Side *side, Unit *unit, char *prop,
+				      int val, int val2, int val3, int val4,
+				      int val5);
+static void broadcast_unit_str_property(Side *side, Unit *unit, char *prop,
+					char *val);
+static void broadcast_add_task(Unit *unit, int pos, Task *task);
+static void broadcast_layer_change(char *layername, Side *side, int x, int y,
+				   int a1, int a2, int a3);
+static void broadcast_packet(char *buf);
+static void save_outgoing_packet(int id, char *inbuf);
+static void save_incoming_packet(int id, char *inbuf);
+static void remove_chars(char *buf, int n);
+static void receive_net_message(char *str);
+static void receive_command(char *str);
+static void receive_action(char *str);
+static void receive_player_prop(char *str);
+static void receive_quit(char *str);
+static void receive_side_prop(char *str);
+static void receive_task(char *str);
+static void receive_unit_prop(char *str);
+static void receive_world_prop(char *str);
+static void receive_run_game(char *str);
+static void receive_game_checksum(char *str);
+static int game_checksum(void);
+static int tohex(int x);
+static int fromhex(int x);
 
 int numremotes;
 
@@ -202,7 +199,7 @@ void
 send_randstate(id)
 int id;
 {
-    sprintf(spbuf, "R%d", randstate);
+    sprintf(spbuf, "R%ld", randstate);
     send_packet(id, spbuf); 
 }
 
@@ -1073,6 +1070,19 @@ Task *task;
     }
 }
 
+void
+net_set_capture_task(Unit *unit, int x, int y)
+{
+    if (my_rid == master_rid) {
+	set_capture_task(unit, x, y);
+    }
+    if (numremotes > 0) {
+	tmptask = create_capture_task(unit, x, y);
+	broadcast_add_task(unit, CLEAR_AGENDA, tmptask);
+	free_task(tmptask);
+    }
+}
+
 void
 net_set_collect_task(unit, m, x, y)
 Unit *unit;
@@ -1148,6 +1158,23 @@ int u2, run;
     }
 }
 
+/* Same as above but handles cases where we resume building an incomplete occ. */
+
+void
+net_resume_build_task(Unit *unit, Unit *unit2, int run)
+{
+    if (my_rid == master_rid) {
+	set_build_task(unit, unit2->type, run);
+	unit->plan->tasks->args[1] = unit2->id;		
+    }
+    if (numremotes > 0) {
+	tmptask = create_build_task(unit, unit2->type, run);
+	tmptask->args[1] = unit2->id;		
+	broadcast_add_task(unit, CLEAR_AGENDA, tmptask);
+	free_task(tmptask);
+    }
+}
+
 void
 net_push_build_task(unit, u2, run)
 Unit *unit;
@@ -1536,7 +1563,7 @@ char *inbuf;
 	/* Add packet to the outgoing queue. */
 	save_outgoing_packet(id, inbuf);
 	Dprintf("OutQ: %d \"%s\"\n", id, inbuf);
-	return;
+	return TRUE;
     }
     Dprintf("Send: %d \"%s\"\n", id, inbuf);
     j = 0;
@@ -1879,11 +1906,12 @@ char *buf;
 	    break;
 	  case 'R':
 	    new_randstate = atoi(buf + 1);
-	    if (new_randstate != randstate)
-	      Dprintf("Rand state change: %d -> %d\n",
-		      randstate, new_randstate);
-	    else
-	      Dprintf("Rand state matches\n");
+	    if (new_randstate != randstate) {
+		Dprintf("Rand state change: %d -> %d\n",
+			randstate, new_randstate);
+	    } else {
+		Dprintf("Rand state matches\n");
+	    }
 	    randstate = new_randstate;
 	    /* This is only issued by the master, no need to rebroadcast. */
 	    should_rebroadcast = FALSE;
@@ -2379,10 +2407,11 @@ char *str;
     reststr = nreststr;
     new_randstate = strtol(reststr, &nreststr, 10);
     set_g_run_serial_number(newsernum);
-    if (new_randstate != randstate)
-      Dprintf("Rand state change: %d -> %d\n", randstate, new_randstate);
-    else
-      Dprintf("Rand state matches\n");
+    if (new_randstate != randstate) {
+	Dprintf("Rand state change: %d -> %d\n", randstate, new_randstate);
+    } else {
+	Dprintf("Rand state matches\n");
+    }
     randstate = new_randstate;
     run_game(maxactions);
 }
@@ -2512,8 +2541,8 @@ int hosting;
 	inport_filename = "client.in";
 	outport_filename = "client.out";
     }
-    inport = fopen(inport_filename, "r");
-    outport = fopen(outport_filename, "w");
+    inport = open_file(inport_filename, "r");
+    outport = open_file(outport_filename, "w");
     if (inport == NULL || outport == NULL)
       run_warning("connection files opening failed");
     /* (should be able to get all the rids from file) */
