@@ -1,5 +1,5 @@
 /* Graphics support not specific to any Xconq interface.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,26 +16,31 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kpublic.h"
+#include "ai.h"
 #include "imf.h"
 #include "ui.h"
-extern void set_contour_interval PARAMS ((VP *vp, int n));
-extern int unit_actually_visible PARAMS ((Side *side, Unit *unit));
-
-extern ImageFamily *get_generic_images PARAMS ((Side *side, char *name));
+extern void set_contour_interval(VP *vp, int n);
+extern int unit_actually_visible(Side *side, Unit *unit);
 
 #ifdef MAC /* temporary */
+#ifdef NEW_HEADERS
+#include <MacTypes.h>
+#else
 #include <Types.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
-#endif
+#endif /* MAC */
 
 extern int error_is_bug;
 
-static void compute_q PARAMS ((void));
-static void calc_view_misc PARAMS ((VP *vp));
-static int blocking_utype PARAMS ((int u, int block));
-static ImageFamily *add_default_terrain_image PARAMS ((ImageFamily *imf, int t));
-static ImageFamily *add_default_unit_image PARAMS ((ImageFamily *imf, int u));
-static ImageFamily *add_default_emblem_image PARAMS ((ImageFamily *imf, int s2));
+static void compute_q(void);
+static int module_name_compare(CONST void *a1, CONST void *a2);
+static int special_strcmp(char *str1, char *str2);
+static void calc_view_misc(VP *vp);
+static int blocking_utype(int u, int block);
+static ImageFamily *add_default_terrain_image(ImageFamily *imf, int t);
+static ImageFamily *add_default_unit_image(ImageFamily *imf, int u);
+static ImageFamily *add_default_emblem_image(ImageFamily *imf, int s2);
 
 /* The two games that should be always be available. */
 
@@ -54,10 +59,14 @@ short mags[NUMPOWERS] = { 1, 2, 4, 8, 16, 32, 64, 128 };
    the cells overlap perfectly at each different magnification, assuming
    that the icons have the right shape and size. */
 
-short hws[NUMPOWERS] = { 1, 2, 4, 10, 24, 44, 88, 174 };
-short hhs[NUMPOWERS] = { 1, 2, 4, 10, 26, 48, 96, 192 };
+short hws[NUMPOWERS] = { 1, 2, 4, 12, 24, 44, 88, 174 };
+short hhs[NUMPOWERS] = { 1, 2, 4, 13, 26, 48, 96, 192 };
 short hcs[NUMPOWERS] = { 1, 2, 4, 10, 20, 37, 74, 148 };
 
+/* These numbers are half the length of a vertical side. */
+
+short halfsides[NUMPOWERS] = { 1, 1, 2, 5, 7, 13, 26, 52 };
+
 /* The sizes of the unit subcells.  This is available drawing area, exact
    unit icon sizes depends on what's available. */
 
@@ -68,7 +77,7 @@ short uws[NUMPOWERS] = { 1, 1, 3, 8, 16, 32, 64, 128 };
 
 /* Full border width. */
 
-short bwid[NUMPOWERS] = { 0, 0, 1, 1, 3, 5, 7, 9 };
+short bwid[NUMPOWERS] = { 0, 0, 1, 2, 3, 5, 7, 9 };
 
 /* Half-width, for narrower inset borders. */
 
@@ -76,7 +85,7 @@ short bwid2[NUMPOWERS] = { 0, 0, 1, 1, 2, 3, 4, 5 };
 
 /* Full connection width. */
 
-short cwid[NUMPOWERS] = { 0, 0, 1, 1, 3, 5, 7, 9 };
+short cwid[NUMPOWERS] = { 0, 0, 1, 2, 3, 5, 7, 9 };
 
 /* Coordinates of the hex borders. */
 /* Note that array has extra column so don't need to wrap index. */
@@ -85,7 +94,7 @@ short bsx[NUMPOWERS][7] = {
     { 0 },
     { 0 },
     {  2,   4,   4,  2,  0,  0,  2 },
-    {  5,  10,  10,  5,  0,  0,  5 },
+    {  6,  12,  12,  6,  0,  0,  6 },
     { 12,  24,  24, 12,  0,  0, 12 },
     { 22,  44,  44, 22,  0,  0, 22 },
     { 44,  88,  88, 44,  0,  0, 44 },
@@ -95,7 +104,7 @@ short bsy[NUMPOWERS][7] = {
     { 0 },
     { 0 },
     {  0,   0,   4,   4,   4,  0,  0 },
-    {  0,   0,  10,  10,  10,  0,  0 },
+    {  0,   3,  10,  13,  10,  3,  0 },
     {  0,   6,  20,  26,  20,  6,  0 },
     {  0,  11,  37,  48,  37, 11,  0 },
     {  0,  21,  75,  96,  75, 21,  0 },
@@ -108,7 +117,7 @@ short lsx[NUMPOWERS][6] = {
     { 0 },
     { 0 },
     {  1,  2,   1,  -1,  -2,  -1 },
-    {  2,  5,   2,  -2,  -5,  -2 },
+    {  3,  6,   3,  -3,  -6,  -3 },
     {  6, 12,   6,  -6, -12,  -6 },
     { 11, 22,  11, -11, -22, -11 },
     { 22, 44,  22, -22, -44, -22 },
@@ -150,7 +159,7 @@ int num_recorded_imfs;
 
 int max_recorded_imfs;
 
-void (*imf_describe_hook) PARAMS ((Side *side, Image *img));
+void (*imf_describe_hook)(Side *side, Image *img);
 
 /* Machinery to find and list all the games that should be listed as
    choices for the user. We don't actually scan library folders
@@ -166,11 +175,8 @@ int numgames = 0;
    modules at the end, plus the default sorting puts initial-lowercased
    names after uppercased ones. */
 
-static int special_strcmp PARAMS ((char *str1, char *str2));
-
 static int
-module_name_compare(a1, a2)
-CONST void *a1, *a2;
+module_name_compare(CONST void *a1, CONST void *a2)
 {
     Module *mp1, *mp2;
     int rslt;
@@ -420,7 +426,7 @@ num_active_displays()
 /* Compute positions at each hex corner, slightly inset. */ 
 
 static void
-compute_q()
+compute_q(void)
 {
     int d, p, w;
 
@@ -440,9 +446,9 @@ compute_q()
 /* Viewport handling. */
 
 VP *
-new_vp()
+new_vp(void)
 {
-    int t, thickest, numcontours;
+    int t, thickest;
     VP *vp;
 
     if (!q_computed) {
@@ -474,14 +480,39 @@ new_vp()
     return vp;
 }
 
+/* xform_cell has been separated into two functions, a core that does the xform
+calculations and a shell that adds on the vertical offset. This makes it possible to
+bypass the vertical offset by calling xform_cell_flat directly. */
+
 /* Given a viewport and a cell, figure out where its UL corner will be. */
 
 void
-xform_cell(vp, x, y, sxp, syp)
-VP *vp;
-int x, y, *sxp, *syp;
+xform_cell(VP *vp, int x, int y, int *sxp, int *syp)
 {
-    int xnw, xw, elev, offset;
+    int xw, elev, offset;
+
+    /* First call the core function. */
+    xform_cell_flat(vp, x, y, sxp, syp);
+
+    /* Then add vertical offset if drawing at an angle. */
+    if (vp->angle != 90) {
+	xw = wrapx(x);
+	elev = (elevations_defined() ? elev_at(xw, y) : 0) - area.avgelev;
+	/* We see the top of the terrain in the cell, not the bottom. */
+	elev += t_thickness(terrain_at(xw, y));
+	/* Exaggerate the vertical scale if requested. */
+	elev *= vp->vertscale;
+	offset = (elev * vp->hh) / vp->cellwidth;
+	*syp -= offset;
+    }
+}
+
+/* Core calculation for xform_cell without vertical offset. */ 
+
+void
+xform_cell_flat(VP *vp, int x, int y, int *sxp, int *syp)
+{
+    int	 xnw;
 
     /* Mac background redrawing uses out-of-area cell positions up to
        8 cells east of the area edge! */
@@ -530,14 +561,6 @@ int x, y, *sxp, *syp;
 		*sxp = *syp = 0;
 		break;
 	    }
-	    xw = wrapx(x);
-	    elev = (elevations_defined() ? elev_at(xw, y) : 0) - area.avgelev;
-	    /* We see the top of the terrain in the cell, not the bottom. */
-	    elev += t_thickness(terrain_at(xw, y));
-	    /* Exaggerate the vertical scale if requested. */
-	    elev *= vp->vertscale;
-	    offset = (elev * vp->hh) / vp->cellwidth;
-	    *syp -= offset;
 	    *sxp -= vp->sx;  *syp -= vp->sy;
 	    return;
 	} else {
@@ -558,26 +581,12 @@ int x, y, *sxp, *syp;
 		  *sxp -= area.width * vp->hw;
 	    }
 	}
-	/* Compute the scaled y. */
 	*syp = (vp->totsh - (vp->hh + y * vp->hch)) - vp->sy;
-	if (vp->angle != 90 /* && vp->draw_elevations */) {
-	    /* Wrapping should not be necessary, should find out why this
-	       fixes probs. */
-	    xw = wrapx(x);
-	    elev = (elevations_defined() ? elev_at(xw, y) : 0) - area.avgelev;
-	    /* We see the top of the terrain in the cell, not the bottom. */
-	    elev += t_thickness(terrain_at(xw, y));
-	    /* Exaggerate the vertical scale if requested. */
-	    elev *= vp->vertscale;
-	    offset = (elev * vp->hh) / vp->cellwidth;
-	    *syp -= offset;
-	}
     } else {
 	/* Always die on this, indicates bugs that must be fixed. */
 	error_is_bug = TRUE;
 	run_error("attempting to xform %d,%d", x, y);
     }
-    *sxp += vp->offsetx;  *syp += vp->offsety;
 }
 
 /* Similarly, but allowing 1/1000ths of cells as input.  Note that
@@ -594,6 +603,17 @@ int x, y, xf, yf, *sxp, *syp;
     *sxp += (xf * vp->hw) / 1000;  *syp += ((1000 - yf) * vp->hch) / 1000;
 }
 
+/* Same as xform_cell_fractional but bypasses vertical offset. */
+
+void
+xform_cell_fractional_flat(vp, x, y, xf, yf, sxp, syp)
+VP *vp;
+int x, y, xf, yf, *sxp, *syp;
+{
+    xform_cell_flat(vp, x, y, sxp, syp);
+    *sxp += (xf * vp->hw) / 1000;  *syp += ((1000 - yf) * vp->hch) / 1000;
+}
+
 void
 xform_unit(vp, unit, sxp, syp, swp, shp)
 VP *vp;
@@ -742,7 +762,6 @@ int sx, sy, *xp, *yp, *xfp, *yfp;
 	}
 	return (in_area(*xp, *yp));
     }
-    sx -= vp->offsetx;  sy -= vp->offsety;
     /* Flip the raw y and then scale to hex coords. */
     syflipped = vp->totsh - (vp->sy + sy);
     *yp = syflipped / vp->hch;
@@ -756,18 +775,11 @@ int sx, sy, *xp, *yp, *xfp, *yfp;
        around to positive values.  This should only ever happens for cylinder
        areas, but doesn't hurt to just adjust all negative values. */
 
-// --- MAC MERIDIAN BUG FIX ------------------------------------
-
     /* Unfortunately, it does hurt. The mac-specific meridian drawing code 
     must be allowed to use negative values for non-cylindrical worlds! */
-
    if(area.xwrap && sxadj < 0)
-
-//    if (sxadj < 0)
-
-// --------------------------------------------------------
-
       sxadj += (2 * area.width * vp->hw);
+
     *xp = sxadj / vp->hw;
     if (xfp) {
 	sxfrac = sxadj - (*xp * vp->hw);
@@ -813,7 +825,8 @@ int sx, sy, *xp, *yp, *dirp;
 }
 
 Unit *
-find_unit_or_occ(vp, unit, usx, usy, usw, ush, sx, sy)
+find_unit_or_occ(side, vp, unit, usx, usy, usw, ush, sx, sy)
+Side *side;
 VP *vp;
 Unit *unit;
 int usx, usy, usw, ush, sx, sy;
@@ -822,10 +835,17 @@ int usx, usy, usw, ush, sx, sy;
     Unit *occ, *rslt;
 
     /* See if the point might be over an occupant. */
-    if (unit->occupant != NULL) {
+    if (unit->occupant != NULL
+	&& (side_controls_unit(side, unit)
+	    /* If side can examine unit in detail it can also see its occs. */
+	    || side_sees_unit(side, unit)
+	    || vp->see_all
+	    || u_see_occupants(unit->type)
+	    || side_owns_occupant(side, unit))) {
 	for_all_occupants(unit, occ) {
 	    xform_unit(vp, occ, &usx1, &usy1, &usw1, &ush1);
-	    rslt = find_unit_or_occ(vp, occ, usx1, usy1, usw1, ush1, sx, sy);
+	    rslt = find_unit_or_occ(side, vp, occ, usx1, usy1, usw1, ush1,
+				    sx, sy);
 	    if (rslt)
 	      return rslt;
 	}
@@ -839,7 +859,8 @@ int usx, usy, usw, ush, sx, sy;
 }
 
 Unit *
-find_unit_at(vp, x, y, sx, sy)
+find_unit_at(side, vp, x, y, sx, sy)
+Side *side;
 VP *vp;
 int x, y, sx, sy;
 {
@@ -848,7 +869,7 @@ int x, y, sx, sy;
     
     for_all_stack(x, y, unit) {
 	xform_unit(vp, unit, &usx, &usy, &usw, &ush);
-	rslt = find_unit_or_occ(vp, unit, usx, usy, usw, ush, sx, sy);
+	rslt = find_unit_or_occ(side, vp, unit, usx, usy, usw, ush, sx, sy);
 	if (rslt)
 	  return rslt;
     }
@@ -856,7 +877,8 @@ int x, y, sx, sy;
 }
 
 int
-nearest_unit(vp, sx, sy, unitp)
+nearest_unit(side, vp, sx, sy, unitp)
+Side *side;
 VP *vp;
 int sx, sy;
 Unit **unitp;
@@ -869,7 +891,7 @@ Unit **unitp;
 	return FALSE;
     }
     if (vp->power > 4) {
-	*unitp = find_unit_at(vp, x, y, sx, sy);
+	*unitp = find_unit_at(side, vp, x, y, sx, sy);
     } else {
 	*unitp = unit_at(x, y);
     }
@@ -1070,19 +1092,14 @@ VP *vp;
 {
     /* Calculate and cache the width in pixels of the whole area, adding an
        an adjustment to account for the "bulge" of hexagon-shaped areas. */
+    /* (but it seems the adjustment is wrong for sizing - should figure
+       out why this seems necessary) */
     vp->totsw = area.width * vp->hw + hexagon_adjust(vp);
     if (area.xwrap && (vp->totsw > (vp->pxw - vp->hw)))
       vp->totsw += (vp->pxw - vp->hw);
     /* Total scaled height is based on center-to-center height, plus
        an adjustment to include the bottom parts of the bottom row. */
     vp->totsh = area.height * vp->hch + (vp->hh - vp->hch);
-    if (vp->center) {
-	vp->offsetx = vp->offsety = 0;
-	if (vp->totsw < vp->pxw)
-	  vp->offsetx = (vp->pxw - vp->totsw) / 2;
-	if (vp->totsh < vp->pxh)
-	  vp->offsety = (vp->pxh - vp->totsh) / 2;
-    }
     vp->sxmin = hexagon_adjust(vp);  vp->symin = 0;
     /* Special-case wide viewports (a la Mac) for cylindrical worlds. */
     if (vp->wide_viewport && area.xwrap)
@@ -1226,9 +1243,9 @@ int u, block;
 void
 plot_meridians(vp, line_callback, text_callback)
 VP *vp;
-void (*line_callback) PARAMS ((int x1, int y1, int x1f, int y1f,
-			       int x2, int y2, int x2f, int y2f));
-void (*text_callback) PARAMS ((int x1, int y1, int x1f, int y1f, char *str));
+void (*line_callback)(int x1, int y1, int x1f, int y1f,
+			       int x2, int y2, int x2f, int y2f);
+void (*text_callback)(int x1, int y1, int x1f, int y1f, char *str);
 {
     int xxmin, ymin, xmax, ymax;
     int lat1, lon1, lat2, lon2, latmin, latmax, lonmin, lonmax, incr, lat, lon;
@@ -1593,7 +1610,7 @@ int sx, sy;
     Feature *feature;
 
     nearest_cell(vp, sx, sy, &x, &y, &xf, &yf);
-    nearest_unit(vp, sx, sy, &unit);
+    nearest_unit(side, vp, sx, sy, &unit);
     if (!in_area(x, y)) {
 	strcpy(tmpbuf, "(nothing)");
 	return;
@@ -2120,7 +2137,7 @@ int x, y;
 /* Given a unit and amounts of supplies desired to transfer, move as much of them
    as possible into the unit's transport. */
 
-int give_supplies_to_one PARAMS ((Unit *unit, Unit *unit2, short *amts, short *rslts));
+int give_supplies_to_one(Unit *unit, Unit *unit2, short *amts, short *rslts);
 
 int
 give_supplies(unit, amts, rslts)
@@ -2227,65 +2244,14 @@ Unit *unit;
 	&& unit->plan->tasks->type == TASK_BUILD)
       return unit->plan->tasks->args[0];
     for_all_unit_types(u) {
-	if (uu_acp_to_create(unit->type, u) > 0)
+	if (uu_acp_to_create(unit->type, u) > 0
+	    && type_allowed_on_side(u, unit->side)
+	    && has_advance_to_build(unit->side, u))
 	  return u;
     }
     return NONUTYPE;
 }
 
-/* Generic image setup. */
-
-static short imf_dir_loaded;
-
-ImageFamily *
-get_generic_images(side, name)
-Side *side;
-char *name;
-{
-    FILE *fp;
-    int cloned = FALSE;
-    ImageFamily *imf;
-
-    imf = get_imf(name);
-    if (imf == NULL)
-      return NULL;
-    if (imf->numsizes > 0 && imf_interp_hook != NULL)
-      imf = (*imf_interp_hook)(imf, NULL, FALSE);
-    if (imf_load_hook != NULL)
-      imf = (*imf_load_hook)(imf);
-    if (imf->numsizes == 0) {
-	/* Maybe collect the names/locations of all image families. */
-	if (!imf_dir_loaded) {
-	    /* (should let name be decided by platform?) */
-	    fp = open_library_file("imf.dir");
-	    if (fp != NULL) {
-		load_image_families(fp, FALSE, NULL);
-		fclose(fp);
-	    } else {
-		init_warning("Cannot open \"%s\", will not use it", "imf.dir");
-	    }
-	    imf_dir_loaded = TRUE;
-	}
-	/* Get a (possibly empty) family. */
-	imf = get_imf(name);
-	if (imf == NULL)
-	  return NULL;
-	if (imf->location != NULL) {
-	    /* Load data filling in the family. */
-	    /* (should use library path list) */
-	    make_pathname(xconq_libs->path, imf->location->name, "", spbuf);
-	    if (load_imf_file(spbuf, NULL)) {
-	    } else if (load_imf_file(imf->location->name, NULL)) {
-	    } else {
-		/* complain here, or not? */
-	    }
-	    if (imf_interp_hook != NULL)
-	      imf = (*imf_interp_hook)(imf, NULL, FALSE);
-	}
-    }
-    return imf;
-}
-
 ImageFamily *
 get_unit_type_images(side, u)
 Side *side;
@@ -2298,7 +2264,7 @@ int u;
       name = u_image_name(u);
     else
       name = u_internal_name(u);
-    imf = get_generic_images(side, name);
+    imf = get_generic_images(name);
     if (imf != NULL && imf->numsizes == 0) {
 	imf->ersatz = TRUE;
 	imf = add_default_unit_image(imf, u);
@@ -2351,7 +2317,7 @@ int t;
       name = t_image_name(t);
     else
       name = t_type_name(t);
-    imf = get_generic_images(side, name);
+    imf = get_generic_images(name);
     if (imf != NULL && imf->numsizes == 0) {
 	imf->ersatz = TRUE;
 	imf = add_default_terrain_image(imf, t);
@@ -2388,7 +2354,7 @@ get_unseen_images(side)
 Side *side;
 {
     if (!empty_string(g_unseen_color())) {
-	unseen_image = get_generic_images(side, g_unseen_color());
+	unseen_image = get_generic_images(g_unseen_color());
 	if (unseen_image != NULL && unseen_image->numsizes == 0) {
 	    /* Appears to have failed - clear the unseen image then. */
 	    unseen_image = NULL;
@@ -2415,7 +2381,7 @@ Side *side, *side2;
 	sprintf(tmpbuf, "s%d", s2);
 	name = copy_string(tmpbuf);
     }
-    imf = get_generic_images(side, name);
+    imf = get_generic_images(name);
     /* If we must have an image, and none were acquired, invoke the default
        image getter. */
     if (imf != NULL && imf->numsizes == 0 && strcmp(name, "none") != 0) {
@@ -2513,8 +2479,6 @@ ImageFamily *imf;
 	DGprintf("    %dx%d", img->w, img->h);
 	if (img->istile)
 	  DGprintf(" tile");
-	if (img->style)
-	  DGprintf(" style %d", img->style);
 	if (imf_describe_hook)
 	  (*imf_describe_hook)(side, img);
 	DGprintf("\n");
@@ -2572,7 +2536,7 @@ int use_both_chars;
     Unit *unit;
     FILE *fp;
 
-    fp = fopen(VIEWFILE, "w");
+    fp = open_file(VIEWFILE, "w");
     if (fp != NULL) {
 	for (y = area.height-1; y >= 0; --y) {
 	    for (i = 0; i < y; ++i)
