@@ -26,8 +26,7 @@ typedef struct a_vp {
     short isometric;
     short isodir;
     short wide_viewport;
-    short center;		/* True if should center area in viewport */
-    short offsetx, offsety;
+    short see_all;
     short draw_terrain;		/* Display solid color terrain on the map? */
     short aux_terrain_types[MAXTTYPES];
     short draw_aux_terrain[MAXTTYPES];
@@ -81,7 +80,7 @@ typedef struct a_line_segment {
 #define ESCAPE_CHAR '\033'
 #define DELETE_CHAR 0x7f
 
-extern short mags[], hws[], hhs[], hcs[], uws[], uhs[];
+extern short mags[], hws[], hhs[], hcs[], halfsides[], uws[], uhs[];
 
 extern short bwid[], bwid2[], cwid[];
 
@@ -121,7 +120,9 @@ extern void xform_unit_self PARAMS ((VP *vp, Unit *unit,
 extern void xform_occupant PARAMS ((VP *vp, Unit *transport, Unit *unit,
 				    int sx, int sy, int sw, int sh,
 				    int *sxp, int *syp, int *swp, int *shp));
-
+extern void xform_cell_flat PARAMS ((VP *vp, int x, int y, int *sxp, int *syp));
+extern void xform_cell_fractional_flat PARAMS ((VP *vp, int x, int y, int xf, int yf,
+					   		int *sxp, int *syp));
 extern void scale_vp PARAMS ((VP *vp, VP *vp2,
 			      int *sxp, int *syp, int *swp, int *shp));
 
@@ -129,7 +130,8 @@ extern int nearest_cell PARAMS ((VP *vp, int sx, int sy,
 				 int *xp, int *yp, int *xfp, int *yfp));
 extern int nearest_boundary PARAMS ((VP *vp, int sx, int sy,
 				     int *xp, int *yp, int *dirp));
-extern int nearest_unit PARAMS ((VP *vp, int sx, int sy, Unit **unitp));
+extern int nearest_unit PARAMS ((Side *side, VP *vp, int sx, int sy,
+				 Unit **unitp));
 
 extern int cell_is_visible PARAMS ((VP *vp, int x, int y));
 extern int cell_is_in_middle PARAMS ((VP *vp, int x, int y));
@@ -163,10 +165,11 @@ extern int draw_winds_here PARAMS ((Side *side, int x, int y));
 
 extern void oneliner PARAMS ((Side *side, VP *vp, int sx, int sy));
 
-extern Unit *find_unit_or_occ PARAMS ((VP *vp, Unit *unit,
+extern Unit *find_unit_or_occ PARAMS ((Side *side, VP *vp, Unit *unit,
 				       int usx, int usy, int usw, int ush,
 				       int sx, int sy));
-extern Unit *find_unit_at PARAMS ((VP *vp, int x, int y, int sx, int sy));
+extern Unit *find_unit_at PARAMS ((Side *side, VP *vp, int x, int y,
+				   int sx, int sy));
 
 extern void pick_a_focus PARAMS ((Side *side, int *xp, int *yp));
 
@@ -216,7 +219,7 @@ extern HelpNode *long_commands_help_node;
 /* Declarations of all the command functions. */
 
 #undef DEF_CMD
-#define DEF_CMD(letter,name,args,FN,help) \
+#define DEF_CMD(letter,name,FN,help) \
   void FN PARAMS ((Side *side));
 
 #include "cmd.def"
