@@ -7,12 +7,12 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void interp_utype_list PARAMS ((short *arr, Obj *lis));
-extern int type_can_sit_on_conn PARAMS ((int u, int x, int y));
+extern void interp_utype_list(short *arr, Obj *lis);
+extern int type_can_sit_on_conn(int u, int x, int y);
 #include "kernel.h"
-extern int num_sides_allowed PARAMS ((int u));
-extern void wreck_unit PARAMS ((Unit *unit));
-extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
+extern int num_sides_allowed(int u);
+extern void wreck_unit(Unit *unit);
+extern int side_owns_occupant(Side *side, Unit *unit);
 
 /* This is not a limit, just sets initial allocation of unit objects
    and how many additional to get if more are needed.  This can be
@@ -23,13 +23,13 @@ extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
 #define INITMAXUNITS 200
 #endif
 
-static void allocate_unit_block PARAMS ((void));
-static int compare_units PARAMS ((Unit *unit1, Unit *unit2));
-static int compare_units_by_keys PARAMS ((const void *e1, const void *e2));
+static void allocate_unit_block(void);
+static int compare_units(Unit *unit1, Unit *unit2);
+static int compare_units_by_keys(const void *e1, const void *e2);
 
-static void leave_cell_aux PARAMS ((Unit *unit, Unit *oldtransport));
+static void leave_cell_aux(Unit *unit, Unit *oldtransport);
 
-static void change_cell_aux PARAMS ((Unit *unit, int x0, int y0, int x, int y));
+static void change_cell_aux(Unit *unit, int x0, int y0, int x, int y);
 
 /* The list of available units. */
 
@@ -99,8 +99,8 @@ int recent_dead_flushed = TRUE;
 static Side *tmpside_classtest;
 
 /* Advanced unit support. */
-short default_size = 0;		/* Default abstract size of unit */
 
+short default_size = 1;		/* Default abstract size of unit */
 short default_usedcells = 0;	/* Number of cells being used by the unit */
 short default_maxcells = 1;	/* Default max number of cells that unit can use */
 short default_curadvance = -1;	/* Default advance being researched by new unit */
@@ -108,8 +108,6 @@ long default_population = 1;	/* Default size of unit's population */
 
 extern void free_used_cells(Unit *unit);
 
-extern int has_advance_to_build(Side *side, int u);
-
 /* Grab a block of unit objects to work with. */
 
 static void
@@ -217,8 +215,6 @@ int type, makebrains;
     for_all_material_types(m)
       newunit->supply[m] = 0;
 
-// --- UNIT PRODUCTION CACHE ---------------------------------------
-
     /* Allocate cache for units last production. */
     if (nummtypes > 0)
         newunit->production = (short *) xmalloc(nummtypes * sizeof(short));
@@ -226,8 +222,6 @@ int type, makebrains;
     for_all_material_types(m)
        newunit->production[m] = 0;
 
-// ------------------------------------------------------------
-
     /* Will allocate tooling state when actually needed. */
     newunit->tooling = NULL;
     /* Will allocate opinions when actually needed. */
@@ -261,13 +255,10 @@ int type, makebrains;
     newunit->maxcells = default_maxcells;
     newunit->population = default_population;
     newunit->curadvance = default_curadvance;
-
-// --- AUTOMATION SUPPORT ---------------------------
-
     newunit->autobuild = FALSE;		
     newunit->autoresearch = FALSE;
-
-// -----------------------------------------------
+    newunit->autoplan = FALSE;
+    newunit->cp_stash = 0;
 
     /* Add to the global unit counts. */
     ++numunits;
@@ -649,7 +640,7 @@ Unit *unit3;
 
 /* Test whether the given position has a connection that the unit may use. */
 
-int can_occupy_conn_1 PARAMS ((Unit *unit, int nx, int ny, int c));
+int can_occupy_conn_1(Unit *unit, int nx, int ny, int c);
 
 int
 can_occupy_conn(unit, nx, ny, nz)
@@ -792,6 +783,8 @@ Unit *transport, *unit;
 {
     int u = unit->type, u2 = transport->type, u3, o;
     int numthistype = 0, numalltypes = 0, occvolume = 0;
+    int numfacilities = 0;
+    int nummobiles = 0;
     int ucap, uucap;
     int numtypes[MAXUTYPES];
     Unit *occ;
@@ -818,6 +811,10 @@ Unit *transport, *unit;
 	++numtypes[occ->type];
 	if (o == u)
 	  ++numthistype;
+	if (u_facility(o))
+	  ++numfacilities;
+	if (mobile(o))
+	  ++nummobiles;
 	/* Only count against fullness if exclusive capacity exceeded. */
 	if (numtypes[o] > uu_capacity_x(u2, o)) {
 	    occvolume += uu_size(o, u2);
@@ -830,6 +827,16 @@ Unit *transport, *unit;
     if (uu_occ_max(u2, u) >= 0
         && numthistype + 1 - uucap > uu_occ_max(u2, u))
       return FALSE;
+	/* Check upper limit on number of facilities. */
+	if (u_facility_total_max(u2) >= 0
+	    && u_facility(u)
+	    && numfacilities + 1 > u_facility_total_max(u2))
+		return FALSE;
+	/* Check upper limit on number of mobiles. */
+	if (u_mobile_total_max(u2) >= 0
+	    && mobile(u)
+	    && nummobiles + 1 > u_mobile_total_max(u2))
+		return FALSE;
     /* Check upper limit on count of occupants of all types. */
     if (u_occ_total_max(u2) >= 0
         && numalltypes + 1 > u_occ_total_max(u2))
@@ -847,6 +854,8 @@ Unit *transport;
 {
     int u2 = transport->type, u3, o;
     int numthistype = 0, numalltypes = 0, occvolume = 0;
+    int numfacilities = 0;
+    int nummobiles = 0;
     int ucap, uucap;
     int numtypes[MAXUTYPES];
     Unit *occ;
@@ -868,6 +877,10 @@ Unit *transport;
 	++numtypes[o];
 	if (o == u)
 	  ++numthistype;
+	if (u_facility(o))
+	  ++numfacilities;
+	if (mobile(o))
+	  ++nummobiles;
 	/* Only count against fullness if exclusive capacity exceeded. */
 	if (numtypes[o] > uu_capacity_x(u2, o)) {
 	    occvolume += uu_size(o, u2);
@@ -880,6 +893,16 @@ Unit *transport;
     if (uu_occ_max(u2, u) >= 0
         && numthistype + 1 - uucap > uu_occ_max(u2, u))
       return FALSE;
+	/* Check upper limit on number of facilities. */
+	if (u_facility_total_max(u2) >= 0
+	    && u_facility(u)
+	    && numfacilities + 1 > u_facility_total_max(u2))
+		return FALSE;
+	/* Check upper limit on number of mobiles. */
+	if (u_mobile_total_max(u2) >= 0
+	    && mobile(u)
+	    && nummobiles + 1 > u_mobile_total_max(u2))
+		return FALSE;
     /* Check upper limit on count of occupants of all types. */
     if (u_occ_total_max(u2) >= 0
         && numalltypes + 1 > u_occ_total_max(u2))
@@ -919,8 +942,6 @@ Unit *unit, *transport;
     int u = unit->type, ustack, u2stack;
     Unit *unit2, *topunit, *prevunit = NULL, *nextunit = NULL;
 
-// --- OVERCROWDED TRANSPORT BUG FIX ------------------------------------ 
-
     /* Don't enter transport if there is no room. */
     if (!can_carry(transport, unit)) {
 	/* Disband the unit if it is incomplete. */
@@ -929,9 +950,6 @@ Unit *unit, *transport;
     		kill_unit(unit, H_UNIT_DISBANDED);
 	return;
      }	
-
-// -----------------------------------------------------------------
-
     if (unit == transport) {
     	run_error("Unit is trying to enter itself");
     }
@@ -1086,9 +1104,7 @@ int x, y;
 }
 
 void
-set_unit_position(unit, x, y, z)
-Unit *unit;
-int x, y;
+set_unit_position(Unit *unit, int x, int y, int z)
 {
     int u, t, tmpz;
 
@@ -1096,7 +1112,7 @@ int x, y;
     unit->x = x;  unit->y = y;  unit->z = z;
     /* Constrain the altitude according to terrain if nonzero. */
     if (unit->z != 0) {
-	if (unit->z & 1 == 0) {
+	if ((unit->z & 1) == 0) {
 	    u = unit->type;
 	    t = terrain_at(x, y);
 	    tmpz = unit->z / 2;
@@ -1110,9 +1126,7 @@ int x, y;
 }
 
 static void
-change_cell_aux(unit, x0, y0, x, y)
-Unit *unit;
-int x0, y0, x, y;
+change_cell_aux(Unit *unit, int x0, int y0, int x, int y)
 {
     int u = unit->type;
     Unit *occ;
@@ -1435,6 +1449,9 @@ Obj *leaf;
     }
 }
 
+/* Inability to build a unit should not preclude its capture etc. Separate calls to
+has_advance_to_build have therefore been introduced everywhere in the code. */
+
 int
 type_allowed_on_side(u, side)
 int u;
@@ -1442,11 +1459,8 @@ Side *side;
 {
     if (side == NULL)
       side = indepside;
-
-    if (!has_advance_to_build(side, u))
-	return FALSE;
-
     if (side->uavail == NULL) {
+
 	int u2;
 
 	side->uavail = (short *) xmalloc(numutypes * sizeof(short));
@@ -1574,6 +1588,13 @@ Side *side;
 	    side->tech[u] = u_tech_from_ownership(u);
 	    /* (should update any displays of tech - how to ensure?) */
 	}
+	/* Turn on automation for units that switch to being independent or under AI control. Note that the 
+	reverse is NOT true. Switching from AI to human control will therefore leave the unit on automatic. */
+	if (side == indepside || side == NULL || !side_has_display(side) || side_has_ai(side)) {
+		unit->autoplan = TRUE;
+		unit->autoresearch = TRUE;
+		unit->autobuild = TRUE;
+	}
     }
     return TRUE;
 }
@@ -1781,7 +1802,7 @@ int reason;
    Dead units stay on the dead unit list for each side until that side has had
    a chance to move.  Then they are finally flushed in a permanent fashion. */ 
 
-static void flush_one_unit PARAMS ((Unit *unit));
+static void flush_one_unit(Unit *unit);
 
 void
 flush_dead_units()
@@ -2700,4 +2721,3 @@ Unit *unit;
 }
 
 #endif /* DESIGNERS */
-
