@@ -1,5 +1,5 @@
 /* Random utilities not classifiable elsewhere.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,10 +8,10 @@ any later version.  See the file COPYING.  */
 
 #include "config.h"
 #include "misc.h"
-extern int select_by_weight PARAMS ((int *arr, int numvals));
-extern void debugging_string PARAMS ((char *buf));
+extern int select_by_weight(int *arr, int numvals);
+extern void debugging_string(char *buf);
 extern int warnings_logged;
-extern void log_warning PARAMS ((char *typ, char *str));
+extern void log_warning(char *typ, char *str);
 #include "lisp.h"
 #include "dir.h"
 
@@ -198,8 +198,8 @@ struct a_malloc {
     int count;
 } *mallocs = NULL;
 
-static void record_malloc PARAMS ((int amt));
-static int malloc_record_compare PARAMS ((const void *m1, const void *m2));
+static void record_malloc(int amt);
+static int malloc_record_compare(const void *m1, const void *m2);
 
 /* Given an amount of memory allocated, record it with the others. */
 
@@ -591,7 +591,7 @@ update_debugging()
 	}
 	/* Reopen the file. */
 	if (ffp == NULL) {
-	    ffp = fopen("Xconq.DebugOut", "a");
+	    ffp = open_file("Xconq.DebugOut", "a");
 	}
 	if (ffp != NULL) {
 	    if (Debug)
@@ -621,7 +621,7 @@ int *flagp;
     }
     /* (Re-)open the debugging transcript file. */
     if (ffp == NULL) {
-	ffp = fopen("Xconq.DebugOut", (firstdebug ? "w" : "a"));
+	ffp = open_file("Xconq.DebugOut", (firstdebug ? "w" : "a"));
 	firstdebug = FALSE;
     }
     if (ffp != NULL && flagp != NULL) {
@@ -726,6 +726,136 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12;
 
 #endif /* DEBUGGING */
 
+struct a_timeline_entry {
+  int start;
+  int end;
+  char *type;
+  int detail1;
+  int detail2;
+};
+
+typedef struct a_timeline_block {
+  struct a_timeline_entry entries[10000];
+  int index;
+  struct a_timeline_block *next;
+} TimelineBlock;
+
+int activity_trace;
+
+static int starting_trace = TRUE;
+
+static TimelineBlock *current_timeline_block;
+
+static char *current_activity;
+
+#ifdef UNIX
+
+#include <sys/time.h>
+
+static struct timeval initial_time;
+#endif
+
+void
+record_activity_start(char *type, int detail)
+{
+    TimelineBlock *tlblock;
+    int usec;
+
+    if (!activity_trace)
+      return;
+    if (current_activity != NULL)
+      run_warning("starting activity %s before %s done\n",
+		  type, current_activity);
+    if (current_timeline_block == NULL) {
+	current_timeline_block =
+	  (TimelineBlock *) xmalloc(sizeof(TimelineBlock));
+    }
+    if (starting_trace) {
+	current_timeline_block->index = 0;
+#ifdef UNIX
+	gettimeofday(&initial_time, NULL);
+#endif
+	starting_trace = FALSE;
+    }
+    tlblock = current_timeline_block;
+    if (tlblock->index >= 9999)
+      return;
+#ifdef UNIX
+    {
+	struct timeval curtime;
+
+	gettimeofday(&curtime, NULL);
+	usec = (curtime.tv_sec - initial_time.tv_sec) * 1000000
+	  + curtime.tv_usec - initial_time.tv_usec;
+	tlblock->entries[tlblock->index].start = usec;
+    }
+#endif
+    tlblock->entries[tlblock->index].type = type;
+    tlblock->entries[tlblock->index].detail1 = detail;
+    current_activity = type;
+}
+
+void
+record_activity_end(char *type, int detail)
+{
+    TimelineBlock *tlblock;
+    int usec;
+
+    if (!activity_trace)
+      return;
+    if (type != current_activity)
+      run_warning("ending activity %s instead of %s\n",
+		  type, current_activity);
+    tlblock = current_timeline_block;
+    if (tlblock->index >= 9999)
+      return;
+#ifdef UNIX
+    {
+	struct timeval curtime;
+
+	gettimeofday(&curtime, NULL);
+	usec = (curtime.tv_sec - initial_time.tv_sec) * 1000000
+	  + curtime.tv_usec - initial_time.tv_usec;
+	tlblock->entries[tlblock->index].end = usec;
+    }
+#endif
+    tlblock->entries[tlblock->index].detail2 = detail;
+    ++(tlblock->index);
+    current_activity = NULL;
+}
+
+void
+dump_activity_trace(void)
+{
+    int i;
+    FILE *fp;
+    TimelineBlock *tlblock;
+    struct a_timeline_entry *entry;
+
+    tlblock = current_timeline_block;
+    if (tlblock == NULL)
+      return;
+    if ((fp = open_file("xconq.trace", "w")) != NULL) {
+	for (i = 0; i < tlblock->index; ++i) {
+	    entry = &(tlblock->entries[i]);
+	    fprintf(fp, "%d %d %s %d %d\n",
+		    entry->start, entry->end - entry->start, entry->type,
+		    entry->detail1, entry->detail2);
+	    if (tlblock->entries[i+1].start > entry->end + 5) {
+		fprintf(fp, "%d %d %s %d %d\n",
+			entry->end + 1,
+			tlblock->entries[i+1].start - entry->end,
+			"gap", 0, 0);
+	    }
+	}
+	fclose(fp);
+    }
+    /* Once we've dumped a trace, prepare to start over. */
+    starting_trace = TRUE;
+}
+
+/* Handling for various kinds of failures. */
+
 void
 #ifdef __STDC__
 init_error(char *str, ...)
@@ -848,10 +978,10 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9;
 }
 
 void
-log_warning(typ, str)
-char *typ, *str;
+log_warning(char *typ, char *str)
 {
     if (ewfp == NULL)
-      ewfp = fopen("Xconq.Warnings", "w");
-    fprintf(ewfp, "%s: %s\n", typ, str);
+      ewfp = open_file("Xconq.Warnings", "w");
+    if (ewfp != NULL)
+      fprintf(ewfp, "%s: %s\n", typ, str);
 }
