@@ -63,15 +63,7 @@ typedef struct a_area {
     short *tmp2;
     short *tmp3;
     struct a_region **regions;
-
-// --- USER LAYER FIX ----------------------------------------
-
     short *user;			/* layer of IDs of units using materials in each cell */
-
-//    struct a_unit **user;	/* Pointer to unit using the resources in a cell */
-
-// -------------------------------------------------------
-
 } Area;
 
 /* Named geographical features. */
@@ -147,24 +139,24 @@ typedef struct a_region_link {
 #define set_user_at(x, y, u) aset(area.user, x, y, u)
 
 /* Test if cell at (x1, y1) is within n steps from (x, y) */
-#define cell_is_within_n_steps(x, y, n, x1, y1) \
+#define cell_is_within_range(x, y, n, x1, y1) \
 	( (x) - (x1) > (n) || (x1) - (x) > (n) || (y) - (y1) > (n) \
 	|| (y1) - (y) > (n) || (x) + (y) - (x1) - (y1) > (n) \
 	|| (x1) + (y1) - (x) - (y) > (n) ) ? FALSE : TRUE
 
 /* Test if cell at (x1, y1) is within reach of unit u */
 #define cell_is_within_reach(u, x1, y1) \
-	cell_is_within_n_steps((u)->x, (u)->y, (u)->reach, (x1), (y1))
+	cell_is_within_range((u)->x, (u)->y, (u)->reach, (x1), (y1))
 
 /* Repeat for all cells at (x1, y1) within n steps from (x, y) */
-#define for_all_cells_within_n_steps(x, y, n, x1, y1) \
+#define for_all_cells_within_range(x, y, n, x1, y1) \
 	for((x1) = (x) - (n); (x1) <= (x) + (n); (x1)++) \
 	for((y1) = max((y) - (n), (y) - (n) + (x) - (x1)); \
 	       (y1) <= min((y) + (n), (y) + (n) + (x) - (x1)); (y1)++)
 
 /* Repeat for all cells at (x1, y1) within reach of unit u */
 #define for_all_cells_within_reach(u, x1, y1) \
-	for_all_cells_within_n_steps((u)->x, (u)->y, (u)->reach, (x1), (y1))
+	for_all_cells_within_range((u)->x, (u)->y, (u)->reach, (x1), (y1))
 
 /* The unit is a raw pointer - this macro is used a *lot*.  This could
    be space-optimized by using a 16-bit unit id. */
@@ -178,6 +170,31 @@ typedef struct a_region_link {
 #define for_all_stack(x,y,var)  \
   for ((var) = unit_at((x), (y)); (var) != NULL;  (var) = (var)->nexthere)
 
+/* Iterate through all units in this cell including occs within occs within occs. */
+
+/* This nifty little macro will climb the occupant : nexthere tree and follow all branches 
+four levels down to find all the occs within occs within occs within units in the stack. */ 
+
+#define for_all_stack_with_occs(x,y,var)  \
+  for ((var) = unit_at((x), (y)); \
+          (var) != NULL; \
+  	 (var) = ((var)->occupant != NULL ? \
+  	 	        (var)->occupant : \
+  		      ((var)->nexthere != NULL ? \
+  		        (var)->nexthere : \
+  		      ((var)->transport != NULL && \
+  		        (var)->transport->nexthere != NULL ? \
+  		        (var)->transport->nexthere : \
+  		      ((var)->transport != NULL && \
+  		        (var)->transport->transport != NULL && \
+  		        (var)->transport->transport->nexthere != NULL ? \
+  		        (var)->transport->transport->nexthere : \
+  		      ((var)->transport != NULL && \
+  		        (var)->transport->transport != NULL && \
+  		        (var)->transport->transport->transport != NULL && \
+  		        (var)->transport->transport->transport->nexthere != NULL ? \
+  		        (var)->transport->transport->transport->nexthere : NULL))))))
+
 /* Test if the terrain has been allocated yet. */
 
 #define terrain_defined() (area.terrain != NULL)
@@ -475,15 +492,9 @@ extern int fn_raw_cloud_at PARAMS ((int x, int y));
 extern int fn_raw_cloud_bottom_at PARAMS ((int x, int y));
 extern int fn_raw_cloud_height_at PARAMS ((int x, int y));
 extern int fn_raw_wind_at PARAMS ((int x, int y));
-
-// ---ADVANCED UNIT SUPPORT -------------------------------------------------------
-
 extern void allocate_area_users PARAMS ((void));
 extern int fn_user_at PARAMS ((int x, int y));
 extern void fn_set_user_at PARAMS ((int x, int y, int val));
-
-// -----------------------------------------------------------------------------
-
 extern void fn_set_terrain_at PARAMS ((int x, int y, int val));
 extern void fn_set_aux_terrain_at PARAMS ((int x, int y, int val));
 extern void fn_set_people_side_at PARAMS ((int x, int y, int val));
