@@ -7,8 +7,9 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern char *escaped_symbol PARAMS ((char *str));
-extern char *escaped_string PARAMS ((char *str));
+extern char *escaped_symbol(char *str);
+extern char *escaped_string(char *str);
+#include "ai.h"
 #include "imf.h"
 extern ImageFamily **recorded_imfs;
 extern int num_recorded_imfs;
@@ -17,117 +18,108 @@ extern int debugging_state_sync;
 
 #define key(x) (keyword_name(x))
 
-static void write_variants PARAMS ((Variant *varray));
-static void write_types PARAMS ((void));
-static void write_tables PARAMS ((int compress));
-static void write_type_name_list PARAMS ((int typ, int *flags, int dim));
+static void write_variants(Variant *varray);
+static void write_types(void);
+static void write_tables(int compress);
+static void write_type_name_list(int typ, int *flags, int dim);
 #if 0
-static void write_type_value_list PARAMS ((int typ, int *flags, int dim,
-					   int (*getter)(int, int), int i));
+static void write_type_value_list(int typ, int *flags, int dim,
+					   int (*getter)(int, int), int i);
 #endif
-static void write_table PARAMS ((char *name, int (*getter)(int, int),
-				 int dflt, int typ1, int typ2, int valtype,
-				 int compress));
-static void write_world PARAMS ((void));
-static void write_areas PARAMS ((Module *module));
-static void write_area_terrain PARAMS ((int compress));
-static void write_area_aux_terrain PARAMS ((int compress));
-static void write_area_features PARAMS ((int compress));
-static void write_area_elevations PARAMS ((int compress));
-static void write_area_people_sides PARAMS ((int compress));
-static void write_area_control_sides PARAMS ((int compress));
-static void write_area_materials PARAMS ((int compress));
-static void write_area_temperatures PARAMS ((int compress));
-static void write_area_clouds PARAMS ((int compress));
-static void write_area_winds PARAMS ((int compress));
+static void write_table(char *name, int (*getter)(int, int), int dflt,
+			int typ1, int typ2, int valtype, int compress);
+static void write_world(void);
+static void write_areas(Module *module);
+static void write_area_terrain(int compress);
+static void write_area_aux_terrain(int compress);
+static void write_area_features(int compress);
+static void write_area_elevations(int compress);
+static void write_area_people_sides(int compress);
+static void write_area_control_sides(int compress);
+static void write_area_materials(int compress);
+static void write_area_temperatures(int compress);
+static void write_area_clouds(int compress);
+static void write_area_winds(int compress);
+static void write_area_users(int compress);
+static void write_globals(void);
+static void write_scorekeepers(void);
+static void write_doctrines(void);
+static void write_sides(Module *module);
+static void write_side_properties(Side *side);
+static void write_standing_orders(Side *side);
+static int fn_terrain_view(int x, int y);
+static int fn_aux_terrain_view(int x, int y);
+static int fn_aux_terrain_view_date(int x, int y);
+static int fn_terrain_view_date(int x, int y);
+static int fn_unit_view(int x, int y);
+static int fn_unit_view_date(int x, int y);
+static int fn_material_view(int x, int y);
+static int fn_material_view_date(int x, int y);
+static int fn_temp_view(int x, int y);
+static int fn_temp_view_date(int x, int y);
+static int fn_cloud_view(int x, int y);
+static int fn_cloud_bottom_view(int x, int y);
+static int fn_cloud_height_view(int x, int y);
+static int fn_cloud_view_date(int x, int y);
+static int fn_wind_view(int x, int y);
+static int fn_wind_view_date(int x, int y);
+static void write_side_view(Side *side, int compress);
+static void write_one_side_view_layer(int propkey, int (*fn)(int x, int y));
+static void write_players(void);
+static void write_player(struct a_player *player);
+static void write_agreements(void);
+static void write_agreement(struct a_agreement *agreement);
+static void write_units(Module *module);
+static void write_unit_properties(Unit *unit);
+static void write_unit_act(Unit *unit);
+static void write_action(Action *action, int id);
+static void write_unit_plan(Unit *unit);
+static void write_task(Task *task);
+static void write_goal(Goal *goal, int keyword);
+static void write_rle(int (*datafn)(int, int), int lo, int hi,
+		      int (*translator)(int), int compress);
+static void write_run(int run, int val);
+static void write_namers(void);
+static void write_history(void);
+static void write_past_unit(PastUnit *pastunit);
+static void write_historical_event(HistEvent *hevt);
+static void write_images(void);
+static int reshaped_point(int x1, int y1, int *x2p, int *y2p);
+static int original_point(int x1, int y1, int *x2p, int *y2p);
 
-// --- ADVANCED UNIT SUPPORT -------------------------------------------
-
-static void write_area_users PARAMS ((int compress));
-
-// -----------------------------------------------------------------
-
-static void write_globals PARAMS ((void));
-static void write_scorekeepers PARAMS ((void));
-static void write_doctrines PARAMS ((void));
-static void write_sides PARAMS ((Module *module));
-static void write_side_properties PARAMS ((Side *side));
-static void write_standing_orders PARAMS ((Side *side));
-static int fn_terrain_view PARAMS ((int x, int y));
-static int fn_aux_terrain_view PARAMS ((int x, int y));
-static int fn_aux_terrain_view_date PARAMS ((int x, int y));
-static int fn_terrain_view_date PARAMS ((int x, int y));
-static int fn_unit_view PARAMS ((int x, int y));
-static int fn_unit_view_date PARAMS ((int x, int y));
-static int fn_temp_view PARAMS ((int x, int y));
-static int fn_temp_view_date PARAMS ((int x, int y));
-static int fn_cloud_view PARAMS ((int x, int y));
-static int fn_cloud_bottom_view PARAMS ((int x, int y));
-static int fn_cloud_height_view PARAMS ((int x, int y));
-static int fn_cloud_view_date PARAMS ((int x, int y));
-static int fn_wind_view PARAMS ((int x, int y));
-static int fn_wind_view_date PARAMS ((int x, int y));
-static void write_side_view PARAMS ((Side *side, int compress));
-static void write_one_side_view_layer PARAMS ((int propkey,
-					       int (*fn)(int x, int y)));
-static void write_players PARAMS ((void));
-static void write_player PARAMS ((struct a_player *player));
-static void write_agreements PARAMS ((void));
-static void write_agreement PARAMS ((struct a_agreement *agreement));
-static void write_units PARAMS ((Module *module));
-static void write_unit_properties PARAMS ((Unit *unit));
-static void write_unit_act PARAMS ((Unit *unit));
-static void write_action PARAMS ((Action *action, int id));
-static void write_unit_plan PARAMS ((Unit *unit));
-static void write_task PARAMS ((Task *task));
-static void write_goal PARAMS ((Goal *goal, int keyword));
-static void write_rle PARAMS ((int (*datafn)(int, int), int lo, int hi,
-			       int (*translator)(int), int compress));
-static void write_run PARAMS ((int run, int val));
-static void write_namers PARAMS ((void));
-static void write_history PARAMS ((void));
-static void write_past_unit PARAMS ((PastUnit *pastunit));
-static void write_historical_event PARAMS ((HistEvent *hevt));
-static void write_images PARAMS ((void));
-static int reshaped_point PARAMS ((int x1, int y1, int *x2p, int *y2p));
-static int original_point PARAMS ((int x1, int y1, int *x2p, int *y2p));
-
-static void start_form PARAMS ((char *hd));
-static void add_to_form PARAMS ((char *x));
-static void add_to_form_no_space PARAMS ((char *x));
-static void add_char_to_form PARAMS ((int x));
-static void add_num_to_form PARAMS ((int x));
-static void add_num_to_form_no_space PARAMS ((int x));
-static void add_num_or_dice_to_form PARAMS ((int x, int valtype));
-static void add_form_to_form PARAMS ((Obj *x));
-static void end_form PARAMS ((void));
-static void newline_form PARAMS ((void));
-static void space_form PARAMS ((void));
-static void add_to_packet PARAMS ((int id, char *str));
-static void flush_write PARAMS ((void));
-static void write_bool_prop PARAMS ((char *name, int value,
-				     int dflt, int nodefaulting,
-				     int addnewline));
-static void write_num_prop PARAMS ((char *name, int value,
-				    int dflt, int nodefaulting,
-				    int addnewline));
-static void write_str_prop PARAMS ((char *name, char *value,
-				    char *dflt, int nodefaulting,
-				    int addnewline));
-static int string_not_default PARAMS ((char *str, char *dflt));
-static void write_lisp_prop PARAMS ((char *name, struct a_obj *value,
-				     struct a_obj *dflt, int nodefaulting,
-				     int as_cdr, int addnewline));
-static void write_utype_value_list PARAMS ((char *name, short *arr,
-					    int dflt, int addnewline));
-static void write_mtype_value_list PARAMS ((char *name, short *arr,
-					    int dflt, int addnewline));
-static void write_atype_value_list PARAMS ((char *name, short *arr,
-					    int dflt, int addnewline));
-static void write_side_value_list PARAMS ((char *name, short *arr,
-					   int dflt, int addnewline));
-static void write_utype_string_list PARAMS ((char *name, char **arr,
-					     char *dflt, int addnewline));
+static void start_form(char *hd);
+static void add_to_form(char *x);
+static void add_to_form_no_space(char *x);
+static void add_char_to_form(int x);
+static void add_num_to_form(int x);
+static void add_num_to_form_no_space(int x);
+static void add_num_or_dice_to_form(int x, int valtype);
+static void add_form_to_form(Obj *x);
+static void end_form(void);
+static void newline_form(void);
+static void space_form(void);
+static void add_to_packet(int id, char *str);
+static void flush_write(void);
+static void write_bool_prop(char *name, int value,
+			    int dflt, int nodefaulting, int addnewline);
+static void write_num_prop(char *name, int value,
+			   int dflt, int nodefaulting, int addnewline);
+static void write_str_prop(char *name, char *value,
+			   char *dflt, int nodefaulting, int addnewline);
+static int string_not_default(char *str, char *dflt);
+static void write_lisp_prop(char *name, struct a_obj *value,
+			    struct a_obj *dflt, int nodefaulting,
+			    int as_cdr, int addnewline);
+static void write_utype_value_list(char *name, short *arr,
+				   int dflt, int addnewline);
+static void write_mtype_value_list(char *name, short *arr,
+				   int dflt, int addnewline);
+static void write_atype_value_list(char *name, short *arr,
+				   int dflt, int addnewline);
+static void write_side_value_list(char *name, short *arr,
+				  int dflt, int addnewline);
+static void write_utype_string_list(char *name, char **arr,
+				    char *dflt, int addnewline);
 
 /* The pointer to the file being written to. */
 
@@ -226,7 +218,7 @@ Module *module;
 	return FALSE;
     }
     if (module->filename != NULL) {
-	wfp = fopen(module->filename, "w");
+	wfp = open_file(module->filename, "w");
     } else {
 	wfp = NULL;
     }
@@ -590,7 +582,7 @@ int typ, *flags, dim;
 
 static void
 write_type_value_list(typ, flags, dim, getter, i)
-int typ, *flags, dim, (*getter) PARAMS ((int, int)), i;
+int typ, *flags, dim, (*getter)(int, int), i;
 {
     int j, first = TRUE, listlen = 0;
 
@@ -619,9 +611,10 @@ struct histo { int count, val; };
 
 /* Sort into *descending* order by count. */
 
+static int histo_compare(CONST void *x, CONST void *y);
+
 static int
-histo_compare(x, y)
-CONST void *x, *y;
+histo_compare(CONST void *x, CONST void *y)
 {
     return ((struct histo *) y)->count - ((struct histo *) x)->count;
 }
@@ -633,7 +626,7 @@ CONST void *x, *y;
 static void
 write_table(name, getter, dflt, typ1, typ2, valtype, compress)
 char *name;
-int (*getter) PARAMS ((int, int)), dflt, typ1, typ2, valtype, compress;
+int (*getter)(int, int), dflt, typ1, typ2, valtype, compress;
 {
     int i, j, k, colvalue, constcol, next;
     int numrandoms, nextrowdiffers, writeconst;
@@ -915,19 +908,11 @@ Module *module;
       write_area_clouds(compress);
     if (all || module->def_area_weather)
       write_area_winds(compress);
-
-// ---ADVANCED UNIT SUPPORT -------------------------------------------------------
-
    /* It is alloc_area_terrain that handles the user layer. */
    if (all || module->def_area_terrain)
       write_area_users(compress);
-
-// -----------------------------------------------------------------------------
-
 }
 
-// ---ADVANCED UNIT SUPPORT -------------------------------------------------------
-
 static void
 write_area_users(int compress)
 {
@@ -939,7 +924,7 @@ write_area_users(int compress)
 	space_form();
 	start_form(key(K_USER));
 	newline_form();
-	write_rle(fn_user_at, NOUSER, nextid - 1, NULL, compress);
+	write_rle(fn_user_at, NOUSER, nextid, NULL, compress);
 	space_form();
 	space_form();
 	end_form();
@@ -947,8 +932,6 @@ write_area_users(int compress)
 	newline_form();
 }
 
-// -----------------------------------------------------------------------------
-
 static void
 write_area_terrain(compress)
 int compress;
@@ -1051,9 +1034,10 @@ int compress;
     newline_form();
 }
 
+static int fn_elevation_at_offset(int x, int y);
+
 static int
-fn_elevation_at_offset(x, y)
-int x, y;
+fn_elevation_at_offset(int x, int y)
 {
     return (elev_at(x, y) - area.minelev);
 }
@@ -1253,8 +1237,7 @@ int compress;
 }
 
 static void
-write_area_winds(compress)
-int compress;
+write_area_winds(int compress)
 {
     if (winds_defined()) {
 	start_form(key(K_AREA));
@@ -1274,10 +1257,10 @@ int compress;
    if they match the compiled-in defaults. */
 
 static void
-write_globals()
+write_globals(void)
 {
     int i, complete = FALSE;
-    Obj *val, *dfltval;
+    Obj *val;
     time_t now;
 
     /* Snapshot realtime values, but only after they've been set up. */
@@ -1468,15 +1451,9 @@ Side *side;
     write_num_prop(key(K_PLAYER), (side->player ? side->player->id : 0), 0, FALSE, TRUE);
     write_num_prop(key(K_DEFAULT_DOCTRINE),
     		   (side->default_doctrine ? side->default_doctrine->id : 0), 0, FALSE, TRUE);
-
-// ---ADVANCES AND TREASURY SUPPORT -------------------------------------------------
-
     write_num_prop(key(K_NUMADVANCES), side->numadvances, 0, FALSE, TRUE);
     write_atype_value_list(key(K_ADVANCE), side->advance, 0, TRUE);
     write_mtype_value_list(key(K_TREASURY), side->treasury, 0, TRUE);
-
-// -----------------------------------------------------------------------------
-
     anyudoctrines = FALSE;
     if (side->udoctrine != NULL) {
       for_all_unit_types(u) {
@@ -1846,7 +1823,7 @@ write_side_view(side, compress)
 
 static void
 write_one_side_view_layer(propkey, fn)
-int propkey, (*fn) PARAMS ((int x, int y));
+int propkey, (*fn)(int x, int y);
 {
     newline_form();
     start_form(key(K_SIDE));
@@ -1897,10 +1874,10 @@ Player *player;
 static void
 write_agreements()
 {
-    Agreement *agreement;
+    Agreement *ag;
 
-    for (agreement = agreementlist; agreement != NULL; agreement = agreement->next) {
-	write_agreement(agreement);
+    for_all_agreements(ag) {
+	write_agreement(ag);
     }
 }
 
@@ -2000,18 +1977,7 @@ Module *module;
 		    || (unit->occupant
 			&& unit->name == NULL
 			&& unit_symbol(unit) == lispnil))
-
-// ---  SAVE GAME BUG FIX ----------------------------------------------------------
-
-/* Now when unit id (nextid) starts with 0, we can't pass 0 as dflt to write_num_prop. If we do, 
-unit 0 will be assigned an already occupied id when the saved game is launced, causing a crash. */
-
-		  write_num_prop(key(K_SHARP), unit->id, NOUNIT, FALSE, FALSE);
-
-//		  write_num_prop(key(K_SHARP), unit->id, 0, FALSE, FALSE);
-
-// -----------------------------------------------------------------------------
-
+		  	write_num_prop(key(K_SHARP), unit->id, 0, FALSE, FALSE);
 		/* Need this to get back into the right transport. */
 		if (unit->transport) {
 		    if (unit_symbol(unit->transport) != lispnil
@@ -2071,9 +2037,7 @@ Unit *unit;
     /* (should do appear x,y also here?) */
     write_num_prop(key(K_DISAPPEAR), unit_disappear_turn(unit), -1, FALSE, FALSE);
     write_lisp_prop(key(K_X), unit_hook(unit), lispnil, FALSE, TRUE, FALSE);
-
-// ---ADVANCED UNIT SUPPORT -------------------------------------------------------
-
+	/* Advanced unit support. */
 	if (u_advanced(unit->type)) {
 		write_num_prop(key(K_SIZE), unit->size, 0, FALSE, FALSE);		
 		write_num_prop(key(K_REACH), unit->reach, 2, FALSE, FALSE);		
@@ -2082,12 +2046,11 @@ Unit *unit;
 		write_num_prop(key(K_CURADVANCE), unit->curadvance, -1, FALSE, FALSE);		
 		write_num_prop(key(K_POPULATION), unit->population, 1, FALSE, FALSE);		
     		write_mtype_value_list(key(K_PRODUCTION), unit->production, 0, FALSE);
+		write_num_prop(key(K_AUTOPLAN), unit->autoplan, FALSE, FALSE, FALSE);		
+		write_num_prop(key(K_CP_STASH), unit->cp_stash, FALSE, FALSE, FALSE);		
 		write_num_prop(key(K_AUTOBUILD), unit->autobuild, FALSE, FALSE, FALSE);		
 		write_num_prop(key(K_AUTORESEARCH), unit->autoresearch, FALSE, FALSE, FALSE);		
 	}
-				
-// ------------------------------------------------------------------------------
-
 }
 
 /* Write out the unit's current actor state. */
@@ -2334,27 +2297,26 @@ write_namers()
 /* The comparison function for the image list just does "strcmp" order
    and *requires* that all image families be named and named uniquely. */
 
-static void sort_all_recorded_images PARAMS ((void));
+static void sort_all_recorded_images(void);
+
+static int image_name_compare(CONST void *imf1, CONST void *imf2);
 
 static int
-image_name_compare(imf1, imf2)
-#ifdef THINK_C
-const
-#endif
-void *imf1, *imf2;
+image_name_compare(CONST void *imf1, CONST void *imf2)
 {
     return strcmp((*((ImageFamily **) imf1))->name,
 		  (*((ImageFamily **) imf2))->name);
 }
 
 static void
-sort_all_recorded_images()
+sort_all_recorded_images(void)
 {
-    qsort(&(recorded_imfs[0]), num_recorded_imfs, sizeof(ImageFamily *), image_name_compare);
+    qsort(&(recorded_imfs[0]), num_recorded_imfs, sizeof(ImageFamily *),
+	  image_name_compare);
 }
 
 static void
-write_images()
+write_images(void)
 {
     int i;
 
@@ -2373,7 +2335,7 @@ write_images()
 
 static void
 write_rle(datafn, lo, hi, translator, compress)
-int (*datafn) PARAMS ((int, int)), lo, hi, (*translator) PARAMS ((int)), compress;
+int (*datafn)(int, int), lo, hi, (*translator)(int), compress;
 {
     int width, height, x, y, x0, y0, run, runval, val, trval;
     int numbad = 0;
