@@ -10,9 +10,8 @@ any later version.  See the file COPYING.  */
 #include "print.h"
 #include "kpublic.h"
 #include "macconq.h"
-extern void enable_command(void);
+
 extern void set_construction_run_length(int len);
-extern Map *front_map(void);
 extern void do_idle_command(int n);
 extern int do_down_command(void);
 extern int do_up_command(void);
@@ -140,7 +139,8 @@ get_a_position(Map **mapp, int *xp, int *yp, int *dirp)
 		position_set_modally = FALSE;
 		*mapp = modal_map;
 		return m_nearest_boundary(*mapp, modal_point.h, modal_point.v, xp, yp, dirp);
-	} else if ((*mapp = map_from_window(FrontWindow())) != NULL) {
+	} else if ((*mapp = frontmap) != NULL) {
+
 		GetMouse(&at);
 		return m_nearest_boundary(*mapp, at.h, at.v, xp, yp, dirp);
 	}
@@ -156,7 +156,8 @@ get_a_unit(Map **mapp, Unit **unitp)
 		position_set_modally = FALSE;
 		*mapp = modal_map;
 		return m_nearest_unit(*mapp, modal_point.h, modal_point.v, unitp);
-	} else if ((*mapp = map_from_window(FrontWindow())) != NULL) {
+	} else if ((*mapp = frontmap) != NULL) {
+
 		GetMouse(&at);
 		return m_nearest_unit(*mapp, at.h, at.v, unitp);
 	}
@@ -175,7 +176,8 @@ get_a_visible_unit(Map **mapp, Unit **unitp)
 		position_set_modally = FALSE;
 		*mapp = modal_map;
 		rslt = m_nearest_unit(*mapp, modal_point.h, modal_point.v, unitp);
-	} else if ((*mapp = map_from_window(FrontWindow())) != NULL) {
+	} else if ((*mapp = frontmap) != NULL) {
+
 		GetMouse(&at);
 		rslt = m_nearest_unit(*mapp, at.h, at.v, unitp);
 	}
@@ -325,10 +327,11 @@ void
 do_build(Side *side)
 {
 	int i;
-	Map *map;  List *list;  UnitCloseup *closeup;
+	List *list;  
+	UnitCloseup *closeup;
 	Unit *unit;
 
-	list = list_from_window(FrontWindow());
+	list = list_from_window(topWindow);
 	if (list != NULL) {
 		unit = selected_unit_in_list(list);
 		if (unit != NULL && can_build_or_help(unit)) {
@@ -337,7 +340,7 @@ do_build(Side *side)
 		}
 		return;
 	}
-	closeup = unit_closeup_from_window(FrontWindow());
+	closeup = unit_closeup_from_window(topFloat);
 	if (closeup != NULL) {
 		unit = closeup->unit;
 		if (unit != NULL && can_build_or_help(unit)) {
@@ -346,10 +349,9 @@ do_build(Side *side)
 		}
 		return;
 	}
-	map = front_map();
-	if (map != NULL) {
-		for (i = 0; i < map->numselections; ++i) {
-			unit = map->selections[i];
+	if (frontmap) {
+		for (i = 0; i < frontmap->numselections; ++i) {
+			unit = frontmap->selections[i];
 			if (unit != NULL && can_build_or_help(unit)) {
 				unit_do_build_2(unit);
 				return;
@@ -367,14 +369,14 @@ unit_do_build_2(Unit *unit)
     extern int editedrunlength;
 
 	modal_construction = TRUE;
-	window_behind_construction = FrontWindow();
+	window_behind_construction = topWindow;
 	if (dside->prefixarg > 0)
 	  set_construction_run_length(dside->prefixarg);
 	if (dside->prefixarg > 0)
 	  editedrunlength = dside->prefixarg;
 	else
 	  editedrunlength = -1;
-	enable_construction();
+	show_construction_window();
 	select_unit_in_construction_window(unit);
 	select_type_in_construction_window(favored_type(unit));
 }
@@ -382,17 +384,16 @@ unit_do_build_2(Unit *unit)
 /* Create and/or bring up the construction planning window. */
 
 void
-enable_construction()
+show_construction_window()
 {
-	window_behind_construction = FrontWindow();
+	window_behind_construction = topWindow;
 	if (constructionwin == nil) {
 		create_construction_window();
-	}
-	if (constructionwin != nil) {
+	} else {
 		reinit_construction_lists();
-		ShowWindow(constructionwin);
-		SelectWindow(constructionwin);
 	}
+	SelectTheWindow(constructionwin);
+	ActivateWindow(constructionwin, true);
 }
 
 void
@@ -451,7 +452,7 @@ do_one_collect(Unit *unit)
 void
 do_copying(Side *side)
 {
-	help_dialog(copying_help_node);
+	show_help_window(copying_help_node);
 }
 
 void
@@ -541,10 +542,8 @@ void
 do_dir(Side *side)
 {
 	int ndirs, dir1, dir2, modif;
-	Map *map;
 
-	map = front_map();
-	if (map != NULL) {
+	if (frontmap) {
 		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
 		if (ndirs >= 1) {
 			tmpdir = dir1;
@@ -553,8 +552,8 @@ do_dir(Side *side)
 			return;
 		}
 		apply_to_all_selected(do_one_dir_move, TRUE);
-		if (map->moveonclick)
-		  map->scrolltocurunit = TRUE;
+		if (frontmap->moveonclick)
+		  frontmap->scrolltocurunit = TRUE;
 	}
 }
 
@@ -578,10 +577,8 @@ void
 do_dir_multiple(Side *side)
 {
 	int ndirs, dir1, dir2, modif;
-	Map *map;
 
-	map = front_map();
-	if (map != NULL) {
+	if (frontmap) {
 		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
 		if (ndirs >= 1) {
 			tmpdir = dir1;
@@ -590,8 +587,8 @@ do_dir_multiple(Side *side)
 			return;
 		}
 		apply_to_all_selected(do_one_dir_multiple_move, TRUE);
-		if (map->moveonclick)
-		  map->scrolltocurunit = TRUE;
+		if (frontmap->moveonclick)
+		  frontmap->scrolltocurunit = TRUE;
 	}
 }
 
@@ -716,7 +713,7 @@ do_end_turn(Side *side)
 {
 	/* <return> is also interpreted by dialogs, so special-case this, depending
 	   on which window was in front. */
-	if (FrontWindow() == constructionwin) {
+	if (topWindow == constructionwin) {
 		Point pt;
 		extern ControlHandle constructbutton;
 		
@@ -791,15 +788,12 @@ do_one_fire_into(Unit *atker)
 void
 do_follow_action(Side *side)
 {
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		map->follow_action = !map->follow_action;
-		if (map->follow_action)
-		  notify(dside, "Following the action on map %d", map->id);
+	if (frontmap) {
+		frontmap->follow_action = !frontmap->follow_action;
+		if (frontmap->follow_action)
+		  notify(dside, "Following the action on map %d", frontmap->id);
 		else
-		  notify(dside, "No longer following action on map %d", map->id);
+		  notify(dside, "No longer following action on map %d", frontmap->id);
 	} else {
 		beep();
 	}
@@ -860,7 +854,7 @@ do_help(Side *side)
 {
     /* Bring up the help window at wherever it was, if already
        created, otherwise will start at topics node. */
-	help_dialog(NULL);
+	show_help_window(NULL);
 }
 
 void
@@ -997,14 +991,12 @@ int
 do_one_occupant(Unit *unit)
 {
 	Unit *nextocc;
-	Map *map;
 
 	nextocc = find_next_occupant(unit);
 	if (nextocc != unit) {
-		map = map_from_window(FrontWindow());
-		if (map != NULL) {
-			unselect_unit_on_map(map, unit);
-			select_unit_on_map(map, nextocc);
+		if (frontmap) {
+			unselect_unit_on_map(frontmap, unit);
+			select_unit_on_map(frontmap, nextocc);
 			/* Overkill, but draw_row alone not as desirable? */
 			update_cell_display(dside, unit->x, unit->y, TRUE);
 		}
@@ -1027,7 +1019,7 @@ do_other(Side *side)
 	if (!doingother) {
 		doingother = TRUE;
 		other_command_prefixarg = side->prefixarg;
-		enable_command();
+		show_command_window();
 		other_command_prefixarg = -1;
 		doingother = FALSE;
 	} else {
@@ -1045,37 +1037,35 @@ TEHandle command_text = nil;
 
 Rect commandtextrect;
 
-void create_command_dialog(void);
+void create_command_window(void);
 
 static void get_command_and_do(void);
 
 void
-enable_command()
+show_command_window()
 {
 	if (commandwin == nil) {
-		create_command_dialog();
-	}
-	if (commandwin != nil) {
-		ShowWindow(commandwin);
-		SelectWindow(commandwin);
+		create_command_window();
 	}
+	SelectTheWindow(commandwin);
+	ActivateWindow(commandwin, true);
 }
 
 void
-create_command_dialog()
+create_command_window()
 {
 	Rect tmprect;
 
 	if (hasColorQD) {
-		commandwin = GetNewCWindow(wCommand, NULL, (WindowPtr) -1L);
+		commandwin = GetNewCWindow(wCommand, NULL, NULL);
 	} else {
-		commandwin = GetNewWindow(wCommand, NULL, (WindowPtr) -1L);
+		commandwin = GetNewWindow(wCommand, NULL, NULL);
 	}
 	dobutton = GetNewControl(cCommandDoButton, commandwin);
 	SetPort(commandwin);
 	tmprect = commandwin->portRect;
 	commandtextrect = tmprect;
-	commandtextrect.bottom -= 30;
+	commandtextrect.bottom -= 25;
 	InsetRect(&commandtextrect, 5, 5);
 	/* Create and clear the TextEdit record. */
 	command_text = TENew(&commandtextrect, &commandtextrect);
@@ -1086,7 +1076,7 @@ create_command_dialog()
 		sprintf(spbuf, "%d ", other_command_prefixarg);
 		TEInsert(spbuf, strlen(spbuf), command_text);
 	}
-	ShowWindow(commandwin);
+	SelectTheWindow(commandwin);
 }
 
 void
@@ -1129,6 +1119,7 @@ do_mouse_down_command(Point mouse, int mods)
 
 	part = FindControl(mouse, commandwin, &control);
 	if (control == dobutton) {
+		TrackControl(control, mouse, NULL);
 		get_command_and_do();
 	} else if (PtInRect(mouse, &commandtextrect)) {
 		TEClick(mouse, mods, command_text);
@@ -1166,11 +1157,12 @@ void
 do_produce(Side *side)
 {
 	int i;
-	Map *map;  List *list;  UnitCloseup *closeup;
+	List *list;  
+	UnitCloseup *closeup;
 	Unit *unit;
 
 	/* Try different alternatives to find a unit. */
-	list = list_from_window(FrontWindow());
+	list = list_from_window(topWindow);
 	if (list != NULL) {
 		unit = selected_unit_in_list(list);
 		if (unit != NULL && can_produce(unit)) {
@@ -1179,7 +1171,7 @@ do_produce(Side *side)
 		}
 		return;
 	}
-	closeup = unit_closeup_from_window(FrontWindow());
+	closeup = unit_closeup_from_window(topFloat);
 	if (closeup != NULL) {
 		unit = closeup->unit;
 		if (unit != NULL && can_produce(unit)) {
@@ -1188,10 +1180,9 @@ do_produce(Side *side)
 		}
 		return;
 	}
-	map = front_map();
-	if (map != NULL) {
-		for (i = 0; i < map->numselections; ++i) {
-			unit = map->selections[i];
+	if (frontmap) {
+		for (i = 0; i < frontmap->numselections; ++i) {
+			unit = frontmap->selections[i];
 			if (unit != NULL && can_produce(unit)) {
 				unit_do_produce_2(unit);
 				return;
@@ -1239,12 +1230,8 @@ void
 do_recenter(Side *side)
 {
 	int avgx, avgy;
-	Map *map;
 
-	map = front_map();
-	/* Only apply recentering if the front map window is also
-	   the overall front window. */
-	if (map != NULL && map->window == FrontWindow()) {
+	if (frontmap) {
 		allsumx = allsumy = 0;
 		unitcount = 0;
 		apply_to_all_selected(add_unit_position, FALSE);
@@ -1253,20 +1240,10 @@ do_recenter(Side *side)
 			return;
 		}
 		avgx = allsumx / unitcount;  avgy = allsumy / unitcount;
-
-// --- CALL NEW FUNCTION INSTEAD ---------------------------
-
-		set_map_focus(map, avgx, avgy);
-
-//		set_focus(map, avgx, avgy);
-
-// ----------------------------------------------------
-
+		set_map_focus(frontmap, avgx, avgy);
 	}
 }
 
-// --- SET_FOCUS REPLACED BY SET_MAP_FOCUS IN MACMAP.C --------
-
 /* Recalculate and redraw everything. */
 
 void
@@ -1456,32 +1433,29 @@ do_set_formation(Side *side)
 	}
 }
 
-// --- NOW CALLS SET_MAP_ANGLE WHICH DOES THE JOB ------------------------------
-
 void
 do_set_view_angle(Side *side)
 {
 	int angle;
-	Map *map;
 
-	map = front_map();
-	if (map != NULL) {
-		angle = map->vp->angle;
+	if (frontmap) {
+		angle = frontmap->vp->angle;
 		if (angle == 90)
 		  angle = 30;
 		else if (angle == 30)
 		  angle = 15;
 		else
 		  angle = 90;
-		map->vp->vertscale = (side->prefixarg < 1 ? 1 : side->prefixarg);
-		set_map_angle(map, angle);
+		/* A suitable vertscale is now computed by set_map_angle. 
+		Only override it if provided with a valid prefixarg. */
+		set_map_angle(frontmap, angle);
+		frontmap->vp->vertscale 
+		   = (side->prefixarg < 1 ? frontmap->vp->vertscale : side->prefixarg);
 	} else {
 		beep();
 	}
 }
 
-// ---------------------------------------------------------------------
-
 void
 do_sleep(Side *side)
 {
@@ -1520,11 +1494,8 @@ do_standing_orders(Side *side)
 void 
 do_survey(Side *side)
 {
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		toggle_survey(map);
+	if (frontmap) {
+		toggle_survey(frontmap);
 	} else {
 		beep();
 	}
@@ -1583,17 +1554,14 @@ do_wake_all(Side *side)
 void
 do_warranty(Side *side)
 {
-	help_dialog(warranty_help_node);
+	show_help_window(warranty_help_node);
 }
 
 void
 do_zoom_in(Side *side)
 {
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		magnify_map(map, 1);
+	if (frontmap) {
+		magnify_map(frontmap, 1);
 	} else {
 		beep();
 	}
@@ -1602,11 +1570,8 @@ do_zoom_in(Side *side)
 void
 do_zoom_out(Side *side)
 {
-	Map *map;
-
-	map = front_map();
-	if (map != NULL) {
-		magnify_map(map, -1);
+	if (frontmap) {
+		magnify_map(frontmap, -1);
 	} else {
 		beep();
 	}
