@@ -18,7 +18,6 @@ any later version.  See the file COPYING.  */
 
 /* --- FUNCTIONS PROTOTYPES -------------------------------------------------*/
 
-extern ImageFamily *get_generic_images PARAMS ((Side *side, char *name));
 extern CIconHandle mac_create_cicn(Image *img);
 extern void mac_init_cicn(Image *img);
 static void build_colors_popup_menu(MenuHandle popMenu);
@@ -40,13 +39,11 @@ int default_featureborders = TRUE;	/* Draw feature (province) borders */
 int default_featurenames = TRUE;		/* Draw feature (province) names */
 int default_simple_borders = TRUE;	/* Do not draw borders to water or to empty lands */
 int default_shorelines = TRUE;		/* Draw shorelines in 3D relief */
-
-// --- USER CONTROL OF EMBLEMS -----------------------------------------
-
+int default_draw_ai = TRUE;		/* Draw AI regions (if present). */
+int alert_256_colors = TRUE;		/* Suggest switch to 256 colors at start. */
+int default_draw_scrollbars = FALSE;	/* Draw scrollbars in maps. */
+int default_draw_cpanel = TRUE;		/* Draw control panel in maps. */
 int default_draw_emblems = TRUE;	/* Draw unit emblems in maps and lists. */
-
-// ------------------------------------------------------------------
-
 int default_draw_topline = TRUE;		/* Draw top line boxes in maps */
 int default_draw_topunit = TRUE;		/* Draw unit info boxes in maps */
 int default_drawothermaps = TRUE;	/* Draw other maps as boxes */
@@ -297,7 +294,7 @@ get_color(char* name)
 	     return blackcolor;
 
 	/* Find (or create) the image family. */
-	imf = get_generic_images(dside, name);
+	imf = get_generic_images(name);
 
 	/* Needed only in the presence of mac-lib. */ 
     	record_imf_get(imf);			
