@@ -89,6 +89,10 @@ int foundimagesfile = FALSE;
 
 int sbarwid = 15;
 
+/* The width of a floating window scrollbar. */
+
+int floatsbarwid = 10;
+
 /* Info about various fonts we use. */
 
 short small_font_id;
@@ -226,11 +230,13 @@ main()
 	init_library_path(NULL);
 	init_ae();
 	/* Acquire Mac-specific files (preferences and resources). */
-	/* Check if any screen has more than 256 colors and propose reduction. */	
-	check_screen_depths();
 	/* Load colors used in absence of saved preferences. */
 	set_default_colors();
 	get_files();
+	/* Check if any screen has more than 256 colors and propose reduction. */	
+	/* Don't check screen if this function has been turned off. */
+	if (alert_256_colors)
+		check_screen_depths();
 	/* A hack to ensure some memory available for error handling. */
 	spare = NewHandle(2000);
 	/* If no Apple Events, go to the splash screen now, otherwise we'll wait
@@ -290,6 +296,7 @@ init_toolbox()
 	InitFonts();
 	FlushEvents(everyEvent, 0);
 	InitWindows();
+	InitFloats();
 	InitMenus();
 	TEInit();
 	InitDialogs(NULL);
@@ -521,24 +528,17 @@ event_loop()
 			SystemTask();
 			gotevent = GetNextEvent(everyEvent, &event);
 		}
-		/* First decide if the event is for a dialog or is just any old event. */
-		if (FrontWindow() != nil && IsDialogEvent(&event)) {
-			/* Handle all the modeless dialogs here. */
-			/* Process any player setup window events that we want to handle
-			   specially (such as keystrokes). */
-			if (playersetupwin != nil && playersetupwin == FrontWindow())
-			  handle_player_setup_event(&event);
-			/* Avoid handling any dialog events to other than the windows that
-			   we know to be dialogs. */
-			if (!(instructionswin == FrontWindow()
-				  || playersetupwin == FrontWindow()))
-			  continue;  
+		/* First test if the event is for playersetupwin, the only modeless dialog left. */
+		if (topWindow 
+		     && playersetupwin == topWindow 
+		     && IsDialogEvent(&event)) {
+			/* Process events we want to handle specially, such as keystrokes. */
+		  	handle_player_setup_event(&event);
+			/* Stop if the dialog just was closed, or DialogSelect will crash! */
+			if (playersetupwin == NULL)
+			    continue;
+			/* Process events that the Dialog Manager handles, such as clicks on buttons. */
 			if (DialogSelect(&event, &dialog, &itemhit)) {
-				if (dialog == instructionswin) {
-					hit_instructions_dialog(dialog, itemhit, &event);
-				}
-				/* Process player setup window events that the Dialog
-				   Manager handles (clicks on buttons for instance). */
 				if (dialog == playersetupwin) {
 					mouse = event.where;
 					SetPort(playersetupwin);
@@ -577,7 +577,7 @@ event_loop()
 			receive_data(0);
 		}
 		/* Blink TE cursors regularly. */
-		if (commandwin != nil && FrontWindow() == commandwin) {
+		if (commandwin != nil && topWindow == commandwin) {
 			TEIdle(command_text);
 		}
 		if (ok_to_exit && want_to_exit) {
@@ -614,6 +614,10 @@ char mouseoverbuf[100];
 
 CursHandle current_cursor;
 
+/* Delay before autoscrolling starts. */
+
+static int delay;
+
 void
 adjust_cursor(Point mouse, RgnHandle region)
 {
@@ -623,60 +627,97 @@ adjust_cursor(Point mouse, RgnHandle region)
 	Map *map;
 	GrafPtr oldport;
 
-	map = map_from_window(FrontWindow());
+	map = frontmap;
 	if (map != NULL) {
 		GetPort(&oldport);
 		SetPort(map->window);
 		GlobalToLocal(&mouse);
-		if (mouse.h > conwid && mouse.h < map->window->portRect.right - sbarwid
-			&& mouse.v > map->toph && mouse.v < map->window->portRect.bottom - sbarwid) {
-			if (map_modal != NO_MODAL) {
-				switch (map_modal) {
-					case ATTACK_MODAL:
-					case FIRE_MODAL:
-					case FIRE_INTO_MODAL:
-						current_cursor = firecursor;
-						break;
-					case MOVE_TO_MODAL:
-					case SET_FORMATION_MODAL:
-					case ADD_TERRAIN_MODAL:
-					case REMOVE_TERRAIN_MODAL:
-					case DISTANCE_MODAL:
-					case ZOOM_MODAL:
-					case GENERIC_MODAL:
-						current_cursor = opencrosscursor;
-						break;
-					default:
-						run_error("unknown modal tool %d", map_modal);
-						break;
-				}
+
+		/* Don't do cursor adjustment for frontmap if we are in a floating window! */ 
+		if (PtInRgn(mouse, map->window->visRgn) != TRUE) {
+	  		SetCursor(&QD(arrow));
+			return;
+		}
+
+		if (mouse.h > map->conw && mouse.h < map->window->portRect.right - map->sbarwid
+			&& mouse.v > map->toph && mouse.v < map->window->portRect.bottom - map->sbarwid) {
+
+			if (mouse.h < map->conw + 40) {
+				current_cursor = leftcursor;				
 				usual = FALSE;
-#ifdef DESIGNERS
-			} else if (dside->designer && tooltype != notool) {
-				current_cursor = adjust_designer_cursor(mouse, region);  usual = FALSE;
-#endif DESIGNERS
-			} else if (map->moveonclick) {
-				if (map->numselections == 1
-				    && (unit = map->selections[0]) != NULL) {
-					/* Calculate the approx dir to here from selected unit. */
-					m_nearest_cell(map, mouse.h, mouse.v, &x, &y);
-					/* Note that we allow x,y here to be outside the world. */
-					if (mobile(unit->type)
-					    && (approxdir = approx_dir(x - unit->x, y - unit->y)) >= 0) {
-						current_cursor = movecursors[approxdir];  usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					scroll_map_window(map, -4, 0);
+			} else if (mouse.h > map->window->portRect.right - map->sbarwid - 40) {
+				current_cursor = rightcursor;
+				usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					scroll_map_window(map, 4, 0);
+			} else if (mouse.v < map->toph + 40) {
+				current_cursor = upcursor;
+				usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					scroll_map_window(map, 0, -4);
+			} else if (mouse.v > map->window->portRect.bottom - map->sbarwid - 40) {
+				current_cursor = downcursor;
+				usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					scroll_map_window(map, 0, 4);
+			} else	 {
+				/* Reset autoscrolling delay. */
+				delay = TickCount();		
+
+				if (map_modal != NO_MODAL) {
+					switch (map_modal) {
+						case ATTACK_MODAL:
+						case FIRE_MODAL:
+						case FIRE_INTO_MODAL:
+							current_cursor = firecursor;
+							break;
+						case MOVE_TO_MODAL:
+						case SET_FORMATION_MODAL:
+						case ADD_TERRAIN_MODAL:
+						case REMOVE_TERRAIN_MODAL:
+						case DISTANCE_MODAL:
+						case ZOOM_MODAL:
+						case GENERIC_MODAL:
+							current_cursor = opencrosscursor;
+							break;
+						default:
+							run_error("unknown modal tool %d", map_modal);
+							break;
+					}
+					usual = FALSE;
+	#ifdef DESIGNERS
+				} else if (dside->designer && tooltype != notool) {
+					current_cursor = adjust_designer_cursor(mouse, region);  usual = FALSE;
+	#endif DESIGNERS
+				} else if (map->moveonclick) {
+					if (map->numselections == 1
+					    && (unit = map->selections[0]) != NULL) {
+						/* Calculate the approx dir to here from selected unit. */
+						m_nearest_cell(map, mouse.h, mouse.v, &x, &y);
+						/* Note that we allow x,y here to be outside the world. */
+						if (mobile(unit->type)
+						    && (approxdir = approx_dir(x - unit->x, y - unit->y)) >= 0) {
+							current_cursor = movecursors[approxdir];  usual = FALSE;
+						} else {
+							current_cursor = nomovecursor;  usual = FALSE;
+						}
+					} else if (map->numselections > 1) {
+						current_cursor = allmovecursor;  usual = FALSE;
+					} else if (map->autoselect) {
+						/* If no units are selected and autoselect is on, then select one unit.
+						   In theory this should never happen, but in practice it does... */
+						map->curunit = find_next_awake_mover(dside, NULL);
+						select_exactly_one_unit(map, map->curunit);
 					} else {
-						current_cursor = nomovecursor;  usual = FALSE;
+						/* (this is a little confusing here if no units are selected, since
+					   	will just be arrow cursor) */
 					}
-				} else if (map->numselections > 1) {
-					current_cursor = allmovecursor;  usual = FALSE;
-				} else if (map->autoselect) {
-					/* If no units are selected and autoselect is on, then select one unit.
-					   In theory this should never happen, but in practice it does... */
-					map->curunit = find_next_awake_mover(dside, NULL);
-					select_exactly_one_unit(map, map->curunit);
-				} else {
-					/* (this is a little confusing here if no units are selected, since
-				   	will just be arrow cursor) */
 				}
 			}
 			/* This isn't really "cursor adjustment", but this is the right place
@@ -696,6 +737,8 @@ adjust_cursor(Point mouse, RgnHandle region)
 				lastmouse = mouse;
 			}
 		} else {
+			/* Reset autoscrolling delay. */
+			delay = TickCount();
 			if (map->toplineh > 0) {
 				if (mouseover != NULL) {
 					mouseover = NULL;
@@ -741,12 +784,18 @@ handle_event(EventRecord *event)
 					SystemClick(event, win);
 					break;
 				case inContent:
-					if (win != FrontWindow()) {
+					if (win != topWindow && win != topFloat) {
 						/* Bring the clicked-on window to the front. */
-						SelectWindow(win);
+						SelectTheWindow(win);
 						update_window(win);
 						/* Fix the menu to match the new front window. */
 						adjust_menus();
+						/* Permit direct click action in non-top floating windows.
+						This is not exactly in line with Apple's interface guidelines
+						but it does make things easier when more than one float is up
+						since you don't have to worry about which one is in front. */
+						if (IsFloating(win))
+						    do_mouse_down(win, event);
 						/* We always want to discard the event now, since clicks in a
 						   windows are often irreversible actions. */
 					} else {
@@ -756,7 +805,7 @@ handle_event(EventRecord *event)
 					break;
 				case inDrag:
 					/* Standard drag behavior, no tricks necessary. */
-					DragWindow(win, event->where, &dragrect);
+					DragTheWindow(win, event);
 					break;
 				case inGrow:
 					grow_window(win, event->where);
@@ -773,11 +822,10 @@ handle_event(EventRecord *event)
 			break;
 		case keyDown:
 		case autoKey:
-			/* First handle the special use of the numeric keypad keys for offscreen scrolling */
-			/* Break if a numeric keypad key was hit to prevent reading of its charCode below */
+			/* First handle the special use of the numeric keypad for offscreen scrolling:
+			break if a keypad key was hit to prevent reading of its charCode below. */
 			if (handle_numeric_keypad(event))
 			  break;
-
 			key = event->message & charCodeMask;
 			/* Check for menukey equivalents. */
 			if (event->modifiers & cmdKey) {
@@ -785,23 +833,20 @@ handle_event(EventRecord *event)
 					adjust_menus();
 					do_menu_command(MenuKey(key));
 				}
-			} else {
-				/* Don't accept keystrokes if no front window (crashes do_keyboard_command). */
-				if (!FrontWindow())
-			  	break;
-				if (event->what == keyDown) {
-					int handled = FALSE;
+			/* '' is the charCode for option + w. */
+			} else if (key == '') {
+				close_window(topFloat);
+			} else if (event->what == keyDown) {
+				int handled = FALSE;
 
-					/* Random keypress, interpret it. */
-					if (FrontWindow() == constructionwin)
-					  handled = do_key_down_construction(key);
-					if (FrontWindow() == commandwin)
-					  handled = do_key_down_command(key);
-					if (!handled)
-					  do_keyboard_command(key);
-				}
+				/* Random keypress, interpret it. */
+				if (topWindow && topWindow == constructionwin)
+				  handled = do_key_down_construction(key);
+				if (topWindow && topWindow == commandwin)
+				  handled = do_key_down_command(key);
+				if (!handled)
+				  do_keyboard_command(key);
 			}
-
 			break;
 		case activateEvt:
 			activate_window((WindowPtr) event->message, event->modifiers & activeFlag);
@@ -817,14 +862,24 @@ handle_event(EventRecord *event)
 				err = DIBadMount(pnt, event->message);
 			}
 			break;
-		case app4Evt:
+		case osEvt:
 		    /* Grab only a single byte. */
 			switch ((event->message >> 24) & 0xFF) {
+				/* Mouse moved evt - do nothing. */
 				case 0xfa:
 					break;
+				/* Suspend/resume evt. */
 				case 1:
+					/* Check for suspend (0) or resume (1). */
 					inbackground = !(event->message & 1);
-					activate_window(FrontWindow(), !inbackground);
+					/* Activate or deactivate the top window. */
+					ActivateWindow(topWindow, !inbackground);
+					/* The floats should also be activated/deactivated! */
+					for_all_windows(win) {
+						if (IsFloating(win)) {
+							ActivateWindow(win, !inbackground);
+						}							
+					}
 					break;
 			}
 			break;
@@ -904,7 +959,7 @@ zoom_window(WindowPtr win, Point where, int part)
 		} else {
 			/* Generic window zooming. */
 			EraseRect(&win->portRect);
-			ZoomWindow(win, part, true);
+			ZoomWindow(win, part, false);
 			InvalRect(&win->portRect);
 		}
 		SetPort(oldport);
@@ -926,14 +981,10 @@ close_window(WindowPtr win)
 			destroy_list(list_from_window(win));
 		} else if (unit_closeup_from_window(win)) {
 			destroy_unit_closeup(unit_closeup_from_window(win));
-#ifdef DESIGNERS
-		} else if (win == designwin) {
-			/* Closing the design palette implies we're done designing. */
-			disable_designing();
-#endif /* DESIGNERS */
 		}
 		/* Remove the window from our sight, will provoke update events. */
-		HideWindow(win);
+		if (win)
+		HideTheWindow(win);
 		/* At least for now, don't actually dispose of the window. */
 	}
 	update_all_map_windows();
@@ -974,6 +1025,8 @@ do_mouse_down(WindowPtr window, EventRecord *event)
 			do_mouse_down_command(mouse, event->modifiers);
 		} else if (window == scoreswin) {
 			do_mouse_down_scores(mouse, event->modifiers);
+		} else if (window == instructionswin) {
+			do_mouse_down_instructions(mouse, event->modifiers);
 #ifdef DESIGNERS
 		} else if (window == designwin) {
 			do_mouse_down_design(mouse, event->modifiers);
@@ -1007,6 +1060,8 @@ activate_window(WindowPtr win, int activate)
 		activate_construction(activate);
 	} else if (win == helpwin) {
 		activate_help(activate);
+	} else if (win == instructionswin) {
+		activate_instructions(activate);
 	} else if (win == noticewin) {
 		activate_notice(activate);
 	} else if (win == commandwin) {
@@ -1045,6 +1100,9 @@ update_window(WindowPtr win)
 		  draw_unit_info(map);
 		/* Then copy map content from the gworld. */
 		copy_from_gworld(map, map->contentrect);	   
+		/* Don't draw scrollbars if they are zero width. */
+		if (map->sbarwid == 0)
+	             controls = FALSE;
 		growbox = TRUE;
 	} else if ((list = list_from_window(win)) != NULL) {
 		draw_list(list);
@@ -1063,6 +1121,9 @@ update_window(WindowPtr win)
 	} else if (win == helpwin) {
 		draw_help();
 		growbox = TRUE;
+	} else if (win == instructionswin) {
+		draw_instructions();
+		growbox = FALSE;
 	} else if (win == noticewin) {
 		draw_notice();
 		growbox = TRUE;
@@ -1084,12 +1145,47 @@ update_window(WindowPtr win)
 		UpdateControls(win, win->visRgn);
 	}
 	if (growbox) {
-		DrawGrowIcon(win);
+		/* We want scrollbar lines only when there are scrollbars! */
+		if(controls)
+			DrawGrowIcon(win);
+		else 	draw_growicon_nolines(win);
 	}
 	EndUpdate(win);
 	SetPort(oldport);
 }
 
+/* Fixes a bug in the Toolbox which causes DrawGrowIcon to draw scrollbar
+lines even if there are no scrollbars in the window. */
+
+void 
+draw_growicon_nolines(WindowPtr win)
+{
+	Rect 		iconrect;
+	RgnHandle 	newclip;
+	RgnHandle 	oldclip;
+	
+	newclip = NewRgn();
+	oldclip = NewRgn();
+	GetClip(oldclip);
+
+	/* Set new clip rgn to GrowIcon. */
+	iconrect = win->portRect;
+	iconrect.left = iconrect.right-15; 
+	iconrect.top = iconrect.bottom-15;
+	SetRectRgn(newclip, iconrect.left, iconrect.top, 
+				       iconrect.right, iconrect.bottom);
+
+	/* Intersect it with old clip rgn. */
+	SectRgn(oldclip, newclip, newclip);
+
+	SetClip(newclip);
+	DrawGrowIcon(win);
+	SetClip(oldclip);
+	DisposeRgn(newclip);
+	DisposeRgn(oldclip);
+
+}
+
 static int last_tick_count = 0;
 
 void
@@ -1099,7 +1195,7 @@ maybe_select_next_unit()
 	Map *map;
 
 	if ((!beforestart && !endofgame)
-	    && (map = map_from_window(FrontWindow())) != NULL
+	    && (map = frontmap) != NULL
 	    && map->autoselect) {
 	    /* Hunt around for a reasonable "next unit" to select. */
 	    /* Scroll over to the current unit if appropriate. */
@@ -1603,7 +1699,7 @@ low_notify(Side *side, char *str)
 /* Kernel callback to update info about the given side. */
 
 void
-update_area_display()
+update_area_display(Side *side)
 {
 }
 
@@ -2316,10 +2412,15 @@ stagger_window(WindowPtr win, int *lasthp, int *lastvp)
 			} else if (!position_already_used(h, v)) {
 				break;
 			}
+			/* Moving this inside the while loop fixed infinite loop hangs with certain values. */
+			*lasthp = h;  *lastvp = v;
 		}
 		MoveWindow(win, h, v, FALSE);
-		*lasthp = h;  *lastvp = v;
 	} else {
+		/* Do move to accomodate Apple Platinum frames (would be more correct 
+		to use strucRgn->rgnBBox here, but it is not yet defined at this point). */
+		MoveWindow(win, 5, GetMBarHeight() + 21, FALSE);
+
 		/* Don't move the first window, but do record its position. */
 		GetPort(&oldport);
 		SetPort(win);
@@ -2420,7 +2521,9 @@ set_standard_state(WindowPtr win, int fullw, int fullh)
 		if (bestscreen == GetMainDevice()) {
 			gdrect.top += GetMBarHeight();
 		}
-		InsetRect(&gdrect, 3, 3);
+		/* Modified for Apple Platinum. */
+		InsetRect(&gdrect, 5, 5);
+		gdrect.top -= 5;
 		gdrect.top += wintitlehgt;
 		screenw = gdrect.right - gdrect.left;  screenh = gdrect.bottom - gdrect.top;
 		if (winrect.left + fullw <= gdrect.right
@@ -2510,8 +2613,8 @@ force_update(WindowPtr win)
 	map = map_from_window(win);
 	if (map) {
 		/* Do this stuff first so that the user has something to look at. */
-		set_map_scrollbars(map);
 		set_content_rect(map);
+		set_map_scrollbars(map);
 
 		/* Draw the map decor. */
 		if (map->conw > 0)
