@@ -24,14 +24,15 @@ any later version.  See the file COPYING.  */
 #endif
 #ifdef NEW_HEADERS
 #include <MacTypes.h>
+#include <MacWindows.h>
 #else
 #include <Types.h>
-#endif
+#include <Windows.h>
+#endif /* NEW_HEADERS */
 #include <Resources.h>
 #include <QuickDraw.h>
 #include <Fonts.h>
 #include <Events.h>
-#include <Windows.h>
 #include <Menus.h>
 #include <TextEdit.h>
 #include <Dialogs.h>
@@ -81,6 +82,10 @@ any later version.  See the file COPYING.  */
 
 #include "macdefs.h"
 
+/* Bring in floating window support. */
+
+#include "Floating.h"
+
 #ifdef MPW_C
 #define QD(whatever) (qd.##whatever)
 #define QDPat(whatever) (&(qd.##whatever))
@@ -210,7 +215,6 @@ typedef struct a_ui {
 typedef struct a_map {
 	short id;				/* Identifying number for the map */
 	VP *vp;                 		/* This map's generic view parameters */
-	short osx, osy;         		/* Tracks shifted origin so can be restored */
 	WindowPtr window;
 	ControlHandle hscrollbar;
 	ControlHandle vscrollbar;
@@ -229,6 +233,7 @@ typedef struct a_map {
 	short toplineh;
 	short topunith;
 	short toph;
+	short sbarwid;
 	short bufx;				/* Actual size of horizontal offscreen scrolling buffer */
 	short bufy;				/* Actual size of vertical offscreen scrolling buffer */
 	short max_bufx;			/* Max size of horizontal offscreen scrolling buffer as set in dialog */
@@ -247,13 +252,8 @@ typedef struct a_map {
 	short draw_topunit;		/* Draw top unit info */	
 	short erase_names;		/* Draw row in update_cell_display to erase name of moving unit */	
 	short drawsizes;		/* Draw unit (city) size numbers */
-
-// --- USER CONTROL OF EMBLEMS -----------------------------------------
-
 	short draw_emblems;
-
-// ------------------------------------------------------------------
-
+	short draw_ai;
 	short see_all;
 	short autoselect;
 	Unit *curunit;
@@ -278,13 +278,7 @@ typedef struct a_list {
 	short id;
 	short sidecolors;		/* Use side colors for units */
 	short drawsizes;		/* Draw unit (city) size numbers */
-
-// --- USER CONTROL OF EMBLEMS -----------------------------------------
-
 	short draw_emblems;
-
-// ------------------------------------------------------------------
-
 	SideMask sides;			/* Sides to show the units of */
 	short completed_units;
 	short incomplete_units;
@@ -464,6 +458,7 @@ extern int hasColorQD;
 extern int minscreendepth;
 extern int maxscreendepth;
 extern int sbarwid;
+extern int floatsbarwid;
 extern WindowPtr worldmapwin;
 extern Map *worldmap;
 extern Map *frontmap;
@@ -515,6 +510,12 @@ extern CursHandle firecursor;
 extern CursHandle sendcursor;
 extern CursHandle receivecursor;
 
+/* Autoscroll cursors. */
+extern CursHandle leftcursor;
+extern CursHandle rightcursor;
+extern CursHandle downcursor;
+extern CursHandle upcursor;
+
 extern enum grays gridgray;
 extern enum grays unseengray;
 extern enum grays bggray;
@@ -580,6 +581,8 @@ extern int unified_cicns;			/* Use BitMap as mask for color PixMap */
 
 extern int default_draw_topline;		/* Draw top line box (toggle function declared elsewhere) */
 extern int default_draw_topunit;		/* Draw unit info box (toggle function declared elsewhere) */
+extern int default_draw_scrollbars;	/* Draw scrollbars in maps. */
+extern int default_draw_cpanel;		/* Draw control panel in maps. */
 extern int default_drawothermaps;	/* Draw other-map boxes */
 extern int default_sidecolors;		/* Use side colors for unit icons */					
 extern int default_iconmasks;		/* Draw unit icon masks */
@@ -588,13 +591,9 @@ extern int default_textmasks;		/* Draw text masks */
 extern int default_featureborders;	/* Draw feature borders */
 extern int default_featurenames;		/* Draw feature names */		
 extern int default_shorelines;		/* Draw 3D shorelines */
-
-// --- USER CONTROL OF EMBLEMS ------------------------------------------------
-
+extern int default_draw_ai;			/* Draw AI regions (if present). */
+extern int alert_256_colors;		/* Suggest switch to 256 colors at start. */
 extern int default_draw_emblems;	/* Draw unit emblems in maps and lists. */
-
-// ------------------------------------------------------------------------
-
 extern int default_simple_borders;	/* Do not draw borders to sea or empty land */
 extern int default_optimize_fonts;	/* Use optimized set of font sizes */
 extern int default_erase_names;		/* Erase names of moving units */
@@ -719,6 +718,9 @@ extern int at_all_visible(Map *map, int x, int y);
 extern int in_middle(Map *map, int x, int y);
 
 extern void xform(Map *map, int x, int y, int *sxp, int *syp);
+extern void xform_flat(Map *map, int x, int y, int *sxp, int *syp);
+extern void xform_fractional(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
+extern void xform_fractional_flat(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
 extern void m_xform_unit(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
 extern void m_xform_unit_self(Map *map, Unit *unit, int *sxp, int *syp, int *swp, int *shp);
 extern void m_xform_occupant(Map *map, Unit *transport, Unit *unit, int sx, int sy, int sw, int sh, int *sxp, int *syp, int *swp, int *shp);
@@ -752,6 +754,8 @@ extern void draw_related_maps(Map *map);
 extern void draw_other_map(Map *map, Map *map2);
 extern void draw_row(Map *map, int x0, int y0, int len, int clearit);
 
+extern void draw_growicon_nolines(WindowPtr win);
+
 extern void draw_selections(Map *map);
 extern void draw_selections_at(Map *map, int x, int y);
 extern void draw_selected_unit(Map *map, Unit *unit);
@@ -765,7 +769,8 @@ extern void draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u
 extern void draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style);
 extern void draw_cell_block(int sx, int sy, int n, int power, int t, int over, int angle, int dosolid);
 extern void calc_best_terrain_images(void);
-extern void draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogrid, int dosolid);
+extern void draw_hex_region(int sx, int sy, int power, int t, int over, int angle,
+							int dogrid, int dosolid, int var);
 extern void draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
 extern void draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
 extern void draw_terrain_sample(WindowPtr win, Rect tmprect, int t);
@@ -799,6 +804,8 @@ extern void set_map_mag(Map *map, int newpower);
 extern void toggle_map_grid(Map *map);
 extern void toggle_map_topline(Map *map);
 extern void toggle_map_topunit(Map *map);
+extern void toggle_map_scrollbars(Map *map);
+extern void toggle_map_cpanel(Map *map);
 extern void toggle_map_other_maps(Map *map);
 extern void toggle_map_lighting(Map *map);
 extern void toggle_map_coverage(Map *map);
@@ -814,13 +821,8 @@ extern void toggle_map_clouds(Map *map);
 extern void toggle_map_storms(Map *map);
 extern void toggle_map_plans(Map *map);
 extern void toggle_map_ai(Map *map);
-
-// --- USER CONTROL OF EMBLEMS --------------------------------------------
-
 extern void toggle_map_draw_emblems(Map *map);
 
-// --------------------------------------------------------------------
-
 extern void do_mouse_down_map_content(Map *map, int h, int v, int mods);
 extern void select_all_dragged_over(Map *map, int h0, int v0, int mods);
 extern void select_area_and_zoom(Map *map, int h0, int v0, int mods);
@@ -921,7 +923,7 @@ extern void quit_the_game(void);
 extern void apply_to_all_selected(int (*fn)(), int beepfailure);
 extern void do_reserve_command(int value, int radius, int recurse);
 extern void do_sleep_command(int value, int radius, int recurse);
-extern void enable_construction(void);
+extern void show_construction_window(void);
 extern int do_move_to_command(void);
 extern void adjust_menus(void);
 
@@ -937,10 +939,13 @@ extern void toggle_profiling(void);
 
 extern char *get_string_from_item(Handle itemhandle);
 
-extern void instructions_dialog(void);
-extern void create_instructions_dialog(void);
-extern int hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt);
+extern void show_instructions_window(void);
+extern void create_instructions_window(void);
+extern void activate_instructions(int activate);
+extern void do_mouse_down_instructions(Point mouse, int mods);
+extern void draw_instructions(void);
 
+extern void show_game_window(void);
 extern void create_game_window(void);
 extern void draw_game(void);
 extern void draw_game_date(void);
@@ -986,6 +991,7 @@ extern UnitCloseup *unit_closeup_from_window(WindowPtr win);
 extern int do_mouse_down_unit_closeup(UnitCloseup *unitcloseup, Point mouse, int mods);
 extern void destroy_unit_closeup(UnitCloseup *unitcloseup);
 
+extern void show_history_window(void);
 extern void create_history_window(void);
 extern void draw_history(void);
 extern void draw_historical(HistEvent *hevt, int y, int drawevt);
@@ -995,7 +1001,7 @@ extern void grow_history(int h, int v);
 extern void zoom_history(int part);
 extern void update_history_window(HistEvent *hevt);
 
-extern void notice_dialog(void);
+extern void show_notice_window(void);
 extern void create_notice_window(void);
 extern void append_notice(char *str);
 extern void draw_notice(void);
@@ -1005,7 +1011,7 @@ extern void do_mouse_down_notice(Point mouse, int mods);
 extern void grow_notice(int h, int v);
 extern void zoom_notice(int part);
 
-extern void scores_dialog(void);
+extern void show_scores_window(void);
 extern void create_scores_window(void);
 extern void append_scores(char *str);
 extern void draw_scores(void);
@@ -1021,7 +1027,7 @@ extern void activate_command(int activate);
 extern int do_key_down_command(int key);
 extern void do_mouse_down_command(Point mouse, int mods);
 
-extern void help_dialog(HelpNode *helpnode);
+extern void show_help_window(HelpNode *helpnode);
 extern void describe_menus(int arg, char *key, TextBuffer *buf);
 extern void describe_mouse(int arg, char *key, TextBuffer *buf);
 extern void describe_help(int arg, char *key, TextBuffer *buf);
@@ -1039,8 +1045,8 @@ extern void enable_designing(int forsure);
 extern void disable_designing(void);
 extern void init_design_cursors(void);
 extern CursHandle adjust_designer_cursor(Point mouse, RgnHandle region);
+extern void show_design_window(void);
 extern void create_design_window(void);
-extern void position_design_window(void);
 extern void draw_design_window(void);
 extern void draw_design_window_tool(enum tooltype tool);
 extern void do_mouse_down_design(Point mouse, int mods);
@@ -1071,7 +1077,7 @@ extern void play_sound(char *soundname);
 extern void query_position_modally(int mode);
 extern void set_position_modally(void);
 
-extern void enable_command(void);
+extern void show_command_window(void);
 
 extern void invert_map(Map *map, int x, int y, int r);
 extern Map *front_map(void);
@@ -1104,13 +1110,8 @@ extern void toggle_erase_names(Map *map);
 extern void toggle_draw_latlong(Map *map);			/* The associated default is declared elsewhere */
 extern void toggle_solid_color_terrain(Map *map);	/* The associated default is declared elsewhere */
 extern void toggle_list_sidecolors(List *list);		/* Uses the same default as toggle_map_sidecolors */
-
-// --- USER CONTROL OF EMBLEMS ----------------------------------------------------
-
 extern void toggle_list_draw_emblems(List *list);	/* Uses the same default as toggle_map_sidecolors */
 
-// ----------------------------------------------------------------------------
-
 extern void read_xcol(void);				/* Read RGB colors form XCol resource */
 extern void write_xcol(void);				/* Write RGB colors to XCol resource */
 extern void check_screen_depths(void);		/* Check for more than 256 colors */
