@@ -35,6 +35,10 @@ CursHandle bordpaintor;
 CursHandle connpaintor;
 CursHandle coatpaintor;
 
+/* This is the number of tools actually used. */
+
+short numtoolsused = 0;
+
 /* This is the width and height of each design tool's entry. */
 
 short dtoolw = 120;
@@ -98,14 +102,7 @@ enable_designing(int forsure)
 	/* Actually change designer status, this will call back to alter displays. */
 	net_become_designer(dside);
 	/* Create and add the designer's palette. */
-	if (designwin == nil) {
-		create_design_window();
-	}
-	if (designwin != nil) {
-		position_design_window();
-		ShowWindow(designwin);
-/*		SelectWindow(designwin); */
-	}
+	show_design_window();
 	/* Force all maps to see_all; designer can disable manually if
 	   desired. */
 	for_all_maps(map) {
@@ -125,7 +122,7 @@ disable_designing()
 	  return;
 	/* Hide (but don't destroy) the designer's palette. */
 	if (designwin != nil) {
-		HideWindow(designwin);
+		HideTheWindow(designwin);
 	}
 	/* Actually change designer status, this will call back to alter all displays. */
 	net_become_nondesigner(dside);
@@ -181,17 +178,33 @@ adjust_designer_cursor(Point mouse, RgnHandle region)
 	return paintcursors[tooltype];
 }
 
+/* This is the top-level access to bring up the design window, can be called
+   anywhere, anytime. */
+
+void
+show_design_window()
+{
+	if (designwin == nil) {
+		create_design_window();
+	} else	 {
+		SelectTheWindow(designwin);
+		ActivateWindow(designwin, true);
+	}
+}
+
 /* Create the designer tool window. */
 
 void
 create_design_window()
 {
+	int i;
+
 	if (hasColorQD) {
-		designwin = GetNewCWindow(wDesign, NULL, (WindowPtr) -1L);
+		designwin = GetNewCWindow(wFloatFixed, NULL, NULL);
 	} else {
-		designwin = GetNewWindow(wDesign, NULL, (WindowPtr) -1L);
+		designwin = GetNewWindow(wFloatFixed, NULL, NULL);
 	}
-	add_window_menu_item("Design", designwin);  /* until this becomes a windoid */
+	SetWTitle(designwin, "\pDesign");
 	/* Make the current side be the one with the display. */
 	cursidenumber = side_number(dside);
 	build_terrain_type_menu();
@@ -207,8 +220,8 @@ create_design_window()
 	feature_add_button = GetNewControl(cFeatureAddButton, designwin);
 	feature_remove_button = GetNewControl(cFeatureRemoveButton, designwin);
 	feature_edit_button = GetNewControl(cFeatureEditButton, designwin);
-	SizeWindow(designwin, 2 * dtoolw - 1, (numtooltypes / 2) * dtoolh - 1, 1);
-	position_design_window();
+	/* Just put the window at the left edge of the screen. */
+	MoveWindow(designwin, 3, 280, true);
 	init_design_cursors();
 	enabledtooltype[notool] = TRUE;
 	enabledtooltype[terraintool] = TRUE;
@@ -222,33 +235,17 @@ create_design_window()
 	enabledtooltype[cloudstool] = any_clouds;
 	enabledtooltype[windstool] = any_wind_variation;
 	enabledtooltype[viewtool] = !all_see_all;
-	ShowWindow(designwin);
-}
 
-/* Try to put the palette alongside the frontmost window. */
-
-void
-position_design_window()
-{
-	Point pt;
-	WindowPtr win;
-	GrafPtr oldport;
-
-	/* (should fix all this) */
-	if ((win = FrontWindow()) != nil) {
-		GetPort(&oldport);
-		SetPort(win);
-		SetPt(&pt, win->portRect.right + 3, win->portRect.top);
-		LocalToGlobal(&pt);
-		SetPort(oldport);
-	} else {
-		SetPt(&pt, 500, 50);
+	/* Count the number of tools in use and size window accordingly. */
+	for (i = 0; i < numtooltypes; ++i) {
+		if (enabledtooltype[i])
+		    ++ numtoolsused;
 	}
-	/* (should make sure is not off the edge of the screen) */
-	if (pt.h + 2 * dtoolw > 640 /* is off screen entirely */) {
-		SetPt(&pt, 640 - 2 * dtoolw - 2, 50);
-	}
-	MoveWindow(designwin, pt.h, pt.v, TRUE);
+	SizeWindow(designwin, dtoolw - 1, numtoolsused * dtoolh - 1, 1);
+
+	/* Make this a floating window! */
+	MakeFloat(designwin);
+	SelectTheWindow(designwin);
 }
 
 void
@@ -278,9 +275,17 @@ draw_design_window_tool(enum tooltype tool)
 	Rect tmprect, imagerect;
 	char imbuf[BUFSIZE];
 	Str255 tmpstr;
+	int i;
 
+	/* Don't draw unused tools. */
+	if (enabledtooltype[i] == FALSE)
+		return;
 	SetRect(&tmprect, 0, 0, dtoolw, dtoolh);
-	OffsetRect(&tmprect, (tool / (numtooltypes/2)) * dtoolw, (tool % (numtooltypes/2)) * dtoolh);
+	/* Don't include unused tools in offset. */
+	for (i = 0; i < tool; ++i) {
+		if (enabledtooltype[i] == TRUE)
+		     OffsetRect(&tmprect, 0, dtoolh);
+	}
 	EraseRect(&tmprect);
 	/* Confine the image to a square subrect on the left side of the window. */
 	imagerect = tmprect;
@@ -495,7 +500,7 @@ draw_design_window_tool(enum tooltype tool)
 void
 do_mouse_down_design(Point mouse, int mods)
 {
-	int oldtool, poppedtool, newutype, newbgttype, toolchoice;
+	int oldtool, poppedtool, newutype, newbgttype, toolchoice, clicked, i;
 	Feature *feature;
 	Rect tmprect;
 	ControlHandle control;
@@ -504,7 +509,17 @@ do_mouse_down_design(Point mouse, int mods)
 
 	tmprect.left = 0;  tmprect.right = dtoolw;
 	oldtool = poppedtool = tooltype;
-	toolchoice = (mouse.v / dtoolh) + (mouse.h > dtoolw ? (numtooltypes / 2) : 0);
+	/* Find the box that was clicked. */
+	clicked = mouse.v / dtoolh;
+	toolchoice = 0;
+	/* Convert it into toolchoice. */ 
+	for(toolchoice = 0; toolchoice < numtooltypes; toolchoice++) {
+		if (toolchoice == clicked && enabledtooltype[toolchoice] == TRUE)
+			break;
+		/* Skip disabled tooltypes. */
+		 if (enabledtooltype[toolchoice] == FALSE)
+		 	++clicked;
+	}
 	FindControl(mouse, designwin, &control);
 	if (control == ttypepopup) {
 		TrackControl(control, mouse, (void *) -1);
@@ -743,8 +758,6 @@ feature_rename_dialog(Feature *feature)
 	SetIText(itemhandle, tmpstr);
 	ShowWindow(win);
 	while (!done) {
-		/* Deactivate the front window. */
-		activate_window(FrontWindow(), FALSE);
 		SetCursor(&QD(arrow));
 		ModalDialog(NULL, &ditem);
 		switch (ditem) {
@@ -1027,8 +1040,6 @@ designer_save_dialog()
 	GetDItem(win, diDesignerSaveCompress, &itemtype, &itemhandle, &itemrect);
 	SetCtlValue((ControlHandle) itemhandle, module->compress_layers);
 	while (!done) {
-		/* Deactivate the front window. */
-		activate_window(FrontWindow(), FALSE);
 		SetCursor(&QD(arrow));
 		ModalDialog(NULL, &ditem);
 		switch (ditem) {
@@ -1135,8 +1146,6 @@ designer_reshape_dialog(Module *module)
 	
 	win = GetNewDialog(dDesignerReshape, NULL, (DialogPtr) -1L);
 	while (!done) {
-		/* Deactivate the front window. */
-		activate_window(FrontWindow(), FALSE);
   		put_number_into_ditem(diDesignerReshapeOrigWidth, area.width);
   		put_number_into_ditem(diDesignerReshapeOrigHeight, area.height);
   		put_number_into_ditem(diDesignerReshapeOrigWorld, world.circumference);
