@@ -9,8 +9,9 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
-static pascal void draw_instructions_text(WindowPtr win, short ditem);
 static pascal void help_vscroll_fn(ControlHandle control, short code);
+static void insert_newlines(TEHandle te, int n);
+
 
 /* Globals for the instructions window. */
 
@@ -32,11 +33,17 @@ ControlHandle helpvscrollbar;
 
 ControlActionUPP help_vscroll_proc;
 
+/* Used in help window. */
 ControlHandle topicsbutton;
 ControlHandle helpbutton;
 ControlHandle prevbutton;
 ControlHandle nextbutton;
 ControlHandle backbutton;
+ControlHandle closebutton;
+
+/* Used in instructions window. */
+ControlHandle openhelpbutton;
+ControlHandle closewinbutton;
 
 static HelpNode *cur_help_node = NULL;
 
@@ -58,49 +65,17 @@ int nodestackpos;
 
 /* The instructions window. */
 
-static pascal void
-draw_instructions_text(WindowPtr win, short ditem)
-{
-	GrafPtr oldport;
-	short itemtype;  Handle itemhandle;  Rect itemrect;
-
-	GetDItem(instructionswin, diInstructionsText, &itemtype, &itemhandle, &itemrect);
-	GetPort(&oldport);
-	SetPort(instructionswin);
-	TextFont(small_font_id);
-	TextSize(small_font_size);
-	TEUpdate(&itemrect, instructions_text);
-	/* This makes the title item draw big. */
-	TextFont(title_font_id);
-	TextSize(title_font_size);
-	SetPort(oldport);
-}
-
-/* Bring up the dialog with instructions on how to play. */
-
 void
-instructions_dialog()
+show_instructions_window()
 {
-	int mainwidth, mainheight;
-	Rect tmprect;
-
 	if (instructionswin == nil) {
-		create_instructions_dialog();
+		create_instructions_window();
+		draw_instructions();
 	}
-	if (first_windows) {
-		get_main_screen_size(&mainwidth, &mainheight);
-		tmprect = instructionswin->portRect;
-		MoveWindow(instructionswin,
-				   mainwidth - (tmprect.right - tmprect.left) - 3,
-				   mainheight - (tmprect.bottom - tmprect.top) - 3,
-				   FALSE);
-	}
-	ShowWindow(instructionswin);
-	SelectWindow(instructionswin);
+	SelectTheWindow(instructionswin);
+	ActivateWindow(instructionswin, true);
 }
 
-void insert_newlines(TEHandle te, int n);
-
 void
 insert_newlines(TEHandle te, int n)
 {
@@ -118,34 +93,64 @@ insert_newlines(TEHandle te, int n)
 	TEInsert(str, strlen(str), te);
 }
 
+/* The non-modal instructions dialog messed up the floating windows, 
+so I have converted it into a real window instead. */
+
 void
-create_instructions_dialog()
+create_instructions_window()
+{
+	/* Create the window, color if possible, since images may be in color. */
+	if (hasColorQD) {	
+		instructionswin = GetNewCWindow(wInstructions, NULL, NULL);
+	} else {
+		instructionswin = GetNewWindow(wInstructions, NULL, NULL);
+	}
+	openhelpbutton = GetNewControl(cHelpButton, instructionswin);
+	MoveControl(openhelpbutton, 175, 200);
+	closewinbutton = GetNewControl(cCloseButton, instructionswin);
+	MoveControl(closewinbutton, 235, 200);
+	add_window_menu_item("Instructions", instructionswin);
+}
+
+/* New function, in part derived from old create_instructions_dialog. */
+
+void
+draw_instructions()
 {
 	int any_instructions = FALSE;
 	Obj *instructions, *rest;
-	char *str;
-	Str255 tmpstr;
-	Rect destrect, viewrect;
-	short itemtype;  Handle itemhandle;  Rect itemrect;
 	Scorekeeper *sk;
+	GrafPtr oldport;
+	Rect textrect;
+	Str255 pstr;
+	char *str;
 
-	if (draw_instructions_text_proc == NULL)
-	  draw_instructions_text_proc = NewUserItemProc(draw_instructions_text);
-
-	instructionswin = GetNewDialog(dInstructions, nil, (DialogPtr) -1L);
-	GetDItem(instructionswin, diInstructionsTitle, &itemtype, &itemhandle, &itemrect);
-	c2p(((mainmodule && mainmodule->title) ? mainmodule->title : ""), tmpstr);
-	SetIText(itemhandle, tmpstr);
-	GetDItem(instructionswin, diInstructionsText, &itemtype, &itemhandle, &itemrect);
-	SetDItem(instructionswin, diInstructionsText, itemtype, (Handle) draw_instructions_text_proc, &itemrect);
-	destrect = itemrect;
-	viewrect = itemrect;
+	GetPort(&oldport);
 	SetPort(instructionswin);
+
+	/* Draw the game title. */
+	c2p(((mainmodule && mainmodule->title) ? mainmodule->title : ""), pstr);
+	TextFont(title_font_id);
+	TextSize(title_font_size);
+	MoveTo(15, 20);
+	DrawString(pstr);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
-	instructions_text = TENew(&destrect, &viewrect);
+
+	/* Draw a horizontal line under the title. */
+	MoveTo(0, 30);
+	LineTo(instructionswin->portRect.right, 30);
+
+	/* Draw the instructions text itself. */
+	textrect = instructionswin->portRect;
+	textrect.left += 15;
+	textrect.right -= 15;
+	textrect.top += 40;
+	textrect.bottom -= 15;
+	instructions_text = TENew(&textrect, &textrect);
 	TESetSelect(0, 32767, instructions_text);
 	TEDelete(instructions_text);
+
 	/* (should generify, add parameter for how to line break) */
 	if (mainmodule == NULL) {
 		run_error("no main module?");
@@ -230,22 +235,37 @@ create_instructions_dialog()
 			TEInsert(str, strlen(str), instructions_text);
 		}
 	}
-	/* This makes the title item draw big. */
-	TextFont(title_font_id);
-	TextSize(title_font_size);
-	add_window_menu_item("Instructions", instructionswin);
+	SetPort(oldport);
 }
 
-int
-hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt)
+/* New function. */
+
+void
+activate_instructions(int activate)
 {
-	switch (itemhit) {
-		case diInstructionsHelp:
-			/* Just jump to the help dialog. */
-			help_dialog(NULL);
-			break;
+	HiliteControl(openhelpbutton, (activate ? 0 : 255));
+	HiliteControl(closewinbutton, (activate ? 0 : 255));
+}
+
+/* New function. Replaces hit_instructions_dialog. */
+
+void
+do_mouse_down_instructions(Point mouse, int mods)
+{
+	ControlHandle control;
+	Handle itemhandle;
+	short part;
+	
+	part = FindControl(mouse, instructionswin, &control);
+	if (control == openhelpbutton) {
+		TrackControl(control, mouse, NULL);
+		close_window(instructionswin);
+	    	show_help_window(NULL);
+	}
+	if (control == closewinbutton) {
+		TrackControl(control, mouse, NULL);
+		close_window(instructionswin);
 	}
-	return TRUE;
 }
 
 /* Help window. */
@@ -254,7 +274,7 @@ hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt)
    anywhere, anytime. */
 
 void
-help_dialog(HelpNode *helpnode)
+show_help_window(HelpNode *helpnode)
 {
 	if (helpwin == nil) {
 		create_help_window();
@@ -263,8 +283,8 @@ help_dialog(HelpNode *helpnode)
 		cur_help_node = helpnode;
 		set_help_content(helpnode);
 	}
-	ShowWindow(helpwin);
-	SelectWindow(helpwin);
+	SelectTheWindow(helpwin);
+	ActivateWindow(helpwin, true);
 }
 
 void
@@ -350,15 +370,17 @@ create_help_window()
 
 	/* Create the window, color if possible, since images may be in color. */
 	if (hasColorQD) {	
-		helpwin = GetNewCWindow(wHelp, NULL, (WindowPtr) -1L);
+		helpwin = GetNewCWindow(wHelp, NULL, NULL);
 	} else {
-		helpwin = GetNewWindow(wHelp, NULL, (WindowPtr) -1L);
+		helpwin = GetNewWindow(wHelp, NULL, NULL);
 	}
 	topicsbutton = GetNewControl(cTopicsButton, helpwin);
 	helpbutton = GetNewControl(cHelpButton, helpwin);
 	prevbutton = GetNewControl(cPrevButton, helpwin);
 	nextbutton = GetNewControl(cNextButton, helpwin);
 	backbutton = GetNewControl(cBackButton, helpwin);
+	closebutton = GetNewControl(cCloseButton, helpwin);
+
 	SetPort(helpwin);
 	TextFont(title_font_id);
 	TextSize(title_font_size);
@@ -491,6 +513,12 @@ void
 activate_help(int activate)
 {
 	HiliteControl(helpvscrollbar, (activate ? 0 : 255));
+	HiliteControl(topicsbutton, (activate ? 0 : 255));
+	HiliteControl(helpbutton, (activate ? 0 : 255));
+	HiliteControl(prevbutton, (activate ? 0 : 255));
+	HiliteControl(nextbutton, (activate ? 0 : 255));
+	HiliteControl(backbutton, (activate ? 0 : 255));
+	HiliteControl(closebutton, (activate ? 0 : 255));
 }
 
 static pascal void
@@ -545,17 +573,26 @@ do_mouse_down_help(Point mouse, int mods)
 
 	part = FindControl(mouse, helpwin, &control);
 	if (control == topicsbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = topics_help_node;
 	} else if (control == helpbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = help_help_node;
 	} else if (control == prevbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = cur_help_node->prev;
 	} else if (control == nextbutton) {
+		TrackControl(control, mouse, NULL);
 		cur_help_node = cur_help_node->next;
 	} else if (control == backbutton) {
+		TrackControl(control, mouse, NULL);
 		if (nodestackpos > 0) {
 			cur_help_node = nodestack[--nodestackpos];
 		}
+	} else if (control == closebutton) {
+		TrackControl(control, mouse, NULL);
+		close_window(helpwin);
+		
 	} else if (control == helpvscrollbar) {
 		if (part != 0) {
 			switch (part) {
@@ -667,7 +704,7 @@ zoom_help(int part)
 		}
 		(*((WStateDataHandle) ((WindowPeek) helpwin)->dataHandle))->stdState = zoomrect;
 	}
-	ZoomWindow(helpwin, part, (helpwin == FrontWindow()));
+	ZoomWindow(helpwin, part, false);
 	h = window_width(helpwin);  v = window_height(helpwin);
 	MoveControl(helpvscrollbar, h - sbarwid, 0);
 	SizeControl(helpvscrollbar, sbarwid + 1, v - sbarwid + 1);
