@@ -1,5 +1,5 @@
 /* Image families for the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -46,9 +46,10 @@ extern void mac_add_ppat(ImageFamily *imf, Handle handle);
 
 static MacImage *init_mac_image(Image *img);
 static void mac_interp_image(ImageFamily *imf, Image *img, int force);
+static void mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int force);
 static void mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump);
-static void mac_copy_bytes(char *data, int numbytes, Handle desthandle, int jump, int rowbytes);
-static CTabHandle interp_ctab(Obj *palette);
+static void mac_copy_bytes(char *data, int numbytes, Handle desthandle, int macrowbytes, int rowbytes);
+static CTabHandle interp_ctab(Image *img);
 static CTabHandle synth_ctab(void);
 static void convert_ppat(Image *img, PixPatHandle colrpat);
 static void convert_cicn(Image *img, CIconHandle colricon, int *hasmono, int *hasmask);
@@ -381,6 +382,39 @@ int force;
 
 void
 mac_interp_image(ImageFamily *imf, Image *img, int force)
+{
+    int i, subi;
+    Image *subimg;
+
+    /* Iterate through any subimages. */
+    if (img->isborder && img->subimages != NULL) {
+		for (i = 0; i < 16; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+		img->numsubimages = 16;
+    } else if (img->isconnection && img->subimages != NULL) {
+		for (i = 0; i < 64; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+		img->numsubimages = 64;
+    } else if (img->istransition && img->subimages != NULL) {
+		for (i = 0; i < 4 * 4; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+		img->numsubimages = 4 * 4;
+    } else if (img->numsubimages > 0 && img->subimages != NULL) {
+		for (i = 0; i < img->numsubimages; ++i) {
+		    mac_interp_image_1(imf, img, img->subimages[i], i, force);
+		}
+    } else {
+		mac_interp_image_1(imf, img, img, 0, force);
+    }
+}
+
+/* Interpret a single image/subimage. */
+
+static void
+mac_interp_image_1(ImageFamily *imf, Image *img, Image *subimg, int subi, int force)
 {
 	int w, h, i, numbytes, bitrowbytes, actualw, actualh, pixelsize, rowbytes;
 	int macrowbytes;
@@ -395,20 +429,20 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 	actualw = img->actualw;  actualh = img->actualh;
 	pixelsize = img->pixelsize;
 	rowbytes = computed_rowbytes(actualw, pixelsize);
-	macimg = get_mac_image(img);
+	macimg = get_mac_image(subimg);
 	/* Mono icons and masks are very similar; digest both here. */
 	monodone = colrdone = maskdone = FALSE;
-	if (img->monodata != lispnil && (img->rawmonodata == NULL || force)) {
+	if (subimg->monodata != lispnil && (subimg->rawmonodata == NULL || force)) {
 		numbytes = h * computed_rowbytes(w, 1);
-		img->rawmonodata = xmalloc(numbytes);
-		interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
+		subimg->rawmonodata = xmalloc(numbytes);
+		interp_bytes(subimg->monodata, numbytes, subimg->rawmonodata, 0);
 	}
-	if (w == 8 && h == 8 && img->rawmonodata != NULL && img->istile) {
+	if (w == 8 && h == 8 && subimg->rawmonodata != NULL && img->istile) {
 		/* Monochrome pattern. */
 		datahandle = NewHandle(8);
 		memset(*datahandle, 0, 8);
 		/* Read exactly 8 bytes. */
-		mac_copy_bytes(img->rawmonodata, 8, datahandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmonodata, 8, datahandle, rowbytes, rowbytes);
 		/* Fill in the monopat. */
 		for (i = 0; i < 8; ++i)
 		  SET_IMG_PAT(macimg, i, (*datahandle)[i]);
@@ -416,44 +450,44 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 		/* Patterns have no masks, but defeat subsequent mask hacking. */
 		monodone = maskdone = TRUE;
 	}
-	if (w == 16 && h == 16 && img->rawmonodata != NULL) {
+	if (w == 16 && h == 16 && subimg->rawmonodata != NULL) {
 		/* Shape is appropriate for a small icon, make one. */
 		sicnhandle = NewHandle(32);
 		memset(*sicnhandle, 0, 32);
 		/* Read exactly 32 bytes. */
-		mac_copy_bytes(img->rawmonodata, 32, sicnhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmonodata, 32, sicnhandle, rowbytes, rowbytes);
 		macimg->monosicn = sicnhandle;
 		monodone = TRUE;
 	}
-	if (w == 32 && h == 32 && img->rawmonodata != NULL) {
+	if (w == 32 && h == 32 && subimg->rawmonodata != NULL) {
 		/* Shape is appropriate for a standard icon, make one. */
 		iconhandle = NewHandle(128);
 		memset(*iconhandle, 0, 128);
 		/* Read exactly 128 bytes. */
-		mac_copy_bytes(img->rawmonodata, 128, iconhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmonodata, 128, iconhandle, rowbytes, rowbytes);
 		macimg->monoicon = iconhandle;
 		monodone = TRUE;
 	}
-	if (img->maskdata != lispnil && (img->rawmaskdata == NULL || force)) {
+	if (subimg->maskdata != lispnil && (subimg->rawmaskdata == NULL || force)) {
 		numbytes = h * computed_rowbytes(w, 1);
-		img->rawmaskdata = xmalloc(numbytes);
-		interp_bytes(img->maskdata, numbytes, img->rawmaskdata, 0);
+		subimg->rawmaskdata = xmalloc(numbytes);
+		interp_bytes(subimg->maskdata, numbytes, subimg->rawmaskdata, 0);
 	}
-	if (w == 16 && h == 16 && img->rawmaskdata != NULL) {
+	if (w == 16 && h == 16 && subimg->rawmaskdata != NULL) {
 		/* Shape is appropriate for a small icon mask, make one. */
 		sicnhandle = NewHandle(32);
 		memset(*sicnhandle, 0, 32);
 		/* Read exactly 32 bytes. */
-		mac_copy_bytes(img->rawmaskdata, 32, sicnhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmaskdata, 32, sicnhandle, rowbytes, rowbytes);
 		macimg->masksicn = sicnhandle;
 		maskdone = TRUE;
 	}
-	if (w == 32 && h == 32 && img->rawmaskdata != NULL) {
+	if (w == 32 && h == 32 && subimg->rawmaskdata != NULL) {
 		/* Shape is appropriate for a standard icon, make one. */
 		iconhandle = NewHandle(128);
 		memset(*iconhandle, 0, 128);
 		/* Read exactly 128 bytes. */
-		mac_copy_bytes(img->rawmaskdata, 128, iconhandle, 0, rowbytes);
+		mac_copy_bytes(subimg->rawmaskdata, 128, iconhandle, rowbytes, rowbytes);
 		macimg->maskicon = iconhandle;
 		maskdone = TRUE;
 	}
@@ -468,7 +502,7 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 		(*pmhandle)->pixelSize = 1;
 		(*pmhandle)->cmpCount = 1;
 		(*pmhandle)->cmpSize = pixelsize;
-		(*pmhandle)->pmTable = interp_ctab(img->palette);
+		(*pmhandle)->pmTable = interp_ctab(img);
 		datahandle = NewHandle(8);
 		(*ppathandle)->patData = datahandle;
 		HLock(datahandle);
@@ -478,49 +512,37 @@ mac_interp_image(ImageFamily *imf, Image *img, int force)
 		macimg->colrpat = ppathandle;
 		colrdone = maskdone = TRUE;
 	}
-	if (img->colrdata != lispnil && (img->rawcolrdata == NULL || force)) {
+	if (subimg->colrdata != lispnil && (subimg->rawcolrdata == NULL || force)) {
 		numbytes = h * computed_rowbytes(w, img->pixelsize);
-		img->rawcolrdata = xmalloc(numbytes);
-		interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
+		subimg->rawcolrdata = xmalloc(numbytes);
+		interp_bytes(subimg->colrdata, numbytes, subimg->rawcolrdata, 0);
 	}
+    if (subimg->rawcolrdata == NULL && img->filedata != lispnil) {
+		make_image_from_file_image(imf, img, subimg, subi);
+    }
 	/* A sufficiently small tiling color image can be a color pattern. */
-	if (hasColorQD && w <= 64 && h <= 64 && img->istile && img->rawcolrdata != NULL) {
-		macimg->colrpat = mac_create_ppat(img);
+	if (hasColorQD && w <= 64 && h <= 64 && img->istile && subimg->rawcolrdata != NULL) {
+		macimg->colrpat = mac_create_ppat(subimg);
 		colrdone = maskdone = TRUE;
 	}
 	/* Finally, a color icon covers all other cases. */
 	if (hasColorQD && w <= 64 && h <= 64 && !img->istile
-	    && (img->rawcolrdata != NULL
+	    && (subimg->rawcolrdata != NULL
 	        || (macimg->monosicn == nil
 	            && macimg->masksicn == nil
 	            && macimg->monoicon == nil
 	            && macimg->maskicon == nil))) {
-	    macimg->colricon = (Handle) mac_create_cicn(img);
+	    macimg->colricon = (Handle) mac_create_cicn(subimg);
 	    macimg->cicn_inited = FALSE;
-		mac_init_cicn(img);
+		mac_init_cicn(subimg);
 		colrdone = TRUE;
 	}
-	if (!monodone && !colrdone) {
+	/* This will mostly happen for images that are too large. */
+	/* Special-case 88x96 because the image library does include
+	   some, for use by other interfaces. */
+	if (!monodone && !colrdone && !(w == 88 && h == 96)) {
 		run_warning("%dx%d image of \"%s\" could not be interpreted", w, h, imf->name);
 	}
-#if 0 /* broken */
-    /* If we have a color pattern of random size, and an 8x8 mono pattern, then we should
-       glue the mono pattern into the color pattern's mono slot.  This requires searching
-       all pairs of images in the family. */
-	for (img = imf->images; img != NULL; img = img->next) {
-		Image *img2;
-		MacImage *macimg2;
-		macimg = get_mac_image(img);
-		if (macimg->colrpat != nil) {
-			for (img2 = imf->images; img2 != NULL; img2 = img2->next) {
-				macimg2 = get_mac_image(img2);
-				if (macimg2->patdefined) {
-					memcpy((char *) &(*(macimg->colrpat)->pat1Data), (char *) &(macimg2->monopat), 8);
-				}
-			}
-		}
-	}
-#endif
 }
 
 PixPatHandle
@@ -550,11 +572,11 @@ mac_create_ppat(Image *img)
 	(*pmhandle)->pixelSize = pixelsize;
 	(*pmhandle)->cmpCount = 1;
 	(*pmhandle)->cmpSize = pixelsize;
-	(*pmhandle)->pmTable = interp_ctab(img->palette);
+	(*pmhandle)->pmTable = interp_ctab(img);
 	numbytes = actualh * rowbytes;
 	datahandle = NewHandle(numbytes);
 	(*ppathandle)->patData = datahandle;
-	mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+	mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes, rowbytes);
 	(*ppathandle)->patXValid = -1;
 	return ppathandle;
 }
@@ -601,7 +623,7 @@ mac_create_cicn(Image *img)
 	(*cicnhandle)->iconPMap.cmpCount = 1;
 	(*cicnhandle)->iconPMap.cmpSize = pixelsize;
 	(*cicnhandle)->iconPMap.planeBytes = 0;
-	(*cicnhandle)->iconPMap.pmTable = interp_ctab(img->palette);
+	(*cicnhandle)->iconPMap.pmTable = interp_ctab(img);
 	(*cicnhandle)->iconPMap.pmReserved = 0;
 	/* Configure the monochrome icon. */
 	SetRect(&((*cicnhandle)->iconBMap.bounds), 0, 0, actualw, actualh);
@@ -622,14 +644,14 @@ mac_create_cicn(Image *img)
 			img->rawcolrdata = xmalloc(xnumbytes);
 			interp_bytes(img->colrdata, xnumbytes, img->rawcolrdata, 0);
 		}
-		mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+		mac_copy_bytes(img->rawcolrdata, numbytes, datahandle, macrowbytes, rowbytes);
 	} else if (img->monodata != lispnil || img->rawmonodata != NULL) {
 		if (img->rawmonodata == NULL) {
 			int xnumbytes = h * computed_rowbytes(w, 1);
 			img->rawmonodata = xmalloc(xnumbytes);
 			interp_bytes(img->monodata, xnumbytes, img->rawmonodata, 0);
 		}
-		mac_copy_bytes(img->rawmonodata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+		mac_copy_bytes(img->rawmonodata, numbytes, datahandle, macrowbytes, rowbytes);
 		/* Make an ersatz color table. */
 		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
 	} else if (img->maskdata != lispnil || img->rawmaskdata != NULL) {
@@ -638,7 +660,7 @@ mac_create_cicn(Image *img)
 			img->rawmaskdata = xmalloc(xnumbytes);
 			interp_bytes(img->maskdata, xnumbytes, img->rawmaskdata, 0);
 		}
-		mac_copy_bytes(img->rawmaskdata, numbytes, datahandle, macrowbytes - rowbytes, rowbytes);
+		mac_copy_bytes(img->rawmaskdata, numbytes, datahandle, macrowbytes, rowbytes);
 		/* Make an ersatz color table. */
 		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
 	} else {
@@ -668,7 +690,7 @@ mac_create_cicn(Image *img)
 			img->rawmonodata = xmalloc(xnumbytes);
 			interp_bytes(img->monodata, xnumbytes, img->rawmonodata, 0);
 		}
-		mac_copy_bytes(img->rawmonodata, numbytes, monohandle, macbitrowbytes - bitrowbytes, bitrowbytes);
+		mac_copy_bytes(img->rawmonodata, numbytes, monohandle, macbitrowbytes, bitrowbytes);
 		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *monohandle, numbytes);
 		HUnlock(monohandle);
 	} else {
@@ -699,7 +721,7 @@ mac_create_cicn(Image *img)
 			img->rawmaskdata = xmalloc(xnumbytes);
 			interp_bytes(img->maskdata, xnumbytes, img->rawmaskdata, 0);
 		}
-		mac_copy_bytes(img->rawmaskdata, numbytes, maskhandle, macbitrowbytes - bitrowbytes, bitrowbytes);
+		mac_copy_bytes(img->rawmaskdata, numbytes, maskhandle, macbitrowbytes, bitrowbytes);
 		memcpy((char *) (*cicnhandle)->iconMaskData, *maskhandle, numbytes);
 		HUnlock(maskhandle);
 	} else {
@@ -722,21 +744,26 @@ mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump)
     HUnlock(desthandle);
 }
 
+/* Mac bitmap/pixmap data usually has a an even number of bytes per row, while
+   generic image data can use the smallest number, whether odd or even.  So
+   when we copy from generic to Mac-specific, we may need to insert an empty
+   byte at the end of each row. */
+
 static void
-mac_copy_bytes(char *data, int numbytes, Handle desthandle, int jump, int rowbytes)
+mac_copy_bytes(char *data, int numbytes, Handle desthandle, int macrowbytes, int rowbytes)
 {
 	int i, j;
 
     HLock(desthandle);
-    if (jump == 0) {
+    if (macrowbytes == rowbytes) {
 		memcpy(*desthandle, data, numbytes);
     } else {
 		j = 0;
 		for (i = 0; i < numbytes; ++i) {
-			(*desthandle)[i] = data[j++];
-			if (jump == 1) {
-				++i;
+			if ((i % macrowbytes) >= rowbytes) {
 				(*desthandle)[i] = 0;
+			} else {
+				(*desthandle)[i] = data[j++];
 			}
 		}
     }
@@ -755,8 +782,26 @@ mac_copy_bytes(char *data, int numbytes, Handle desthandle, int jump, int rowbyt
    the first (and only) PlotCIcon call. To prevent fragmentation, the handles are
    also moved to the top of the heap zone before this is done. */
 
+void mac_init_cicn_1(Image *img);
+
 void
 mac_init_cicn(Image *img)
+{
+	int subi;
+	CIconHandle cicnhandle;
+	MacImage *macimg;
+
+	if (img->numsubimages > 0) {
+		for (subi = 0; subi < img->numsubimages; ++subi) {
+			mac_init_cicn_1(img->subimages[subi]);
+		}
+	} else {
+		mac_init_cicn_1(img);
+	}
+}
+
+void
+mac_init_cicn_1(Image *img)
 {
 	CIconHandle cicnhandle;
 	MacImage *macimg;
@@ -780,68 +825,73 @@ mac_init_cicn(Image *img)
 	}
 }
 
-/* Given a list of color table entries, create and return a color table. */
+/* Given an image, create and return a color table, using either its Lisp palette
+   or the raw palette, whichever has colors in it. */
 
 static CTabHandle
-interp_ctab(Obj *palette)
+interp_ctab(Image *img)
 {
 	int len, i, r, g, b;
 	char *colorname;
-	Obj *head, *rest, *color;
+	Obj *palette, *head, *rest, *color;
 	ColorSpec *ctdata;
 	CTabHandle ctabhandle;
 
+	palette = img->palette;
 	if (!consp(palette)) {
 		/* (should look up named palette) */
 	}
-	len = length(palette);
-	if (len == 0)
+	if (palette == lispnil && img->rawpalette == NULL)
 	  return nil;
+	len = (palette != lispnil ? length(palette) : img->numcolors);
 	ctabhandle = (CTabHandle) NewHandle(8 + len * 8);
 	HLock((Handle) ctabhandle);
 	(*ctabhandle)->ctFlags = 0;
 	(*ctabhandle)->ctSeed = GetCTSeed();
 	(*ctabhandle)->ctSize = len - 1;
 	ctdata = (ColorSpec *) &((*ctabhandle)->ctTable);
-	for (i = 0, rest = palette; i < len; ++i, rest = cdr(rest)) {
-		head = car(rest);
-		ctdata[i].value = c_number(car(head));
-		color = cdr(head);
-		if (symbolp(car(color)) || stringp(car(color))) {
-			colorname = c_string(car(color));
-			if (strcmp(colorname, "white") == 0) {
-				ctdata[i].rgb.red   = 65535;
-				ctdata[i].rgb.green = 65535;
-				ctdata[i].rgb.blue  = 65535;
-			} else if (strcmp(colorname, "black") == 0) {
-				ctdata[i].rgb.red   = 0;
-				ctdata[i].rgb.green = 0;
-				ctdata[i].rgb.blue  = 0;
-			} else if (strcmp(colorname, "red") == 0) {
-				ctdata[i].rgb.red   = 65535;
-				ctdata[i].rgb.green = 0;
-				ctdata[i].rgb.blue  = 0;
-			} else if (strcmp(colorname, "green") == 0) {
-				ctdata[i].rgb.red   = 0;
-				ctdata[i].rgb.green = 65535;
-				ctdata[i].rgb.blue  = 0;
-			} else if (strcmp(colorname, "blue") == 0) {
-				ctdata[i].rgb.red   = 0;
-				ctdata[i].rgb.green = 0;
-				ctdata[i].rgb.blue  = 65535;
+	if (palette != lispnil) {
+		for (i = 0, rest = palette; i < len; ++i, rest = cdr(rest)) {
+			head = car(rest);
+			ctdata[i].value = c_number(car(head));
+			color = cdr(head);
+			if (symbolp(car(color)) || stringp(car(color))) {
+				colorname = c_string(car(color));
+				/* Look for predefined color names. */
+				if (strcmp(colorname, "white") == 0) {
+					r = 65535;  g = 65535;  b = 65535;
+				} else if (strcmp(colorname, "black") == 0) {
+					r = 0;  g = 0;  b = 0;
+				} else if (strcmp(colorname, "red") == 0) {
+					r = 65535;  g = 0;  b = 0;
+				} else if (strcmp(colorname, "green") == 0) {
+					r = 0;  g = 65535;  b = 0;
+				} else if (strcmp(colorname, "blue") == 0) {
+					r = 0;  g = 0;  b = 65535;
+				} else {
+					init_warning("No color named \"%s\" found, substituting gray",
+								 colorname);
+					r = g = b = i * 5000;
+				}
+			} else if (numberp(car(color))) {
+				/* The usual case is a triple of numbers. */
+				r = c_number(car(color));
+				g = c_number(cadr(color));
+				b = c_number(car(cddr(color)));
 			} else {
-				init_warning("No color named \"%s\" found, substituting gray",
-							 colorname);
-				ctdata[i].rgb.red   = i * 5000;
-				ctdata[i].rgb.green = i * 5000;
-				ctdata[i].rgb.blue  = i * 5000;
+				init_warning("palette color is not a name or set of numbers, substituting black");
+				r = g = b = 0;
 			}
-		} else if (numberp(car(color))) {
-			ctdata[i].rgb.red   = c_number(car(color));
-			ctdata[i].rgb.green = c_number(cadr(color));
-			ctdata[i].rgb.blue  = c_number(car(cddr(color)));
-		} else {
-			init_warning("palette color is not a name or set of numbers, ignoring");
+			ctdata[i].rgb.red   = r;
+			ctdata[i].rgb.green = g;
+			ctdata[i].rgb.blue  = b;
+		}
+	} else {
+		for (i = 0; i < len; ++i) {
+			ctdata[i].value = img->rawpalette[i * 4];
+			ctdata[i].rgb.red   = img->rawpalette[i * 4 + 1];
+			ctdata[i].rgb.green = img->rawpalette[i * 4 + 2];
+			ctdata[i].rgb.blue  = img->rawpalette[i * 4 + 3];
 		}
 	}
 	HUnlock((Handle) ctabhandle);
