@@ -1,5 +1,5 @@
 /* Initialization for the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -194,6 +194,12 @@ CursHandle watchcursor;
 CursHandle sendcursor;
 CursHandle receivecursor;
 
+/* Autoscrolling cursors. */
+CursHandle leftcursor;
+CursHandle rightcursor;
+CursHandle downcursor;
+CursHandle upcursor;
+
 /* This value is the current progress cursor. */
 
 int curcurs = 0;
@@ -283,6 +289,11 @@ init_cursors()
 	sendcursor = GetCursor(cSendPacket);
 	receivecursor = GetCursor(cReceivePacket);
 
+	leftcursor = GetCursor(cLeft);
+	rightcursor = GetCursor(cRight);
+	downcursor = GetCursor(cDown);
+	upcursor = GetCursor(cUp);
+
 	/* Designer-related cursors will be done later, if needed at all. */
 
 	dotdotdotpicture = (PicHandle) GetResource('PICT', 134);
@@ -1763,7 +1774,8 @@ player_setup_dialog()
 	selectedplayer = 0;
 	set_player_setup_button_states();
 	draw_default_button(playersetupwin, diPlayerSetupOK);
-	ShowWindow(playersetupwin);
+	/* This is a normal window dressed out as dialog. Handle accordingly. */
+	SelectTheWindow(playersetupwin);
 	return TRUE;
 }
 
@@ -2106,14 +2118,6 @@ launch_game()
 	}
 	/* Set up the images we'll be needing. */
 	init_terrain_images();
-
-// --- BUG FIX ----------------------------------------------
-
-	/* Should not be called twice. */
-//	init_unit_images();
-
-// --------------------------------------------------------
-
 	init_side_colors();
 	/* Trial side/player assignment, will create the default player if nececessary. */
 	make_trial_assignments();
@@ -2139,7 +2143,7 @@ launch_game_2()
 	if (area.width == 0 || area.height == 0)
 	  set_area_shape(DEFAULTWIDTH, DEFAULTHEIGHT, TRUE);
 
-// ---ALSO ASSIGN A REASONABLE CIRCUMFERENCE IF STILL UNDEFINED -------------------
+	/* Also assign a reasonable circumference if still undefined. */
 
 	/* Use default circumference for all those real-earth games in which it is undefined (zero). */
 	/* However, don't allow the circumference to be less than the area width. */
@@ -2147,8 +2151,6 @@ launch_game_2()
 	if (world.circumference == 0)
 		world.circumference = max(area.width, DEFAULTCIRCUMFERENCE);
 
-// ---------------------------------------------------------------------
-
 	/* About to do lengthy init calcs, show the progress bar. */
 	open_progress_dialog();
 	/* Synthesize the remainder of the initial game setup. */
@@ -2289,7 +2291,7 @@ announce_progress(int pctdone)
 	if (progresswin == nil)
 	  return;
 	/* (should notice that stop button was hit and use to escape from program) */
-	SelectWindow(progresswin);
+	SelectTheWindow(progresswin);
 	lastprogress = progress;
 	progress = min(max(0, pctdone), 100);
 	/* Redraw the progress bar app-defined item, but only if there was
@@ -2313,7 +2315,7 @@ finish_lengthy_process()
 
 	if (progresswin == nil)
 	  return;
-	SelectWindow(progresswin);
+	SelectTheWindow(progresswin);
 	/* Replace the text with a "reassuring" but uninformative message. */
 	GetDItem(progresswin, diProgressText, &itemtype, &itemhandle, &itemrect);
 	SetIText(itemhandle, "\pKeeping busy...");
@@ -2370,7 +2372,7 @@ init_ui(Side *side)
 void
 init_display()
 {
-	int p;
+	int p, mainwidth;
 	Handle handle;
 	
 	/* If dside never got assigned, try some hasty repairs. */
@@ -2444,19 +2446,24 @@ init_display()
 		   shown; (should) adjust to fit the screen size. */
 		first_windows = TRUE;
 		/* The game window is always useful. */
-		create_game_window();
-		ShowWindow(gamewin);
+		show_game_window();
 		/* The notice window is always useful. */
-		create_notice_window();
-		ShowWindow(noticewin);
+		show_notice_window();
 		/* This will be the main playing display, so it should be up. */
 		create_map(default_power, mapnum++);
+		/* Create world map of suitable size by default. */
+		get_main_screen_size(&mainwidth, 0);
+		for (p = NUMPOWERS - 1; p > 0; p--) {
+			if (area.width * hws[p] <= mainwidth / 4) { 
+				break;
+			}
+		}
+		create_map(p, 0);
 		/* This needs to come up so that players unfamiliar with the game
 		   will have some guidance. */
 		if (show_instructions) {
-			instructions_dialog();
+			show_instructions_window();
 		}
-
 		first_windows = FALSE;
 	}
 #ifdef DESIGNERS
@@ -2474,11 +2481,15 @@ void
 init_unit_images()
 {
 	int u;
+	ImageFamily *imf;
 
 	uimages = (ImageFamily **) xmalloc(numutypes * sizeof(ImageFamily *));
 
 	for_all_unit_types(u) {
-		uimages[u] = get_unit_type_images(dside, u);
+		imf = get_unit_type_images(dside, u);
+		if (DebugG)
+		  describe_imf(dside, "unit type", u_type_name(u), imf);
+		uimages[u] = imf;
 	}
 	init_unit_cicns();
 }
@@ -2488,17 +2499,21 @@ init_unit_images()
 void
 init_terrain_images()
 {
-	int 			t;
-	Image 		*img;
-	MacImage 		*macimg;
-	PixMapHandle 	pmhandle;
-	CTabHandle 	ctabhandle;
-	ColorSpec 		*ctdata;
+	int t;
+	Image *img;
+	ImageFamily *imf;
+	MacImage *macimg;
+	PixMapHandle pmhandle;
+	CTabHandle ctabhandle;
+	ColorSpec *ctdata;
 
 	/* First load terrain images. */
 	timages = (ImageFamily **) xmalloc(numttypes * sizeof(ImageFamily *));
 	for_all_terrain_types(t) {
-		timages[t] = get_terrain_type_images(dside, t);
+		imf = get_terrain_type_images(dside, t);
+		if (DebugG)
+		  describe_imf(dside, "terrain type", t_type_name(t), imf);
+		timages[t] = imf;
 	}
 
 	/* Then optionally load terrain colors. */
