@@ -88,13 +88,7 @@ create_list()
 	list->largeicons = FALSE; /* (should be a preference) */
 	list->sidecolors = default_sidecolors;	/* same default as for maps */
 	list->drawsizes = default_drawsizes;	/* same default as for maps */
-
-// --- USER CONTROL OF EMBLEMS --------------------------------------------
-
 	list->draw_emblems = default_draw_emblems;	/* same default as for maps */
-
-// ---------------------------------------------------------------------
-
 	/* (should share with toggle routine?) */
 	list->entryspacing = (list->largeicons ? largeentryspacing : smallentryspacing);
 	list->firstvisible = 0;
@@ -103,9 +97,11 @@ create_list()
 	list->next = listlist;
 	listlist = list;
 	/* Make a window for the list, give it scrollbars. */
-	list->window = GetNewWindow(wList, nil, (WindowPtr) -1L);
+	if (hasColorQD)
+		list->window = GetNewCWindow(wList, nil, NULL);
+	else	list->window = GetNewWindow(wList, nil, NULL);
 	stagger_window(list->window, &lastlisth, &lastlistv);
-	ShowWindow(list->window);
+	SelectTheWindow(list->window);
 	SetPort(list->window);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
@@ -281,9 +277,10 @@ draw_list_contents(List *list)
 	for (line = list->firstvisible; line <= list->lastvisible; ++line) {
 		draw_unit_list_entry(list, line, FALSE);
 	}
-	HiliteControl(list->vscrollbar, ((numvisunits < list->numunits) ? 0 : 255));
+	/* The HiliteControl calls at this point caused a display bug. They were not
+	necessary since UpdateControls is smart enough to figure out if a scrollbar
+	should be displayed as active or not depending on the window size. */
 	viswidth = winrect.right - winrect.left - sbarwid;
-	HiliteControl(list->hscrollbar, ((viswidth < maxlistwidth) ? 0 : 255));
 }
 
 void
@@ -494,7 +491,7 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 	/* Draw the task. */
 	MoveTo(x, texty);
 	if (unit->plan && unit->plan->tasks) {
-		task_desc(spbuf, unit->side, unit->plan->tasks);
+		task_desc(spbuf, unit->side, unit, unit->plan->tasks);
 	} else {
 		strcpy(spbuf, "  -  ");
 	}
@@ -553,7 +550,7 @@ zoom_list(List *list, int part)
 		set_standard_state(listwin, maxlistwidth, maxh);
 	}
 	EraseRect(&listwin->portRect);
-	ZoomWindow(listwin, part, true);
+	ZoomWindow(listwin, part, false);
 	adjust_list_decor(list);
 }
 
@@ -567,9 +564,9 @@ adjust_list_decor(List *list)
 	w = list->window->portRect.right - list->window->portRect.left;
 	h = list->window->portRect.bottom - list->window->portRect.top;
 	MoveControl(list->vscrollbar, w - sbarwid, listtoph - 1);
-	SizeControl(list->vscrollbar, sbarwid + 1, h - listtoph - sbarwid + 1);
+	SizeControl(list->vscrollbar, sbarwid + 1, h - listtoph - sbarwid + 2);
 	MoveControl(list->hscrollbar, fixedfieldwidth - 1, h - sbarwid);
-	SizeControl(list->hscrollbar, w - fixedfieldwidth - sbarwid + 1, sbarwid + 1);
+	SizeControl(list->hscrollbar, w - fixedfieldwidth - sbarwid + 2, sbarwid + 1);
 	set_list_scrollbars(list);
 }
 
@@ -658,7 +655,7 @@ list_hscroll_fn(ControlHandle control, short code)
 
 #if 1
 
-/* --- NEW CODE THAT FIXES HORISONTAL SCROLLING BUG -----------------------------
+/* 	New code that fixes horisontal scrolling bug. */
 
 	/* Return to avoid flicker if stuck at the end of the scrollbar. */
 	if (curvalue == oldvalue) return;
@@ -670,7 +667,7 @@ list_hscroll_fn(ControlHandle control, short code)
 	draw_list(curlist);
 	EndUpdate(listwin);
 
-#elif
+#else
 
 	/* Scroll the already-drawn bits. */
 	tmprgn = NewRgn();
@@ -691,6 +688,11 @@ list_hscroll_fn(ControlHandle control, short code)
 void
 scroll_list_using_key(List *list, char key)
 {
+	GrafPtr	oldport;
+
+	GetPort(&oldport);
+	/* Set current port to the list window. */
+	SetPort(list->window);
 	curlist = list;
 	switch (key) {
 		case 0x7D:		/* Down Arrow */
@@ -713,6 +715,7 @@ scroll_list_using_key(List *list, char key)
 			beep();
 			break;
 	}
+	SetPort(oldport);
 }
 
 
@@ -830,8 +833,6 @@ toggle_list_sidecolors(List *list)
 	force_update(list->window);
 }
 
-// --- USER CONTROL OF EMBLEMS --------------------------------------------
-
 void
 toggle_list_draw_emblems(List *list)
 {
@@ -839,8 +840,6 @@ toggle_list_draw_emblems(List *list)
 	force_update(list->window);
 }
 
-// ---------------------------------------------------------------------
-
 void
 update_unit_in_lists(Unit *unit)
 {
@@ -949,30 +948,14 @@ activate_list(List *list, int activate)
 {
 	Rect growRect;
 
+	GrafPtr oldport;
+
 	if (activate) {
 		HiliteControl(list->vscrollbar, 0);
 		HiliteControl(list->hscrollbar, 0);
-#if 0
-		/* the controls must be redrawn on activation: */
-		(*(list->vscrollbar))->contrlVis = 255;
-		(*(list->hscrollbar))->contrlVis = 255;
-		InvalRect(&(*(list->vscrollbar))->contrlRect);
-		InvalRect(&(*(list->hscrollbar))->contrlRect);
-#endif
-		/* The growbox needs to be redrawn on activation. */
-		growRect = list->window->portRect;
-		/* adjust for the scrollbars */
-		growRect.top = growRect.bottom - sbarwid;
-		growRect.left = growRect.right - sbarwid;
-		InvalRect(&growRect);
 	} else {
-		/* The scrollbars must be hidden on deactivation. */
 		HiliteControl(list->vscrollbar, 255);
 		HiliteControl(list->hscrollbar, 255);
-/*		HideControl(list->vscrollbar);
-		HideControl(list->hscrollbar); */
-		/* The growbox should be changed immediately on deactivation. */
-		DrawGrowIcon(list->window);
 	}
 }
 
