@@ -23,14 +23,13 @@ any later version.  See the file COPYING.  */
 
 static void draw_feature_names(Map *map, Rect drawrect);
 static void update_grid_leakage(Map *map);
-
+static void adjust_latlong_interval(Map *map);
 static void draw_meridians(Map *map, Rect drawrect);
 static void draw_info_text(Map *map, int x, int y, int len, char *buf);
 static void draw_unit_names_only(Map *map, int x, int y);
 
 extern void draw_gridlines(Map *map, int x, int y);
 extern void draw_shorelines(Map *map, int x, int y);
-extern void xform_fractional(Map *map, int x, int y, int xf, int yf, int *sxp, int *syp);
 
 Map *frontmap = NULL;
 
@@ -76,9 +75,6 @@ PicHandle blcontrols = nil;
 
 /* Note: In at least one case (longitude text in draw_meridians) it is clearly necessary */
 /* to do a double UNWRAP (both add and substract sxmax) in order to cover all cases. */
-/* This may well turn out be true also for the UNWRAP macros below. */
-
-// --- PERFORM DOUBLE UNWRAP FOR ALL THREE MACROS ---------------------------- 
 
 /* Repeat (function) both if sx < 0 and if sx is within one hex from sxmax. */
 #define UNWRAP_SX(map, sx, function) { \
@@ -94,18 +90,6 @@ PicHandle blcontrols = nil;
 	} \
 }
 
-/* Repeat (function) once using sx shifted one area width to the right. */
-/*
-#define UNWRAP_SX(map, sx, function) { \
-	(function); \
-	if (area.xwrap && (sx) < 0) { \
-		(sx) += (map)->vp->sxmax; \
-		(function); \
-		(sx) -= (map)->vp->sxmax; \
-	} \
-}
-*/
-
 /* Repeat (function) both if rect.left < 0 and if rect.right > sxmax. */
 #define UNWRAP_RECT(map, rect, function) { \
 	(function); \
@@ -120,18 +104,6 @@ PicHandle blcontrols = nil;
 	} \
 }
 
-/* Repeat (function) once using rect offset one area width to the right. */
-/*
-#define UNWRAP_RECT(map, rect, function) { \
-	(function); \
-	if (area.xwrap && (rect).left < 0) { \
-		OffsetRect(&(rect), (map)->vp->sxmax, 0); \
-		(function); \
-		OffsetRect(&(rect), -(map)->vp->sxmax, 0); \
-	} \
-}
-*/
-
 /* Repeat (function) both if polyBBox.left < 0 and if polyBBox.right > sxmax. */
 #define UNWRAP_POLY(map, poly, function) { \
 	(function); \
@@ -146,20 +118,6 @@ PicHandle blcontrols = nil;
 	} \
 }
 
-/* Repeat (function) once using poly offset one area width to the right. */
-/*
-#define UNWRAP_POLY(map, poly, function) { \
-	(function); \
-	if (area.xwrap && (*(*(poly))).polyBBox.left < 0) { \
-		OffsetPoly((poly), (map)->vp->sxmax, 0); \
-		(function); \
-		OffsetPoly((poly), -(map)->vp->sxmax, 0); \
-	} \
-}
-*/
-
-// -----------------------------------------------------------------------
-
 /* Given a map and a cell, compute the pixel coords of the cell's UL corner.
    This is the core routine that relates cells and pixels. */
 
@@ -169,8 +127,8 @@ Map *map;
 int x, y, *sxp, *syp;
 {
 	xform_cell(map->vp, x, y, sxp, syp);
-	/* Shift the basic viewport result to account for both origin-shift and map decor. */
-	*sxp += map->osx + map->conw;  *syp += map->osy + map->toph;
+	/* Shift the viewport for map decor. */
+	*sxp += map->conw; *syp += map->toph;
 }
 
 void
@@ -179,8 +137,30 @@ Map *map;
 int x, y, xf, yf, *sxp, *syp;
 {
 	xform_cell_fractional(map->vp, x, y, xf, yf, sxp, syp);
-	/* Shift the basic viewport result to account for both origin-shift and map decor. */
-	*sxp += map->osx + map->conw;  *syp += map->osy + map->toph;
+	/* Shift the viewport for map decor. */
+	*sxp += map->conw; *syp += map->toph;
+}
+
+/* These two functions bypass the vertical offset when viewing at an angle. */
+
+void
+xform_flat(map, x, y, sxp, syp)
+Map *map;
+int x, y, *sxp, *syp;
+{
+	xform_cell_flat(map->vp, x, y, sxp, syp);
+	/* Shift the viewport for map decor. */
+	*sxp += map->conw; *syp += map->toph;
+}
+
+void
+xform_fractional_flat(map, x, y, xf, yf, sxp, syp)
+Map *map;
+int x, y, xf, yf, *sxp, *syp;
+{
+	xform_cell_fractional_flat(map->vp, x, y, xf, yf, sxp, syp);
+	/* Shift the viewport for map decor. */
+	*sxp += map->conw; *syp += map->toph;
 }
 
 void
@@ -190,8 +170,8 @@ Unit *unit;
 int *sxp, *syp, *swp, *shp;
 {
 	xform_unit(map->vp, unit, sxp, syp, swp, shp);
-	/* Shift the basic viewport result to account for both origin-shift and map decor. */
-	*sxp += map->osx + map->conw;  *syp += map->osy + map->toph;
+	/* Shift the viewport for map decor. */
+	*sxp += map->conw; *syp += map->toph;
 }
 
 void
@@ -201,8 +181,8 @@ Unit *unit;
 int *sxp, *syp, *swp, *shp;
 {
 	xform_unit_self(map->vp, unit, sxp, syp, swp, shp);
-	/* Shift the basic viewport result to account for both origin-shift and map decor. */
-	*sxp += map->osx + map->conw;  *syp += map->osy + map->toph;
+	/* Shift the viewport for map decor. */
+	*sxp += map->conw; *syp += map->toph;
 }
 
 void
@@ -212,10 +192,10 @@ Unit *transport, *unit;
 int sx, sy, sw, sh, *sxp, *syp, *swp, *shp;
 {
 	/* Transform the coordinates back to relative values. */
-	sx -= map->osx + map->conw;  sy -= map->osy + map->toph;
+	sx -= map->conw;  sy -= map->toph;
 	xform_occupant(map->vp, transport, unit, sx, sy, sw, sh, sxp, syp, swp, shp);
-	/* Shift the basic viewport result to account for both origin-shift and map decor. */
-	*sxp += map->osx + map->conw;  *syp += map->osy + map->toph;
+	/* Shift the viewport for map decor. */
+	*sxp += map->conw; *syp += map->toph;
 }
 
 /* Un-transform screen coordinates (as supplied by mouse perhaps) into
@@ -242,7 +222,7 @@ m_nearest_boundary(Map *map, int sx, int sy, int *xp, int *yp, int *dirp)
 int
 m_nearest_unit(Map *map, int sx, int sy, Unit **unitp)
 {
-	return nearest_unit(map->vp, sx - map->conw, sy - map->toph, unitp);
+	return nearest_unit(dside, map->vp, sx - map->conw, sy - map->toph, unitp);
 }
 
 /* This tests whether the given cell might possibly be visible on the given map.
@@ -303,9 +283,17 @@ create_map(int power, int id)
 	pick_a_focus(dside, &x, &y);
 	set_view_focus(map->vp, x, y);
 	/* Set default values for the display controls. */
-	map->conw = conwid;
-	map->toplineh = (default_draw_topline ? tophgt : 0);
-	map->topunith = (default_draw_topunit ? topunithgt : 0);
+	if (map == worldmap) {
+		map->conw = 0;		/* No decor for the world map. */
+		map->toplineh = 0;
+		map->topunith = 0;
+		map->sbarwid = floatsbarwid;	/* Turned off later, needed to position sbars. */
+	} else {
+		map->conw = (default_draw_cpanel ? conwid : 0);
+		map->toplineh = (default_draw_topline ? tophgt : 0);
+		map->topunith = (default_draw_topunit ? topunithgt : 0);
+		map->sbarwid = sbarwid;
+	}
 	map->toph = map->toplineh + map->topunith;
 	/* Mac viewport needs different xform style, let the common code know about this. */
 	map->vp->wide_viewport = TRUE;
@@ -339,16 +327,9 @@ create_map(int power, int id)
 	map->vp->draw_clouds = FALSE;
 	map->vp->draw_storms = TRUE;
 	map->vp->draw_grid = default_draw_grid;
-	
-	/* Display AI info by default if there is an AI present. */
-	map->vp->draw_ai = side_has_ai(dside);
-
-// --- USER CONTROL OF EMBLEMS -----------------------------------------
-
+	/* Display AI info if we have one and draw_ai is on by default. */
+	map->vp->draw_ai = default_draw_ai && side_has_ai(dside);
 	map->draw_emblems = default_draw_emblems;
-
-// ------------------------------------------------------------------
-
 	map->sidecolors = default_sidecolors;
 	map->iconmasks = default_iconmasks;
 	map->boxmasks = default_boxmasks;
@@ -379,18 +360,8 @@ create_map(int power, int id)
 	if (map == worldmap)
 		map->vp->draw_other_maps = TRUE;
 	map->vp->latlong_interval = default_latlong_interval;
-	/* Override default and choose larger interval if spacing would make too cluttered. */
-
-// --- FIXES CRASHES WHEN LAUNCHING FLATTOPS ETC -----------------------------------
-
-	if (map->vp->latlong_interval > 0 && world.circumference > 0)
-
-// -------------------------------------------------------------------------
-
-		while (((map->vp->latlong_interval * world.circumference * map->vp->hw) / 21600) < 32)
-			map->vp->latlong_interval *= 2;
-
-
+	/* Adjust the default latlong interval so that it is appropriate for this magnification. */
+	adjust_latlong_interval(map);
 	map->autoselect = defaultautoselect;
 	map->moveonclick = defaultmoveonclick;
 	map->drawsizes = default_drawsizes;
@@ -401,53 +372,96 @@ create_map(int power, int id)
 	map->next = maplist;
 	maplist = map;
 	if (hasColorQD) {
-		win = GetNewCWindow(wMap, nil, (WindowPtr) -1L);
+		win = GetNewCWindow((map == worldmap ? wFloatResize : wMap), NULL, NULL);
 	} else {
-		win = GetNewWindow(wMap, nil, (WindowPtr) -1L);
+		win = GetNewWindow((map == worldmap ? wFloatResize : wMap), NULL, NULL);
 	}
 	map->window = win;
-	stagger_window(win, &lastmaph, &lastmapv);
-	if (first_windows) {
-		get_main_screen_size(&mainwidth, &mainheight);
-		/* Use the main screen size if full-sized map desired, else
-		   leave room for game window at side and notices at bottom. */
-		SizeWindow(win,
-				   mainwidth - (fullsize_map ? 0 : gamewinw) - 10,
-				   mainheight - GetMBarHeight() - 16  - (fullsize_map ? 0 : 150) - 7,
+	if (map == worldmap) {
+	
+		Point winTL;
+	
+		/* Make worldmapwin just big enough to contain the area. */
+		SizeWindow(win, 
+				area.width * hws[power], 
+				area.height * hcs[power], 
+			FALSE);
+		/* Put it in the right corner of the frontmap if it exists. */
+		if (frontmap) {
+			winTL.h = frontmap->window->portRect.right - area.width * hws[power] + 7;
+			winTL.v = GetMBarHeight() + 15 + 21;
+		/* Else put it in the right corner of the main screen. */
+		} else {					
+			get_main_screen_size(&mainwidth, &mainheight);
+			winTL.h = mainwidth - area.width * hws[power] - 3;
+			winTL.v = GetMBarHeight() + 15;
+		}
+		/* Move it out of the way of game window if necessary. */
+		if (IsVisible(gamewin)) {
+			winTL.h -= gamewinw + 8;
+		}
+		MoveWindow(win, winTL.h, winTL.v, false);
+	} else {
+		stagger_window(win, &lastmaph, &lastmapv);
+		if (first_windows) {
+			get_main_screen_size(&mainwidth, &mainheight);
+			/* Use the main screen size if full-sized map desired, else
+			   leave room for game window at side and notices at bottom. */
+			SizeWindow(win,
+					   /* Make room for edges under Apple Platinum theme. */
+					   mainwidth - (fullsize_map ? 0 : gamewinw + 12) - 10,
+					   mainheight - GetMBarHeight() - 21 - (fullsize_map ? 0 : 150) - 7, 
 				   FALSE);
+			/* Make roon for edges under Apple Platinum theme. */
+			MoveWindow(win, 5, GetMBarHeight() + 21, FALSE);
+		}
 	}
-	ShowWindow(win);
 	SetPort(win);
+	/* Make the world map floating. */	
+	if (map == worldmap)
+	    MakeFloat(win);
+	/* MakeFloat does not hilite the world map, it only brings it to front. */ 
+	SelectTheWindow(win);
 
 	/* Use window title "World Map" if world map, else use the map number. */
 	if (map == worldmap) {
 		worldmapwin = win;
-		sprintf(spbuf, "World Map");
+		SetWTitle(win, "\pWorld Map");
 	} else {
 		sprintf(spbuf, "Map %d", map->id);
+		add_window_menu_item(spbuf, win);
 	}
-	add_window_menu_item(spbuf, win);
 	set_content_rect(map);
 	m_center_on_focus(map);
 	/* Make the scrollbars. */
 	vscrollrect = map->window->portRect;
 	vscrollrect.top -= 1;
-	vscrollrect.bottom -= sbarwid - 1;
-	vscrollrect.left = vscrollrect.right - sbarwid;
+	vscrollrect.bottom -= map->sbarwid - 1;
+	vscrollrect.left = vscrollrect.right - map->sbarwid;
 	vscrollrect.right += 1;
 	map->vscrollbar =
 		NewControl(win, &vscrollrect, "\p", 1,
 			 map->vp->sy, map->vp->symin, map->vp->symax, scrollBarProc, 0L);
 	hscrollrect = win->portRect;
-	hscrollrect.top = hscrollrect.bottom - sbarwid;
+	hscrollrect.top = hscrollrect.bottom - map->sbarwid;
 	hscrollrect.bottom += 1;
 	hscrollrect.left += map->conw;
-	hscrollrect.right -= sbarwid - 1;
+	hscrollrect.right -= map->sbarwid - 1;
 	map->hscrollbar =
 		NewControl(win, &hscrollrect, "\p", 1,
 			 map->vp->sx, map->vp->sxmin, map->vp->sxmax, scrollBarProc, 0L);
+	/* Turn off scrollbars that should be off now when they have been 
+	positioned correctly. */
+	if (map == worldmap)	
+		map->sbarwid = 0;	
+	else	map->sbarwid = (default_draw_scrollbars ? sbarwid : 0);
+	if (map->sbarwid == 0) {
+		SizeControl(map->vscrollbar, 0, 0);
+		SizeControl(map->hscrollbar, 0, 0);
+	}
 	set_map_scrollbars(map);
 	++nummaps;
+	draw_growicon_nolines(map->window);
 
 	/* Save the onscreen gworld */
 	GetGWorld(&map->windowPortPtr,&map->deviceHdl);
@@ -464,13 +478,14 @@ create_map(int power, int id)
 		/* Return zero to indicate no map was created */
 		return NULL;			
 	}
-	
+	/* Make the new map frontmap. */
+	if (map != worldmap)
+		frontmap = map;	
 	/* Optimize gworld size and position and also draw the map */
 	update_gworld(map);
 
 	/* Finally draw the new other-map boxes */
 	draw_related_maps(map);	
-
 	return map;
 }
 
@@ -480,27 +495,28 @@ void
 set_content_rect(Map *map)
 {
 	map->contentrect = map->window->portRect;
-	map->contentrect.left += map->conw;  map->contentrect.top += map->toph;
-	map->contentrect.right -= sbarwid;  map->contentrect.bottom -= sbarwid;
+	map->contentrect.left += map->conw;  
+	map->contentrect.top += map->toph;
+	map->contentrect.right -= map->sbarwid;  
+	map->contentrect.bottom -= map->sbarwid;
 	set_view_size(map->vp,
 				  map->contentrect.right - map->contentrect.left,
 				  map->contentrect.bottom - map->contentrect.top);
 }
 
 /* Adjust the appearance and thumb of the scroll bars to reflect the map.  This is
-   needed whenever the map is adjusted under program control, such as when magnifying
-   or scrolling to a specified location. */
+needed whenever the map is adjusted under program control, such as when magnifying
+or scrolling to a specified location. */
 
 void
 set_map_scrollbars(Map *map)
 {
 	int hilite;
 	
-	/* Scale the scrollbars to circumvent the 32K limit on control values. */
-
-	/* This hack solves the scrolling problem for big maps at high magnification */
-	/* Note that all direct access of the scrollbar values from now on first must */
-	/* simulate the scaling that is applied below in order to get correct values! */
+	/* Scale the scrollbars to circumvent the 32K limit on control values. This hack 
+	solves the scrolling problem for big maps at high magnification. Note that all direct
+	access of the scrollbar values from now on first must simulate the scaling that is 
+	applied below in order to get correct values! */
 
 	int	scaled_sxmin, scaled_sx, scaled_sxmax, scaled_symin, scaled_sy, scaled_symax;
 
@@ -513,17 +529,11 @@ set_map_scrollbars(Map *map)
 		scaled_sxmin /= 2;	
 		scaled_sx /= 2;
 	}
-
 	/* Apply the scaled values */
 	SetCtlMin(map->hscrollbar, scaled_sxmin);
 	SetCtlMax(map->hscrollbar, scaled_sxmax);
 	SetCtlValue(map->hscrollbar, scaled_sx);
-
-	hilite = (map->vp->sxmin != map->vp->sxmax);
-	/* Adjust the hiliting of the scrollbar, but only if the window is in front,
-	   otherwise the scrollbar should remain unhilited. */
-	if (map->window == FrontWindow())
-	  HiliteControl(map->hscrollbar, (hilite ? 0 : 255));
+	/* (HiliteControl call removed - caused display bug). */
 	DGprintf("Hscroll (%shilite) is %d -- %d -- %d\n",
 			 (hilite ? "" : "no "), GetCtlMin(map->hscrollbar),
 	         GetCtlValue(map->hscrollbar), GetCtlMax(map->hscrollbar));
@@ -538,15 +548,11 @@ set_map_scrollbars(Map *map)
 		scaled_symin /= 2;	
 		scaled_sy /= 2;
 	}
-
 	/* Apply the scaled values */
 	SetCtlMin(map->vscrollbar, scaled_symin);
 	SetCtlMax(map->vscrollbar, scaled_symax);
 	SetCtlValue(map->vscrollbar, scaled_sy);
-
-	hilite = (map->vp->symin != map->vp->symax);
-	if (map->window == FrontWindow())
-	  HiliteControl(map->vscrollbar, (hilite ? 0 : 255));
+	/* (HiliteControl call removed- caused display bug). */
 	set_content_rect(map);
 	DGprintf("Vscroll (%shilite) is %d -- %d -- %d\n",
 			 (hilite ? "" : "no "), GetCtlMin(map->vscrollbar),
@@ -560,10 +566,42 @@ void
 set_map_power(Map *map, int power)
 {
 	set_view_power(map->vp, power);
+
+	/* Make sure the latlong interval is appropriate for the new magnification. */
+	adjust_latlong_interval(map);
+
 	if (power >= 4 && cellrgns[power] == nil)
 	  make_cell_clip(power);
 }
 
+void
+adjust_latlong_interval(Map *map)
+{
+	/* Skip if meridians are not defined. */
+	if (world.circumference <= 0)
+		return;
+	/* First set the interval so that it corresponds to 64 pixels. */
+	map->vp->latlong_interval = 64 * 21600 / (world.circumference * map->vp->hw);
+	/* Then adjust it upwards to an even number of degrees or minutes. */
+	if (map->vp->latlong_interval <= 10)
+	      	map->vp->latlong_interval = 10;
+	else if (map->vp->latlong_interval <= 30)
+		map->vp->latlong_interval = 30;
+	else if (map->vp->latlong_interval <= 60)
+		map->vp->latlong_interval = 60;
+	else if (map->vp->latlong_interval <= 120)
+		map->vp->latlong_interval = 120;
+	else if (map->vp->latlong_interval <= 300)
+		map->vp->latlong_interval = 300;
+	else if (map->vp->latlong_interval <= 600)
+		map->vp->latlong_interval = 600;
+	else if (map->vp->latlong_interval <= 1200)
+		map->vp->latlong_interval = 1200;
+	else if (map->vp->latlong_interval <= 3600)
+		map->vp->latlong_interval = 3600;
+	else	map->vp->latlong_interval = 0;
+}
+
 /* This is the old set_focus from maccmd.c moved here and renamed for consistency. */
 
 void
@@ -597,6 +635,9 @@ set_map_angle(Map *map, int angle)
 	/* Erase other-map boxes in other windows. */
 	draw_related_maps(map);
 
+	/* Compute a suitable vertscale value. */
+	map->vp->vertscale = max(500, area.maxelev) / 100 ;
+
 	/* Change the view angle. */
 	set_view_angle(map->vp, angle);
 
@@ -625,7 +666,9 @@ grow_map(Map *map, int w, int h)	/* Manual resizing by dragging the corner */
 	adjust_map_decor(map);
 
 	/* Dont forget this little guy. */
-	DrawGrowIcon(map->window);
+	if (map->sbarwid < 15)
+		draw_growicon_nolines(map->window);
+	else	DrawGrowIcon(map->window);
 
 	/* Update maps in background immediately. */
 	for_all_maps(map2) {
@@ -647,21 +690,23 @@ zoom_map(Map *map, int part)	/* Resizing by clicking in the zoom box */
 
 	if (part == inZoomOut) {
 	      set_standard_state(map->window,
-		      area.width  * map->vp->hw  + map->conw + sbarwid + 1,
-		      area.height * map->vp->hch + map->vp->hh - map->vp->hch + sbarwid + map->toph + 1);
+		      area.width  * map->vp->hw  + map->conw + map->sbarwid,
+		      area.height * map->vp->hch + map->vp->hh - map->vp->hch + map->sbarwid + map->toph);
 	}
 	
 	/* Erase other-map boxes in other windows. */
 	draw_related_maps(map);
 
 	/* Zoom the window. */
-	ZoomWindow(map->window, part, true);
+	ZoomWindow(map->window, part, false);
 
 	/* Resize scrollbars. */
 	adjust_map_decor(map);
 
 	/* Dont forget this little guy. */
-	DrawGrowIcon(map->window);
+	if (map->sbarwid < 15)
+		draw_growicon_nolines(map->window);
+	else	DrawGrowIcon(map->window);
 
 	/* Update maps in background immediately. */
 	for_all_maps(map2) {
@@ -686,10 +731,10 @@ adjust_map_decor(Map *map)
 
 	w = mapwin->portRect.right - mapwin->portRect.left;
 	h = mapwin->portRect.bottom - mapwin->portRect.top;
-	MoveControl(map->hscrollbar, map->conw - 1, h - sbarwid);
-	SizeControl(map->hscrollbar, w - map->conw - sbarwid + 2, sbarwid + 1);
-	MoveControl(map->vscrollbar, w - sbarwid, -1);
-	SizeControl(map->vscrollbar, sbarwid + 1, h - sbarwid + 1 + 1);
+	MoveControl(map->hscrollbar, map->conw - 1, h - map->sbarwid);
+	SizeControl(map->hscrollbar, w - map->conw - map->sbarwid + 2, map->sbarwid + 1);
+	MoveControl(map->vscrollbar, w - map->sbarwid, -1);
+	SizeControl(map->vscrollbar, map->sbarwid + 1, h - map->sbarwid + 1 + 1);
 }
 
 void
@@ -725,35 +770,43 @@ activate_map(map, activate)
 Map *map;
 int activate;
 {
+ 	EvQEl *myQElPtr;
+	int below = FALSE;
+	WindowPtr win;
 	Rect growRect;
-
+	Map *map2;
+ 
 	if (activate) {
+		/* Hilite controls in the activated map. */
 		HiliteControl(map->vscrollbar, 0);
 		HiliteControl(map->hscrollbar, 0);
-		/* Controls need to be redrawn on activation. */
-		(*(map->vscrollbar))->contrlVis = 255;
-		(*(map->hscrollbar))->contrlVis = 255;
-		InvalRect(&(*(map->vscrollbar))->contrlRect);
-		InvalRect(&(*(map->hscrollbar))->contrlRect);
-		/* The growbox needs to be redrawn on activation. */
-		growRect = map->window->portRect;
-		/* adjust for the scrollbars */
-		growRect.top = growRect.bottom - sbarwid;
-		growRect.left = growRect.right - sbarwid;
-		InvalRect(&growRect);
-		/* Remember this as the map most recently in front, even if it moves behind
-		   later. */
-		frontmap = map;
+		/* Make it the new frontmap. */
+		if (map != worldmap) {
+			frontmap = map;
+			/* Unhilite the scrollbars in all maps below this one. This is necessary 
+			as the window manager cheks only if the next window below the current
+			one is active and should receive a deactivate event. Since we always hilite
+			the controls in the frontmap it may escape deactivation if located further 
+			down. */
+			for_all_windows(win) {
+				map2 = map_from_window(win);
+				if (map2 && below) {
+					HiliteControl(map2->vscrollbar, 255);
+					HiliteControl(map2->hscrollbar, 255);
+				}			
+				/* Set the below flag. */
+				if (map2 == map)
+					below = TRUE;
+			}
+		}
 	} else {
-		/* The scrollbars must be deactivated. */
-		HiliteControl(map->vscrollbar, 255);
-		HiliteControl(map->hscrollbar, 255);
-#if 0  /* We don't want to hide them though, because the window bg is not white. */
-		HideControl(map->vscrollbar);
-		HideControl(map->hscrollbar);
-#endif
-		/* The growbox should be changed immediately on deactivation. */
-		DrawGrowIcon(map->window);
+		/* Check the current frontmap. */
+		frontmap = front_map();
+		/* Inactivate the map only if it lost frontmap status. */
+		if (map != frontmap) {
+			HiliteControl(map->vscrollbar, 255);
+			HiliteControl(map->hscrollbar, 255);
+		}
 	}
 }
 
@@ -773,18 +826,31 @@ map_from_window(WindowPtr window)
 	return NULL;
 }
 
-/* Return the frontmost map, whether or not it's the frontmost window. */
+/* Return the frontmost map, whether or not it's the frontmost window. This
+function is now called only when the front map actually changes. The variable
+frontmap is used everywhere else. Note that frontmap->window does not have
+to be the FrontWindow, or even the (non-floating) topWindow. It can be any
+window in the stack. Nevertheless, it is always active and responds to keyboard 
+commands. This is non-standard behaviour for a Mac window, but it makes the 
+play a lot easier. */
 
 Map *
 front_map()
 {
+	WindowPtr win;
 	Map *map;
 
-	map = map_from_window(FrontWindow());
-	if (map != NULL)
-	  return map;
-	if (frontmap != NULL)
-	  return frontmap;
+	/* Search all non-floating windows from top down 
+	until we find a map. */
+	for_all_windows(win) {
+		if (IsFloating(win))
+		    continue;
+		map = map_from_window(win);
+		/* front_map is called once before the worldmap 
+		is made floating! */ 
+		if (map && map != worldmap)
+			return map;
+	}
 	return NULL;
 }
 
@@ -846,23 +912,30 @@ Map *map;
 			}
 		}
 	}
-	if (frontmap == map)
-	  frontmap = NULL;
 	free_vp(map->vp);
 	free(map->selections);
 
 	/* World map crash fix. */
 	if (worldmap == map)
 	  worldmap = NULL;
-	/* Decrement mapnum, but only if this was normal map. */
-	else
-	  --mapnum;
+	/* Decrement mapnum, but only if this was a normal map. */
+	else 
+	   --mapnum;
 	/* Always decrement nummaps. */
 	--nummaps;
 	/* Trash the gworld. */
 	DisposeGWorld(map->gworldPortPtr);
-	/* This is NOT done by close_windows! */
-	DisposeWindow(map->window);
+	/* This is NOT done by close_window! */
+	DisposeTheWindow(map->window);
+	/* Find the new frontmap if necessary. */
+	if (frontmap == map) {
+	    frontmap = front_map();
+	    /* Hilite its scrollbars if it exists. */
+	    if (frontmap) {
+	    	HiliteControl(frontmap->vscrollbar, 0);
+		HiliteControl(frontmap->hscrollbar, 0);
+	     }
+	}
 	free(map);
 }
 
@@ -1014,14 +1087,8 @@ Map *map;
 		   will clip, but clipped drawing is still expensive). */
 		/* could test by drawing viewport rect as lines... */
 
-// --- FIX OCCASIONAL UNDRAWN HEXES AT LEFT END OF MAP ------------------------
-
+		/* (-1) fixes occasional undrawn cells at left end of map. */
 		x1 = vx - (y - vy) / 2 - 1;
-
-//		x1 = vx - (y - vy) / 2;
-
-// -------------------------------------------------------------------
-
 		x2 = x1 + vw + 2;
 		/* If the area doesn't wrap, then we might have to stop
 		   drawing before we reach the edge of the viewport. */
@@ -1109,15 +1176,10 @@ draw_meridians(Map *map, Rect drawrect)
     	int 	latdeg, latmit, londeg, lonmit, x, y, xf, yf, sx, sy;
     	char 	mitbuf[4];
 	Rect 	portrect; 
-	
-// --- CRASH FIX ---------------------------------------------------------
 
 	/* Don't draw meridians for flat or undefined worlds. */
 	if (world.circumference <= 0)
 		return;
-
-// --------------------------------------------------------------------
-
 	/* Compute the interval between meridians in pixels. */
 	incr = map->vp->latlong_interval;
 	interval = (incr * world.circumference * map->vp->hw) / 21600;
@@ -1153,16 +1215,8 @@ draw_meridians(Map *map, Rect drawrect)
 	lonmax -= lonmax % incr;
 	latmax  -= latmax % incr;
 
-// --- WRAPPING BUG FIX --------------------------------------------------
-
 	/* Start longitude drawing at Greenwich if the area is wrapped. */
 	if (area.xwrap) {
-
-//	/* Start longitude drawing at Greenwich if the map is wrapped. */
-//	if (map->xwrap) {
-
-// --------------------------------------------------------------------
-
 		lonmin = 0;
 		lonmax = 21600 - 21600 % incr;
 	}
@@ -1187,7 +1241,7 @@ draw_meridians(Map *map, Rect drawrect)
 		/* Skip if not inside area. */
 		if (!inside_area(wrapx(x), y))
 			continue;
-		xform_fractional(map, x, y, xf, yf, &sx, &sy);
+		xform_fractional_flat(map, x, y, xf, yf, &sx, &sy);
 		/* Skip latitudes that fall outside drawrect. */ 
 		if (sy < drawrect.top || sy > drawrect.bottom)
 			continue;
@@ -1265,7 +1319,7 @@ draw_meridians(Map *map, Rect drawrect)
 		/* Skip if not inside area. */
 		if (!inside_area(wrapx(x), y))
 			continue;
-		xform_fractional(map, x, y, xf, yf, &sx, &sy);
+		xform_fractional_flat(map, x, y, xf, yf, &sx, &sy);
 		/* Wrap sx if necessary. */
 		while (area.xwrap && sx < 0)
 			sx += map->vp->sxmax;
@@ -1277,54 +1331,31 @@ draw_meridians(Map *map, Rect drawrect)
 		/* Clip corners to area for non-wrapped maps. */
 		if (!area.xwrap) {
 			/* Compute top endpoint. */
-
-// --- COSMETIC FIX OF CORNER CLIPPING ----------------------------------------
-
 			if (in_area(0,  ycen + 2 * x - 1))						
 			    	 xform(map, 0, ycen + 2 * x - 1, 0, &top);
 			else if (in_area(2 * x - xend, ycen + 2 * (xend - x) - 1))	
 				 xform(map, 2 * x - xend, ycen + 2 * (xend - x) - 1, 0, &top);
-
-//			if (in_area(0,  ycen + 2 * x))						
-//			    	 xform(map, 0, ycen + 2 * x, 0, &top);
-//			else if (in_area(2 * x - xend, ycen + 2 * (xend - x)))	
-//				 xform(map, 2 * x - xend, ycen + 2 * (xend - x), 0, &top);
-
-// --------------------------------------------------------------------
-
 			/* Compute bottom endpoint. */
 			if (in_area(2 * x, ycen - 2 * x))					
 				 xform(map, 2 * x, ycen - 2 * x, 0, &bottom);	
 			else if (in_area(xend, ycen - 2 * (xend - x)))				
 				 xform(map, xend, ycen - 2 * (xend - x), 0, &bottom);
 		}
+		/* Make sure again there is room for top and bottom text. */
+		top = max(portrect.top + 16, top);
+		bottom = min(portrect.bottom - 16, bottom);
 
+		/* Clip to the redrawn area. */
 		top = max(drawrect.top, top);
 		bottom = min(drawrect.bottom, bottom);
-
-// --- GARBAGE OUTSIDE AREA BUG FIX -----------------------------------------
-
+		/* Fixes garbage outside area. */
 		if (top > bottom)
 			continue;
 
-// --------------------------------------------------------------------
-
 		/* Draw the longitude. */
 		MoveTo(sx, top);
 		LineTo(sx, bottom);
 		
-// --- REMOVE - SX IS ALWAYS BETWEEN 0 AND SXMAX ------------------------------
-
-		/* UNWRAP longitude if necessary. */
-/*		if (area.xwrap && sx < 0) {
-			sx += map->vp->sxmax;
-			MoveTo(sx, top);
-			LineTo(sx, bottom);
-			sx -= map->vp->sxmax;
-		}
-
-// --------------------------------------------------------------------		
-		
 		/* Skip the rest if no longitude text is necessary. */		
 		if (top == drawrect.top && bottom == drawrect.bottom)
 			continue;
@@ -1590,11 +1621,12 @@ void
 draw_top_line(Map *map)
 {
 	int numchars, strwid;
+	GrafPtr oldport;
 	Rect tmprect;
 
-	/* It would be safest to set the grafport explicitly here, but in
-	   the interests of efficiency, don't bother until somebody can
-	   prove it's necessary. */
+	/* Set the port. */
+ 	GetPort(&oldport);
+	SetPort(map->window);
 	/* Set up to draw the current date. */
 	TextSize(small_font_size);
 	TextFont(small_font_id);
@@ -1616,6 +1648,8 @@ draw_top_line(Map *map)
 	/* Draw a line dividing this from other map content. */
 	MoveTo(tmprect.left, tmprect.bottom);
 	Line(tmprect.right - tmprect.left, 0);
+	/* Restore grafport. */
+	SetPort(oldport);
 }
 
 void
@@ -1920,7 +1954,7 @@ draw_selections_at(Map *map, int x, int y)
 	GrafPtr	oldport;
 
 	/* Skip this for undrawn units. */
-	if (!cell_is_in_gworld(map, unit->x, unit->y))
+	if (!cell_is_in_gworld(map, x, y))
 		return;
 
 	xw = wrapx(x);
@@ -2118,6 +2152,9 @@ Unit *unit;
 	} else {
 		PenPat(QDPat(white));
 	}
+	/* Draw selected enemy units in red. */
+	if (!side_controls_unit(dside, unit))
+		ForeColor(redColor);
 	UNWRAP_RECT(map, tmprect, 
 		FrameRect(&tmprect));
 	InsetRect(&tmprect, 1, 1);
@@ -2131,6 +2168,7 @@ Unit *unit;
 	UNWRAP_RECT(map, tmprect, 
 		FrameRect(&tmprect));
 	PenNormal();
+	ForeColor(blackColor);
 	DGprintf("draw selection of %s at %d,%d\n", unit_desig(unit));
 }
 
@@ -2159,35 +2197,24 @@ Unit *unit;
 
 	/* Clip to map content */
 	tmpRect = map->contentrect;
-
-	/* Nessary to do the offset the since we are drawing directly to the screen */
-	OffsetRect(&tmpRect, map->vp->sx - map->conw, map->vp->sy - map->toph);
-
 	ClipRect(&tmpRect);
 
-	/* Old SHIFT_ORIGIN macro, now used only here. */
-	map->osx = map->vp->sx - map->conw;
-	map->osy = map->vp->sy - map->toph;
-	SetOrigin(map->osx, map->osy);
-
 	if (map->vp->uw >= 32) {
-
 		/* Use bigger box enclosing also occs for selection box */
 		m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-
 		if (map->numselections == 1
 			&& sw < 16
 			&& (unit->transport != NULL) ) {
-
 			m_xform_unit(map, unit->transport, &sx, &sy, &sw, &sh);
-
 			sx1 = sx;  sy1 = sy;  sw1 = sw;  sh1 = sh;
 		}
 	} else {
 		xform(map, unit->x, unit->y, &sx, &sy);
 		/* Adjust to unit part of cell. */
-		sx += (map->vp->hw - map->vp->uw) / 2;  sy += (map->vp->hh - map->vp->uh) / 2;
-		sw = map->vp->uw;  sh = map->vp->uh;
+		sx += (map->vp->hw - map->vp->uw) / 2;  
+		sy += (map->vp->hh - map->vp->uh) / 2;
+		sw = map->vp->uw;  
+		sh = map->vp->uh;
 	}
 	/* Draw a highlighting rectangle. */
 	SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
@@ -2200,6 +2227,9 @@ Unit *unit;
 	} else {
 		PenPat(QDPat(white));
 	}
+	/* Draw selected enemy units in red. */
+	if (!side_controls_unit(dside, unit))
+		ForeColor(redColor);
 	FrameRect(&tmprect);
 	if (area.xwrap) {
 		int rest = map->vp->pxw - tmprect.left;
@@ -2242,10 +2272,7 @@ Unit *unit;
 	}
 	PenNormal();
 	PenPat(QDPat(black));
-
-	/* Old RESET_ORIGIN macro, now used only here. */
-	map->osx = 0;  map->osy = 0;
-	SetOrigin(map->osx, map->osy);
+	ForeColor(blackColor);
 
 	SetClip(tmprgn);
 	DisposeRgn(tmprgn);
@@ -2325,8 +2352,8 @@ update_cell(Map *map, int x, int y)
 	/* Set destRect to enclose the redrawn row of cells. */
 	SetRect(&destRect, 
 			sx - 2, sy - 2,				/* Add 2 for W & N shorelines. */ 	
-			sx + cells * map->vp->hw, 
-			sy + map->vp->hh + 4);		/* Add 4 for SE & SW shorelines. */
+			sx + cells * map->vp->hw + 2,	/* Add 2 more pixels. */ 
+			sy + 2 * map->vp->hh);		/* Add the entire row below. */
 
 	GO_OFFSCREEN(map);
 
@@ -2379,7 +2406,8 @@ update_cell(Map *map, int x, int y)
 
 	/* Redraw any partially erased feature names. */
 	if (features_defined() && map->featurenames && map->vp->hh > 3)
-		draw_feature_names(map, sourceRect);
+		UNWRAP_RECT(map, sourceRect, 
+					draw_feature_names(map, sourceRect));
 
 	/* Redraw any meridians that will fall within destRect. */ 
 	if (map->vp->draw_latlong && map->vp->latlong_interval > 0) {
@@ -2387,7 +2415,8 @@ update_cell(Map *map, int x, int y)
 		/* (black color is always restored by draw_meridians). */
 		if (DebugG)
 			ForeColor(redColor);
-		draw_meridians(map, sourceRect);
+		UNWRAP_RECT(map, sourceRect, 
+					draw_meridians(map, sourceRect));
 	}
 	
 	RETURN_ONSCREEN(map);
@@ -2425,14 +2454,8 @@ draw_unit_names_only(Map *map, int x, int y)
 	unit = unit_at(xw, y);
 	if (unit == NULL)
 	  return;
-
-// --- CRASHING BUG FIX ------------------------------------------------
-
 	if (!inside_area(x, y))
 	  return;
-
-// -----------------------------------------------------------------
-
 	xform(map, x, y, &sx, &sy);
 
 	/* Draws unit names at small mags where contents of grouping boxes is not shown */
