@@ -10,12 +10,14 @@ any later version.  See the file COPYING.  */
    such as clicking, dragging, selecting, scrolling and toggling items. */
      
 #include "conq.h"
-extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
 #include "kpublic.h"
 #include "macconq.h"
+
+extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
+
 extern void set_position_modally(void);
 extern void city_dialog(Map *map, Unit *unit);
-extern void global_advance_dialog(Side *side, Unit *unit, int advance);
+extern void do_closeup_mi(void);
 
 static int selrect;
 static int downx, downy, downdir;
@@ -96,7 +98,7 @@ scroll_best_map_to_unit(Unit *unit, int bringtofront)
 		SetPort(oldport);
 	}
 	if (bringtofront) {
-		SelectWindow(bestmap->window);
+		SelectTheWindow(bestmap->window);
 		update_window(bestmap->window);
 		adjust_menus();
 	}
@@ -195,7 +197,7 @@ do_mouse_down_map(Map *map, Point mouse, int mods)
 				part = TrackControl(control, mouse, map_scroll_proc);
 				break;
 		}
-	} else if (mouse.h <= conwid) {
+	} else if (mouse.h <= map->conw) {
 		/* Interpret as a control panel hit. */
 		do_mouse_down_map_control_panel(map, mouse.h, mouse.v, mods);
 	} else {
@@ -209,10 +211,12 @@ do_mouse_down_map_control_panel(Map *map, int h, int v, int mods)
 	int winh = map->window->portRect.bottom - map->window->portRect.top;
 
 	/* (should better organize tests here) */
-	if (between(winh - 2 * sbarwid, v, winh)) {
-		switch ((winh - v) / sbarwid) {
+
+	/* Use 15 instead of map->sbarwid here, since the latter may be zero. */
+	if (between(winh - 2 * 15, v, winh)) {
+		switch ((winh - v) / 15) {
 			case 0:
-				magnify_map(map, ((h < conwid / 2) ? -1 : 1));
+				magnify_map(map, ((h < map->conw / 2) ? -1 : 1));
 				break;
 #if 0
 			/* This was too confusing here, so it's now flushed.  However, the
@@ -227,28 +231,28 @@ do_mouse_down_map_control_panel(Map *map, int h, int v, int mods)
 	} else if ((v - 32) < 5 * 15) {
 		switch ((v - 32) / 15) {
 			case 0:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_awake_mover(map);
 				} else {
 					select_next_awake_mover(map);
 				}
 				break;
 			case 1:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_mover(map);
 				} else {
 					select_next_mover(map);
 				}
 				break;
 			case 2:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_actor(map);
 				} else {
 					select_next_actor(map);
 				}
 				break;
 			case 3:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_unit(map);
 				} else {
 					select_next_unit(map);
@@ -358,8 +362,8 @@ set_map_mag(Map *map, int newpower)
 		m_center_on_focus(map);
 		set_map_scrollbars(map);
 
-		/* Update the gworld (and the map) */
-		update_gworld(map);
+		/* Update the entire map */
+		force_map_update(map);
 
 		/* Redraw other-map boxes in other windows */
 		draw_related_maps(map);
@@ -377,16 +381,25 @@ toggle_map_grid(Map *map)
 void
 toggle_map_topline(Map *map)
 {
-	Rect tmprect;
-	RgnHandle tmprgn;
-	GrafPtr oldport;
 	int oldtoph = map->toph;
+	GrafPtr oldport;
 
+	/* Setting the port fixed a display bug. */
+	GetPort(&oldport);
+	SetPort(map->window);
 	map->draw_topline = !map->draw_topline;
 	map->toplineh = (map->draw_topline ? tophgt : 0);
 	/* Erase other-map boxes in other windows. */
 	draw_related_maps(map);
 	map->toph = map->toplineh + map->topunith;
+
+#if 1
+
+	/* This is a temp fix. It should be possible to get the code below to work! */
+	force_map_update(map);
+
+#else
+
 	/* Update without scrolling if we are expanding and the buffer is too small. */ 
 	if (map->bufy - map->offsety < oldtoph - map->toph) {
 		set_content_rect(map);
@@ -398,18 +411,23 @@ toggle_map_topline(Map *map)
 		  draw_unit_info(map);
 	/* Always scroll the map if we are shrinking, or if the buffer is big enough. */
 	} else update_resized_map(map);
+
+#endif
+
 	/* Redraw other-map boxes in other windows. */
 	draw_related_maps(map);
+	SetPort(oldport);
 }
 
 void
 toggle_map_topunit(Map *map)
 {
-	int oldtoph;
-	Rect tmprect;
-	RgnHandle tmprgn;
 	GrafPtr oldport;
+	int oldtoph;
 
+	/* Setting the port fixed a display bug. */
+	GetPort(&oldport);
+	SetPort(map->window);
 	oldtoph = map->toph;
 	map->draw_topunit = !map->draw_topunit;
 	map->topunith = (map->draw_topunit ? topunithgt : 0);
@@ -429,6 +447,41 @@ toggle_map_topunit(Map *map)
 	} else update_resized_map(map);
 	/* Redraw other-map boxes in other windows. */
 	draw_related_maps(map);
+	SetPort(oldport);
+}
+
+void
+toggle_map_scrollbars(Map *map)
+{
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+
+	if (map->sbarwid)
+		map->sbarwid = 0;
+	else if (map == worldmap)
+		map->sbarwid = floatsbarwid;
+	else	map->sbarwid = sbarwid;
+	
+	adjust_map_decor(map);
+	force_map_update(map);
+
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
+}
+
+void
+toggle_map_cpanel(Map *map)
+{
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+
+	/* Toggle the panel. */
+	map->conw = (map->conw > 0 ? 0 : conwid);
+	adjust_map_decor(map);
+	force_map_update(map);
+
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
 }
 
 void
@@ -570,8 +623,6 @@ toggle_map_sidecolors(Map *map)
 	force_map_update(map);
 }
 
-// --- USER CONTROL OF EMBLEMS ------------------------------------------
-
 void
 toggle_map_draw_emblems(Map *map)
 {
@@ -579,8 +630,6 @@ toggle_map_draw_emblems(Map *map)
 	force_map_update(map);
 }
 
-// ------------------------------------------------------------------
-
 void
 toggle_iconmasks(Map *map)
 {
@@ -670,6 +719,23 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 	if (inside_area(downx, downy)) {
 		map->vp->vcx = downx;  map->vp->vcy = downy;
 	}
+	/* First handle refocus clicks in world map. */
+	if (map == worldmap) {
+		WindowPtr	 win;
+		Map *nextmap;
+
+		/* Find the next frontmost map. */
+		for_all_windows(win) {
+			if (map_from_window(win) != NULL
+			     && win != FrontWindow()) {
+			       	nextmap = map_from_window(win);
+				/* Scroll next map to the new focus. */
+				set_map_focus(nextmap, downx, downy);		
+				break;
+			}
+		}
+		return;
+	}		
 	if (map_modal != NO_MODAL) {
 		switch (map_modal) {
 			case ATTACK_MODAL:
@@ -720,12 +786,14 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		if (map->moveonclick && map->autoselect) {
 			unselect_all(map);
 			m_nearest_unit(map, h, v, &unit);
-			if (unit != NULL && (side_controls_unit(dside, unit) || endofgame)) {
-				/* The nearest unit will become the "current unit". */
+			/* Always select the unit if it exists and we may examine it. */
+			if (side_sees_unit(dside, unit)) {
 				map->curunit = unit;
 				select_unit_on_map(map, unit);
 				draw_selections(map);
-				move_on_drag(map, unit, mods);
+				/* But only move on drag if we control it. */
+				if (side_controls_unit(dside, unit))
+					move_on_drag(map, unit, mods);
 			} else {
 				select_all_dragged_over(map, h, v, mods);
 				/* Pick the first of the multiple selection as the "current unit". */
@@ -736,7 +804,9 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		} else {
 			anysuccess = FALSE;
 			for (i = 0; i < map->numselections; ++i) {
-				if ((unit = map->selections[i]) != NULL) {
+				/* Only move those selected units that we control. */ 
+				unit = map->selections[i];
+				if (side_controls_unit(dside, unit)) {
 					rslt = move_the_selected_unit(map, unit, h, v);
 					if (rslt)
 					  anysuccess = TRUE;
@@ -747,7 +817,9 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		}
 	} else if (mods & optionKey) {
 		for (i = 0; i < map->numselections; ++i) {
-			if ((unit = map->selections[i]) != NULL) {
+			/* Only fire those selected units that we control. */ 
+			unit = map->selections[i];
+			if (side_controls_unit(dside, unit)) {
 				fire_the_selected_unit(map, unit, h, v);
 			}
 		}
@@ -766,77 +838,45 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		} else {
 			select_all_dragged_over(map, h, v, mods);
 		}
-
-/* The old control-clicking code for new focus has been incorporated as a special case below. */
-
 	} else if (mods & controlKey) {
-		int done = FALSE;
-		/* First check all cities. */
-		for_all_units(unit) {		    	
-			/* Open city dialog if clicking on a city. */
-			if (u_advanced(unit->type) && unit->x == downx && unit->y == downy) {
+		m_nearest_unit(map, h, v, &unit);
+		/* Open city dialog or closeup if clicking on a unit that we may examine. */
+		if (side_sees_unit(dside, unit)) {
 
-// --- NOW HANDLED IN DIALOG --------------------------------------------
-
-				/* Break if clicked city not your own unless debugging. */
-//				if (unit->side != dside & !(Debug || DebugG || DebugM))
-//					break;
-
-// ------------------------------------------------------------------
+			UnitCloseup *unitcloseup;
 
+			if (u_advanced(unit->type)) {
 				city_dialog(map, unit);
-				done = TRUE;
-				break;
-			}
-		}
-
-		/* Go on to check non-cities. */		
-		if (!done) {
-			for_all_units(unit) {		    	
-			/* Open advance dialog if clicking on a non-city unit. */
-				if (!u_advanced(unit->type) && unit->x == downx && unit->y == downy) {
-
-// --- NOW HANDLED IN DIALOG --------------------------------------------
-
-					/* Break if clicked unit not your own unless debugging. */
-//					if (unit->side != dside & !(Debug || DebugG || DebugM))
-//						break;
-
-// ------------------------------------------------------------------
-
-					/* Use (-1) to signal that no discovery just was made. */
-					global_advance_dialog(unit->side, NULL, -1);
-					done = TRUE;
-					break;
+			} else { 	
+				if ((unitcloseup = find_unit_closeup(unit)) != NULL) {
+					SelectTheWindow(unitcloseup->window);
+				} else {
+					create_unit_closeup(unit);
 				}
 			}
-		}				
-
 		/* Else focus on clicked spot and select closest own unit. */
-		if (!done) {
-			long unitdist, mindist = 32000;
-			/* Focus on clicked point */
+		} else {
+			long	unitdist, mindist = 32000;
+
+			/* Focus on the clicked point. */
 			set_map_focus(map, downx, downy);
-			if (dside->actionvector) {
-				/* Find nearest own unit and select it. */
-			    for (i = 0; i < dside->actionvector->numunits; ++i) {
-			    	unit = (dside->actionvector->units)[i].unit;
-		   			if (could_be_next_unit(unit) && side_controls_unit(dside, unit)) {
-						/* Use least square method */
-						unitdist = (downx - unit->x) * (downx - unit->x) + 
-							        (downy - unit->y) * (downy - unit->y);
-						if (unitdist < mindist) {
-							mindist = unitdist;
-							unselect_all(map);
-							map->curunit = unit;
-						}
-				    }
-			    }
+			for_all_side_units(dside, unit) {
+   				if (side_controls_unit(dside, unit)) {
+					/* Use least square method. */
+					unitdist = (downx - unit->x) * (downx - unit->x) + 
+					        	        (downy - unit->y) * (downy - unit->y);
+					if (unitdist < mindist) {
+						mindist = unitdist;
+						map->curunit = unit;
+					}
+		    		}
+		    	}
+			if (map->curunit) {
+				unselect_all(map);
+				select_unit_on_map(map, map->curunit);
+				draw_selections(map);
 			}
 		}
-		select_unit_on_map(map, map->curunit);
-		draw_selections(map);
-
 	} else {
 		/* Interpret an unmodified mouse down. */
 #ifdef DESIGNERS
@@ -848,7 +888,9 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 			/* Usually will only be one to move, but be general anyway. */
 			anysuccess = FALSE;
 			for (i = 0; i < map->numselections; ++i) {
-				if ((unit = map->selections[i]) != NULL) {
+				/* Only move those selected units that we control. */ 
+				unit = map->selections[i];
+				if (side_controls_unit(dside, unit)) {
 					rslt = move_the_selected_unit(map, unit, h, v);
 					if (rslt)
 					  anysuccess = TRUE;
@@ -860,12 +902,19 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		} else {
 			unselect_all(map);
 			m_nearest_unit(map, h, v, &unit);
-			if (unit != NULL && (side_controls_unit(dside, unit) || endofgame)) {
+			/* Always select the unit if it exists and we may examine it. */
+			if (side_sees_unit(dside, unit)) {
+				map->curunit = unit;
 				select_unit_on_map(map, unit);
 				draw_selections(map);
-				move_on_drag(map, unit, mods);
+				/* But only move on drag if we control it. */
+				if (side_controls_unit(dside, unit))
+					move_on_drag(map, unit, mods);
 			} else {
 				select_all_dragged_over(map, h, v, mods);
+				/* Pick the first of the selection as the current unit. */
+				if (map->numselections > 0)
+					map->curunit = map->selections[0];
 			}
 		}
 	}
@@ -1124,8 +1173,8 @@ select_all_units_in_rect(Map *map, Rect *rectptr)
 	/* Now look at all the plausible units and see if any's image intersects the rect. */
 	for_all_units(unit) {
 		if (in_play(unit)
-			&& (side_controls_unit(dside, unit) || endofgame)
-			&& (rectissmall || unit->transport == NULL)) {
+		     && side_sees_unit(dside, unit)
+		     && (rectissmall || unit->transport == NULL)) {
 			m_xform_unit_self(map, unit, &sx, &sy, &sw, &sh);
 			SetRect(&unitrect, sx, sy, sx + sw, sy + sh);
 			if (SectRect(&unitrect, rectptr, &tmprect)) {
