@@ -37,8 +37,6 @@ static void draw_region(Map *map, int x, int y);
    The macros should be applied to all drawing functions that take an sx value, rect
    or poly as a parameter. */
 
-// --- PERFORM DOUBLE UNWRAP FOR ALL THREE MACROS ----------------------------- 
-
 /* Repeat (function) both if sx < 0 and if sx is within one hex from sxmax. */
 #define UNWRAP_SX(map, sx, function) { \
 	(function); \
@@ -53,18 +51,6 @@ static void draw_region(Map *map, int x, int y);
 	} \
 }
 
-/* Repeat (function) once using sx shifted one area width to the right. */
-/*
-#define UNWRAP_SX(map, sx, function) { \
-	(function); \
-	if (area.xwrap && (sx) < 0) { \
-		(sx) += (map)->vp->sxmax; \
-		(function); \
-		(sx) -= (map)->vp->sxmax; \
-	} \
-}
-*/
-
 /* Repeat (function) both if rect.left < 0 and if rect.right > sxmax. */
 #define UNWRAP_RECT(map, rect, function) { \
 	(function); \
@@ -79,18 +65,6 @@ static void draw_region(Map *map, int x, int y);
 	} \
 }
 
-/* Repeat (function) once using rect offset one area width to the right. */
-/*
-#define UNWRAP_RECT(map, rect, function) { \
-	(function); \
-	if (area.xwrap && (rect).left < 0) { \
-		OffsetRect(&(rect), (map)->vp->sxmax, 0); \
-		(function); \
-		OffsetRect(&(rect), -(map)->vp->sxmax, 0); \
-	} \
-}
-*/
-
 /* Repeat (function) both if polyBBox.left < 0 and if polyBBox.right > sxmax. */
 #define UNWRAP_POLY(map, poly, function) { \
 	(function); \
@@ -105,20 +79,6 @@ static void draw_region(Map *map, int x, int y);
 	} \
 }
 
-/* Repeat (function) once using poly offset one area width to the right. */
-/*
-#define UNWRAP_POLY(map, poly, function) { \
-	(function); \
-	if (area.xwrap && (*(*(poly))).polyBBox.left < 0) { \
-		OffsetPoly((poly), (map)->vp->sxmax, 0); \
-		(function); \
-		OffsetPoly((poly), -(map)->vp->sxmax, 0); \
-	} \
-}
-*/
-
-// -----------------------------------------------------------------------
-
 /* For these accessors, x must already be wrapped. */
 
 #define cell_overlay(x, y)  \
@@ -146,7 +106,7 @@ static void draw_region(Map *map, int x, int y);
 
 #define cell_style(map, x, y, power)  \
   ((map->see_all || terrain_visible(dside, x, y) || unseen_image != NULL || !in_area(x, y))  \
-   ? (power >= 4 ? usepolygons : useblocks)  \
+   ? (power >= 3 ? usepolygons : useblocks)  \
    : dontdraw);
 
 /* x0 may be negative here. */
@@ -470,13 +430,13 @@ draw_row(Map *map, int x0, int y, int len, int clearit)
 void
 draw_terrain_row(Map *map, int x0, int y, int len)
 {
-	int 	x0w = wrapx(x0), x, xw, x1 = x0, sx, sy, i = 0;
+	int 	x0w = wrapx(x0), x, xw, x1 = x0, sx, sy, i = 0, quasirand;
 	int 	style, segstyle, terr, segterr, over, segover;
 	int 	dosolid = map->solid_color_terrain;
 	int 	dofill = map->vp->draw_cell_pats;
 	int 	pwr = map->vp->power;
 	int	ang = map->vp->angle;
-	int 	dogrid = FALSE; 
+	int 	dogrid = FALSE;
 
 	/* Don't use quick grid method if it would leak. */ 
 	if (map->vp->draw_grid &! map->leaky_grid) 
@@ -519,8 +479,9 @@ draw_terrain_row(Map *map, int x0, int y, int len)
 					case usepictures:
 						break;
 					case usepolygons:
+						quasirand = (11 * x1 + (13 * y) + 7) * 41;
 						UNWRAP_SX(map, sx, 
-							draw_hex_region(sx, sy, pwr, segterr, segover, ang, dogrid, dosolid));
+							draw_hex_region(sx, sy, pwr, segterr, segover, ang, dogrid, dosolid, quasirand));
 				}
 			}
 			/* Set up for the next segment. */
@@ -543,6 +504,9 @@ draw_cliffs(Map *map, int x, int y)
 	RGBColor hexcolor, oldcolor;
 
 	xw = wrapx(x);
+	/* Prevents crashes when trying to get thickness of BACKTTYPE. */
+	if (!in_area(xw, y))
+		return;
 	t = cell_terrain(map, xw, y);
 	elev = (elevations_defined() ? elev_at(xw, y) : 0) + t_thickness(t);
 	drawit = FALSE;
@@ -677,18 +641,13 @@ draw_contours(Map *map, int x, int y)
 			MoveTo(lines[i].sx1, lines[i].sy1); 
 			LineTo(lines[i].sx2, lines[i].sy2);
 		}
-
-// --- ALSO UNWRAP IF SX > SXMAX -----------------------------
-
-		else if (area.xwrap && sx > map->vp->sxmax) {
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
 			lines[i].sx1 -= map->vp->sxmax;
 			lines[i].sx2 -= map->vp->sxmax;
 			MoveTo(lines[i].sx1, lines[i].sy1); 
 			LineTo(lines[i].sx2, lines[i].sy2);
 		}
-
-// -----------------------------------------------------
-
 	}
 	ForeColor(blackColor);
 }
@@ -769,18 +728,13 @@ draw_gridlines(Map *map, int x, int y)
 			LineTo(sx + dx2, sy + dy2 - 1);
 			sx -= map->vp->sxmax;
 		}
-
-// --- ALSO UNWRAP IF SX > SXMAX -----------------------------
-
-		else if (area.xwrap && sx > map->vp->sxmax) {
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
 			sx -= map->vp->sxmax;
 			MoveTo(sx + dx1, sy + dy1 - 1);
 			LineTo(sx + dx2, sy + dy2 - 1);
 			sx += map->vp->sxmax;
 		}
-
-// -----------------------------------------------------
-
 	}
 	PenNormal();
 	ForeColor(blackColor);
@@ -795,7 +749,7 @@ draw_shorelines(Map *map, int x, int y)
 	int	t, elev, offset = 0, cliffs = FALSE;
 	int	pwr = map->vp->power;
 	int	ang = map->vp->angle;
-	int	wid = bwid2[pwr];
+	int	wid = bwid[pwr];
 	int	wid2 = wid / 2;
 	int	xw = wrapx(x);
 
@@ -876,7 +830,7 @@ draw_shorelines(Map *map, int x, int y)
 		if (dir == NORTHWEST || dir == NORTHEAST) {
 			if (!t_liquid(terrain_at(xw, y)))
 			  dx1 += 1; 
-			else if (pwr > 3)
+			else if (pwr > 2)
 			  dy1 -= 1;
 		}
 
@@ -887,10 +841,11 @@ draw_shorelines(Map *map, int x, int y)
 		    dy2 /= 2;
 		if (ang == 15) 
 		    dy2 /= 4;
-		if (dir == NORTHWEST || dir == NORTHEAST) {
+		if (dir == NORTHWEST /*|| dir == NORTHEAST*/) {
 			if (!t_liquid(terrain_at(xw, y)))
 			  dx2 += 1; 
-			else if (pwr > 3)
+			else if (pwr > 2)
+			if (pwr > 2)
 			  dy2 -= 1;
 		}
 
@@ -905,18 +860,13 @@ draw_shorelines(Map *map, int x, int y)
 			LineTo(sx + dx2 - wid2, sy + dy2 - 1 + adjust);
 			sx -= map->vp->sxmax;
 		}
-
-// --- ALSO UNWRAP IF SX > SXMAX -----------------------------
-
-		else if (area.xwrap && sx > map->vp->sxmax) {
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
 			sx -= map->vp->sxmax;
 			MoveTo(sx + dx1 - wid2, sy + dy1 - 1 + adjust);
 			LineTo(sx + dx2 - wid2, sy + dy2 - 1 + adjust);
 			sx += map->vp->sxmax;
 		}
-
-// -----------------------------------------------------
-
 	}
 	PenNormal();
 	ForeColor(blackColor);
@@ -1031,18 +981,13 @@ draw_feature_boundary_simple(Map *map, int x, int y)
 			LineTo(sx + dx2 - wid2, sy + dy2 - wid2);
 			sx -= map->vp->sxmax;
 		}
-
-// --- ALSO UNWRAP IF SX > SXMAX -----------------------------
-
-		else if (area.xwrap && sx > map->vp->sxmax) {
+		/* Trigger unwrapping already within 1 hex from right edge. */
+		else if (area.xwrap && sx > map->vp->sxmax - map->vp->hw) {
 			sx -= map->vp->sxmax;
 			MoveTo(sx + dx1 - wid2, sy + dy1 - wid2);
 			LineTo(sx + dx2 - wid2, sy + dy2 - wid2);
 			sx += map->vp->sxmax;
 		}
-
-// -----------------------------------------------------
-
 	}
 	PenNormal();
 	ForeColor(blackColor);
