@@ -30,19 +30,11 @@ static pascal void scores_vscroll_fn(ControlHandle control, short code);
 static void calc_history_layout(void);
 static void set_history_scrollbar(void);
 
-// --- NOW DECLARED IN MACCONQ.H ------------------------------------------
-
-// extern char **current_colorscheme;		/* Color schemes by name (red,pink,blue) */
-									/* Defined in maccolors.c */
-
-// --------------------------------------------------------------------
-
-
 /* Globals for the game window. */
 
 WindowPtr gamewin = nil;
 
-int gamewinw = 200;
+int gamewinw = 160;
 int gamedatehgt = 18;
 int gameclockoffset;
 int gameclockhgt = 15;
@@ -107,7 +99,7 @@ static char *nummrows;
 
 int closeup_spacing = 16;
 
-int closeupwinwid = 200;
+int closeupwinwid = 240;
 
 /* Globals for the history window. */
 
@@ -153,6 +145,19 @@ ControlActionUPP scores_vscroll_proc;
 
 /* The game progress window. */
 
+/* This is the top-level access to bring up the game window, can be called
+   anywhere, anytime. */
+
+void
+show_game_window()
+{
+	if (gamewin == nil) {
+		create_game_window();
+	} 
+	SelectTheWindow(gamewin);
+	ActivateWindow(gamewin, true);
+}
+
 /* Create the game progress window. */
 
 void
@@ -163,10 +168,11 @@ create_game_window()
 
 	/* Create the window, color if possible, since emblems may be in color. */
 	if (hasColorQD) {	
-		gamewin = GetNewCWindow(wGame, NULL, (WindowPtr) -1L);
+		gamewin = GetNewCWindow(wFloatFixed, NULL, NULL);
 	} else {
-		gamewin = GetNewWindow(wGame, NULL, (WindowPtr) -1L);
+		gamewin = GetNewWindow(wFloatFixed, NULL, NULL);
 	}
+	SetWTitle(gamewin, "\pGame");
 	gametophgt = gamedatehgt;
 	if (g_rt_per_turn() > 0 || g_rt_for_game() > 0) {
 		gameclockoffset = gametophgt;
@@ -190,13 +196,17 @@ create_game_window()
 	SizeWindow(gamewin, gamewinw, gametophgt + gamenumsides * gamesidehgt, 1);
 	if (first_windows) {
 		get_main_screen_size(&screenwidth, NULL);
-		MoveWindow(gamewin, screenwidth - gamewinw - 3, 40, FALSE);
+		/* Tweaked to fit Apple Platinum theme. */
+		MoveWindow(gamewin, screenwidth - gamewinw - 3, 41 + 15, FALSE);
 	}
 	/* Get handles to useful sicns. */
 	aisicnhandle = GetNamedResource('SICN', "\pmplayer");
 	facesicnhandle[0] = GetNamedResource('SICN', "\phostile");
 	facesicnhandle[1] = GetNamedResource('SICN', "\pneutral");
 	facesicnhandle[2] = GetNamedResource('SICN', "\pfriendly");
+
+	/* Finally make it floating. */
+	MakeFloat(gamewin);
 }
 
 char *last_status;
@@ -357,13 +367,13 @@ draw_game_side(Side *side2)
 	TextFace(0);
 	if (side_has_ai(side2) && side2->ingame) {
 		/* Show that the side is run by an AI. */
-		plot_sicn(gamewin, 182, sy + 2, aisicnhandle, 0, TRUE, srcOr);
+		plot_sicn(gamewin, 142, sy + 2, aisicnhandle, 0, TRUE, srcOr);
 	}
 	if (side2 != dside
 	    && side2->ingame
 	    && (side_has_ai(side2) || side_has_display(side2))) {
 		/* Indicate attitude of other side. */
-		plot_sicn(gamewin, 164, sy + 2,
+		plot_sicn(gamewin, 124, sy + 2,
 			facesicnhandle[feeling_towards(side2, dside)], 0, TRUE, srcOr);
 	}
 	last_status[s2] = -1;
@@ -563,9 +573,9 @@ create_construction_window()
 	Rect listrect, tmprect;
 
 	if (hasColorQD) {
-		constructionwin = GetNewCWindow(wConstruction, NULL, (WindowPtr) -1L);
+		constructionwin = GetNewCWindow(wConstruction, NULL, NULL);
 	} else {
-		constructionwin = GetNewWindow(wConstruction, NULL, (WindowPtr) -1L);
+		constructionwin = GetNewWindow(wConstruction, NULL, NULL);
 	}
 	constructbutton = GetNewControl(cConstructButton, constructionwin);
 	developbutton = GetNewControl(cResearchButton, constructionwin);
@@ -599,11 +609,10 @@ create_construction_window()
 		get_main_screen_size(NULL, &mainheight);
 		tmprect = constructionwin->portRect;
 		MoveWindow(constructionwin,
-				   4,
-				   mainheight - (tmprect.bottom - tmprect.top) - 3,
+				   313,
+				   mainheight - (tmprect.bottom - tmprect.top) - 7,
 				   FALSE);
 	}
-	ShowWindow(constructionwin);
 }
 
 /* Build the list of constructing units and the list of constructible types. */
@@ -636,7 +645,9 @@ init_construction_lists()
 	  possibletypes = (int *) xmalloc(numutypes * sizeof(int));
 	numposstypes = 0;
 	for_all_unit_types(u) {
-		if (construction_possible(u) && type_allowed_on_side(u, dside)) {
+		if (construction_possible(u) 
+		    && has_advance_to_build(dside, u)
+		    && type_allowed_on_side(u, dside)) {
 			LAddRow(1, tmpcell.v, construction_type_list);
 			constructible_desc(spbuf, dside, u, NULL);
 			LSetCell(spbuf, strlen(spbuf), tmpcell, construction_type_list);
@@ -817,7 +828,7 @@ do_mouse_down_construction(Point mouse, int mods)
 				net_push_build_task(unit, u, currunlength);
 				update_construction_unit_list(unit);
 				if (modal_construction && window_behind_construction != nil) {
-					SelectWindow(window_behind_construction);
+					SelectTheWindow(window_behind_construction);
 					update_window(window_behind_construction);
 				}
 				window_behind_construction = NULL;
@@ -833,7 +844,7 @@ do_mouse_down_construction(Point mouse, int mods)
 				net_push_develop_task(unit, u, u_tech_to_build(u));
 				update_construction_unit_list(unit);
 				if (modal_construction && window_behind_construction != nil) {
-					SelectWindow(window_behind_construction);
+					SelectTheWindow(window_behind_construction);
 					update_window(window_behind_construction);
 				}
 				window_behind_construction = NULL;
@@ -1047,10 +1058,12 @@ adjust_construction_controls()
 	if (unit != NULL) {
 		u = get_selected_construction_type();
 		if (u != NONUTYPE) {
-			if (uu_acp_to_create(unit->type, u) > 0)
-			  canconstruct = TRUE;
+			if (uu_acp_to_create(unit->type, u) > 0
+	    		  && type_allowed_on_side(u, unit->side)
+	    		  && has_advance_to_build(unit->side, u))
+			     canconstruct = TRUE;
 			if (uu_acp_to_develop(unit->type, u) > 0)
-			  candevelop = TRUE;
+			     candevelop = TRUE;
 		}
 	}
 	HiliteControl(constructbutton, (canconstruct ? 0 : 255));
@@ -1111,7 +1124,7 @@ zoom_construction(int part)
 		}
 		(*((WStateDataHandle) ((WindowPeek) constructionwin)->dataHandle))->stdState = zoomrect;
 	}
-	ZoomWindow(constructionwin, part, (constructionwin == FrontWindow()));
+	ZoomWindow(constructionwin, part, false);
 	adjust_construction_items();
 	/* This will force a full redraw at the next update. */
 	InvalRect(&constructionwin->portRect);
@@ -1192,7 +1205,7 @@ side_rename_dialog(Side *side)
 				/* Trash and reload all emblem cicns at the top of the heap zone,
 				   so as to avoid fragmentation */				
 				reinit_all_emblem_images();
-				current_colorscheme[side_number(side)] = side->colorscheme;
+				current_colorscheme[side_number(side)] = copy_string(side->colorscheme);
 				dissect_colorscheme(side_number(side));
 				ui_update_state();
 				save_preferences();
@@ -1326,12 +1339,29 @@ create_unit_closeup(Unit *unit)
 	u = unit->type;
 	unitcloseup->unit = unit;
 	if (hasColorQD) {
-		win = GetNewCWindow(wUnitCloseup, nil, (WindowPtr) -1L);
+		win = GetNewCWindow(wFloatFixed, NULL, NULL);
 	} else {
-		win = GetNewWindow(wUnitCloseup, nil, (WindowPtr) -1L);
+		win = GetNewWindow(wFloatFixed, NULL, NULL);
 	}
 	unitcloseup->window = win;
-	stagger_window(unitcloseup->window, &lastunitcloseuph, &lastunitcloseupv);
+
+	/* Put it adjacent to the clicked or list-selected unit. */
+	if (frontmap) {
+		GrafPtr oldport;
+		Point upoint;
+		int sx, sy;
+						
+		GetPort(&oldport);
+		SetPort(frontmap->window);
+		xform(frontmap, unit->x + 2, unit->y - 2, &sx, &sy);
+		upoint.h = sx; upoint.v = sy;
+		LocalToGlobal(&upoint);
+		MoveWindow(win, upoint.h, upoint.v, true);
+		SetPort(oldport);
+	} else {
+		/* This only happens if no maps are open and a closeup is chosen from a list. */
+		stagger_window(unitcloseup->window, &lastunitcloseuph, &lastunitcloseupv);
+	}
 	preferred_closeup_size(u, &w, &h);
 	SizeWindow(win, w, h, 1);
 	sprintf(spbuf, "Closeup on %s", short_unit_handle(unit));
@@ -1339,7 +1369,8 @@ create_unit_closeup(Unit *unit)
 	unitcloseup->next = unitcloseuplist;
 	unitcloseuplist = unitcloseup;
 	/* We're now ready to show this closeup to the world. */
-	ShowWindow(win);
+	MakeFloat(win);
+	SelectTheWindow(win);
 }
 
 /* Compute the right size for a unit's closeup.  More complicated units
@@ -1351,7 +1382,8 @@ preferred_closeup_size(int u, int *widp, int *hgtp)
 	int wid = closeupwinwid, hgt, m, u2, count;
 
 	closeup_spacing = small_line_spacing + 1;
-	hgt = 4 + 32 + 4 + 3 * closeup_spacing;
+	/* Add 4 pixels at bottom to make room for 3rd task. */
+	hgt = 4 + 32 + 4 + 3 * closeup_spacing + 4;
 	if (nummrows == NULL) {
 		/* Compute and cache the space needed to display each unit's supply. */
 		nummrows = xmalloc(numutypes);
@@ -1384,14 +1416,16 @@ draw_unit_closeup(UnitCloseup *unitcloseup)
 	GrafPtr oldport;
 	WindowPtr win = unitcloseup->window;
 	Unit *unit = unitcloseup->unit;
+	Map	*map;
 
+	update_cell_display(dside, unit->x, unit->y, TRUE);
 	if (!active_display(dside))
 	  return;
-	if (!in_play(unit) || !(side_controls_unit(dside, unit) || endofgame)) {
-		/* If the unit is no longer alive and ours, shut down the window. */
+	/* We now permit peeking at enemy units. */
+	if (!in_play(unit)) { 
 		remove_window_menu_item(win);
 		destroy_unit_closeup(unitcloseup);
-		HideWindow(win);
+		HideTheWindow(win);
 		return;
 	}
 	GetPort(&oldport);
@@ -1453,32 +1487,61 @@ draw_unit_closeup(UnitCloseup *unitcloseup)
 	}
 	/* Draw the unit's plan, if it has one. */
 	if (unit->plan) {
+
 		Task *task;
 		Plan *plan = unit->plan;
 		
-		plan_desc(infobuf, unit);
+		/* plan_desc now also includes goal_desc on the same line. To avoid 
+		repetition only the first half of plan_desc is therefore included below. */ 
+		strcpy(infobuf, "Plan: ");
+		strcat(infobuf, plantypenames[plan->type]);
+		if (plan->waitingfortasks)
+		  strcat(infobuf, " Waiting");
+		if (plan->asleep)
+		  strcat(infobuf, " Asleep");
+		if (plan->reserve)
+		  strcat(infobuf, " Reserve");
+		if (plan->delayed)
+		  strcat(infobuf, " Delay");
+		if (!plan->aicontrol)
+		  strcat(infobuf, " NoAI");
+		if (plan->supply_is_low)
+		  strcat(infobuf, " SupplyLow");
+		if (plan->tasks) {
+			int i = 0;
+			for_all_tasks(plan, task)
+				++i;
+			tprintf(infobuf, " (%d task%s)", i, (i == 1 ? "" : "s"));
+		} 
 		sy += closeup_spacing;
 		MoveTo(sx, sy);
 		DrawText(infobuf, 0, strlen(infobuf));
     	if (plan->maingoal) {
 			strcpy(infobuf, "Goal: ");
-    		goal_desc(infobuf+strlen(infobuf), plan->maingoal);
+	    		goal_desc(tmpbuf, plan->maingoal);
+			strncat(infobuf, tmpbuf, strlen(tmpbuf));
 			sy += closeup_spacing;			
 			MoveTo(sx, sy);
 			DrawText(infobuf, 0, strlen(infobuf));
     	}
     	if (plan->formation) {
 			strcpy(infobuf, "Formation: ");
-    		goal_desc(infobuf+strlen(infobuf), plan->formation);
+    			goal_desc(infobuf+strlen(infobuf), plan->formation);
 			sy += closeup_spacing;			
 			MoveTo(sx, sy);
 			DrawText(infobuf, 0, strlen(infobuf));
     	}
-    	for_all_tasks(plan, task) {
-			task_desc(infobuf, unit->side, unit, task);
-			sy += closeup_spacing;			
-			MoveTo(sx, sy);
-			DrawText(infobuf, 0, strlen(infobuf));
+		if (plan->tasks) {
+			int i = 0;
+	    		for_all_tasks(plan, task) {
+				++i;
+				sprintf(infobuf, "Task %d: ", i);
+				task_desc(tmpbuf, unit->side, unit, task);			
+				strcat(infobuf, tmpbuf);
+				sy += closeup_spacing;			
+				MoveTo(sx, sy);
+				DrawText(infobuf, 0, strlen(infobuf));
+			}
 		}
 	}
 	SetPort(oldport);
@@ -1507,8 +1570,16 @@ do_mouse_down_unit_closeup(UnitCloseup *unitcloseup, Point mouse, int mods)
 	part = FindControl(mouse, window, &control);
 	if (0 /* some control */) {
 	} else {
-		/* This just forces a redraw of the window - kind of crude. */
-		update_unit_display(dside, unitcloseup->unit, TRUE);
+	
+		Unit	*unit;
+		Map	*map;
+
+		unit = unitcloseup->unit;
+		for_all_maps(map) {
+			unselect_all(map);
+			select_unit_on_map(map, unit);
+		}
+		update_cell_display(dside, unit->x, unit->y, TRUE);
 		return TRUE;
 	}
 }
@@ -1517,7 +1588,11 @@ void
 destroy_unit_closeup(UnitCloseup *unitcloseup)
 {
 	UnitCloseup *unitcloseup2;
-	
+	Unit	*unit;
+	Map	*map;
+
+	unit = unitcloseup->unit;
+	update_cell_display(dside, unit->x, unit->y, TRUE);
 	if (unitcloseuplist == unitcloseup) {
 		unitcloseuplist = unitcloseup->next;
 	} else {
@@ -1533,12 +1608,25 @@ destroy_unit_closeup(UnitCloseup *unitcloseup)
 
 /* History window. */
 
+/* This is the top-level access to bring up the history window, can be called
+   anywhere, anytime. */
+
+void
+show_history_window()
+{
+	if (historywin == nil) {
+		create_history_window();
+	}
+	SelectTheWindow(historywin);
+	ActivateWindow(historywin, true);
+}
+
 void
 create_history_window()
 {
 	Rect vscrollrect;
 
-	historywin = GetNewWindow(wHistory, NULL, (WindowPtr) -1L);
+	historywin = GetNewWindow(wHistory, NULL, NULL);
 	SetPort(historywin);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
@@ -1553,7 +1641,6 @@ create_history_window()
 			 					0, 0, 100, scrollBarProc, 0L);
 	total_history_lines = update_total_hist_lines(dside);
 	set_history_scrollbar();
-	ShowWindow(historywin);
 }
 
 static void
@@ -1767,7 +1854,7 @@ zoom_history(int part)
 		}
 		(*((WStateDataHandle) ((WindowPeek) historywin)->dataHandle))->stdState = zoomrect;
 	}
-	ZoomWindow(historywin, part, (historywin == FrontWindow()));
+	ZoomWindow(historywin, part, false);
 	move_history_scrollbar(window_width(historywin), window_height(historywin));
 	/* This will force a full redraw at the next update. */
 	InvalRect(&historywin->portRect);
@@ -1787,13 +1874,13 @@ move_history_scrollbar(int h, int v)
    anywhere, anytime. */
 
 void
-notice_dialog()
+show_notice_window()
 {
 	if (noticewin == nil) {
 		create_notice_window();
 	}
-	ShowWindow(noticewin);
-	SelectWindow(noticewin);
+	SelectTheWindow(noticewin);
+	ActivateWindow(noticewin, true);
 }
 
 void
@@ -1804,43 +1891,38 @@ create_notice_window()
 
 	/* Create the window, color if possible, since images may be in color. */
 	if (hasColorQD) {	
-		noticewin = GetNewCWindow(wNotice, NULL, (WindowPtr) -1L);
+		noticewin = GetNewCWindow(wFloatResize, NULL, NULL);
 	} else {
-		noticewin = GetNewWindow(wNotice, NULL, (WindowPtr) -1L);
+		noticewin = GetNewWindow(wFloatResize, NULL, NULL);
 	}
+	SetWTitle(noticewin, "\pNotices");
+	SizeWindow(noticewin, 300, 120, true);
 	SetPort(noticewin);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
 	h = window_width(noticewin);  v = window_height(noticewin);
-	SetRect(&viewrect, 5, 5, h - sbarwid, v - sbarwid); 
+	SetRect(&viewrect, 5, 5, h - floatsbarwid, v - floatsbarwid); 
 	destrect = viewrect;
 	notice_text = TENew(&destrect, &destrect);
 	/* Set up a vertical scrollbar. */
 	vscrollrect = noticewin->portRect;
-	vscrollrect.top = 5;
-	vscrollrect.bottom -= sbarwid - 1;
-	vscrollrect.left = vscrollrect.right - sbarwid;
+	vscrollrect.top -= 1;
+	vscrollrect.bottom -= floatsbarwid - 1;
+	vscrollrect.left = vscrollrect.right - floatsbarwid;
 	vscrollrect.right += 1;
 	notice_v_scrollbar =
 		NewControl(noticewin, &vscrollrect, "\p", TRUE, 0, 0, 0, scrollBarProc, 0L);
 	HiliteControl(notice_v_scrollbar, 0);
-
-// --- WINDOW MENU BUG FIX ---------------------------------------
-
-	/* The Notices window is a permanent item (#2) so it should not be added here. */
-
-//	add_window_menu_item("Notices", noticewin);
-
-// ------------------------------------------------------------
-
 	if (1 /* position notices at bottom of main screen */) {
 		get_main_screen_size(NULL, &mainheight);
 		tmprect = noticewin->portRect;
 		MoveWindow(noticewin,
-				   4,
-				   mainheight - (tmprect.bottom - tmprect.top) - 3,
+				   3,
+				   mainheight - (tmprect.bottom - tmprect.top) - 5,
 				   FALSE);
 	}
+	/* Finally make it floating. */
+	MakeFloat(noticewin);
 }
 
 void
@@ -2013,11 +2095,11 @@ grow_notice(int h, int v)
 {
 	EraseRect(&noticewin->portRect);
 	SizeWindow(noticewin, h, v, 1);
-	MoveControl(notice_v_scrollbar, h - sbarwid, 5);
-	SizeControl(notice_v_scrollbar, sbarwid + 1, v - 5 - sbarwid + 1);
-	(*notice_text)->viewRect.right = h - sbarwid;
-	(*notice_text)->viewRect.bottom = v - sbarwid;
-	(*notice_text)->destRect.right = h - sbarwid;
+	MoveControl(notice_v_scrollbar, h - floatsbarwid, -1);
+	SizeControl(notice_v_scrollbar, floatsbarwid + 1, v + 1 - floatsbarwid + 1);
+	(*notice_text)->viewRect.right = h - floatsbarwid;
+	(*notice_text)->viewRect.bottom = v - floatsbarwid;
+	(*notice_text)->destRect.right = h - floatsbarwid;
 	TECalText(notice_text);
 	InvalRect(&noticewin->portRect);
 }					
@@ -2048,14 +2130,14 @@ zoom_notice(int part)
 		zoomrect.top += titleh;
 		(*((WStateDataHandle) ((WindowPeek) noticewin)->dataHandle))->stdState = zoomrect;
 	}
-	ZoomWindow(noticewin, part, (noticewin == FrontWindow()));
+	ZoomWindow(noticewin, part, false);
 	h = window_width(noticewin);  v = window_height(noticewin);
-	MoveControl(notice_v_scrollbar, h - sbarwid, 0);
-	SizeControl(notice_v_scrollbar, sbarwid + 1, v - sbarwid + 1);
+	MoveControl(notice_v_scrollbar, h - floatsbarwid, -1);
+	SizeControl(notice_v_scrollbar, floatsbarwid + 1, v + 1 - floatsbarwid + 1);
 	adjust_notice_scrollbar();
-	(*notice_text)->viewRect.right = h - sbarwid;
-	(*notice_text)->viewRect.bottom = v - sbarwid;
-	(*notice_text)->destRect.right = h - sbarwid;
+	(*notice_text)->viewRect.right = h - floatsbarwid;
+	(*notice_text)->viewRect.bottom = v - floatsbarwid;
+	(*notice_text)->destRect.right = h - floatsbarwid;
 	TECalText(notice_text);
 	/* This will force a full redraw at the next update. */
 	InvalRect(&noticewin->portRect);
@@ -2067,7 +2149,7 @@ zoom_notice(int part)
    anywhere, anytime. */
 
 void
-scores_dialog()
+show_scores_window()
 {
     extern char *get_scores(Side *side);
 
@@ -2075,8 +2157,8 @@ scores_dialog()
 		create_scores_window();
 		append_scores(get_scores(dside));
 	}
-	ShowWindow(scoreswin);
-	SelectWindow(scoreswin);
+	SelectTheWindow(scoreswin);
+	ActivateWindow(scoreswin, true);
 }
 
 void
@@ -2087,9 +2169,9 @@ create_scores_window()
 
 	/* Create the window, color if possible, since images may be in color. */
 	if (hasColorQD) {	
-		scoreswin = GetNewCWindow(wScores, NULL, (WindowPtr) -1L);
+		scoreswin = GetNewCWindow(wScores, NULL, NULL);
 	} else {
-		scoreswin = GetNewWindow(wScores, NULL, (WindowPtr) -1L);
+		scoreswin = GetNewWindow(wScores, NULL, NULL);
 	}
 	SetPort(scoreswin);
 	TextFont(small_font_id);
@@ -2315,7 +2397,7 @@ zoom_scores(int part)
 		zoomrect.top += titleh;
 		(*((WStateDataHandle) ((WindowPeek) scoreswin)->dataHandle))->stdState = zoomrect;
 	}
-	ZoomWindow(scoreswin, part, (scoreswin == FrontWindow()));
+	ZoomWindow(scoreswin, part, false);
 	h = window_width(scoreswin);  v = window_height(scoreswin);
 	MoveControl(scores_v_scrollbar, h - sbarwid, 0);
 	SizeControl(scores_v_scrollbar, sbarwid + 1, v - sbarwid + 1);
