@@ -9,6 +9,7 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kpublic.h"
 #include "tkconq.h"
+Unit *embarkation_unit(Unit *unit);
 
 void really_do_design(Side *side);
 
@@ -17,11 +18,11 @@ static void aux_add_terrain_2(Side *side, Map *map, int cancel);
 static void do_add_terrain_2(Side *side, Map *map, int t);
 static void aux_attack(Side *side, Map *map, int cancel);
 static void aux_build(Side *side, Map *map, int cancel);
-static void impl_build(Side *side, Unit *unit, int u2, int n);
+static void aux_build_2(Side *side, Map *map, int cancel);
+static void impl_build(Side *side, Unit *unit, int u2, int x, int y, int n);
 static void aux_distance(Side *side, Map *map, int cancel);
 static void aux_fire_at(Side *side, Map *map, int cancel);
 static void aux_give_unit(Side *side, Map *map, int cancel);
-static void aux_move_look(Side *side, Map *map);
 static void aux_move_dir(Side *side, Map *map, Unit *unit);
 static void aux_message(Side *side, Map *map, int cancel);
 static void aux_move_to(Side *side, Map *map, int cancel);
@@ -263,7 +264,14 @@ do_build(Side *side)
 	break;
       case 1:
 	/* Only one type to build - do it. */
-	impl_build(side, unit, ufirst, map->prefixarg);
+	if (u_speed(unit->type) > 0
+	    || uu_create_range(unit->type, ufirst) > 0) {
+	    map->argunitid = unit->id;
+	    map->inptype = ufirst;
+	    ask_position(side, map, "Where to build?", aux_build_2);
+	} else {
+	    impl_build(side, unit, ufirst, unit->x, unit->y, map->prefixarg);
+	}
 	break;
       default:
 	/* Player has to choose a type to build. */
@@ -293,9 +301,16 @@ int cancel;
 	if (u2 != NONUTYPE) {
 	    unit = find_unit(map->argunitid);
 	    if (in_play(unit) && side_controls_unit(side, unit)) {
-		impl_build(side, unit, u2, map->prefixarg);
+		if (u_speed(unit->type) > 0
+		    || uu_create_range(unit->type, u2) > 0) {
+		    map->inptype = u2;
+		    ask_position(side, map, "Where to build?", aux_build_2);
+		} else {
+		    impl_build(side, unit, u2, unit->x, unit->y,
+			       map->prefixarg);
+		}
 	    } else {
-		cmd_error(side, "unit disappeared!?");
+		cmd_error(side, "unit disappeared!");
 	    }
 	}
     } else {
@@ -306,10 +321,38 @@ int cancel;
 }
 
 static void
-impl_build(side, unit, u2, n)
+aux_build_2(side, map, cancel)
 Side *side;
-Unit *unit;
-int u2, n;
+Map *map;
+int cancel;
+{
+    int x, y;
+    Unit *unit;
+
+    if (cancel)
+      return;
+    if (map->inpch == '?') {
+	notify(side, "Pick a location where you would like to build.");
+	map->modalhandler = aux_build_2;
+	return;
+    }
+    if (grok_position(side, map, &x, &y, NULL)) {
+	unit = find_unit(map->argunitid);
+	if (unit == NULL) {
+	    cmd_error(side, "Unit disappeared!");
+	} else if (!inside_area(x, y)) {
+	    cmd_error(side, "Cannot build here.");
+	} else {
+	    impl_build(side, unit, map->inptype, x, y, map->prefixarg);
+	}
+    } else {
+        beep(side);
+	map->modalhandler = aux_build_2;
+    }
+}
+
+static void
+impl_build(Side *side, Unit *unit, int u2, int x, int y, int n)
 {
     if (n < 0)
       n = unit_doctrine(unit)->construction_run[u2];
@@ -317,7 +360,11 @@ int u2, n;
       n = 99;
     notify(side, "%s will build %d %s.",
 	   unit_handle(side, unit), n, u_type_name(u2));
-    net_push_build_task(unit, u2, n);
+    net_set_build_task(unit, u2, n);
+    /* We may need to move to get to where we're supposed to build. */
+    if (distance(x, y, unit->x, unit->y) > uu_create_range(unit->type, u2)) {
+	net_push_move_to_task(unit, x, y, uu_create_range(unit->type, u2));
+    }
 }
 
 void
@@ -559,29 +606,39 @@ void
 do_embark(Side *side)
 {
     Map *map = side->ui->curmap;
-    Unit *transport, *occ;
-    Unit *unit = map->curunit;
+    Unit *unit = map->curunit, *transport;
 
     REQUIRE_UNIT(map);
+    transport = embarkation_unit(unit);
+    if (transport != NULL) {
+	net_prep_enter_action(unit, unit, transport);
+	return;
+    }
+    cmd_error(side, "Nothing for %s to enter!", unit_handle(dside, unit));
+}
+
+/* (should be generic) */
+Unit *
+embarkation_unit(Unit *unit)
+{
+    Unit *transport, *occ;
+
     /* look for the first possible transport */
     for_all_stack(unit->x, unit->y, transport) {
 	/* make sure its not the transport we're in and we can enter it */
 	if (transport != unit->transport &&
 	    valid(check_enter_action(unit, unit, transport))) {
-	    net_prep_enter_action(unit, unit, transport);
-	    return;
+	    return transport;
 	}
-
 	/* check the occupants too */
 	for_all_occupants(transport, occ) {
 	    if (occ != unit->transport &&
 		valid(check_enter_action(unit, unit, occ))) {
-		net_prep_enter_action(unit, unit, occ);
-		return;
+		return occ;
 	    }
 	}
     }
-    cmd_error(side, "Nothing for this unit to enter!");
+    return NULL;
 }
 
 void
@@ -599,15 +656,15 @@ do_fire(Side *side)
 
     REQUIRE_UNIT(map);
 
-#if 0
-    if (map->frombutton) {
+    if (map->inpch == '\0') {
+      cmd_error(side, "must use keystroke");
+    }
+    if (0 /*map->frombutton*/) {
 	map->argunitid = unit->id;
-	save_cur(side, map);
 	ask_position(side, map, "Fire at what unit? [click to set]",
 		     aux_fire_at);
 	return;
     }
-#endif
 
     sx = map->inpsx;  sy = map->inpsy;
 
@@ -722,7 +779,7 @@ do_give(Side *side)
     Map *map = side->ui->curmap;
     int something = FALSE;
     int n, u, m, r, gift, actual;
-    Unit *unit = map->curunit, *main = NULL;
+    Unit *unit = map->curunit, *mainunit = NULL;
     char buf[BUFSIZE];
 
     if (nummtypes == 0) {
@@ -731,23 +788,23 @@ do_give(Side *side)
     }
     REQUIRE_UNIT(map);
     u = unit->type;
-    main = unit->transport;
-    if (main == NULL) {
+    mainunit = unit->transport;
+    if (mainunit == NULL) {
 	cmd_error(side, "Nothing to give to here!");
 	return;
     }
-    m = main->type;
+    m = mainunit->type;
     n = (map->prefixarg < 0 ? 1 : map->prefixarg);
 
     buf[0] = '\0';
     for_all_material_types(r) {
-	gift = (n < 0 ? (um_storage_x(m, r) - main->supply[r]) : n);
+	gift = (n < 0 ? (um_storage_x(m, r) - mainunit->supply[r]) : n);
 	if (gift > 0) {
 	    something = TRUE;
 	    /* Be stingy if we're low */
 	    if (2 * unit->supply[r] < um_storage_x(u, r))
 	      gift = max(1, gift/2);
-	    actual = transfer_supply(unit, main, r, gift);
+	    actual = transfer_supply(unit, mainunit, r, gift);
 	    tprintf(buf, " %d %s", actual, m_type_name(r));
 	}
     }
@@ -762,11 +819,12 @@ do_give(Side *side)
 
 /* (but giving to indep should be tested, otherwise might kill unit) */
 
+static void really_do_give_unit(Unit *unit, Side *side2);
+
 void
 do_give_unit(Side *side)
 {
     Map *map = side->ui->curmap;
-    Unit *unit = map->curunit;
     Side *side2;
 
     REQUIRE_UNIT(map);
@@ -794,6 +852,7 @@ aux_give_unit(Side *side, Map *map, int cancel)
     }
 }
 
+static void
 really_do_give_unit(Unit *unit, Side *side2)
 {
     if (unit->side == side2)
@@ -848,38 +907,11 @@ do_map(Side *side)
     if (map == NULL)
       return;
     vp = widget_vp(map);
-    if (strcmp(cmdargstr, "clouds") == 0) {
-	vp->draw_clouds = !vp->draw_clouds;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "control") == 0) {
-	vp->draw_control = !vp->draw_control;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "cover") == 0) {
-	vp->draw_cover = !vp->draw_cover;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "elevations") == 0) {
-	vp->draw_elevations = !vp->draw_elevations;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "fboundaries") == 0) {
-	vp->draw_feature_boundaries = !vp->draw_feature_boundaries;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "fnames") == 0) {
-	vp->draw_feature_names = !vp->draw_feature_names;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "grid") == 0) {
-	vp->draw_grid = !vp->draw_grid;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "iso") == 0) {
+    if (strcmp(cmdargstr, "iso") == 0) {
 	vp->isometric = TRUE;
 	if (map->prefixarg >= 0)
 	  vp->vertscale = map->prefixarg;
 	redraw_map(map);
-    } else if (strcmp(cmdargstr, "lighting") == 0) {
-	vp->draw_lighting = !vp->draw_lighting;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "lines") == 0) {
-	toggle_mapw_lines(map);
-	redraw_map(map);
     } else if (strcmp(cmdargstr, "meridians") == 0) {
 	vp->draw_latlong = !vp->draw_latlong;
 	if (map->prefixarg > 0)
@@ -888,9 +920,6 @@ do_map(Side *side)
     } else if (strcmp(cmdargstr, "over") == 0) {
 	vp->isometric = FALSE;
 	redraw_map(map);
-    } else if (strcmp(cmdargstr, "people") == 0) {
-	vp->draw_people = !vp->draw_people;
-	redraw_map(map);
     } else if (strcmp(cmdargstr, "plan-none") == 0) {
 	if (map->curunit)
 	  net_set_unit_plan_type(dside, map->curunit, PLAN_NONE);
@@ -909,9 +938,6 @@ do_map(Side *side)
     } else if (strcmp(cmdargstr, "plan-random") == 0) {
 	if (map->curunit)
 	  net_set_unit_plan_type(dside, map->curunit, PLAN_RANDOM);
-    } else if (strcmp(cmdargstr, "polygons") == 0) {
-	toggle_mapw_polygons(map);
-	redraw_map(map);
     } else if (strcmp(cmdargstr, "rotl") == 0) {
 	vp->isodir = left_dir(vp->isodir);
 	printf("Looking %s\n", dirnames[vp->isodir]);
@@ -927,42 +953,9 @@ do_map(Side *side)
 	}
 	map->see_all = !map->see_all;
 	redraw_map(map);
-    } else if (strcmp(cmdargstr, "temperature") == 0) {
-	vp->draw_temperature = !vp->draw_temperature;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "transitions") == 0) {
-	toggle_mapw_transitions(map);
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "tstyle-solid") == 0) {
-	map->terrain_style = 0;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "tstyle-pattern") == 0) {
-	map->terrain_style = 1;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "tstyle-image") == 0) {
-	map->terrain_style = 2;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "unames") == 0) {
-	vp->draw_names = !vp->draw_names;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "unit-color") == 0) {
-	int u;
-	map->unit_style = 0;
-	for_all_unit_types(u)
-	  side->ui->uimages[u]->preferred_style = 0;
-	redraw_map(map);
     } else if (strcmp(cmdargstr, "unit-colorize") == 0) {
 	map->colorize_units = !map->colorize_units;
 	redraw_map(map);
-    } else if (strcmp(cmdargstr, "unit-silhouette") == 0) {
-	int u;
-	map->unit_style = 1;
-	for_all_unit_types(u)
-	  side->ui->uimages[u]->preferred_style = 1;
-	redraw_map(map);
-    } else if (strcmp(cmdargstr, "winds") == 0) {
-	vp->draw_winds = !vp->draw_winds;
-	redraw_map(map);
     } else if (*cmdargstr == '!') {
 	eval_tcl_cmd("%s", cmdargstr + 1);
     } else {
@@ -1460,7 +1453,8 @@ do_resign(Side *side)
 	notify(side, "You are already out of the game.");
 	beep(side);
     } else {
-	ask_bool(side, map, "You really want to resign?", FALSE, aux_resign_b);
+	ask_bool(side, map, "Are you sure you want to resign now?",
+		 FALSE, aux_resign_b);
     }
 }
 
@@ -1700,7 +1694,6 @@ do_sleep(Side *side)
 void
 do_standing_orders(Side *side)
 {
-    Map *map = side->ui->curmap;
     int rslt;
 
     if (cmdargstr) {
@@ -1835,13 +1828,13 @@ do_world_map(Side *side)
 void
 do_zoom_in(Side *side)
 {
-    zoom_in_out(side, side->ui->curmap, 1);
+    eval_tcl_cmd("zoom_in_out 1");
 }
 
 void
 do_zoom_out(Side *side)
 {
-    zoom_in_out(side, side->ui->curmap, -1);
+    eval_tcl_cmd("zoom_in_out -1");
 }
 
 #ifdef DESIGNERS
@@ -1850,8 +1843,6 @@ void
 do_design(Side *side)
 {
     Map *map = side->ui->curmap;
-
-    int u;
     Map *map2;
 
     if (!side->designer) {
@@ -1888,7 +1879,6 @@ int cancel;
 void
 really_do_design(Side *side)
 {
-    int u;
     Map *map;
 
     net_become_designer(side);
@@ -1914,7 +1904,11 @@ do_profile(Side *side)
 void
 do_trace(Side *side)
 {
-    cmd_error(side, "Not implemented.");
+    extern int activity_trace;
+
+    if (activity_trace)
+      dump_activity_trace();
+    activity_trace = !activity_trace;
 }
 
 #endif /* DEBUGGING */
