@@ -1,5 +1,5 @@
 /* Definitions of image families for the Tk interface to Xconq.
-   Copyright (C) 1998 Stanley T. Shebs.
+   Copyright (C) 1998, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,9 +23,11 @@ typedef struct a_tk_image {
     XColor **cmap;
 } TkImage;
 
-extern TkImage *init_tk_image PARAMS ((Image *img));
-extern TkImage *get_tk_image PARAMS ((Image *img));
-extern ImageFamily *tk_load_imf PARAMS ((ImageFamily *imf));
-extern ImageFamily *tk_interp_imf PARAMS ((ImageFamily *imf, Image *img,
-					   int force));
-extern void reverse_bit_endianness PARAMS ((char *rawdata, int numbytes));
+extern int use_clip_mask;
+
+extern TkImage *init_tk_image(Image *img);
+extern TkImage *get_tk_image(Image *img);
+extern ImageFamily *tk_load_imf(ImageFamily *imf);
+extern ImageFamily *tk_interp_imf(ImageFamily *imf, Image *img, int force);
+extern void reverse_bit_endianness(char *rawdata, int numbytes);
+extern ImageFamily *tk_find_imf(char *name);
