@@ -8,25 +8,27 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "tkconq.h"
+Unit *embarkation_unit(Unit *unit);
 
-extern ImageFamily *get_generic_images();
+void init_other_images(void);
+void really_do_design(Side *side);
 
 XColor *request_color(char *name);
 void set_colors(void);
 void init_emblem(Side *side2);
 
+int can_give_take(Unit *unit);
+
 #ifndef __STDC__
 void abort ();
 #endif
 
-Pixmap tranpics[NUMPOWERS];
 Pixmap fuzzpics[NUMPOWERS];
-Pixmap bordpics[NUMPOWERS];
-Pixmap connpics[NUMPOWERS];
-Pixmap rrpics[NUMPOWERS];
 Pixmap windpics[5][NUMDIRS];
 Pixmap antpic;
 
+ImageFamily *generic_transition;
+
 short utype_indexes[MAXUTYPES];
 short mtype_indexes[MAXMTYPES];
 
@@ -39,29 +41,7 @@ static void init_unit_images(void);
 static void init_terrain_images(void);
 static void init_emblem_images(void);
 static void report_missing_images(void);
-static void tk_describe_image PARAMS ((Side *side, Image *img));
-
-#if 0
-static char *variant_default PARAMS ((Variant *var));
-static char *variant_name PARAMS ((Variant *var));
-static void implement_variant PARAMS ((char *value, Variant *var));
-static int valid_variant PARAMS ((char *value, Variant *var));
-static int type_player PARAMS ((char *value));
-static void init_assignw_player PARAMS ((Assignw *assignw, Player *player));
-static int can_open_display PARAMS ((char *dpyname));
-static int boolean_lisp PARAMS ((char *value));
-#endif
-
-extern char *tclbuf;
-
-static int loop, chosen_game;
-
-/* we could share this buffers with other modules */
-char buf1[BUFSIZE], buf2[BUFSIZE], buf3[BUFSIZE], errormess[BUFSIZE];
-
-Obj *variants;
-
-char *imflib = IMFLIB;
+static void tk_describe_image(Side *side, Image *img);
 
 /* Display and window globals used in callbacks from generic imf
    handling code. */
@@ -79,12 +59,17 @@ int min_h_for_unit_image = 16;
 /* various bitmap definitions. */
 #include "bitmaps/lookglass.b"
 #include "bitmaps/lookmask.b"
+#if 0
 #include "bitmaps/movecurs.b"
 #include "bitmaps/movemask.b"
+#endif
 #include "bitmaps/shootcurs.b"
 #include "bitmaps/shootmask.b"
 #include "bitmaps/buildcurs.b"
 #include "bitmaps/buildmask.b"
+#include "bitmaps/return.b"
+#include "bitmaps/sleep.b"
+#include "bitmaps/delay.b"
 
 #include "bitmaps/closer.b"
 #include "bitmaps/farther.b"
@@ -92,11 +77,6 @@ int min_h_for_unit_image = 16;
 #include "bitmaps/rotl.b"
 #include "bitmaps/rotr.b"
 
-#if 0
-#include "bitmaps/gray.b"
-#include "bitmaps/darkgray.b"
-#endif
-
 #include "bitmaps/hex16.b"
 #include "bitmaps/hex16b.b"
 #include "bitmaps/hex32.b"
@@ -104,22 +84,10 @@ int min_h_for_unit_image = 16;
 #include "bitmaps/hex64.b"
 #include "bitmaps/hex64b.b"
 
-#include "bitmaps/tran16.b"
-#include "bitmaps/tran32.b"
-
 #include "bitmaps/fuzz16.b"
 #include "bitmaps/fuzz32.b"
 #include "bitmaps/fuzz64.b"
 
-#include "bitmaps/bord16.b"
-#include "bitmaps/bord32.b"
-#include "bitmaps/bord64.b"
-
-#include "bitmaps/conn16.b"
-#include "bitmaps/conn32.b"
-#include "bitmaps/rr16.b"
-#include "bitmaps/rr32.b"
-
 #include "bitmaps/hex16iso.b"
 #include "bitmaps/hex32iso.b"
 #include "bitmaps/hex64iso.b"
@@ -196,6 +164,7 @@ Side *side;
     if (side_wants_display(side)) {
         side->ui = (UI *) xmalloc(sizeof(UI));
 	Dprintf("One UI is %d bytes.\n", sizeof(UI));
+	get_preferences();
     } else {
 	side->ui = NULL;
     }
@@ -208,11 +177,10 @@ Side *side;
 int toplevel_display_used = FALSE;
 
 void
-init_display()
+init_display(void)
 {
-    int argc, p, u, t, s, depth;
-    char *dpyname, *str;
-    Display *dpy;
+    int p, u, t, s;
+    char *dpyname;
 
     dpyname = dside->player->displayname;
 
@@ -225,15 +193,8 @@ init_display()
 
     /* Set up things shared by all the windows. */
     dside->ui->dflt_color_terr_images = TRUE;
-    dside->ui->dflt_color_unit_images = TRUE;
     dside->ui->dflt_color_embl_images = TRUE;
     dside->ui->default_latlong_interval = 600;
-#if 0
-    dside->ui->bonw = BLACKONWHITE;
-    depth = XDisplayCells(dpy, dside->ui->screen);
-    DGprintf("%d different colors available ...\n", depth);
-    dside->ui->monochrome = (depth <= 2);
-#endif
     for (p = 0; p < NUMPOWERS; ++p) {
 	for_all_terrain_types(t) {
 	    /* Decide what display technique to use at each power. */
@@ -255,6 +216,10 @@ init_display()
     set_colors();
     tmp_root_window = Tk_MainWindow(interp);
     tmp_valid = TRUE;
+
+    if (DebugG == 2)
+      XSynchronize(Tk_Display(tmp_root_window), True);
+
     init_unit_images();
     init_terrain_images();
     init_emblem_images();
@@ -308,7 +273,7 @@ init_display()
    the graphics interface to supply a substitute. */
 
 void
-set_colors()
+set_colors(void)
 {
     int	t, i;
 
@@ -342,7 +307,7 @@ set_colors()
    space that should go to images. */
 
 static void
-set_optional_colors()
+set_optional_colors(void)
 {
     XColor *color;
 
@@ -415,11 +380,10 @@ Side *side;
    one for each kind of mode or tool.  Cursors should always be 16x16. */
 
 static void
-init_cursors()
+init_cursors(void)
 {
     int i;
     Tk_Window rootwin = Tk_MainWindow(interp);
-    Display *dpy = Tk_Display(rootwin);
 
     Tk_MakeWindowExist(rootwin);
 
@@ -529,7 +493,7 @@ request_color(char *name)
 	/* Now try the imf library. */
 	if (rslt == NULL) {
 	    int dummy, red, grn, blu;
-	    ImageFamily *imf = get_generic_images(dside, name);
+	    ImageFamily *imf = get_generic_images(name);
 	    Image *img;
 	    XColor col;
 
@@ -553,11 +517,10 @@ request_color(char *name)
 }
 
 static void
-init_bitmaps()
+init_bitmaps(void)
 {
-    int i, dir;
+    int dir;
     Tk_Window rootwin = Tk_MainWindow(interp);
-    Display *dpy = Tk_Display(rootwin);
 
     Tk_MakeWindowExist(rootwin);
 
@@ -587,15 +550,6 @@ init_bitmaps()
     dside->ui->bhexpics[6] =
       Tk_GetBitmap(interp, rootwin, Tk_GetUid("hex64b"));
 
-    Tk_DefineBitmap(interp, Tk_GetUid("tran16"),
-		    tran16_bits, tran16_width, tran16_height);
-    tranpics[4] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("tran16"));
-    Tk_DefineBitmap(interp, Tk_GetUid("tran32"),
-		    tran32_bits, tran32_width, tran32_height);
-    tranpics[5] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("tran32"));
-
     Tk_DefineBitmap(interp, Tk_GetUid("fuzz16"),
 		    fuzz16_bits, fuzz16_width, fuzz16_height);
     fuzzpics[4] =
@@ -609,38 +563,6 @@ init_bitmaps()
     fuzzpics[6] =
       Tk_GetBitmap(interp, rootwin, Tk_GetUid("fuzz64"));
 
-
-    Tk_DefineBitmap(interp, Tk_GetUid("bord16"),
-		    bord16_bits, bord16_width, bord16_height);
-    bordpics[4] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("bord16"));
-    Tk_DefineBitmap(interp, Tk_GetUid("bord32"),
-		    bord32_bits, bord32_width, bord32_height);
-    bordpics[5] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("bord32"));
-    Tk_DefineBitmap(interp, Tk_GetUid("bord64"),
-		    bord64_bits, bord64_width, bord64_height);
-    bordpics[6] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("bord64"));
-
-    Tk_DefineBitmap(interp, Tk_GetUid("conn16"),
-		    conn16_bits, conn16_width, conn16_height);
-    connpics[4] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("conn16"));
-    Tk_DefineBitmap(interp, Tk_GetUid("conn32"),
-		    conn32_bits, conn32_width, conn32_height);
-    connpics[5] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("conn32"));
-
-    Tk_DefineBitmap(interp, Tk_GetUid("rr16"),
-		    rr16_bits, rr16_width, rr16_height);
-    rrpics[4] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("rr16"));
-    Tk_DefineBitmap(interp, Tk_GetUid("rr32"),
-		    rr32_bits, rr32_width, rr32_height);
-    rrpics[5] =
-      Tk_GetBitmap(interp, rootwin, Tk_GetUid("rr32"));
-
     Tk_DefineBitmap(interp, Tk_GetUid("hex16iso"),
 		    hex16iso_bits, hex16iso_width, hex16iso_height);
     dside->ui->hexisopics[4] =
@@ -745,9 +667,12 @@ init_bitmaps()
 		    kill_bits, kill_width, kill_height);
     dside->ui->hitpics[2] =
       Tk_GetBitmap(interp, rootwin, Tk_GetUid("kill"));
+    /* Set up three shades of gray stipple. */
+    dside->ui->grays[darkgray] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("gray75"));
     dside->ui->grays[gray] =
       Tk_GetBitmap(interp, rootwin, Tk_GetUid("gray50"));
-    dside->ui->grays[darkgray] =
+    dside->ui->grays[lightgray] =
       Tk_GetBitmap(interp, rootwin, Tk_GetUid("gray25"));
     Tk_DefineBitmap(interp, Tk_GetUid("dotdotdot"),
 		    dots_bits, dots_width, dots_height);
@@ -764,6 +689,12 @@ init_bitmaps()
 		    buildcursor_bits, buildcursor_width, buildcursor_height);
     Tk_DefineBitmap(interp, Tk_GetUid("looking_glass"),
 		    lookglass_bits, lookglass_width, lookglass_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("return"),
+		    return_bits, return_width, return_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("sleep"),
+		    sleep_bits, sleep_width, sleep_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("delay"),
+		    delay_bits, delay_width, delay_height);
     Tk_DefineBitmap(interp, Tk_GetUid("closer"),
 		    closer_bits, closer_width, closer_height);
     Tk_DefineBitmap(interp, Tk_GetUid("farther"),
@@ -776,10 +707,32 @@ init_bitmaps()
 		    rotr_bits, rotr_width, rotr_height);
     Tk_DefineBitmap(interp, Tk_GetUid("laurels"),
 		    laurels_bits, laurels_width, laurels_height);
+#ifdef DESIGNERS
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_cell"),
+		    curscell_bits, curscell_width, curscell_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_unit"),
+		    cursunit_bits, cursunit_width, cursunit_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_people"),
+		    curspeop_bits, curspeop_width, curspeop_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_control"),
+		    curspeop_bits, curspeop_width, curspeop_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_feature"),
+		    cursfeat_bits, cursfeat_width, cursfeat_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_material"),
+		    cursmaterial_bits, cursmaterial_width, cursmaterial_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_elev"),
+		    curselev_bits, curselev_width, curselev_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_temp"),
+		    curstemp_bits, curstemp_width, curstemp_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_clouds"),
+		    cursclouds_bits, cursclouds_width, cursclouds_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("paint_winds"),
+		    curswinds_bits, curswinds_width, curswinds_height);
+#endif
 }
 
 static void
-init_unit_images()
+init_unit_images(void)
 {
     int u;
     ImageFamily *imf;
@@ -806,8 +759,10 @@ init_unit_images()
 
 /* Collect images for all the terrain types. */
 
+static void init_shades(int t);
+
 static void
-init_terrain_images()
+init_terrain_images(void)
 {
     int t, p;
     ImageFamily *imf;
@@ -824,6 +779,7 @@ init_terrain_images()
 	/* Precalculate which images to use at which magnifications. */
 	for (p = 0; p < NUMPOWERS; ++p) {
 	    timg = best_image(dside->ui->timages[t], hws[p], hhs[p]);
+	    dside->ui->besttimages[p][t] = timg;
 	    if (timg != NULL) {
 		tkimg = (TkImage *) timg->hook;
 		if (tkimg != NULL) {
@@ -859,6 +815,7 @@ init_terrain_images()
     }
 }
 
+static void
 init_shades(int t)
 {
     XColor col, *newcolor, *basecolor = dside->ui->cellcolor[t];
@@ -892,7 +849,7 @@ init_shades(int t)
 /* Set up a side's view of everybody else's colors and emblems. */
 
 static void
-init_emblem_images()
+init_emblem_images(void)
 {
     Side *side2;
     
@@ -954,29 +911,42 @@ init_emblem(Side *side2)
     dside->ui->eimages_loaded[s2] = TRUE;
 }
 
-init_other_images()
+/* Collect random other images via the image family system, such as
+   blasts, terrain transitions, etc. */
+
+void
+init_other_images(void)
 {
     ImageFamily *imf;
     Tk_Window tkwin = Tk_MainWindow(interp);
 
     tmp_root_window = tkwin;
     tmp_valid = TRUE;
-    imf = get_generic_images(dside, "miss");
+    imf = get_generic_images("miss");
     record_imf_get(imf);
     if (DebugG)
       describe_imf(dside, "blast", side_desig(dside), imf);
     dside->ui->blastimages[0] = imf;
-    imf = get_generic_images(dside, "hit");
+    imf = get_generic_images("hit");
     record_imf_get(imf);
     if (DebugG)
       describe_imf(dside, "blast", side_desig(dside), imf);
     dside->ui->blastimages[1] = imf;
-    imf = get_generic_images(dside, "kill");
+    imf = get_generic_images("kill");
     record_imf_get(imf);
     if (DebugG)
       describe_imf(dside, "blast", side_desig(dside), imf);
-    tmp_valid = FALSE;
     dside->ui->blastimages[2] = imf;
+    /* Get terrain transition bitmaps, but only if clip masks are
+       available. */
+    if (use_clip_mask) {
+	imf = get_generic_images("transition");
+	record_imf_get(imf);
+	if (DebugG)
+	  describe_imf(dside, "transition", side_desig(dside), imf);
+	generic_transition = imf;
+    }
+    tmp_valid = FALSE;
 }
 
 /* Describe (for debugging) Tk-specific parts of an image. */
@@ -992,7 +962,8 @@ tk_describe_image(Side *side, Image *img)
 	       tkimg->mono, tkimg->colr, tkimg->mask, tkimg->solid);
 }
 
-update_view_controls_info()
+void
+update_view_controls_info(void)
 {
     eval_tcl_cmd("update_view_controls_info %d %d %d %d %d %d %d %d %d",
 		 all_see_all,
@@ -1006,15 +977,17 @@ update_view_controls_info()
 		 clouds_defined());
 }
 
-
+void
 update_action_controls_info(Map *map)
 {
-    int canact = FALSE, canplan = FALSE, canmove = FALSE, canreturn = FALSE;
-    int canbuild = FALSE, canrepair = FALSE;
-    int canattack = FALSE, canfire = FALSE;
+    int canact = FALSE, canplan = FALSE, canmove = FALSE;
+    int canbuild = FALSE, canattack = FALSE;
+    int rslt;
     Unit *unit;
+    char flagsbuf[BUFSIZE];
 
     unit = map->curunit;
+    strcpy(flagsbuf, "{");
     if (in_play(unit)) {
 	if (u_acp(unit->type) > 0)
 	  canact = TRUE;
@@ -1022,27 +995,64 @@ update_action_controls_info(Map *map)
 	  canplan = TRUE;
 	if (can_move_at_all(unit)) {
 	    canmove = TRUE;
-	    if (nummtypes > 0)
-	      canreturn = TRUE;
+	    if (nummtypes > 0) /* needs to be more precise */
+	      strcat(flagsbuf, " can_return");
 	}
-	if ((can_create(unit) || can_complete(unit))
-	    && (unit->transport == NULL
-		|| !completed(unit->transport)
-		|| uu_occ_can_build(unit->transport->type, unit->type))
+	if (((can_create(unit) || can_complete(unit))
+	     && (unit->transport == NULL
+		 || !completed(unit->transport)
+		 || uu_occ_can_build(unit->transport->type, unit->type)))
 	    || can_develop(unit)) {
 	    canbuild = TRUE;
 	}
 	if (can_repair(unit)) {
-	    canrepair = TRUE;
+	    strcat(flagsbuf, " can_repair");
 	}
 	if (can_attack(unit)) {
 	    canattack = TRUE;
 	}
 	if (can_fire(unit)) {
-	    canfire = TRUE;
+	    strcat(flagsbuf, " can_fire");
+	}
+	if (can_detonate(unit)) {
+	    strcat(flagsbuf, " can_detonate");
+	}
+	if (unit->transport != NULL) {
+	    strcat(flagsbuf, " can_disembark");
+	}
+	if (can_disband(unit)) {
+	    strcat(flagsbuf, " can_disband");
+	}
+	if (can_add_terrain(unit)) {
+	    strcat(flagsbuf, " can_add_terrain");
+	}
+	if (can_add_terrain(unit)) {
+	    strcat(flagsbuf, " can_remove_terrain");
+	}
+	if (can_give_take(unit)) {
+	    strcat(flagsbuf, " can_give_take");
+	}
+	if (embarkation_unit(unit) != NULL) {
+	    strcat(flagsbuf, " can_embark");
+	}
+	rslt = check_transfer_part_action(unit, unit, unit->hp / 2, NULL);
+	if (valid(rslt)) {
+	    strcat(flagsbuf, " can_detach");
 	}
     }
-    eval_tcl_cmd("update_action_controls_info %d %d %d %d %d %d %d",
-		 canact, canplan, canmove, canreturn,
-		 canbuild, canattack, canfire);
+    strcat(flagsbuf, "}");
+    eval_tcl_cmd("update_action_controls_info %d %d %d %d %d %s",
+		 canact, canplan, canmove, canbuild, canattack, flagsbuf);
+}
+
+int
+can_give_take(Unit *unit)
+{
+    int u = unit->type, m;
+
+    for_all_material_types(m) {
+	if (um_storage_x(u, m) > 0)
+	  return TRUE;
+    }
+    return FALSE;
 }
