@@ -1,5 +1,5 @@
 /* This program converts a collection of X bitmaps into imf format.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,13 +10,7 @@ any later version.  See the file COPYING.  */
 #include "misc.h"
 #include "lisp.h"
 #include "imf.h"
-#include "xutil.h"
-
-#include <string.h>
-
-extern int numimages;
-
-extern ImageFamily **images;
+#include "xcutil.h"
 
 char tmpbuf[500];
 
@@ -24,9 +18,7 @@ int numremotes;
 int my_rid;
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char *argv[])
 {
     int i;
 
