@@ -459,7 +459,7 @@ UnitCloseup *unitcloseup;
 	buffer_append(1);
 	if (plan->tasks) {
 	    for (task = plan->tasks; task != NULL; task = task->next) {
-		task_desc(buffer, unit->side, task);
+		task_desc(buffer, unit->side, unit, task);
 		buffer_append(1);
 	    }
 	}
@@ -2005,7 +2005,7 @@ Task *task;
 	XtVaSetValues(ordi->task, XtNlabel, taskdefns[task->type].name, NULL);
 
 	/* get task description, chop task name and separator */
-	task_desc(buffer, side, task);
+	task_desc(buffer, side, NULL, task);
 	lab = buffer + strlen(taskdefns[task->type].name);
 	while (*lab == ' ' || *lab == ',')
 	    lab++;
@@ -3005,7 +3005,7 @@ XtPointer call_data;
 	    beep(side);
 	}
 	/* get task description, chop task name and separator */
-	task_desc(buffer, side, task);
+	task_desc(buffer, side, NULL, task);
 	lab = buffer + strlen(taskdefns[task->type].name);
 	while (*lab == ' ' || *lab == ',')
 	  lab++;
