@@ -1,5 +1,5 @@
 /* Commands for the X11 interface to Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -929,11 +929,6 @@ Side *side;
     if (strcmp(cmdargstr, "close") == 0) {
 	/* (should trash map and popdown) */
 	notify(side, "can't close a map yet");
-    } else if (strcmp(cmdargstr, "color") == 0) {
-	side->ui->pref_color_unit_images = TRUE;
-	for_all_unit_types(u)
-	  side->ui->uimages[u]->preferred_style = 0;
-	do_refresh(side);
     } else if (strncmp(cmdargstr, "contour-color=", 14) == 0) {
 	side->ui->contour_color = request_color(side, cmdargstr+14);
 	do_refresh(side);
@@ -966,11 +961,6 @@ Side *side;
 	} else {
 	    cmd_error(side, "Reverse video is only for monochrome!");
 	}
-    } else if (strcmp(cmdargstr, "silhouette") == 0) {
-	side->ui->pref_color_unit_images = FALSE;
-	for_all_unit_types(u)
-	  side->ui->uimages[u]->preferred_style = 1;
-	do_refresh(side);
     } else if (strcmp(cmdargstr, "solid") == 0) {
 	side->ui->pref_solid_color_terrain = TRUE;
 	do_refresh(side);
