@@ -8,13 +8,19 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #define side_has_local_display(side) ((side)->ui != NULL)
-extern void notify_instructions PARAMS ((void));
+extern void notify_instructions(void);
 extern int sendnow;
+extern void auto_pick_new_research(Unit *unit);
+extern int auto_pick_new_build_task(Unit *unit);
+extern void auto_pick_new_plan(Unit *unit);
 #include "kpublic.h"
 #include "cmdline.h"
 extern char *args_used;
 extern char *default_player_spec;
-#include "xconq.h"
+
+#include "tcltk/tkconq.h"
+
+#include <unistd.h>
 
 extern int hosting;
 extern char *connection_method_name;
@@ -22,13 +28,21 @@ extern int connection_method;
 
 int you_player;
 
+void host_the_game(char *option);
+void try_join_game(char *hostport);
+
 /* Local function declarations. */
 
-extern void init_all_displays PARAMS ((void));
+extern void init_all_displays(void);
 
-void init_x_signal_handlers PARAMS ((void));
-static int handle_x_error PARAMS ((Display *dpy, XErrorEvent *evt));
-static int handle_xio_error PARAMS ((Display *dpy));
+void init_x_signal_handlers(void);
+static int handle_x_error(Display *dpy, XErrorEvent *evt);
+static int handle_xio_error(Display *dpy);
+
+void unit_advance_dialog(Unit *unit);
+void global_advance_dialog(Side *side, Unit *unit, int s);
+int unit_build_dialog(Unit *unit);
+void unit_plan_dialog(Unit *unit);
 
 int announced = FALSE;
 
@@ -37,9 +51,7 @@ char *announcemsg = NULL;
 /* The main program. */
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char *argv[])
 {
     extern uid_t games_uid;
 
@@ -77,38 +89,16 @@ char *argv[];
     }
 
     if (option_game_to_join != NULL) {
-	int rslt;
-
 	parse_command_line(argc, argv, variant_options);
 	set_variants_from_options();
 	parse_command_line(argc, argv, player_options);
 	set_players_from_options();
 	/* Go through once more and complain about anything not used. */
 	parse_command_line(argc, argv, leftover_options);
-	hosting = FALSE;
-	open_remote_connection(option_game_to_join);
-	if (connection_method > 0) {
-	    sendnow = TRUE;
-	    rslt = send_join(default_player_spec);
-	    sendnow = FALSE;
-	    if (rslt) {
-		master_rid = 1;
-		while (TRUE) {
-		    receive_data(0);
-		    if (!beforestart) {
-			break;
-		    }
-		}
-	    } else {
-		close_remote_connection();
-		fprintf(stderr, "Unable to join game \"%s\", exiting\n",
-			option_game_to_join);
-		exit(1);
-	    }
-	}
+	try_join_game(option_game_to_join);
     } else {
 	if (option_popup_new_game_dialog) {
-	    check_player_displays();
+	    /* nothing to do */
 	} else {
 	    load_all_modules();
 	    check_game_validity();
@@ -117,40 +107,13 @@ char *argv[];
 	    parse_command_line(argc, argv, player_options);
 	    set_players_from_options();
 	    make_trial_assignments();
-	    check_player_displays();
 	    /* Complain about anything that's left. */
 	    parse_command_line(argc, argv, leftover_options);
 	    /* (still need to merge some databases derived from display) */
 	}
     }
     if (option_game_to_host != NULL) {
-	hosting = TRUE;
-	(assignments[0].player)->rid = 1;
-	(assignments[1].player)->remotewanted = "x";
-	open_remote_connection(option_game_to_host);
-	numremotewaiting = 1;
-	master_rid = 1;
-	while (numremotewaiting > 0) {
-	    int i;
-	    Player *player;
-
-	    receive_data(0);
-	    numremotewaiting = 0;
-	    for (i = 0; i < numsides; ++i) {
-		if (assignments[i].side && (assignments[i].side)->ingame) {
-		    player = assignments[i].player;
-		    if (player->remotewanted != NULL && player->rid == 0) {
-			player->displayname = copy_string(":0.0");
-			player->rid = 2; /* for now */
-			sendnow = TRUE;
-			send_version(player->rid);
-			sendnow = FALSE;
-			sleep(1);
-			download_to_player(player);
-		    }
-		}
-	    }
-	}
+	host_the_game(option_game_to_host);
     }
     if (option_game_to_join == NULL) {
 	launch_game();
@@ -185,6 +148,67 @@ add_default_player()
     return player;
 }
 
+void
+host_the_game(char *option)
+{
+    int i;
+    Player *player;
+
+    hosting = TRUE;
+    for (i = 0; i < numsides; ++i) {
+	if (assignments[i].side && (assignments[i].side)->ingame) {
+	    player = assignments[i].player;
+	    if (player->displayname != NULL) {
+		player->rid = 1;
+	    } else if (player->name) {
+		player->remotewanted = player->name;
+	    }
+	}
+    }
+    open_remote_connection(option);
+    master_rid = 1;
+    while (numremotewaiting > 0) {
+	receive_data(0);
+	numremotewaiting = 0;
+	for (i = 0; i < numsides; ++i) {
+	    if (assignments[i].side && (assignments[i].side)->ingame) {
+		player = assignments[i].player;
+		if (player->remotewanted != NULL && player->rid == 0) {
+		    ++numremotewaiting;
+		}
+	    }
+	}
+    }
+}
+
+void
+try_join_game(char *hostport)
+{
+    int rslt;
+
+    hosting = FALSE;
+    open_remote_connection(copy_string(hostport));
+    if (connection_method > 0) {
+	sendnow = TRUE;
+	rslt = send_join(default_player_spec);
+	sendnow = FALSE;
+	if (rslt) {
+	    master_rid = 1;
+	    while (TRUE) {
+		receive_data(0);
+		if (!beforestart) {
+		    break;
+		}
+	    }
+	} else {
+	    close_remote_connection();
+	    fprintf(stderr, "Unable to join game \"%s\", exiting\n",
+		    option_game_to_join);
+	    exit(1);
+	}
+    }
+}
+
 /* An init error needs to have the command re-run. */
 
 void
@@ -332,8 +356,7 @@ finish_lengthy_process()
 /* This should be called before any sort of normal exit. */
 
 void
-exit_xconq(side)
-Side *side;
+exit_xconq(Side *side)
 {
     close_displays();
     printf("\n%s\n", exit_commentary(side));
@@ -345,6 +368,27 @@ void
 add_remote_player(name)
 char *name;
 {
+    int i;
+    Player *player;
+
+    if (name == NULL)
+      name = "somebody";
+    for (i = 0; i < numsides; ++i) {
+	if (assignments[i].side && (assignments[i].side)->ingame) {
+	    player = assignments[i].player;
+	    if (player->remotewanted != NULL && player->rid == 0) {
+		player->displayname = copy_string(name);
+		player->rid = 2; /* for now */
+		sendnow = TRUE;
+		send_version(player->rid);
+		sendnow = FALSE;
+		sleep(1);
+		download_to_player(player);
+		/* (should do user feedback here?) */
+		return;
+	    }
+	}
+    }
 }
 
 int
@@ -372,7 +416,7 @@ init_all_displays()
     for_all_sides(side) {
 	if (side_has_display(side)) {
 	    if (side_has_local_display(side))
-	      init_display(side);
+	      init_display();
 	    ++numdisplays;
 	}
     }
@@ -397,6 +441,28 @@ int id;
     }
 }
 
-unit_advance_dialog(Unit *unit) { auto_pick_new_research(unit); }
-global_advance_dialog(Side *side, Unit *unit, int s) { auto_pick_new_research(unit); }
-unit_build_dialog(Unit *unit, Task *task) { return 0; }
+void
+unit_advance_dialog(Unit *unit)
+{
+    auto_pick_new_research(unit);
+}
+
+void
+global_advance_dialog(Side *side, Unit *unit, int s)
+{
+    auto_pick_new_research(unit);
+}
+
+int
+unit_build_dialog(Unit *unit)
+{
+    int u = auto_pick_new_build_task(unit);
+
+    return u;
+}
+
+void
+unit_plan_dialog(Unit *unit)
+{
+    auto_pick_new_plan(unit);
+}
