@@ -34,14 +34,6 @@ any later version.  See the file COPYING.  */
 #include "ui.h"
 #include "print.h"
 
-#ifndef IMFLIB
-#ifdef VMS
-#define IMFLIB "[-.lib-x11]"
-#else /* not VMS */
-#define IMFLIB "../lib-x11"
-#endif /* VMS */
-#endif /* IMFLIB */
-
 enum grayshade {
   black,
   gray,
