@@ -1987,7 +1987,7 @@ Map *map;
 	plan_desc(infobuf, unit);
 	draw_info_text(side, map, 0, 3, len, infobuf);
 	for (task = unit->plan->tasks; task != NULL; task = task->next) {
-	    task_desc(infobuf, unit->side, task);
+	    task_desc(infobuf, unit->side, unit, task);
 	    draw_info_text(side, map, 0, row++, len, infobuf);
 	}
     }
