@@ -1,5 +1,5 @@
 /* Map handling for the X11 interface to Xconq.
-   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -428,12 +428,9 @@ char *geospec;
 	       color and a 0 bit in the background color.
 	       That's why we pass in 1 & 0 to draw_unit_image() here for
 	       fgcolor & bgcolor. */
-	    side->ui->uimages[i - 1]->preferred_style = 1;
 	    draw_unit_image(side, pic, 0, 0,
 			    min_w_for_unit_image, min_h_for_unit_image,
 			    i - 1, -1, 1, 0, 0);
-	    side->ui->uimages[i - 1]->preferred_style =
-	      (side->ui->pref_color_unit_images ? 0 : 1);
 
 	    /* Name the widget. */
 	    build_name(wbuff, "utype_", u_type_name(i - 1));
