@@ -165,7 +165,6 @@ typedef struct a_ui {
     short dflt_color_terr_images;
     short dflt_color_embl_images;
     short pref_solid_color_terrain;
-    short pref_color_unit_images;
     short default_latlong_interval;
     GC gc;			/* a tmp graphics context for this display */
     GC textgc;			/* foreground on background text */
@@ -396,7 +395,6 @@ typedef struct a_map {
     Pixmap panner_pix;
     /* How to draw the map. */
     short seeall;		/* True if viewing world data directly */
-    short use_color_unit_images;
     short use_color_terr_images;
     short use_color_embl_images;
     short fullpanel;
